/* Domino widget style for KDE 3
   Copyright (C) 2006 Michael Lentner <michaell@gmx.net>

   based on the KDE style "Plastik":
   Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

   based on the KDE style "dotNET":
   Copyright (C) 2001-2002, Chris Lee <clee@kde.org>
				Carsten Pfeiffer <pfeiffer@kde.org>
				Karol Szwed <gallium@kde.org>
   Drawing routines completely reimplemented from KDE3 HighColor, which was
   originally based on some stuff from the KDE2 HighColor.

   based on drawing routines of the style "Keramik":
   Copyright (c) 2002 Malte Starostik <malte@kde.org>
		 (c) 2002,2003 Maksim Orlovich <mo002j@mail.rochester.edu>
   based on the KDE3 HighColor Style
   Copyright (C) 2001-2002 Karol Szwed		<gallium@kde.org>
		 (C) 2001-2002 Fredrik Hglund	<fredrik@kde.org>
   Drawing routines adapted from the KDE2 HCStyle,
   Copyright (C) 2000 Daniel M. Duley		<mosfet@kde.org>
		 (C) 2000 Dirk Mueller			<mueller@kde.org>
		 (C) 2001 Martijn Klingens		<klingens@kde.org>
   Progressbar code based on KStyle,
   Copyright (C) 2001-2002 Karol Szwed <gallium@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifndef __DOMINO_H
#define __DOMINO_H

#include <kstyle.h>
#include <qintcache.h>
#include <ktoolbarbutton.h>
#include "misc.h"
#include "data.h"
#include <qpixmapcache.h>

#define u_arrow -4,1, 2,1, -3,0, 1,0, -2,-1, 0,-1, -1,-2
#define d_arrow -4,-2, 2,-2, -3,-1, 1,-1, -2,0, 0,0, -1,1
#define l_arrow 0,-3, 0,3,-1,-2,-1,2,-2,-1,-2,1,-3,0
#define r_arrow -2,-3,-2,3,-1,-2, -1,2,0,-1,0,1,1,0

#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)


class DominoStyle : public KStyle
{
	Q_OBJECT

public:
	DominoStyle();
	virtual ~DominoStyle();

	void polish(QApplication* app );
	void polish(QWidget* widget );
	void polish( QPalette &p );
	void unPolish(QWidget* widget );
	
	void drawKStylePrimitive(KStylePrimitive kpe,
				QPainter* p,
				const QWidget* widget,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags = Style_Default,
				const QStyleOption& = QStyleOption::Default ) const;

	void drawPrimitive(PrimitiveElement pe,
				QPainter *p,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags = Style_Default,
				const QStyleOption &opt = QStyleOption::Default ) const;

	void drawControl(ControlElement element,
				QPainter *p,
				const QWidget *widget,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags = Style_Default,
				const QStyleOption& = QStyleOption::Default ) const;

	void drawControlMask( ControlElement, QPainter *, const QWidget *, const QRect &, const QStyleOption &) const;

	void drawComplexControl(ComplexControl control,
				QPainter *p,
				const QWidget *widget,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags = Style_Default,
				SCFlags controls = SC_All,
				SCFlags active = SC_None,
				const QStyleOption& = QStyleOption::Default ) const;

	int pixelMetric(PixelMetric m,
				const QWidget *widget = 0 ) const;

	QRect subRect(SubRect r,
				const QWidget *widget ) const;

	QRect querySubControlMetrics(ComplexControl control,
				const QWidget *widget,
				SubControl subcontrol,
				const QStyleOption &opt = QStyleOption::Default ) const;

	void drawComplexControlMask(QStyle::ComplexControl c,
				QPainter *p,
				const QWidget *w,
				const QRect &r,
				const QStyleOption &o=QStyleOption::Default) const;

	QSize sizeFromContents(QStyle::ContentsType t,
				const QWidget *w,
				const QSize &s,
				const QStyleOption &o) const;

	int styleHint(StyleHint, const QWidget * = 0,
				const QStyleOption & = QStyleOption::Default,
				QStyleHintReturn * = 0 ) const;
	
	QPixmap stylePixmap(StylePixmap stylepixmap, const QWidget* widget, const QStyleOption& opt) const;
	
	void drawFocusIndicator(QPainter *p,
				const QRect &r,
				int flags,
				const QColorGroup cg,
				bool enabled,
				const QPixmap * pixmap,
				const QString & text,
				int /*len = -1*/,
				const QColor * highlight = 0,
				bool drawLine = true) const
	{
		
		p->setPen(cg.foreground());
		
		if(drawLine) {
			
			QPixmap edges = tintImage(uic_findImage("indicatorEdges"), cg.foreground());
			bitBlt(p->device(), r.x(), r.bottom()-1, &edges, 0, 0, 2, 1, Qt::CopyROP);
			bitBlt(p->device(), r.right()-2, r.bottom()-1, &edges, 2, 0, 2, 1, Qt::CopyROP);
			p->drawLine(r.x()+2, r.bottom()-1, r.right()-3, r.bottom()-1);
		}
		else {
			drawItem( p, QRect(r.x(), r.y()-1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x(), r.y()+1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x()+1, r.y(), r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x()-1, r.y(), r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x()-1, r.y()-1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x()+1, r.y()-1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem( p, QRect(r.x()-1, r.y()+1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
			drawItem(p, QRect(r.x()+1, r.y()+1, r.width(), r.height()), flags, cg, enabled, pixmap, text, -1, highlight);
		}
	};
	
	struct DSurface {
		int numGradients;
		QColor g1Color1;
		QColor g1Color2;
		QColor g2Color1;
		QColor g2Color2;
		QColor background;
// 		QColor disabled;
		int g1Top;
		int g1Bottom;
		int g2Top;
		int g2Bottom;
	} btnSurface, tabTopSurface, activeTabTopSurface, tabBottomSurface, activeTabBottomSurface,
			scrollBarSurface, scrollBarGrooveSurface, headerSurface, checkItemSurface;
	
	
	void setScrollBarSurface(DSurface &type) { scrollBarSurface = type; };
	void setScrollBarGrooveSurface(DSurface &type) { scrollBarGrooveSurface = type; };
	void setBtnSurface(DSurface &type) { btnSurface = type; };
	void setCheckItemSurface(DSurface &type) { checkItemSurface = type; };
	void setHeaderSurface(DSurface &type) { headerSurface = type; };
	void setTabTopSurface(DSurface &type) { tabTopSurface = type; };
	void setActiveTabTopSurface(DSurface &type) { activeTabTopSurface = type; };
	void setTabBottomSurface(DSurface &type) { tabBottomSurface = type; };
	void setActiveTabBottomSurface(DSurface &type) { activeTabBottomSurface = type; };
	
	void removeCachedSbPix(const QColor &color) {
		QPixmapCache::remove(color.name()+"ah");
		QPixmapCache::remove(color.name()+"av");
		QPixmapCache::remove(color.name()+"sh");
		QPixmapCache::remove(color.name()+"sv");
	};
	
	void renderSurface(QPainter *p,
			   const QRect &r,
			   const QColor &backgroundColor,
			   const QColor &buttonColor,
			   const QColor &highlightColor,
			   const uint flags = Draw_Left|Draw_Right|Draw_Top|Draw_Bottom|
					   Round_UpperLeft|Round_UpperRight|Round_BottomLeft|Round_BottomRight|Is_Horizontal) const;

	QPixmap* scrollBarCap_top;
	QPixmap* scrollBarCap_bottom;
	QPixmap* scrollBarCap_left;
	QPixmap* scrollBarCap_right;
	
	virtual void renderScrollBarCaps(const QColor &capsColor);

	

protected:
	
	enum TabPosition
	{
	First ,
	Middle,
	Last
	};

	enum ColorType
	{
	ButtonContour,
	DragButtonContour,
	DragButtonSurface,
	PanelContour,
	PanelLight,
	PanelLight2,
	PanelDark,
	PanelDark2,
	CheckMark
	};

	enum WidgetState
	{
	IsEnabled,
	IsPressed,	   // implies IsEnabled
	IsHighlighted, // implies IsEnabled
	IsDisabled
	};
	
	enum SurfaceBorder
	{
		TopBorder ,
		BottomBorder,
	};
	
	// the only way i see to provide all these options
	// to renderContour/renderSurface...
	enum SurfaceFlags {
	Draw_Left =			 0x00000001,
	Draw_Right =		 0x00000002,
	Draw_Top =			 0x00000004,
	Draw_Bottom =		 0x00000008,
	Is_Horizontal =		 0x00000200, // surface
	Is_Highlight =		 0x00000400, // surface
	Is_Default =			 0x00000800,
	Is_Disabled =		 0x00001000,
	Round_UpperLeft =	 0x00002000,
	Round_UpperRight =	 0x00004000,
	Round_BottomLeft =	 0x00008000,
	Round_BottomRight = 0x00010000,
	Square_UpperLeft =	 0x00000010,
	Square_UpperRight =	 0x10000020,
	Square_BottomLeft =	 0x00000040,
	Square_BottomRight = 0x00000080,
	
	
	Draw_AlphaBlend =	0x00020000,
	Is_ComboBox =		0x00080000,
	Is_Button =			0x00100000,
	Is_CheckItem =		0x00200000,
	Is_RadioButton =		0x00400000,
	Is_TabTop =			0x00000100,
	Is_TabBottom =		0x01000000,
	Is_ActiveTabTop =	0x00040000,
	Is_ActiveTabBottom =	0x00800000,
	Is_ScrollBarBtn =		0x02000000,
	Is_Header =			0x04000000,
	Is_ScrollBarGroove =	0x08000000,
	Is_Slider =			0x20000000,
	Is_Down =			0x40000000
	};
	
	void renderContour(QPainter *p,
			const QRect &r,
			const QColor &backgroundColor,
			const QColor &contourColor,
			const uint flags = Draw_Left|Draw_Right|Draw_Top|Draw_Bottom|
				Round_UpperLeft|Round_UpperRight|Round_BottomLeft|Round_BottomRight) const;

	void renderButtonShadow(QPainter *p, const QRect &r, const uint flags = Draw_Left|Draw_Right|Draw_Top|Draw_Bottom|
			Round_UpperLeft|Round_UpperRight|Round_BottomLeft|Round_BottomRight|Is_Horizontal) const;
	
	QPixmap* renderLineEditShadow(QWidget* widget, QRect rect, QColor bg, uint flags, bool reverse);
	
	inline void renderPixel(QPainter *p,
		const QPoint &pos,
		const int alpha,
		const QColor &color,
		const QColor &background = QColor(),
		bool fullAlphaBlend = true) const;

	void renderButton(QPainter *p,
			  const QRect &r,
			  const QColorGroup &g,
			  bool sunken = false,
			  bool mouseOver = false,
			  bool horizontal = true,
			  bool enabled = true,
			  bool khtmlMode = false) const;

	void renderHeader(QPainter *p,
			  const QRect &r,
			  const QColorGroup &g,
			  bool sunken = false,
			  bool mouseOver = false,
			  bool horizontal = true,
			  bool enabled = true) const;

	void renderPanel(QPainter *p,
			 const QRect &r,
			 const QColorGroup &g,
			 const bool pseudo3d = true,
			 const bool sunken = true) const;

	void renderGradient(QPainter *p,
			const QRect &r,
			const QColor &c1,
			const QColor &c2,
			bool horizontal = true) const;

	void renderTab(QPainter *p,
			const QRect &r,
			const QColorGroup &g,
			bool mouseOver = false,
			const bool selected = false,
			const bool bottom = false,
			const TabPosition pos = Middle,
			const bool triangular = false,
			const bool cornerWidget = false,
			const bool konqTab = false) const;
	
	bool eventFilter(QObject *, QEvent *);

	QWidget* hoverWidget;

	QPixmap* checkMark;
	QPixmap* radioIndicator;
	QPixmap* checkbox;
	QPixmap* radiobutton;
	QPixmap* popupFrame;
	QPixmap* border1;
	QGroupBox* designerGroupBox;

protected slots:
	void khtmlWidgetDestroyed(QObject* w);
	//Animation slots.
	void updateProgressPos();
	void updateScrollPos(); // smoothscroll
	
	void progressBarDestroyed(QObject* bar);
	inline QColor getColor(const QColorGroup &cg, const ColorType t, const bool enabled = true)const;
	inline QColor getColor(const QColorGroup &cg, const ColorType t, const WidgetState s)const;
	inline QColor getColor(const DSurface &ds, const SurfaceBorder border)const;
	inline QColor dGetColor(const QColor &color, const int value) const;
	inline QColor lightenColor(const QColor &color, const int value) const;
	inline QColor darkenColor(const QColor &color, const int value) const;
	void delayedGroupBoxUpdate();

	
private:
	// Disable copy constructor and = operator
	DominoStyle( const DominoStyle & );
	DominoStyle& operator=( const DominoStyle & );

	bool kickerMode;
	bool designerMode;
	bool konquerorMode;
	
	mutable bool flatMode;
	bool sideRepaint;
	int _contrast;
	bool _animateProgressBar;
	bool _indicateFocus;
	bool _drawTriangularExpander;
	bool _customCheckMarkColor;
	bool _tintGroupBoxBackground;
	bool _drawGroupBoxDark;
	bool _drawGroupBoxBright;
	bool _smoothScrolling;
	QColor _scrollBarCapsColor;
	QColor _checkMarkColor;
	QColor _toolTipColor;
	bool _customToolTipColor; // unused
	int _buttonHeightAdjustment;
	bool _customPopupMenuColor;
	QColor _popupMenuColor;
	bool _centerTabs;
	QColor _focusIndicatorColor;
	QColor _focusIndicatorBtnColor;
	bool _drawUnderline;
	
	bool konqTab;
	int minTabLength;
	int tabBarContrast;
	QColor tabContour;
	
	mutable int sliderlen;
	
	// smoothscrolling
	int scrollDistance;
	QScrollView* scrollWidget;
	QTimer* scrollTimer;
	
	bool hoveredToolBoxTab;
	
	// track khtml widgets.
	QMap<const QWidget*,bool> khtmlWidgets;

	//Animation support.
	QMap<QWidget*, int> progAnimWidgets;
	

	// pixmap cache.
	enum CacheEntryType {
		cSurface,
		cGradientTile,
		cAlphaDot
	};
	
	struct CacheEntry
	{
		CacheEntryType type;
		int width;
		int height;
		QRgb c1Rgb;
		QRgb c2Rgb;
		bool horizontal;
	
		QPixmap* pixmap;
	
		CacheEntry(CacheEntryType t, int w, int h, QRgb c1, QRgb c2 = 0,
			bool hor = false, QPixmap* p = 0 ):
			type(t), width(w), height(h), c1Rgb(c1), c2Rgb(c2), horizontal(hor), pixmap(p)
		{}
	
		~CacheEntry()
		{
			delete pixmap;
		}
	
		int key()
		{
			// create an int key from the properties which is used to refer to entries in the QIntCache.
			// the result may not be 100% correct as we don't have so much space in one integer -- use
			// == operator after find to make sure we got the right one. :)
			return horizontal ^ (type<<1) ^ (width<<5) ^ (height<<10) ^ (c1Rgb<<19) ^ (c2Rgb<<22);
		}
	
		bool operator == (const CacheEntry& other)
		{
			bool match = (type == other.type) &&
				(width	 == other.width) &&
				(height == other.height) &&
				(c1Rgb == other.c1Rgb) &&
				(c1Rgb == other.c1Rgb) &&
				(horizontal = other.horizontal);
// 				   if(!match) {
// 					   qDebug("operator ==: CacheEntries don't match!");
// 					   qDebug("width: %d\t\tother width: %d", width, other.width);
// 					   qDebug("height: %d\t\tother height: %d", height, other.height);
// 					   qDebug("fgRgb: %d\t\tother fgRgb: %d", fgRgb, other.fgRgb);
// 					   qDebug("bgRgb: %d\t\tother bgRgb: %d", bgRgb, other.bgRgb);
// 					   qDebug("surfaceFlags: %d\t\tother surfaceFlags: %d", surfaceFlags, other.surfaceFlags);
// 				   }
			return match;
		}
	};
	QIntCache<CacheEntry> *pixmapCache;

	// For KPE_ListViewBranch
	mutable QBitmap *verticalLine;
	mutable QBitmap *horizontalLine;
	
	// For progress bar animation
	QTimer *animationTimer;
	
	QPixmap* button;
	QPixmap* buttonVlines;
	QPixmap* buttonHlines;
	QPixmap* buttonShadowSquare;
	QPixmap* buttonShadowRound;
	QPixmap* buttonShadowHlines;
	QPixmap* buttonShadowVlines;
};



// DominoXXX classes to access protected members
class DominoQGroupBox : public QGroupBox
{
	friend class DominoStyle;
	public:
		DominoQGroupBox( QWidget* parent=0, const char* name=0 )
	: QGroupBox( parent, name ) {}
};

class DominoQWidget : public QWidget
{
	friend class DominoStyle;
	public:
		DominoQWidget( QWidget* parent=0, const char* name=0 )
	: QWidget( parent, name ) {}
	
};

class DominoQFrame : public QFrame
{
	friend class DominoStyle;
	public:
		DominoQFrame( QFrame* parent=0, const char* name=0 )
	: QFrame( parent, name ) {}
};


// QWhatsThis
class DominoQWhatsThat : public QWidget
{
	friend class DominoStyle;
	public:
		DominoQWhatsThat(): QWidget() {}
		// The text in qwhatsthat is private, but we can get it from an uninitialized qstring variable.
		// This is ugly, but there is no other way.
		QString text;
};

class DominoKToolBarButton : public KToolBarButton
{
	friend class DominoStyle;
	public:
		DominoKToolBarButton(): KToolBarButton() {}
};

class DominoQListBoxItem : public QListBoxItem
{
	friend class DominoStyle;
	public:
		DominoQListBoxItem(): QListBoxItem() {}
};




#endif // __DOMINO_H

