/*
   Copyright (C) 2006 Michael Lentner <michaell@gmx.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/


bool DominoStyle::eventFilter(QObject *obj, QEvent *ev)
{
	//return false;
	//qDebug("className:   %s  --  name:  %s", obj->className(), obj->name());
	

	if(dynamic_cast<QComboBox*>(obj)) {
		
		if(ev->type() == QEvent::Paint) {
			
			QComboBox* cb = dynamic_cast<QComboBox*>(obj);
			if(khtmlWidgets.contains(cb))
				return false;
			
			const QColorGroup & g = cb->colorGroup();
			QPainter p(cb);
			
			QStyle::SFlags flags = QStyle::Style_Default;
			if (cb->isEnabled())
				flags |= QStyle::Style_Enabled;
			if (cb->hasFocus())
				flags |= QStyle::Style_HasFocus;
			
			if(!cb->listBox()) {
				drawComplexControl( QStyle::CC_ComboBox, &p, cb, cb->rect(), g,
						flags, (uint)QStyle::SC_All, /*(cb->arrowDown ? */QStyle::SC_ComboBoxArrow /*: QStyle::SC_None )*/);
				
				QRect re = querySubControlMetrics( QStyle::CC_ComboBox, cb, QStyle::SC_ComboBoxEditField );
				re = QStyle::visualRect(re, cb);
				p.setClipRect( re );
				
				QString str = cb->text(cb->currentItem());
				const QPixmap *pix = cb->pixmap(cb->currentItem());
				if ( !str.isNull() ) {
					p.save();
					p.setFont(cb->font());
					QFontMetrics fm(cb->font());
					int x = re.x(), y = re.y() + fm.ascent();
					if( pix )
						x += pix->width() + 5;
					p.drawText( x, y, str );
					p.restore();
				}
				if ( pix ) {
					p.fillRect( re.x(), re.y(), pix->width() + 4, re.height(),
							cb->colorGroup().brush( QColorGroup::Base ) );
					p.drawPixmap( re.x() + 2, re.y() +
							( re.height() - pix->height() ) / 2, *pix );
				}
			}
			else {
				
				drawComplexControl( QStyle::CC_ComboBox, &p, cb, cb->rect(), g,
						flags, (uint)QStyle::SC_All, /*(d->arrowDown ? */QStyle::SC_ComboBoxArrow/* : QStyle::SC_None )*/);
				QRect re = querySubControlMetrics( QStyle::CC_ComboBox, cb, QStyle::SC_ComboBoxEditField );
				re = QStyle::visualRect(re, cb);
				p.setClipRect( re );
				
				if ( !cb->editable() ) {
					DominoQListBoxItem * item = (DominoQListBoxItem*)cb->listBox()->item( cb->currentItem() );
					if ( item ) {
						int itemh = item->height( cb->listBox() );
						p.translate( re.x(), re.y() +(re.height()- itemh)/2);
						//p.translate( re.x(), re.y()+1+re.height()/2- itemh/2);
						item->paint( &p );
					}
				}
				else if ( cb->listBox() && cb->listBox()->item( cb->currentItem() ) ) {
					QListBoxItem * item = cb->listBox()->item( cb->currentItem() );
					const QPixmap *pix = item->pixmap();
					if ( pix ) {
						if(re.width() < pix->width())
							return true;
						QColor color = cb->isEnabled() ? cb->lineEdit()->paletteBackgroundColor() : cb->lineEdit()->palette().active().background();
						QPixmap* shadowPix = renderLineEditShadow(cb, QRect(0, 0, pix->width() + 3, re.height()), color, Draw_Left, QApplication::reverseLayout());

						if(!QApplication::reverseLayout()) {
							int x = re.x();
							int y = re.y();
							int w = pix->width() + 4;
							int h = re.height()+re.y();
							
							QRegion mask(x, y, w, h);
							mask -= QRegion(x, y, 2, 1);
							mask -= QRegion(x, y+1, 1, 1);
							mask -= QRegion(x, h-1, 2, 1);
							mask -= QRegion(x, h-2, 1, 1);
							p.setClipRegion(mask);
						}
						bitBlt(shadowPix, 2, ( shadowPix->height() - pix->height() ) / 2, pix, 0, 0, -1, -1, Qt::CopyROP);
						p.drawPixmap(re.x(), re.y(), *shadowPix);
					}
				}
			}
			
			return !strcmp(cb->name(), "dominoPreviewWidget") ? false : true; // ###
			
		}
		return false;
	}

	// don't repaint the combobox if the mouse moves over the listbox.
	if(dynamic_cast<QListBox*>(obj) && dynamic_cast<QComboBox*>(obj->parent())) {
		
		if(dynamic_cast<QComboBox*>(obj->parent())) {
			if(ev->type() == QEvent::Show) {
				QComboBox* cb = dynamic_cast<QComboBox*>(obj->parent());
				cb->repaint(false);
				cb->setUpdatesEnabled(false);
				return false;
			}
			if(ev->type() == QEvent::Hide) {
				QComboBox* cb = dynamic_cast<QComboBox*>(obj->parent());
				cb->setUpdatesEnabled(true);
				cb->repaint(false);
				return false;
			}
		}
		// don't return here
	}
	
	
	if (dynamic_cast<QLineEdit*>(obj) || (dynamic_cast<QFrame*>(obj) && dynamic_cast<QFrame*>(obj)->frameShape() == QFrame::LineEditPanel)) {
		
		QWidget* lineEdit = dynamic_cast<QWidget*>(obj);
		bool khtmlMode = !strcmp(lineEdit->name(), "__khtml");
		bool isLineEdit = dynamic_cast<QLineEdit*>(obj);
		bool isReadOnly = isLineEdit && dynamic_cast<QLineEdit*>(obj)->isReadOnly();
		if((isReadOnly || khtmlMode) && dynamic_cast<QFrame*>(obj)->frameWidth() <= 0) {
			return false;
		}
		
		bool isEnabled = lineEdit->isEnabled();
		bool isSpinWidget = false;
		bool isComboBox = false;
		bool hasPixmap = false;
		QRect r = lineEdit->rect();
		
		if(dynamic_cast<QSpinWidget*>(lineEdit->parentWidget()))
			isSpinWidget = true;
		else if(dynamic_cast<QComboBox*>(lineEdit->parentWidget())) {
			isComboBox = true;
			QComboBox* cb = dynamic_cast<QComboBox*>(lineEdit->parentWidget());
			if(cb->count() > 0 && cb->pixmap(cb->currentItem()))
				hasPixmap = true;
		}
		
		if(!isSpinWidget && !isComboBox && static_cast<QFrame*>(obj)->frameShape() == QFrame::NoFrame)
			return false;
		
		int x, y, w, h;
		r.rect(&x, &y, &w, &h);

		switch(ev->type()) {
			case QEvent::Show:
			case QEvent::Resize:
				
				if((isSpinWidget || isComboBox)) {
					QRegion mask(r);
					if(!hasPixmap && !QApplication::reverseLayout()) {
						mask -= QRegion(x, y, 2, 1);
						mask -= QRegion(x, y+1, 1, 1);
						mask -= QRegion(x, h-1, 2, 1);
						mask -= QRegion(x, h-2, 1, 1);
						lineEdit->setMask(mask);
					}
					else if(QApplication::reverseLayout()) {
						mask -= QRegion(w-2, y, 2, 1);
						mask -= QRegion(w-1, y+1, 1, 1);
						mask -= QRegion(w-1, h-2, 1, 1);
						mask -= QRegion(w-2, h-1, 2, 1);
						lineEdit->setMask(mask);
					}
					else if(isComboBox)
						lineEdit->clearMask();
				}
				return false;
			case QEvent::Paint: {
					const QColorGroup cg = lineEdit->colorGroup();
					const QColor bg = !isEnabled ? cg.background() : lineEdit->paletteBackgroundColor();
					QPainter p;
				
					if((isSpinWidget || isComboBox)) {
						if(w <1 || h <1)
							return false;
						
						uint flags = hasPixmap ? Draw_Right : Draw_Left|Draw_Right;
						QPixmap* lineEditPix = renderLineEditShadow(lineEdit, lineEdit->rect(), bg, flags, QApplication::reverseLayout());
						lineEdit->setUpdatesEnabled(false);
						lineEdit->setErasePixmap(*lineEditPix);
						lineEdit->setUpdatesEnabled(true);
						delete lineEditPix;
						return false;
					}
					
					// normal lineEdit
					
					QPixmap* lineEditEdges = 0;
					if(!khtmlMode && (lineEditEdges = QPixmapCache::find(bg.name()+cg.background().name()+"lEditEdges"))) {
					}
					else {
						if(khtmlMode) {
							QPixmap insidePix = tintImage(uic_findImage("lineedit4inside"), bg);
							bitBlt(lineEdit, x, y, &insidePix, 0, 0, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, w-4, y, &insidePix, 4, 0, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, x, h-4, &insidePix, 0, 4, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, w-4, h-4, &insidePix, 4, 4, 4, 4, Qt::CopyROP);
							
							QPixmap edgesPix = uic_findImage("lineedit4edges");
							bitBlt(lineEdit, x, y, &edgesPix, 0, 0, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, w-4, y, &edgesPix, 4, 0, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, x, h-4, &edgesPix, 0, 4, 4, 4, Qt::CopyROP);
							bitBlt(lineEdit, w-4, h-4, &edgesPix, 4, 4, 4, 4, Qt::CopyROP);
						}
						else {
							lineEditEdges = new QPixmap(8, 8);
							lineEditEdges->fill(cg.background());
							
							QPixmap tmpPix = tintImage(uic_findImage("lineedit4inside"), bg);
							bitBlt(lineEditEdges, 0, 0, &tmpPix, 0, 0, 8, 8, Qt::CopyROP);
							
							QPixmap tmpPix2 = uic_findImage("lineedit4edges");
							bitBlt(lineEditEdges, 0, 0, &tmpPix2, 0, 0, 8, 8, Qt::CopyROP);
							
							QPixmapCache::insert(bg.name()+cg.background().name()+"lEditEdges", *lineEditEdges);
						}
					}
					
					QPixmap* lineEditPix = 0;
					
					if(khtmlMode)
						p.begin(lineEdit);
					else {
						lineEditPix = new QPixmap(w, h);
						p.begin(lineEditPix);
						
						bitBlt(lineEditPix, x, y, lineEditEdges, 0, 0, 4, 4, Qt::CopyROP);
						bitBlt(lineEditPix, w-4, y, lineEditEdges, 4, 0, 4, 4, Qt::CopyROP);
						bitBlt(lineEditPix, x, h-4, lineEditEdges, 0, 4, 4, 4, Qt::CopyROP);
						bitBlt(lineEditPix, w-4, h-4, lineEditEdges, 4, 4, 4, 4, Qt::CopyROP);
						
					}
					
					p.fillRect(QRect(x+4, y, w-8, h), bg);
					p.fillRect(QRect(x, y+4, 4, h-8), bg);
					p.fillRect(QRect(w-4, y+4, 4, h-8), bg);
					
					QColor top_1 = alphaBlendColors(QColor(26, 26, 26), bg, 125);
					QColor top_2 = alphaBlendColors(QColor(18, 18, 18), bg, 50);
					QColor top_3 = alphaBlendColors(QColor(16, 16, 16), bg, 6);
					QColor bottom = alphaBlendColors(QColor(0, 0, 0), bg, 30);
						
					QColor left_1 = alphaBlendColors(QColor(0, 0, 0), bg, 78);
					QColor left_2 = alphaBlendColors(QColor(0, 0, 0), bg, 10);
						
					p.setPen(top_1);
					p.drawLine(x+4, y, w-5, y);
					p.setPen(top_2);
					p.drawLine(x+4, y+1, w-5, y+1);
					p.setPen(top_3);
					p.drawLine(x+4, y+2, w-5, y+2);
					p.setPen(bottom);
					p.drawLine(x+4, h-1, w-5, h-1);
						
					p.setPen(left_1);
					p.drawLine(x, y+4, x, h-4);
					p.drawLine(w-1, y+4, w-1, h-4);
					p.setPen(left_2);
					p.drawLine(x+1, y+4, x+1, h-4);
					p.drawLine(w-2, y+4, w-2, h-4);
					p.end();
					
					
					if(!khtmlMode) {
						if(isLineEdit) {
							lineEdit->setUpdatesEnabled(false);
							lineEdit->setErasePixmap(*lineEditPix);
							lineEdit->erase(QRegion(r)-QRegion(x+2, y+2, w-4, h-4));
							lineEdit->setUpdatesEnabled(true);
						}
						else {
							bitBlt(lineEdit, 0, 0, lineEditPix, 0, 0, -1, -1, Qt::CopyROP);
						}
						delete lineEditPix;
					}
					return false;
				}
			case QEvent::ApplicationPaletteChange:
			case QEvent::PaletteChange:
				lineEdit->setBackgroundMode(QWidget::PaletteBase);
				break;
			case QEvent::Wheel:
			case QEvent::Hide:
			case 7:
				// jump to smoothscrolling
				if(_smoothScrolling)
					break;
			default:
				return false;
		}
	}
	
	
	
	// smoothscrolling
	if(dynamic_cast<QScrollView*>(obj) && _smoothScrolling) {
		if(ev->type() == QEvent::Wheel) {
			QWheelEvent* e = (QWheelEvent*)ev;
			QScrollView* sv = dynamic_cast<QScrollView*>(obj);
			bool pageSteps = false;
			int pageStep = sv->verticalScrollBar()->pageStep();
			int lineStep = sv->verticalScrollBar()->lineStep();
			int step = QMIN( QApplication::wheelScrollLines()*lineStep, pageStep );
			if ( ( e->state() & ControlButton ) || ( e->state() & ShiftButton ) ) {
				step = pageStep;
				pageSteps = true;
			}
			int dy = (e->delta()*step)/120;
			e->accept();
			scrollWidget = sv;
			scrollDistance += dy;
			
			// don't accumulate to much
			if(scrollDistance < 0) {
				scrollDistance = pageSteps? QMAX(scrollDistance, pageStep*2-(pageStep*4)): scrollDistance;
				int minDistance = sv->verticalScrollBar()->maxValue() - sv->verticalScrollBar()->value();
				scrollDistance = QMAX(scrollDistance, minDistance-(minDistance*2));
			}
			else {
				scrollDistance = pageSteps? QMIN(scrollDistance, pageStep*2): scrollDistance;
				scrollDistance = QMIN(scrollDistance, sv->verticalScrollBar()->value());
			}
			
			if(scrollDistance != 0)
				scrollTimer->start(pageSteps? 4: 14);
			return true;
		}
		if(ev->type() == 7) { // ?? 7 = QEvent::KeyRelease
			if(scrollTimer->isActive() && ((QKeyEvent*)ev)->key() == Qt::Key_Shift) {
				scrollTimer->stop();
				scrollDistance = 0;
			}
			return false;
		}
		if(ev->type() == QEvent::Hide || ev->type() == QEvent::Show) {
			scrollDistance = 0;
		}
		
		// don't return here
	}


	if ( dynamic_cast<QScrollBar*>(obj) ) {
		if(ev->type() == QEvent::WindowActivate) {
			QScrollBar* sb = dynamic_cast<QScrollBar*>(obj);
			if(sb->backgroundMode() == Qt::NoBackground && !sb->testWState(WState_HasMouse)) {
				sb->setBackgroundMode(Qt::PaletteBackground);
			}
			return true;
		}
		if(ev->type() == QEvent::Show) {
			// no flicker in konqi and konsole, hits maybe others, too.
			if(obj->parent()->parent() && (!strcmp(obj->parent()->parent()->name(), "KonqFrame") || !strcmp(obj->parent()->parent()->name(), "tab pages"))) {
				dynamic_cast<QScrollBar*>(obj)->setBackgroundMode(Qt::NoBackground);
			}
			return true;
		}
		if(ev->type() == QEvent::Resize) {
			QScrollBar* sb = dynamic_cast<QScrollBar*>(obj);
			if(sb->backgroundMode() != Qt::NoBackground && sb->isVisible()) {
				sb->setBackgroundMode(Qt::NoBackground);
			}
		}
		return false;
	}


// 	// Track show events for progress bars
// 	if ( _animateProgressBar && dynamic_cast<QProgressBar*>(obj) )
// 	{
// 		if ((ev->type() == QEvent::Show) && !animationTimer->isActive())
// 		{
// 			animationTimer->start( 50, false );
// 		}
// 		return false;
// 	}
	
	if (dynamic_cast<QStatusBar*>(obj)) {
			
		if(ev->type() == QEvent::PaletteChange){
			QStatusBar* sb = dynamic_cast<QStatusBar*>(obj);
			sb->setPaletteBackgroundColor(qApp->palette().active().background());
			sb->setPaletteForegroundColor(qApp->palette().active().foreground());
			return true;
		}
		return false;
	}
	
	if (dynamic_cast<QGroupBox*>(obj) && dynamic_cast<QFrame*>(obj)->frameShape() == QFrame::GroupBoxPanel) {
		QGroupBox* gb = dynamic_cast<QGroupBox*>(obj);
		
		switch(ev->type()) {
			case QEvent::Paint: {
				QPainter p(gb);
				if(gb->isFlat()) {
					if(designerMode && gb->erasePixmap())
						gb->setPaletteBackgroundColor(gb->parentWidget()->paletteBackgroundColor());
					
					QRect r = gb->rect();
					QString title = gb->title();
					int titleHeight = gb->fontMetrics().height();
					int titleWidth = gb->fontMetrics().width(title);
					if(title.contains("&"))
						titleWidth -= gb->fontMetrics().width("&");
					p.setPen(qApp->palette().active().background().dark(120));

					p.drawLine(0, titleHeight/2-2, r.width()/2-titleWidth/2-4, titleHeight/2-2);
					p.drawLine(r.width()/2+titleWidth/2+4, titleHeight/2-2, r.width(), titleHeight/2-2);
					
					QColorGroup cg = gb->parentWidget()->colorGroup();
					drawItem(&p, QRect(r.width()/2-titleWidth/2, -1, r.width(), titleHeight), ShowPrefix, cg, Style_Enabled, 0, title, -1, &cg.foreground());
					p.setPen(cg.foreground());
				}
				else if(!gb->erasePixmap()) {
					if(designerMode) {
						designerGroupBox = gb;
						QTimer::singleShot(500, this, SLOT(delayedGroupBoxUpdate()));
					}
					else { // stylepreview kcm
						QEvent e(QEvent::PaletteChange);
						QApplication::sendEvent(gb, &e);
					}
				}
				((DominoQGroupBox*)gb)->drawContents(&p);
				return true;
			}
			case QEvent::Show:
				if(gb->erasePixmap())
					return true;
			case QEvent::Resize:
			case QEvent::ApplicationPaletteChange:
			case QEvent::PaletteChange:
			case QEvent::ParentPaletteChange: {
				
				if(gb->isFlat() || gb->lineWidth() <= 0) {
					if(designerMode)
						gb->setPaletteBackgroundColor(gb->parentWidget()->paletteBackgroundColor());
					return false;
				}
				
				if(gb->layout() && gb->layout()->margin() < 7)
					gb->layout()->setMargin(7);
				
				QRect r = gb->rect();
				QColor outsideColor = gb->parentWidget()->paletteBackgroundColor();
				QColor insideColor;
				if(_tintGroupBoxBackground) {
					if(_drawGroupBoxDark)
						insideColor = darkenColor(outsideColor, 10);
					else
						insideColor = lightenColor(outsideColor, 10);
				}
				else
					insideColor = outsideColor;
				
				QString title = gb->title();
				bool isCheckable = gb->isCheckable();
				int titleHeight = gb->fontMetrics().height();
				int x = r.x();
				int y = isCheckable ? r.y()+titleHeight+5-1 : (title.isNull() || title.isEmpty()) ? QMAX(r.y()-1, 0) : r.y()+titleHeight-1;
				int w = r.width();
				int h = r.height();
				int addition = 50; // ugly, but else we will see tiles while resizing.
				
				if(h-y < 26 || w < 26)
					return false; // don't overlap the pixmaps
				
				gb->setBackgroundMode(Qt::PaletteBackground);
				
				bool paletteChanged = ev->type() == QEvent::ApplicationPaletteChange ||
						ev->type() == QEvent::ParentPaletteChange; /*|| ev->type() == QEvent::PaletteChange*/ // PaletteChange crashes the preview in kcm style
				if(paletteChanged) {
					gb->setPaletteBackgroundColor(insideColor);
				}
				
				
				if(isCheckable && paletteChanged ) {
					((QCheckBox*)gb->child("qt_groupbox_checkbox", 0, false))->setPaletteBackgroundColor(outsideColor);
				}
				
				
				QPixmap* gbPix = 0;
				if((gbPix = QPixmapCache::find(outsideColor.name()+insideColor.name()+"groupbox" ))) {
				}
				else {
					gbPix = new QPixmap(26, 25);
					gbPix->fill(outsideColor);
					QPainter painter(gbPix);
					
					QRegion mask(8, 2, 8, 1);
					mask += QRegion(7, 3, 10, 1);
					mask += QRegion(6, 4, 12, 1);
					mask += QRegion(5, 5, 14, 1);
					mask += QRegion(4, 6, 16, 1);
					mask += QRegion(3, 7, 18, 5);
					
					mask += QRegion(3, 12, 20, 5);
					mask += QRegion(4, 17, 18, 1);
					mask += QRegion(5, 18, 16, 1);
					mask += QRegion(6, 19, 14, 1);
					mask += QRegion(7, 20, 12, 1);
					mask += QRegion(8, 21, 10, 1);
					
					painter.setClipRegion(mask);
					painter.fillRect(QRect(0, 0, 26, 25), insideColor);
					painter.setClipping(false);
					
					QPixmap groupBoxPix = uic_findImage("gb");
					
					bitBlt(gbPix, 0, 0, &groupBoxPix, 0, 0, 12, 12, Qt::CopyROP);
					bitBlt(gbPix,12, 0, &groupBoxPix, 12, 0, 12, 12, Qt::CopyROP);
					bitBlt(gbPix, 0, 12, &groupBoxPix, 0, 12, 13, 13, Qt::CopyROP);
					bitBlt(gbPix, 13, 12, &groupBoxPix, 13, 12, 13, 13, Qt::CopyROP);

					QPixmapCache::insert(outsideColor.name()+insideColor.name()+"groupbox", gbPix);
				}
				
				QPixmap buffer(r.width()+addition, r.height()+addition);
				buffer.fill(outsideColor);
				
				QPainter p(&buffer);
				p.fillRect(QRect(x+4, y+4, w-8, h-8), insideColor);
				
				bitBlt(&buffer, 0, y, gbPix, 0, 0, 12, 12, Qt::CopyROP);
				bitBlt(&buffer, w-12, y, gbPix, 12, 0, 12, 12, Qt::CopyROP);
				bitBlt(&buffer, 0, h-13, gbPix, 0, 12, 13, 13, Qt::CopyROP);
				bitBlt(&buffer, w-13, h-13, gbPix, 13, 12, 13, 13, Qt::CopyROP);
				
				int titleWidth = gb->fontMetrics().width(title);
				if(title.contains("&"))
					titleWidth -= gb->fontMetrics().width("&");
				drawItem(&p, QRect(r.width()/2-titleWidth/2, -1, r.width(), titleHeight), ShowPrefix, gb->colorGroup(), true, 0, title, -1, &qApp->palette().active().foreground());
				
				
				QColor c1 = alphaBlendColors(QColor(255,255,255), outsideColor, 12);
				QColor c2 = alphaBlendColors(QColor(0,0,0), outsideColor, 85);
				QColor c3 = alphaBlendColors(QColor(0,0,0), outsideColor, 42);
				QColor c4 = alphaBlendColors(QColor(0,0,0), insideColor, 12);
				QColor c5 = alphaBlendColors(QColor(253,253,253), outsideColor, 85);
				QColor c6 = alphaBlendColors(QColor(252,252,252), outsideColor, 42);
				QColor c7 = alphaBlendColors(QColor(244,244,244), outsideColor, 12);
				QColor c8 = alphaBlendColors(QColor(255,255,255), outsideColor, 42);
				QColor c9 = alphaBlendColors(QColor(0,0,0), insideColor, 42);
				// top
				p.setPen(c1);
				p.drawLine(r.left()+12, y, r.right()-12, y);
				p.setPen(c2);
				p.drawLine(r.left()+12, y+1, r.right()-12, y+1);
				p.setPen(c9);
				p.drawLine(r.left()+12, y+2, r.right()-12, y+2);
				p.setPen(c4);
				p.drawLine(r.left()+12, y+3, r.right()-12, y+3);
				// bottom
				p.drawLine(r.left()+13, r.bottom()-3, r.right()-12, r.bottom()-3);
				p.setPen(c5);
				p.drawLine(r.left()+13, r.bottom()-2, r.right()-12, r.bottom()-2);
				p.setPen(c6);
				p.drawLine(r.left()+13, r.bottom()-1, r.right()-12, r.bottom()-1);
				p.setPen(c7);
				p.drawLine(r.left()+13, r.bottom(), r.right()-12, r.bottom());
				// left
				p.setPen(c1);
				p.drawLine(r.left(), y+12, r.left(), r.bottom()-13);
				p.setPen(c8);
				p.drawLine(r.left()+1, y+12, r.left()+1, r.bottom()-13);
				p.setPen(c3);
				p.drawLine(r.left()+2, y+12, r.left()+2, r.bottom()-13);
				p.setPen(c4);
				p.drawLine(r.left()+3, y+12, r.left()+3, r.bottom()-13);
				// right
				p.setPen(c4);
				p.drawLine(r.right()-3, y+12, r.right()-3, r.bottom()-13);
				p.setPen(c3);
				p.drawLine(r.right()-2, y+12, r.right()-2, r.bottom()-13);
				p.setPen(c6);
				p.drawLine(r.right()-1, y+12, r.right()-1, r.bottom()-13);
				p.setPen(c7);
				p.drawLine(r.right(), y+12, r.right(), r.bottom()-13);
				
				
				gb->setUpdatesEnabled(false);
				gb->setErasePixmap(buffer);
				gb->setUpdatesEnabled(true);
				
				return false;
			}
			default:
				return false;
		}
	}
	
	if(obj->isA("QDockWindowResizeHandle")) {
		if(ev->type() == QEvent::PaletteChange)
			static_cast<QWidget*>(obj)->setBackgroundMode(Qt::PaletteBackground);
		return false;
	}
		
	if(obj->isA("QWhatsThat")) { // QWhatsThis
		
		if (ev->type() == QEvent::Show) {
			dynamic_cast<QWidget*>(obj)->setBackgroundMode(Qt::NoBackground);
			return true;
		}
		
		if (ev->type() == QEvent::Paint) {
			DominoQWhatsThat* dwt = (DominoQWhatsThat*)obj;
			QString text = dwt->text;
			QRect r = dwt->rect();
			
			int x = r.x();
			int y = r.y();
			int w = r.width();
			int h = r.height();
			
			QPointArray pa(8);
			pa.setPoint(0, x+1,y+4); // left-top
			pa.setPoint(1, x+4,y+1);
			pa.setPoint(2, w-4,y+1); // right-top
			pa.setPoint(3, w-1,y+4);
			pa.setPoint(4, w-1,h-5); // right-bottom
			pa.setPoint(5, w-5,h-1);
			pa.setPoint(6, x+4,h-1); // left-bottom
			pa.setPoint(7, x+1,h-5);
			
			QPainter p(dwt);
			p.setClipRegion(pa);
			renderGradient(&p, QRect(r.left(), r.top(), r.width(), r.height()), _toolTipColor.light(110), _toolTipColor.dark(105), true);
			p.setClipping(false);
			
			QColor tabContour2 = qApp->palette().active().background().dark(150);
			
			p.save();
			p.setPen(tabContour2);
			p.drawLine(x+7,y, w-8,y); // top
			p.drawLine(x+7,h-1, w-8,h-1); // bottom
			p.drawLine(x,y+7, x,h-8); // left
			p.drawLine(w-1,y+7, w-1,h-8); // right
			p.restore();
			
			bitBlt(dwt, x, y, border1, 0, 0, 7, 7, Qt::CopyROP);
			bitBlt(dwt, w-7, y, border1, 7, 0, 7, 7, Qt::CopyROP);
			bitBlt(dwt, x, h-7, border1, 0, 7, 7, 7, Qt::CopyROP);
			bitBlt(dwt, w-7, h-7, border1, 7, 7, 7, 7, Qt::CopyROP);
			
			r.addCoords( 12, 8, -12, -8 );
			
			if(QStyleSheet::mightBeRichText( text )) {
				QSimpleRichText* srt = new QSimpleRichText( text, dwt->font());
				srt->draw(&p, r.x(), r.y(), r, qApp->palette().active(), 0);
			}
			else {
				r.addCoords( 2, 3, 0, 0 );
				p.drawText( r, AlignAuto + AlignTop + WordBreak + ExpandTabs, text );
			}
			
			return true;
		}
		return false;
	}
	
	
	if(obj->isA("KonqFileTip")) {
		
		if (ev->type() == QEvent::Show || ev->type() == QEvent::Paint) {
			
			QWidget* widget = dynamic_cast<QWidget*>(obj);
			QRect r = widget->rect();
			
			QPixmap pix(r.size());
			QPainter p(&pix);
			renderGradient(&p, QRect(r.left(), r.top(), r.width(), r.height()), _toolTipColor.light(110), _toolTipColor.dark(105), true);
			
			if(ev->type() == QEvent::Show) {
				dynamic_cast<QFrame*>(widget)->setFrameStyle(QFrame::NoFrame);
				widget->setBackgroundMode(Qt::NoBackground);
				
				QObjectList *list = const_cast<QObjectList *>(widget->children());
				QObjectListIt it( *list );
				QObject * o;
				while ( (o = it.current()) != 0 ) {
					++it;
					if(dynamic_cast<QLabel*>(o)) {
						QLabel* l = dynamic_cast<QLabel*>(o);
						l->setBackgroundOrigin(QWidget::ParentOrigin);
						l->setPaletteBackgroundPixmap(pix);
					}
				}
			}
			p.end();
			
			if(ev->type() == QEvent::Paint) {
				
				int x = r.x();
				int y = r.y();
				int w = r.width();
				int h = r.height();
				
				QPointArray pa(8);
				pa.setPoint(0, x+1,y+4); // left-top
				pa.setPoint(1, x+4,y+1);
				pa.setPoint(2, w-4,y+1); // right-top
				pa.setPoint(3, w-1,y+4);
				pa.setPoint(4, w-1,h-5); // right-bottom
				pa.setPoint(5, w-5,h-1);
				pa.setPoint(6, x+4,h-1); // left-bottom
				pa.setPoint(7, x+1,h-5);
				
				p.begin(widget);
				p.setClipRegion(pa);
				p.drawPixmap(0, 0, pix);
				p.setClipping(false);
				
				QColor tabContour2 = qApp->palette().active().background().dark(150);
				p.setPen(tabContour2);
				p.drawLine(x+7,y, w-8,y); // top
				p.drawLine(x+7,h-1, w-8,h-1); // bottom
				p.drawLine(x,y+7, x,h-8); // left
				p.drawLine(w-1,y+7, w-1,h-8); // right
				
				bitBlt(widget, x, y, border1, 0, 0, 7, 7, Qt::CopyROP);
				bitBlt(widget, w-7, y, border1, 7, 0, 7, 7, Qt::CopyROP);
				bitBlt(widget, x, h-7, border1, 0, 7, 7, 7, Qt::CopyROP);
				bitBlt(widget, w-7, h-7, border1, 7, 7, 7, 7, Qt::CopyROP);
				
			}
			return true;
		}
		return false;
	}
	
	if(obj->isA("QTipLabel")) {
		
		if (ev->type() == QEvent::Show) {
			QLabel* l = dynamic_cast<QLabel*>(obj);
			l->resize( l->width()+2, l->height()+2);
			l->setBackgroundMode(Qt::NoBackground);
			
			int x, y, w, h;
			l->rect().rect(&x, &y, &w, &h);
			
			QRegion mask(l->rect());
			mask -= QRegion(x, y, 3, 1);
			mask -= QRegion(x, y+1, 2, 1);
			mask -= QRegion(x, y+2, 1, 1);
			mask -= QRegion(w-3, y, 3, 1);
			mask -= QRegion(w-2, y+1, 2, 1);
			mask -= QRegion(w-1, y+2, 1, 1);
			mask -= QRegion(w-1, h-3, 1, 1);
			mask -= QRegion(w-2, h-2, 2, 1);
			mask -= QRegion(w-3, h-1, 3, 1);
			mask -= QRegion(x, h-3, 1, 1);
			mask -= QRegion(x, h-2, 2, 1);
			mask -= QRegion(x, h-1, 3, 1);
			l->setMask(mask);
			return true;
		}
		if (ev->type() == QEvent::Paint) {
			QLabel* label = dynamic_cast<QLabel*>(obj);
			QRect r = label->rect();
			
			int x = r.x();
			int y = r.y();
			int w = r.width();
			int h = r.height();
			
			QPointArray pa(8);
			pa.setPoint(0, x+1,y+4); // left-top
			pa.setPoint(1, x+4,y+1);
			pa.setPoint(2, w-4,y+1); // right-top
			pa.setPoint(3, w-1,y+4);
			pa.setPoint(4, w-1,h-5); // right-bottom
			pa.setPoint(5, w-5,h-1);
			pa.setPoint(6, x+4,h-1); // left-bottom
			pa.setPoint(7, x+1,h-5);
			
			QPainter p(label);
			p.setClipRegion(pa);
			
			renderGradient(&p, QRect(r.left(), r.top(), r.width(), r.height()), _toolTipColor.light(110), _toolTipColor.dark(105), true);
			p.setClipping(false);
			
			QColor tabContour2 = qApp->palette().active().background().dark(150);
			p.setPen(tabContour2);
			p.drawLine(x+7,y, w-8,y); // top
			p.drawLine(x+7,h-1, w-8,h-1); // bottom
			p.drawLine(x,y+7, x,h-8); // left
			p.drawLine(w-1,y+7, w-1,h-8); // right
			
			bitBlt(label, x, y, border1, 0, 0, 7, 7, Qt::CopyROP);
			bitBlt(label, w-7, y, border1, 7, 0, 7, 7, Qt::CopyROP);
			bitBlt(label, x, h-7, border1, 0, 7, 7, 7, Qt::CopyROP);
			bitBlt(label, w-7, h-7, border1, 7, 7, 7, 7, Qt::CopyROP);
			
			
			QSimpleRichText* srt = new QSimpleRichText(label->text(), label->font());
			srt->setWidth(r.width()-5);
			srt->draw(&p, r.x()+4, r.y(), r, qApp->palette().active(),0);
			
			delete srt;
		}
		return true;
	}
	
	if (obj->isA("KonqCheckBox")) {
		
		if (ev->type() == QEvent::Show || ev->type() == QEvent::ApplicationPaletteChange) { // at least doesn't flicker in the unchecked state ;-)
			QButton* btn = dynamic_cast<QButton*>(obj);
			QPixmap pix(btn->size());
			pix.fill(qApp->palette().active().background());
			QPainter p(&pix);
			drawControl( CE_CheckBox, &p, btn, QRect(0,0,18,19), qApp->palette().active(), Style_Default|Style_Off, QStyleOption::Default);
			btn->setErasePixmap(pix);
			return true;
		}
		if (ev->type() == QEvent::Paint) {
			QButton* btn = dynamic_cast<QButton*>(obj);
			btn->erase();
			SFlags flags = Style_Default;
			if ( btn->isDown() )
				flags |= Style_Down;
			if ( btn->state() == QButton::On )
				flags |= Style_On;
			else if ( btn->state() == QButton::Off )
				flags |= Style_Off;
			QPainter p(btn);
			drawPrimitive(PE_CheckMark, &p, QRect(0,0,16,17), qApp->palette().active(), flags);
			return true;
		}
		return false;
	}
	
	if (dynamic_cast<QTabWidget*>(obj)) {
		QTabWidget* tw = dynamic_cast<QTabWidget*>(obj);
		if(tw->isA("KonqFrameTabs")) {
			
			QColorGroup cg = qApp->palette().active();
			switch (ev->type()) {
				case QEvent::Paint: {
					QRect r = tw->rect();
					int top, height;
					if(tw->tabPosition() == QTabWidget::Top) {
						top = r.top();
						height =  r.height()-tw->currentPage()->height();
					}
					else {
						top = r.top()+tw->currentPage()->height();
						height = r.height();
					}
					
					QPainter p(tw);
					p.setPen(cg.background());
					p.drawLine(r.left(), top, r.width(), top);
					
					p.setPen(cg.background().dark(120));
					p.drawLine(r.left(), top+1, r.width(), top+1);
					
					p.setPen(cg.background().dark(tabBarContrast));
					p.drawLine(r.left(), top+2, r.width(), top+2);
					return true;
				}
				case QEvent::Show:
				case QEvent::PaletteChange:
				case QEvent::ApplicationPaletteChange: {
					if(tw->eraseColor() != cg.background().dark(tabBarContrast-20))
						tw->setEraseColor(cg.background().dark(tabBarContrast-20));
					return true;
				}
				default:
					return false;
			}
		}
		return false;
 	}
	
	if(dynamic_cast<KToolBarButton*>(obj)) {
		switch(ev->type()) {
			case QEvent::Paint: {
				KToolBar *bar = dynamic_cast<KToolBar*>(obj->parent());
				KToolBarButton* btn = dynamic_cast<KToolBarButton*>(obj);
				int index = bar->itemIndex(btn->id());
				
				if(sideRepaint == false) {
					sideRepaint = true;
					if(bar->getButton(bar->idAt(index-1))) {
						(bar->getButton(bar->idAt(index-1)))->repaint(false);
					}
					if(bar->getButton(bar->idAt(index+1))) {
						(bar->getButton(bar->idAt(index+1)))->repaint(false);
					}
					sideRepaint = false;
				}
				
				
				SFlags flags = QStyle::Style_Default;
				SCFlags active = QStyle::SC_None;
				if(btn->isOn()) flags |= QStyle::Style_On;
				if (btn->isDown()) {
					flags  |= QStyle::Style_Down;
					active |= QStyle::SC_ToolButton;
				}
				if (btn->isEnabled())  flags |= QStyle::Style_Enabled;
				if (btn->isEnabled() && btn->hasMouse())    flags |= QStyle::Style_Raised;
				if (btn->hasFocus())   flags |= QStyle::Style_HasFocus;
				//SCFlags controls,
				
				QPainter p(btn);
				drawComplexControl(QStyle::CC_ToolButton, &p, btn, btn->rect(), btn->colorGroup(), flags, QStyle::SC_ToolButton, active, QStyleOption());
				return true;
			}
			case QEvent::MouseButtonPress: {
				KToolBarButton* btn = dynamic_cast<KToolBarButton*>(obj);
				btn->setUpdatesEnabled(false);
				if(!btn->isOn()) btn->setOn(true);
				btn->setUpdatesEnabled(true);
				return false;
			}
			case QEvent::MouseButtonRelease: {
				KToolBarButton* btn = dynamic_cast<KToolBarButton*>(obj);
				btn->setUpdatesEnabled(false);
				if(btn->isOn())  btn->setOn(false);
				btn->setUpdatesEnabled(true);
				return false;
			}
			case QEvent::MouseButtonDblClick: {
				KToolBarButton* btn = dynamic_cast<KToolBarButton*>(obj);
				if(!btn->isOn()) btn->setOn(true);
				return false;
			}
			default:
				return false;
		}
	}

// 	if(dynamic_cast<KToolBar*>(obj)) {
// 
// 		if (ev->type() == QEvent::ChildRemoved) {
// 			KToolBar *bar = dynamic_cast<KToolBar*>(obj);
// 			int c = bar->count();
// 			if(bar->getButton(bar->idAt(c-1))) {
// 				(bar->getButton(bar->idAt(c-1)))->repaint(false);
// 			}
// 			return false;
// 		}
// 		return false;
// 	}
	
	if(dynamic_cast<QRadioButton*>(obj) && !strcmp(obj->name(), "__khtml")) {
		if (ev->type() == QEvent::Paint) {
			QPixmap* pix;
			if((pix = QPixmapCache::find("radiobutton"))) {
			}
			else {
				pix = new QPixmap(uic_findImage("radiobutton"));
				QPixmapCache::insert("radiobutton", pix);
			}
			bitBlt(static_cast<QRadioButton*>(obj), 0, 0, pix, 0, 0, -1, -1, Qt::CopyROP);
			return false;
		}
		if (ev->type() == QEvent::Show) {
			QRadioButton* rb = static_cast<QRadioButton*>(obj);
			QBitmap bm(rb->size());
			bm.fill(color0);
			rb->setMask(bm);
		}
		return false;
	}
	
	if(dynamic_cast<QCheckBox*>(obj) && !strcmp(obj->name(), "__khtml")) {
		if (ev->type() == QEvent::Paint) {
			QCheckBox* cb = static_cast<QCheckBox*>(obj);
			QBitmap bm(cb->size());
			bm.fill(color0);
			cb->setMask(bm);

			QPixmap* pix;
			if((pix = QPixmapCache::find("checkbox"))) {
			}
			else {
				pix = new QPixmap(uic_findImage("checkbox"));
				QPixmapCache::insert("checkbox", pix);
			}
			bitBlt(cb, 0, 0, pix, 0, 0, -1, -1, Qt::CopyROP);
			
			return false;
		}
		return false;
	}
	
	if(dynamic_cast<QPopupMenu*>(obj)) {
		if(ev->type() == QEvent::Show) {
			QWidget* widget = static_cast<QWidget*>(obj);
			int x, y, w, h;
			widget->rect().rect(&x, &y ,&w, &h);
			
			QRegion mask(widget->rect());
			mask -= QRegion(x, y, 2 , 1);
			mask -= QRegion(x, y+1, 1 , 1);
			mask -= QRegion(w-2, y, 2 , 1);
			mask -= QRegion(w-1, y+1, 1 , 1);
			mask -= QRegion(w-2, h-1, 2 , 1);
			mask -= QRegion(w-1, h-2, 1 , 1);
			mask -= QRegion(x, h-1, 2 , 1);
			mask -= QRegion(x, h-2, 1 , 1);
			widget->setMask(mask);
			return true;
		}
		if(ev->type() == QEvent::Hide)
			static_cast<QWidget*>(obj)->setBackgroundMode(NoBackground);
		return false;
	}
	
	if(obj->isA("QDockWindowHandle") || obj->isA("AppletHandleDrag"))  {
		if ((ev->type() == QEvent::Enter) &&  static_cast<QWidget*>(obj)->isEnabled()) {
			QWidget* w = static_cast<QWidget*>(obj);
			hoverWidget = w;
			w->repaint(true);
			return false;
		}
		if (ev->type() == QEvent::Leave && static_cast<QWidget*>(obj) == hoverWidget) {
			hoverWidget = 0;
			static_cast<QWidget*>(obj)->repaint(true);
			return false;
		}
		return false;
	}
	

	if (obj->isA("KDockButton_Private")) {
		if (ev->type() == QEvent::Paint) {
			QPixmap* btnPix = 0;
			QButton* btn = static_cast<QButton*>(obj);
			if((btnPix = QPixmapCache::find(btn->paletteBackgroundColor().name()+btn->name()))) {
			}
			else {
				QPixmap* tmpPix = new QPixmap(uic_findImage(btn->name()));
				if(!tmpPix)
					return false;
				btnPix = new QPixmap(12, 24);
				btnPix->fill(btn->paletteBackgroundColor());
				bitBlt(btnPix, 0, 0, tmpPix, 0, 0, -1, -1, Qt::CopyROP);
				QPixmapCache::insert(btn->paletteBackgroundColor().name()+btn->name(), btnPix);
			}
			
			if(btn->isOn() || btn->isDown())
				bitBlt(btn, 0, 1, btnPix, 0, 12, 12, 12, Qt::CopyROP);
			else
				bitBlt(btn, 0, 1, btnPix, 0, 0, 12, 12, Qt::CopyROP);
			return true;
		}
		return false;
	}

	if(dynamic_cast<QButton*>(obj) && obj->parent() && obj->parent()->isA("QToolBox")) {
		QWidget* w = dynamic_cast<QWidget*>(obj);
		if(!w->isEnabled())
			return false;
		if(ev->type() == QEvent::Enter) {
			hoveredToolBoxTab = true;
			w->repaint(false);
			return false;
		}
		if(ev->type() == QEvent::Leave) {
			hoveredToolBoxTab = false;
			w->repaint(false);
			return false;
		}
		if(ev->type() == QEvent::Move) {
			hoveredToolBoxTab = false;
			return false;
		}
		return false;
	}
	if(dynamic_cast<QHeader*>(obj)) {
		switch(ev->type()) {
			case QEvent::ApplicationPaletteChange:
			case QEvent::PaletteChange:
				dynamic_cast<QHeader*>(obj)->setBackgroundMode(Qt::PaletteBackground);
			default:
				break;
		}
		return false;
	}
	if(dynamic_cast<QDockWindow*>(obj)) {
		switch(ev->type()) {
			case QEvent::LayoutHint: {
				QDockWindow* dw = dynamic_cast<QDockWindow*>(obj);
				if(!dw->widget())
					return false;
				if(!dw->area()) {
					dw->setFrameStyle(QFrame::Panel|QFrame::Sunken);
					dw->setLineWidth(1);
					QRect r = dw->rect();
					QRegion mask(r);
					mask -= QRegion(0, 0, 1, 1);
					mask -= QRegion(r.width()-1, 0, 1, 1);
					mask -= QRegion(0, r.height()-1, 1, 1);
					mask -= QRegion(r.width()-1, r.height()-1, 1, 1);
					dw->setMask(mask);
				}
				else if(dw->frameStyle() == QFrame::Panel|QFrame::Sunken) {
					dw->setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
					dw->setLineWidth(2);
					dw->clearMask();
				}
			}
			default:
				break;
				
		}
	}
	else if(obj->isA("QWorkspaceChild")) {
		switch(ev->type()) {
			case QEvent::Show:
			case QEvent::Resize: {
				QWidget* w = static_cast<QWidget*>(obj);
				QRect r = w->rect();
				QRegion mask(r);
				mask -= QRegion(0, 0, 1, 1);
				mask -= QRegion(r.width()-1, 0, 1, 1);
				mask -= QRegion(0, r.height()-1, 1, 1);
				mask -= QRegion(r.width()-1, r.height()-1, 1, 1);
				w->setMask(mask);
			}
			default:
				break;
		}
	}
	

	if (dynamic_cast<QFrame*>(obj)) { // at last because other widgets may include a frame.
		
		QFrame* f = dynamic_cast<QFrame*>(obj);
		const QRect r = f->rect();
		
		if (ev->type() == QEvent::Paint) {
			
			switch(f->frameShape()){
				case QFrame::HLine: {
					QPainter p(f);
					p.setPen(f->paletteBackgroundColor().dark(120));
					p.drawLine(0,0,f->frameRect().width(),0);
					return true;
				}
				case QFrame::VLine: {
					QPainter p(f);
					p.setPen(f->paletteBackgroundColor().dark(120));
					p.drawLine(0,0,0,f->frameRect().height());
					return true;
				}
				case QFrame::WinPanel:
				case QFrame::Panel: {
					if(f->parentWidget() && f->parentWidget()->isA("KDockSplitter")) {
						if(!f->erasePixmap()) {
							bool horizontal = (r.width() > r.height());
							QPixmap pix(r.size());
							QPainter p(&pix);
							renderGradient(&p, r, lightenColor(f->paletteBackgroundColor(), 40), darkenColor(f->paletteBackgroundColor(), 30), horizontal);
							f->setErasePixmap(pix);
						}
						return true;
					}
					
					QPainter p(f);
					QColor frameColor;
					if(f->frameShape() == QFrame::Panel && f->frameShadow() == QFrame::Plain) // used as a selection in kspread
						frameColor = f->paletteBackgroundColor().dark(160);
					else if(dynamic_cast<QListBox*>(obj) && dynamic_cast<QComboBox*>(obj->parent()))
						frameColor = f->paletteBackgroundColor().dark(190);
					else
						frameColor = f->paletteBackgroundColor().dark(120);
					
						
					p.setPen(frameColor);
					p.drawRect(f->rect());
					
					QPaintEvent* e = (QPaintEvent*)ev;
					p.setClipRegion(e->region().intersect(f->contentsRect()));
					DominoQFrame* df = (DominoQFrame*)f;
					df->drawContents(&p);
					
					DominoQWidget* dw = (DominoQWidget*)f;
					QPaintEvent* event = new QPaintEvent(e->region().intersect(f->contentsRect()));
					dw->paintEvent(event);
					return true;
				}
				case QFrame::ToolBarPanel: {
					if(f->isA("KonqSidebarTree")) {
						QPainter p(f);
						p.setPen(qApp->palette().active().background().dark(120));
						p.drawRect(f->rect());
						return true;
					}
					return false;
				}
				case QFrame::Box: {
					if(f->parent() && !strcmp(f->parent()->name(), "KonqFrame")) {
						QPainter p(f);
						p.fillRect(r.width()-16, r.height()-16, 15, 15, qApp->palette().active().background()); // 15x15 scrollview cornerwidget gap
						return true;
					}
					else if(konquerorMode && f->isA("KHTMLView")) {
						QPainter p(f);
						p.setPen(qApp->palette().active().background().dark(120));
						p.drawRect(f->rect());
						return true;
					}
				}
				default:
					return false;
			}
		}
		
		if (ev->type() == QEvent::Show || ev->type() == QEvent::ApplicationPaletteChange) { // ApplicationPaletteChange for style changes
			
			switch(f->frameShape()){
				case QFrame::TabWidgetPanel:
					f->setLineWidth(3);
					f->setMargin(3);
					f->setFrameRect(QRect(r.x()-2, r.y(), r.width()+4, r.height())); // ### lprof grows infinitely here
					return true;
				case QFrame::Box:
					if(f->parent() && !strcmp(f->parent()->name(), "KonqFrame")) {
						f->setEraseColor(qApp->palette().active().background().dark(120));
						f->erase();
						return true;
					}
				case QFrame::Panel:
					if(ev->type() == QEvent::ApplicationPaletteChange && f->parentWidget() && f->parentWidget()->isA("KDockSplitter")) {
						f->setBackgroundMode(Qt::PaletteBackground);
						return true;
					}
				default:
					return false;
			}
			return false;
		}
	}
	
	// nothing matched
	return false;
	
}
