/*
Copyright (C) 2006 Michael Lentner <michaell@gmx.net>

based on the Plastik configuration dialog:
Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

based on the Keramik configuration dialog:
Copyright (c) 2003 Maksim Orlovich <maksim.orlovich@kdemail.net>

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/

#include <qapplication.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qsettings.h>
#include <kglobal.h>
#include <klocale.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <kcolorbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qscrollbar.h>
#include <qheader.h>
#include <qlistview.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qstyle.h>
#include <qpainter.h>
#include <qframe.h>
#include <qlistbox.h>
#include <qheader.h>
#include <kinputdialog.h>
#include <kstandarddirs.h>
#include <qdir.h>
#include <qbuttongroup.h>
#include <kmessagebox.h>
#include <qtabbar.h>

#include "dominoconf.h"
#include <../domino.h>
#include <../misc.cpp>
extern "C"
{
	KDE_EXPORT QWidget* allocate_kstyle_config(QWidget* parent)
	{
		KGlobal::locale()->insertCatalogue("kstyle_domino_config");
		return new DominoStyleConfig(parent);
	}
}



DominoStyleConfig::DominoStyleConfig(QWidget* parent): QWidget(parent)
{
	
	
 //Should have no margins here, the dialog provides them
	QVBoxLayout* layout = new QVBoxLayout(this, 0, 0);
	KGlobal::locale()->insertCatalogue("kstyle_domino_config");

	QTabWidget *tabWidget = new QTabWidget(this);
	//tabWidget->setMargin(7);
	
	layout->addWidget(tabWidget);
	
////////////////////////////////////////////////////////////////////////////
// tab 1
////////////////////////////////////////////////////////////////////////////
	
	QScrollView* sv = new QScrollView(tabWidget);
	QWidget* scrollWidget = new QWidget(sv->viewport());
	sv->addChild(scrollWidget);
	sv->setResizePolicy( QScrollView::AutoOneFit );
	QVBoxLayout* page1layout = new QVBoxLayout(scrollWidget);
	page1layout->setMargin(5);
	tabWidget->addTab(sv, "Misc");
	

	//animateProgressBar = new QCheckBox(i18n("Animate progress bars"), scrollWidget);
	centerTabs = new QCheckBox(i18n("Center tabs"), scrollWidget);
	drawTriangularExpander = new QCheckBox(i18n("Triangular tree expander"), scrollWidget);
	smoothScrolling = new QCheckBox(i18n("Smoother scrolling"), scrollWidget);
	
	QHBox *hbox5 = new QHBox(scrollWidget);
	buttonHeightAdjustmentLabel = new QLabel(hbox5);
	buttonHeightAdjustment = new DSpinBox(-5, 5, 1, hbox5);
	buttonHeightAdjustment->setAlignRight();
	buttonHeightAdjustment->setSuffix(" px");
	
	customCheckMarkColor = new QCheckBox(i18n("Custom checkmark color"), scrollWidget);
	QHBox *hbox3 = new QHBox(scrollWidget);
	hbox3->layout()->addItem(new QSpacerItem(20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum) );
	checkMarkColor = new KColorButton(hbox3);
	
	customToolTipColor = new QCheckBox(i18n("Custom tooltip color"), scrollWidget);
	QHBox *hbox4 = new QHBox(scrollWidget);
	hbox4->layout()->addItem(new QSpacerItem(20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum) );
	toolTipColor = new KColorButton(hbox4);
	
	customPopupMenuColor = new QCheckBox(i18n("Custom popupmenu color"), scrollWidget);
	QHBox *hbox6 = new QHBox(scrollWidget);
	hbox6->layout()->addItem(new QSpacerItem(20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum) );
	popupMenuColor = new KColorButton(hbox6);
	tintGroupBoxBackground = new QButtonGroup(1, Qt::Horizontal, i18n("Tint groupbox background"), scrollWidget);
	tintGroupBoxBackground->setCheckable(true);
	drawGroupBoxDark = new QRadioButton(i18n("Darker background"), tintGroupBoxBackground);
	drawGroupBoxBright = new QRadioButton(i18n("Brighter background"), tintGroupBoxBackground);
	
	// focusIndicator
	indicateFocus = new QGroupBox(scrollWidget);
	indicateFocus->setCheckable( TRUE );
	indicateFocus->setColumnLayout(0, Qt::Vertical );
	indicateFocus->layout()->setSpacing( 6 );
	indicateFocus->layout()->setMargin( 11 );
	QGridLayout* indicateFocusLayout = new QGridLayout( indicateFocus->layout(), 2, 2 );
	indicateFocusLayout->setAlignment( Qt::AlignTop );
	
	indicatorType = new QButtonGroup(1, Qt::Vertical, i18n("Indicator type:"), indicateFocus);
	drawUnderline = new QRadioButton(i18n("Underline"), indicatorType);
	drawColoredOutline = new QRadioButton(i18n("Colored outline"), indicatorType);
	QHBoxLayout* hbox11 = new QHBoxLayout(0, 0, 6);
	hbox11->addItem( new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
	hbox11->addWidget( indicatorType );
	hbox11->addItem( new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
	
	QHBoxLayout* hbox7 = new QHBoxLayout(0, 0, 6);
	labelIndicatorColor = new QLabel(indicateFocus);
	hbox7->addWidget( labelIndicatorColor );
	hbox7->addItem( new QSpacerItem( 21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum ) );
	indicatorColor = new KColorButton( indicateFocus );
	indicatorColor->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed );
	hbox7->addWidget( indicatorColor );
	
	QHBoxLayout* hbox8 = new QHBoxLayout(0, 0, 5);
	labelIndicatorBtnColor = new QLabel(indicateFocus );
	hbox8->addWidget( labelIndicatorBtnColor );
	hbox8->addItem( new QSpacerItem( 21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum ) );
	indicatorBtnColor = new KColorButton( indicateFocus );
	indicatorBtnColor->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed );
	indicatorBtnColor->setFocusPolicy(QWidget::NoFocus);
	hbox8->addWidget( indicatorBtnColor );
	
	
	QFrame* prFrame = new QFrame( indicateFocus);
	prFrame->setPaletteBackgroundColor(qApp->palette().active().background());
	prFrame->setFrameShape( QFrame::StyledPanel );
	prFrame->setFrameShadow( QFrame::Raised );
	QVBoxLayout* prFrameLayout = new QVBoxLayout( prFrame, 3, 6);
	
	QHBoxLayout* hbox9 = new QHBoxLayout(0, 0, 6);
	hbox9->addItem(new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ));
	indicatorPrevWidget = new IndicatorPrevWidget( prFrame );
	int width = indicatorPrevWidget->fontMetrics().width(i18n("focused"));
	indicatorPrevWidget->setMinimumWidth(width+8);
	hbox9->addWidget( indicatorPrevWidget );
	hbox9->addItem(new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ));
	
	QHBoxLayout* hbox10 = new QHBoxLayout(0, 0, 6);
	hbox10->addItem(new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ));
	indicatorPrevButton = new IndicatorPrevButton( prFrame );
	width = indicatorPrevButton->fontMetrics().width(i18n("focused"));
	indicatorPrevButton->setMinimumWidth(width+16);
	hbox10->addWidget( indicatorPrevButton );
	hbox10->addItem(new QSpacerItem( 61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum ));
	prFrameLayout->addLayout(hbox9);
	prFrameLayout->addLayout(hbox10);

	
	indicateFocusLayout->addMultiCellLayout( hbox11, 0, 0, 0, 2 );
	indicateFocusLayout->addLayout(hbox7, 1, 0);
	indicateFocusLayout->addLayout(hbox8, 2, 0);
	indicateFocusLayout->addMultiCellWidget( prFrame, 1, 2, 1, 2 );

	connect(indicatorType, SIGNAL(clicked(int)), indicatorPrevButton, SLOT(update()));
	connect(indicatorType, SIGNAL(clicked(int)), indicatorPrevWidget, SLOT(update()));
	
	connect(drawColoredOutline, SIGNAL(toggled(bool)), indicatorColor, SLOT(setEnabled(bool)));
	connect(drawColoredOutline, SIGNAL(toggled(bool)), indicatorBtnColor, SLOT(setEnabled(bool)));
	//
	

	
//	page1layout->add(animateProgressBar);
	page1layout->add(centerTabs);
	page1layout->add(drawTriangularExpander);
	page1layout->add(smoothScrolling);
	page1layout->add(hbox5);
	page1layout->add(customCheckMarkColor);
	page1layout->add(hbox3);
	page1layout->add(customToolTipColor);
	page1layout->add(hbox4);
	page1layout->add(customPopupMenuColor);
	page1layout->add(hbox6);
	page1layout->add(tintGroupBoxBackground);
	page1layout->add(indicateFocus);
	
	page1layout->addStretch(1);
	
	
	QWidget *page2 = new QWidget();
	tabWidget->addTab(page2, "Custom Gradients");
	
	QVBoxLayout* page2layout = new QVBoxLayout(page2);

////////////////////////////////////////////////////////////////////////////
// tab 2 (custom Gradients)
////////////////////////////////////////////////////////////////////////////

	tabWidget2 = new QTabWidget( page2, "tabWidget2" );
	page2layout->add(tabWidget2);

////////////////////////////////////////////////////////////////////////////
// tab 2.1 (buttons)
////////////////////////////////////////////////////////////////////////////

	tab1 = new QWidget( tabWidget2, "tab1" );
	tab1Layout = new QVBoxLayout( tab1, 11, 6, "tab1Layout"); 
	
	groupBox22 = new QGroupBox( tab1, "groupBox22" );
	groupBox22->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox22->sizePolicy().hasHeightForWidth() ) );
	groupBox22->setColumnLayout(0, Qt::Vertical );
	groupBox22->layout()->setSpacing( 0 );
	groupBox22->layout()->setMargin( 11 );
	groupBox22Layout = new QGridLayout( groupBox22->layout() );
	groupBox22Layout->setAlignment( Qt::AlignTop );
	spacer6 = new QSpacerItem( 83, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22Layout->addItem( spacer6, 3, 3 );

	textLabel1_2 = new QLabel( groupBox22, "textLabel1_2" );
	textLabel1_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2->sizePolicy().hasHeightForWidth() ) );
	textLabel1_2->setTextFormat( QLabel::PlainText );

	groupBox22Layout->addWidget( textLabel1_2, 3, 4 );

	kColorButton27_2 = new KColorButton( groupBox22, "kColorButton27_2" );
	kColorButton27_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, kColorButton27_2->sizePolicy().hasHeightForWidth() ) );
	kColorButton27_2->setMinimumSize( QSize( 75, 0 ) );

	groupBox22Layout->addWidget( kColorButton27_2, 3, 5 );
	spacer3 = new QSpacerItem( 12, 25, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22Layout->addItem( spacer3, 3, 6 );

	groupBox10_2_2_2 = new QGroupBox( groupBox22, "groupBox10_2_2_2" );
	groupBox10_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, groupBox10_2_2_2->sizePolicy().hasHeightForWidth() ) );
	groupBox10_2_2_2->setMinimumSize( QSize( 170, 0 ) );
	groupBox10_2_2_2->setColumnLayout(0, Qt::Vertical );
	groupBox10_2_2_2->layout()->setSpacing( 6 );
	groupBox10_2_2_2->layout()->setMargin( 11 );
	groupBox10_2_2_2Layout = new QVBoxLayout( groupBox10_2_2_2->layout() );
	groupBox10_2_2_2Layout->setAlignment( Qt::AlignTop );


	pushButton16 = new PreviewPushButton(groupBox10_2_2_2, "dominoPreviewWidget" );
	if(strcmp(pushButton16->style().name(), "domino"))
		pushButton16->setStyle("domino");
	groupBox10_2_2_2Layout->addWidget( pushButton16 );

	comboBox17 = new PreviewComboBox( FALSE, groupBox10_2_2_2, "dominoPreviewWidget" );
	if(strcmp(comboBox17->style().name(), "domino"))
		comboBox17->setStyle("domino");
	groupBox10_2_2_2Layout->addWidget( comboBox17 );

	spinBox44 = new PreviewSpinBox( groupBox10_2_2_2, "dominoPreviewWidget" );
	if(strcmp(spinBox44->style().name(), "domino")) {
		spinBox44->setStyle("domino");
		((QSpinWidget*)spinBox44)->setStyle("domino");
	}

	spinBox44->setAlignRight();
	groupBox10_2_2_2Layout->addWidget( spinBox44 );

	groupBox22Layout->addMultiCellWidget( groupBox10_2_2_2, 0, 1, 0, 0 );
	spacer43 = new QSpacerItem( 5, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22Layout->addMultiCell( spacer43, 0, 1, 1, 1 );

	groupBox8_2_2_2 = new QGroupBox( groupBox22, "groupBox8_2_2_2" );
	groupBox8_2_2_2->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox8_2_2_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(btn1GradientChanged()));
	groupBox8_2_2_2->setColumnLayout(0, Qt::Vertical );
	groupBox8_2_2_2->layout()->setSpacing( 7 );
	groupBox8_2_2_2->layout()->setMargin( 4 );
	groupBox8_2_2_2Layout = new QGridLayout( groupBox8_2_2_2->layout() );
	groupBox8_2_2_2Layout->setAlignment( Qt::AlignTop );

	textLabel4_3_2_2_2 = new QLabel( groupBox8_2_2_2, "textLabel4_3_2_2_2" );
	textLabel4_3_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2Layout->addWidget( textLabel4_3_2_2_2, 1, 0 );

	spinBox2_2_2_2_2 = new DSpinBox( groupBox8_2_2_2, "spinBox2_2_2_2_2" );
	spinBox2_2_2_2_2->setAlignRight();
	spinBox2_2_2_2_2->setMaxValue( 100 );

	groupBox8_2_2_2Layout->addWidget( spinBox2_2_2_2_2, 1, 1 );

	textLabel2_2_2_2_2 = new QLabel( groupBox8_2_2_2, "textLabel2_2_2_2_2" );
	textLabel2_2_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2Layout->addWidget( textLabel2_2_2_2_2, 1, 2 );

	kColorButton1_2_2_2_2 = new KColorButton( groupBox8_2_2_2, "kColorButton1_2_2_2_2" );

	groupBox8_2_2_2Layout->addWidget( kColorButton1_2_2_2_2, 1, 3 );

	textLabel3_3_2_2_2 = new QLabel( groupBox8_2_2_2, "textLabel3_3_2_2_2" );
	textLabel3_3_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2Layout->addWidget( textLabel3_3_2_2_2, 0, 0 );

	spinBox1_2_2_2_2 = new DSpinBox( groupBox8_2_2_2, "spinBox1_2_2_2_2" );
	spinBox1_2_2_2_2->setAlignRight();
	spinBox1_2_2_2_2->setMaxValue( 100 );

	groupBox8_2_2_2Layout->addWidget( spinBox1_2_2_2_2, 0, 1 );

	textLabel1_4_2_2_2 = new QLabel( groupBox8_2_2_2, "textLabel1_4_2_2_2" );
	textLabel1_4_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2Layout->addWidget( textLabel1_4_2_2_2, 0, 2 );

	kColorButton2_4_2_2_2 = new KColorButton( groupBox8_2_2_2, "kColorButton2_4_2_2_2" );

	groupBox8_2_2_2Layout->addWidget( kColorButton2_4_2_2_2, 0, 3 );

	groupBox22Layout->addMultiCellWidget( groupBox8_2_2_2, 0, 0, 2, 6 );

	groupBox9_2_2_2 = new QGroupBox( groupBox22, "groupBox9_2_2_2" );
	groupBox9_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox9_2_2_2->sizePolicy().hasHeightForWidth() ) );
	groupBox9_2_2_2->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox9_2_2_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(btn2GradientChanged()));
	groupBox9_2_2_2->setChecked( FALSE );
	groupBox9_2_2_2->setColumnLayout(0, Qt::Vertical );
	groupBox9_2_2_2->layout()->setSpacing( 7 );
	groupBox9_2_2_2->layout()->setMargin( 4 );
	groupBox9_2_2_2Layout = new QGridLayout( groupBox9_2_2_2->layout() );
	groupBox9_2_2_2Layout->setAlignment( Qt::AlignTop );

	textLabel4_4_2_2_2 = new QLabel( groupBox9_2_2_2, "textLabel4_4_2_2_2" );
	textLabel4_4_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2Layout->addWidget( textLabel4_4_2_2_2, 1, 0 );

	spinBox2_3_2_2_2 = new DSpinBox( groupBox9_2_2_2, "spinBox2_3_2_2_2" );
	spinBox2_3_2_2_2->setAlignRight();
	spinBox2_3_2_2_2->setMaxValue( 100 );

	groupBox9_2_2_2Layout->addWidget( spinBox2_3_2_2_2, 1, 1 );

	textLabel3_4_2_2_2 = new QLabel( groupBox9_2_2_2, "textLabel3_4_2_2_2" );
	textLabel3_4_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2Layout->addWidget( textLabel3_4_2_2_2, 0, 0 );

	spinBox1_3_2_2_2 = new DSpinBox( groupBox9_2_2_2, "spinBox1_3_2_2_2" );
	spinBox1_3_2_2_2->setAlignRight();
	spinBox1_3_2_2_2->setMaxValue( 100 );

	groupBox9_2_2_2Layout->addWidget( spinBox1_3_2_2_2, 0, 1 );

	kColorButton1_3_2_2_2 = new KColorButton( groupBox9_2_2_2, "kColorButton1_3_2_2_2" );
	kColorButton1_3_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, kColorButton1_3_2_2_2->sizePolicy().hasHeightForWidth() ) );

	groupBox9_2_2_2Layout->addWidget( kColorButton1_3_2_2_2, 1, 3 );

	kColorButton2_5_2_2_2 = new KColorButton( groupBox9_2_2_2, "kColorButton2_5_2_2_2" );

	groupBox9_2_2_2Layout->addWidget( kColorButton2_5_2_2_2, 0, 3 );

	textLabel1_5_2_2_2 = new QLabel( groupBox9_2_2_2, "textLabel1_5_2_2_2" );
	textLabel1_5_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2Layout->addWidget( textLabel1_5_2_2_2, 0, 2 );

	textLabel2_3_2_2_2 = new QLabel( groupBox9_2_2_2, "textLabel2_3_2_2_2" );
	textLabel2_3_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2Layout->addWidget( textLabel2_3_2_2_2, 1, 2 );

	groupBox22Layout->addMultiCellWidget( groupBox9_2_2_2, 1, 1, 2, 6 );
	spacer46 = new QSpacerItem( 20, 7, QSizePolicy::Minimum, QSizePolicy::Fixed );
	groupBox22Layout->addItem( spacer46, 2, 4 );
	spacer2 = new QSpacerItem( 82, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22Layout->addItem( spacer2, 3, 2 );
	tab1Layout->addWidget( groupBox22 );

	layout9 = new QHBoxLayout( 0, 0, 6, "layout9"); 
	spacer7 = new QSpacerItem( 270, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout9->addItem( spacer7 );

	textLabel7_4 = new QLabel( tab1, "textLabel7_4" );
	layout9->addWidget( textLabel7_4 );

	comboBox3_4 = new QComboBox( FALSE, tab1, "comboBox3_4" );
	layout9->addWidget( comboBox3_4 );

	pushButton4_4 = new QPushButton( tab1, "pushButton4_4" );
	connect(pushButton4_4, SIGNAL(clicked()), SLOT(copyColorsToButton()));
	layout9->addWidget( pushButton4_4 );
	tab1Layout->addLayout( layout9 );
	tabWidget2->insertTab( tab1, QString::fromLatin1("") );
	
/////////////////////////////////////////////////
// tab 2.2 (tabs)
/////////////////////////////////////////////////
	   tab2 = new QWidget( tabWidget2, "tab2" );
	   tab2Layout = new QVBoxLayout( tab2, 11, 6, "tab2Layout"); 

	   groupBox22_2 = new QGroupBox( tab2, "groupBox22_2" );
	   groupBox22_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox22_2->sizePolicy().hasHeightForWidth() ) );
	   groupBox22_2->setColumnLayout(0, Qt::Vertical );
	   groupBox22_2->layout()->setSpacing( 0 );
	   groupBox22_2->layout()->setMargin( 11 );
	   groupBox22_2Layout = new QGridLayout( groupBox22_2->layout() );
	   groupBox22_2Layout->setAlignment( Qt::AlignTop );
	   spacer46_2 = new QSpacerItem( 20, 7, QSizePolicy::Minimum, QSizePolicy::Fixed );
	   groupBox22_2Layout->addItem( spacer46_2, 2, 5 );
	   spacer43_2 = new QSpacerItem( 5, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	   groupBox22_2Layout->addMultiCell( spacer43_2, 0, 1, 1, 1 );

	   groupBox8_2_2_2_2 = new QGroupBox( groupBox22_2, "groupBox8_2_2_2_2" );
	   groupBox8_2_2_2_2->setCheckable( TRUE );
	   connect(((QCheckBox*)groupBox8_2_2_2_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(tab1GradientChanged()));
	   groupBox8_2_2_2_2->setColumnLayout(0, Qt::Vertical );
	   groupBox8_2_2_2_2->layout()->setSpacing( 7 );
	   groupBox8_2_2_2_2->layout()->setMargin( 4 );
	   groupBox8_2_2_2_2Layout = new QGridLayout( groupBox8_2_2_2_2->layout() );
	   groupBox8_2_2_2_2Layout->setAlignment( Qt::AlignTop );

	   textLabel4_3_2_2_2_2 = new QLabel( groupBox8_2_2_2_2, "textLabel4_3_2_2_2_2" );
	   textLabel4_3_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox8_2_2_2_2Layout->addWidget( textLabel4_3_2_2_2_2, 1, 0 );

	   spinBox2_2_2_2_2_2 = new DSpinBox( groupBox8_2_2_2_2, "spinBox2_2_2_2_2_2" );
	   spinBox2_2_2_2_2_2->setAlignRight();
	   spinBox2_2_2_2_2_2->setMaxValue( 100 );

	   groupBox8_2_2_2_2Layout->addWidget( spinBox2_2_2_2_2_2, 1, 1 );

	   textLabel2_2_2_2_2_2 = new QLabel( groupBox8_2_2_2_2, "textLabel2_2_2_2_2_2" );
	   textLabel2_2_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox8_2_2_2_2Layout->addWidget( textLabel2_2_2_2_2_2, 1, 2 );

	   kColorButton1_2_2_2_2_2 = new KColorButton( groupBox8_2_2_2_2, "kColorButton1_2_2_2_2_2" );

	   groupBox8_2_2_2_2Layout->addWidget( kColorButton1_2_2_2_2_2, 1, 3 );

	   textLabel3_3_2_2_2_2 = new QLabel( groupBox8_2_2_2_2, "textLabel3_3_2_2_2_2" );
	   textLabel3_3_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox8_2_2_2_2Layout->addWidget( textLabel3_3_2_2_2_2, 0, 0 );

	   spinBox1_2_2_2_2_2 = new DSpinBox( groupBox8_2_2_2_2, "spinBox1_2_2_2_2_2" );
	   spinBox1_2_2_2_2_2->setAlignRight();
	   spinBox1_2_2_2_2_2->setMaxValue( 100 );

	   groupBox8_2_2_2_2Layout->addWidget( spinBox1_2_2_2_2_2, 0, 1 );

	   textLabel1_4_2_2_2_2 = new QLabel( groupBox8_2_2_2_2, "textLabel1_4_2_2_2_2" );
	   textLabel1_4_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox8_2_2_2_2Layout->addWidget( textLabel1_4_2_2_2_2, 0, 2 );

	   kColorButton2_4_2_2_2_2 = new KColorButton( groupBox8_2_2_2_2, "kColorButton2_4_2_2_2_2" );

	   groupBox8_2_2_2_2Layout->addWidget( kColorButton2_4_2_2_2_2, 0, 3 );

	   groupBox22_2Layout->addMultiCellWidget( groupBox8_2_2_2_2, 0, 0, 2, 6 );

	   groupBox9_2_2_2_2 = new QGroupBox( groupBox22_2, "groupBox9_2_2_2_2" );
	   groupBox9_2_2_2_2->setCheckable( TRUE );
	   connect(((QCheckBox*)groupBox9_2_2_2_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(tab2GradientChanged()));
	   groupBox9_2_2_2_2->setChecked( FALSE );
	   groupBox9_2_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox9_2_2_2_2->sizePolicy().hasHeightForWidth() ) );
	   groupBox9_2_2_2_2->setColumnLayout(0, Qt::Vertical );
	   groupBox9_2_2_2_2->layout()->setSpacing( 7 );
	   groupBox9_2_2_2_2->layout()->setMargin( 4 );
	   groupBox9_2_2_2_2Layout = new QGridLayout( groupBox9_2_2_2_2->layout() );
	   groupBox9_2_2_2_2Layout->setAlignment( Qt::AlignTop );

	   textLabel4_4_2_2_2_2 = new QLabel( groupBox9_2_2_2_2, "textLabel4_4_2_2_2_2" );
	   textLabel4_4_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox9_2_2_2_2Layout->addWidget( textLabel4_4_2_2_2_2, 1, 0 );

	   spinBox2_3_2_2_2_2 = new DSpinBox( groupBox9_2_2_2_2, "spinBox2_3_2_2_2_2" );
	   spinBox2_3_2_2_2_2->setButtonSymbols( QSpinBox::UpDownArrows );
	   spinBox2_3_2_2_2_2->setAlignRight();
	   spinBox2_3_2_2_2_2->setMaxValue( 100 );

	   groupBox9_2_2_2_2Layout->addWidget( spinBox2_3_2_2_2_2, 1, 1 );

	   textLabel3_4_2_2_2_2 = new QLabel( groupBox9_2_2_2_2, "textLabel3_4_2_2_2_2" );
	   textLabel3_4_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox9_2_2_2_2Layout->addWidget( textLabel3_4_2_2_2_2, 0, 0 );

	   spinBox1_3_2_2_2_2 = new DSpinBox( groupBox9_2_2_2_2, "spinBox1_3_2_2_2_2" );
	   spinBox1_3_2_2_2_2->setAlignRight();
	   spinBox1_3_2_2_2_2->setMaxValue( 100 );

	   groupBox9_2_2_2_2Layout->addWidget( spinBox1_3_2_2_2_2, 0, 1 );

	   kColorButton1_3_2_2_2_2 = new KColorButton( groupBox9_2_2_2_2, "kColorButton1_3_2_2_2_2" );
	   kColorButton1_3_2_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, kColorButton1_3_2_2_2_2->sizePolicy().hasHeightForWidth() ) );

	   groupBox9_2_2_2_2Layout->addWidget( kColorButton1_3_2_2_2_2, 1, 3 );

	   kColorButton2_5_2_2_2_2 = new KColorButton( groupBox9_2_2_2_2, "kColorButton2_5_2_2_2_2" );

	   groupBox9_2_2_2_2Layout->addWidget( kColorButton2_5_2_2_2_2, 0, 3 );

	   textLabel1_5_2_2_2_2 = new QLabel( groupBox9_2_2_2_2, "textLabel1_5_2_2_2_2" );
	   textLabel1_5_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox9_2_2_2_2Layout->addWidget( textLabel1_5_2_2_2_2, 0, 2 );

	   textLabel2_3_2_2_2_2 = new QLabel( groupBox9_2_2_2_2, "textLabel2_3_2_2_2_2" );
	   textLabel2_3_2_2_2_2->setTextFormat( QLabel::PlainText );

	   groupBox9_2_2_2_2Layout->addWidget( textLabel2_3_2_2_2_2, 1, 2 );

	   groupBox22_2Layout->addMultiCellWidget( groupBox9_2_2_2_2, 1, 1, 2, 6 );
	   spacer6_2 = new QSpacerItem( 83, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	   groupBox22_2Layout->addItem( spacer6_2, 3, 3 );

	   textLabel1_2_2 = new QLabel( groupBox22_2, "textLabel1_2_2" );
	   textLabel1_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2_2->sizePolicy().hasHeightForWidth() ) );
	   textLabel1_2_2->setTextFormat( QLabel::PlainText );

	   groupBox22_2Layout->addWidget( textLabel1_2_2, 3, 4 );
	   spacer2_2 = new QSpacerItem( 82, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	   groupBox22_2Layout->addItem( spacer2_2, 3, 2 );
	   spacer3_2 = new QSpacerItem( 12, 25, QSizePolicy::Fixed, QSizePolicy::Minimum );
	   groupBox22_2Layout->addItem( spacer3_2, 3, 6 );

	   kColorButton27_2_2 = new KColorButton( groupBox22_2, "kColorButton27_2_2" );
	   kColorButton27_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, kColorButton27_2_2->sizePolicy().hasHeightForWidth() ) );
	   kColorButton27_2_2->setMinimumSize( QSize( 75, 0 ) );

	   groupBox22_2Layout->addWidget( kColorButton27_2_2, 3, 5 );

	   groupBox10_2_2_2_2 = new QGroupBox( groupBox22_2, "groupBox10_2_2_2_2" );
	   groupBox10_2_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, groupBox10_2_2_2_2->sizePolicy().hasHeightForWidth() ) );
	   groupBox10_2_2_2_2->setMinimumSize( QSize( 170, 0 ) );
	   groupBox10_2_2_2_2->setColumnLayout(0, Qt::Vertical );
	   groupBox10_2_2_2_2->layout()->setSpacing( 6 );
	   groupBox10_2_2_2_2->layout()->setMargin( 11 );
	   groupBox10_2_2_2_2Layout = new QVBoxLayout( groupBox10_2_2_2_2->layout() );
	   groupBox10_2_2_2_2Layout->setAlignment( Qt::AlignTop );
	   
	   prevTopTabWidget = new QTabWidget(groupBox10_2_2_2_2);
	   if(strcmp(prevTopTabWidget->style().name(), "domino"))
		   prevTopTabWidget->setStyle("domino");
	   tabBarTop = new PreviewTabBar(0);
	   ((DominoQTabWidget*)prevTopTabWidget)->setTabBar(tabBarTop);
	   
	   QWidget* prevTopPage1 = new QWidget(prevTopTabWidget);
	   QWidget* prevTopPage2 = new QWidget(prevTopTabWidget);
	   QWidget* prevTopPage3 = new QWidget(prevTopTabWidget);
	   
	   prevTopTabWidget->addTab(prevTopPage1, "  1  ");
	   prevTopTabWidget->addTab(prevTopPage2, "  2  ");
	   prevTopTabWidget->addTab(prevTopPage3, "  3  ");
	   prevTopTabWidget->page(0)->parentWidget()->setPaletteBackgroundColor(qApp->palette().active().background());
	   prevTopTabWidget->setCurrentPage(1);
	   
	   
	   prevBottomTabWidget = new QTabWidget(groupBox10_2_2_2_2);
	   if(strcmp(prevBottomTabWidget->style().name(), "domino"))
		   prevBottomTabWidget->setStyle("domino");
	   tabBarBottom = new PreviewTabBar(0);
	   ((DominoQTabWidget*)prevBottomTabWidget)->setTabBar(tabBarBottom);
	   prevBottomTabWidget->setTabPosition(QTabWidget::Bottom);
	   
	   QWidget* prevBottomPage1 = new QWidget(prevBottomTabWidget);
	   QWidget* prevBottomPage2 = new QWidget(prevBottomTabWidget);
	   QWidget* prevBottomPage3 = new QWidget(prevBottomTabWidget);
	   
	   prevBottomTabWidget->addTab(prevBottomPage1, "  1  ");
	   prevBottomTabWidget->addTab(prevBottomPage2, "  2  ");
	   prevBottomTabWidget->addTab(prevBottomPage3, "  3  ");
	   prevBottomTabWidget->setCurrentPage(1);
	   
	   groupBox10_2_2_2_2Layout->addWidget(prevTopTabWidget);
	   groupBox10_2_2_2_2Layout->addWidget(prevBottomTabWidget);
	   
	   
	   groupBox22_2Layout->addMultiCellWidget( groupBox10_2_2_2_2, 0, 1, 0, 0 );
	   tab2Layout->addWidget( groupBox22_2 );

	   layout9_2 = new QHBoxLayout( 0, 0, 6, "layout9_2"); 
	   
	   tabComboLayout = new QHBoxLayout( 0, 0, 6, "tabComboLayout");
	   tabPosCombo = new QComboBox( FALSE, tab2, "tabPosCombo" );
	   tabComboLayout->addWidget( tabPosCombo );
	   
	   tabStateCombo = new QComboBox( FALSE, tab2, "tabStateCombo" );
	   tabComboLayout->addWidget( tabStateCombo );
	   
	   QSpacerItem* spacer_combo2 = new QSpacerItem( 20, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	   tabComboLayout->addItem( spacer_combo2 );
	   
	   layout9_2->addLayout( tabComboLayout );
	   
	   
	   spacer7_2 = new QSpacerItem( 270, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	   layout9_2->addItem( spacer7_2 );

	   textLabel7_4_2 = new QLabel( tab2, "textLabel7_4_2" );
	   layout9_2->addWidget( textLabel7_4_2 );
	   
	   comboBox3_4_2 = new QComboBox( FALSE, tab2, "comboBox3_4_2" );
	   layout9_2->addWidget( comboBox3_4_2 );

	   pushButton4_4_2 = new QPushButton( tab2, "pushButton4_4_2" );
	   connect(pushButton4_4_2, SIGNAL(clicked()), SLOT(copyColorsToTab()));
	   layout9_2->addWidget( pushButton4_4_2 );
	   
	   tab2Layout->addLayout( layout9_2 );
	   tabWidget2->insertTab( tab2, QString::fromLatin1("") );
	   
	   
	
	
/////////////////////////////////////////////////
// tab 2.3 (scrollbars)
/////////////////////////////////////////////////
	tab3 = new QWidget( tabWidget2, "tab3" );
	tab3Layout = new QVBoxLayout( tab3, 11, 6, "tab3Layout"); 

	groupBox22_2_2 = new QGroupBox( tab3, "groupBox22_2_2" );
	groupBox22_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox22_2_2->sizePolicy().hasHeightForWidth() ) );
	groupBox22_2_2->setColumnLayout(0, Qt::Vertical );
	groupBox22_2_2->layout()->setSpacing( 0 );
	groupBox22_2_2->layout()->setMargin( 11 );
	groupBox22_2_2Layout = new QGridLayout( groupBox22_2_2->layout() );
	groupBox22_2_2Layout->setAlignment( Qt::AlignTop );

	groupBox8_2_2_2_2_4 = new QGroupBox( groupBox22_2_2, "groupBox8_2_2_2_2_4" );
	groupBox8_2_2_2_2_4->setCheckable( TRUE );
	groupBox8_2_2_2_2_4->setColumnLayout(0, Qt::Vertical );
	groupBox8_2_2_2_2_4->layout()->setSpacing( 7 );
	groupBox8_2_2_2_2_4->layout()->setMargin( 4 );
	groupBox8_2_2_2_2_4Layout = new QGridLayout( groupBox8_2_2_2_2_4->layout() );
	groupBox8_2_2_2_2_4Layout->setAlignment( Qt::AlignTop );
	connect(((QCheckBox*)groupBox8_2_2_2_2_4->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(scrollBar1GradientChanged()));

	textLabel4_3_2_2_2_2_4 = new QLabel( groupBox8_2_2_2_2_4, "textLabel4_3_2_2_2_2_4" );
	textLabel4_3_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4Layout->addWidget( textLabel4_3_2_2_2_2_4, 1, 0 );

	spinBox2_2_2_2_2_2_4 = new DSpinBox( groupBox8_2_2_2_2_4, "spinBox2_2_2_2_2_2_4" );
	spinBox2_2_2_2_2_2_4->setAlignRight();
	spinBox2_2_2_2_2_2_4->setMaxValue( 100 );

	groupBox8_2_2_2_2_4Layout->addWidget( spinBox2_2_2_2_2_2_4, 1, 1 );

	textLabel2_2_2_2_2_2_4 = new QLabel( groupBox8_2_2_2_2_4, "textLabel2_2_2_2_2_2_4" );
	textLabel2_2_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4Layout->addWidget( textLabel2_2_2_2_2_2_4, 1, 2 );

	kColorButton1_2_2_2_2_2_4 = new KColorButton( groupBox8_2_2_2_2_4, "kColorButton1_2_2_2_2_2_4" );

	groupBox8_2_2_2_2_4Layout->addWidget( kColorButton1_2_2_2_2_2_4, 1, 3 );

	textLabel3_3_2_2_2_2_4 = new QLabel( groupBox8_2_2_2_2_4, "textLabel3_3_2_2_2_2_4" );
	textLabel3_3_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4Layout->addWidget( textLabel3_3_2_2_2_2_4, 0, 0 );

	spinBox1_2_2_2_2_2_4 = new DSpinBox( groupBox8_2_2_2_2_4, "spinBox1_2_2_2_2_2_4" );
	spinBox1_2_2_2_2_2_4->setAlignRight();
	spinBox1_2_2_2_2_2_4->setMaxValue( 100 );

	groupBox8_2_2_2_2_4Layout->addWidget( spinBox1_2_2_2_2_2_4, 0, 1 );

	textLabel1_4_2_2_2_2_4 = new QLabel( groupBox8_2_2_2_2_4, "textLabel1_4_2_2_2_2_4" );
	textLabel1_4_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4Layout->addWidget( textLabel1_4_2_2_2_2_4, 0, 2 );

	kColorButton2_4_2_2_2_2_4 = new KColorButton( groupBox8_2_2_2_2_4, "kColorButton2_4_2_2_2_2_4" );

	groupBox8_2_2_2_2_4Layout->addWidget( kColorButton2_4_2_2_2_2_4, 0, 3 );

	groupBox22_2_2Layout->addMultiCellWidget( groupBox8_2_2_2_2_4, 0, 0, 2, 6 );
	spacer46_2_2 = new QSpacerItem( 20, 7, QSizePolicy::Minimum, QSizePolicy::Fixed );
	groupBox22_2_2Layout->addItem( spacer46_2_2, 2, 5 );

	groupBox9_2_2_2_2_4 = new QGroupBox( groupBox22_2_2, "groupBox9_2_2_2_2_4" );
	groupBox9_2_2_2_2_4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox9_2_2_2_2_4->sizePolicy().hasHeightForWidth() ) );
	groupBox9_2_2_2_2_4->setCheckable( TRUE );
	groupBox9_2_2_2_2_4->setChecked( FALSE );
	groupBox9_2_2_2_2_4->setColumnLayout(0, Qt::Vertical );
	groupBox9_2_2_2_2_4->layout()->setSpacing( 7 );
	groupBox9_2_2_2_2_4->layout()->setMargin( 4 );
	groupBox9_2_2_2_2_4Layout = new QGridLayout( groupBox9_2_2_2_2_4->layout() );
	groupBox9_2_2_2_2_4Layout->setAlignment( Qt::AlignTop );

	connect(((QCheckBox*)groupBox9_2_2_2_2_4->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(scrollBar2GradientChanged()));
	
	textLabel4_4_2_2_2_2_4 = new QLabel( groupBox9_2_2_2_2_4, "textLabel4_4_2_2_2_2_4" );
	textLabel4_4_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4Layout->addWidget( textLabel4_4_2_2_2_2_4, 1, 0 );

	spinBox2_3_2_2_2_2_4 = new DSpinBox( groupBox9_2_2_2_2_4, "spinBox2_3_2_2_2_2_4" );
	spinBox2_3_2_2_2_2_4->setAlignRight();
	spinBox2_3_2_2_2_2_4->setMaxValue( 100 );

	groupBox9_2_2_2_2_4Layout->addWidget( spinBox2_3_2_2_2_2_4, 1, 1 );

	textLabel3_4_2_2_2_2_4 = new QLabel( groupBox9_2_2_2_2_4, "textLabel3_4_2_2_2_2_4" );
	textLabel3_4_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4Layout->addWidget( textLabel3_4_2_2_2_2_4, 0, 0 );

	spinBox1_3_2_2_2_2_4 = new DSpinBox( groupBox9_2_2_2_2_4, "spinBox1_3_2_2_2_2_4" );
	spinBox1_3_2_2_2_2_4->setAlignRight();
	spinBox1_3_2_2_2_2_4->setMaxValue( 100 );

	groupBox9_2_2_2_2_4Layout->addWidget( spinBox1_3_2_2_2_2_4, 0, 1 );

	kColorButton1_3_2_2_2_2_4 = new KColorButton( groupBox9_2_2_2_2_4, "kColorButton1_3_2_2_2_2_4" );
	kColorButton1_3_2_2_2_2_4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, kColorButton1_3_2_2_2_2_4->sizePolicy().hasHeightForWidth() ) );

	groupBox9_2_2_2_2_4Layout->addWidget( kColorButton1_3_2_2_2_2_4, 1, 3 );

	kColorButton2_5_2_2_2_2_4 = new KColorButton( groupBox9_2_2_2_2_4, "kColorButton2_5_2_2_2_2_4" );

	groupBox9_2_2_2_2_4Layout->addWidget( kColorButton2_5_2_2_2_2_4, 0, 3 );

	textLabel1_5_2_2_2_2_4 = new QLabel( groupBox9_2_2_2_2_4, "textLabel1_5_2_2_2_2_4" );
	textLabel1_5_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4Layout->addWidget( textLabel1_5_2_2_2_2_4, 0, 2 );

	textLabel2_3_2_2_2_2_4 = new QLabel( groupBox9_2_2_2_2_4, "textLabel2_3_2_2_2_2_4" );
	textLabel2_3_2_2_2_2_4->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4Layout->addWidget( textLabel2_3_2_2_2_2_4, 1, 2 );

	groupBox22_2_2Layout->addMultiCellWidget( groupBox9_2_2_2_2_4, 1, 1, 2, 6 );

	groupBox10_2_2_2_2_4 = new QGroupBox( groupBox22_2_2, "groupBox10_2_2_2_2_4" );
	groupBox10_2_2_2_2_4->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, groupBox10_2_2_2_2_4->sizePolicy().hasHeightForWidth() ) );
	groupBox10_2_2_2_2_4->setMinimumSize( QSize( 170, 0 ) );
	groupBox10_2_2_2_2_4->setColumnLayout(0, Qt::Vertical );
	groupBox10_2_2_2_2_4->layout()->setSpacing( 6 );
	groupBox10_2_2_2_2_4->layout()->setMargin( 11 );
	groupBox10_2_2_2_2_4Layout = new QGridLayout( groupBox10_2_2_2_2_4->layout() );
	groupBox10_2_2_2_2_4Layout->setAlignment( Qt::AlignTop );
	spacer35 = new QSpacerItem( 112, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox10_2_2_2_2_4Layout->addItem( spacer35, 0, 0 );

	scrollBar5 = new PreviewScrollBar( groupBox10_2_2_2_2_4, "dominoPreviewWidget" );
	if(strcmp(scrollBar5->style().name(), "domino"))
		scrollBar5->setStyle("domino");

	scrollBar5->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding);
	scrollBar5->setOrientation( QScrollBar::Vertical );
	scrollBar5->setMaxValue(50);
	groupBox10_2_2_2_2_4Layout->addWidget( scrollBar5, 0, 1 );

	scrollBar6 = new PreviewScrollBar( groupBox10_2_2_2_2_4, "dominoPreviewWidget" );
	if(strcmp(scrollBar6->style().name(), "domino"))
		scrollBar6->setStyle("domino");
	scrollBar6->setOrientation( QScrollBar::Horizontal );
	scrollBar6->setMaxValue(50);
	groupBox10_2_2_2_2_4Layout->addWidget( scrollBar6, 1, 0 );
	
	//////////////////////////////////// caps color
	capsColorButton = new KColorButton(groupBox22_2_2, "capsColorButton");
	connect(capsColorButton, SIGNAL(changed(const QColor&)),this,  SLOT(capsColorChanged(const QColor&)));
	capsLabel = new QLabel(groupBox22_2_2);
	capsHBoxLayout = new QHBoxLayout(0);
	capsHBoxLayout->addWidget(capsLabel);
	capsHBoxLayout->addWidget(capsColorButton);
	groupBox22_2_2Layout->addLayout(capsHBoxLayout, 3, 2 );

	//////////////////////////////////
	groupBox22_2_2Layout->addMultiCellWidget( groupBox10_2_2_2_2_4, 0, 1, 0, 0 );
	spacer43_2_2 = new QSpacerItem( 5, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2Layout->addItem( spacer43_2_2, 1, 1 );
	spacer3_2_2 = new QSpacerItem( 12, 25, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2Layout->addItem( spacer3_2_2, 3, 6 );
	spacer6_2_2 = new QSpacerItem( 83, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2Layout->addItem( spacer6_2_2, 3, 3 );

	kColorButton27_2_2_2 = new KColorButton( groupBox22_2_2, "kColorButton27_2_2_2" );
	kColorButton27_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, kColorButton27_2_2_2->sizePolicy().hasHeightForWidth() ) );
	kColorButton27_2_2_2->setMinimumSize( QSize( 75, 0 ) );

	groupBox22_2_2Layout->addWidget( kColorButton27_2_2_2, 3, 5 );
	spacer2_2_2 = new QSpacerItem( 82, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2Layout->addItem( spacer2_2_2, 3, 2 );

	textLabel1_2_2_2 = new QLabel( groupBox22_2_2, "textLabel1_2_2_2" );
	textLabel1_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2_2_2->sizePolicy().hasHeightForWidth() ) );
	textLabel1_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox22_2_2Layout->addWidget( textLabel1_2_2_2, 3, 4 );
	tab3Layout->addWidget( groupBox22_2_2 );

	layout6 = new QHBoxLayout( 0, 0, 6, "layout6");
	comboBox7 = new QComboBox( FALSE, tab3, "comboBox7" );
	layout6->addWidget( comboBox7 );
	spacer7_2_3 = new QSpacerItem( 246, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout6->addItem( spacer7_2_3 );

	textLabel7_4_2_3 = new QLabel( tab3, "textLabel7_4_2_3" );
	layout6->addWidget( textLabel7_4_2_3 );

	comboBox3_4_2_3 = new QComboBox( FALSE, tab3, "comboBox3_4_2_3" );
	layout6->addWidget( comboBox3_4_2_3 );

	pushButton4_4_2_3 = new QPushButton( tab3, "pushButton4_4_2_3" );
	connect(pushButton4_4_2_3, SIGNAL(clicked()), SLOT(copyColorsToScrollBar()));
	
	layout6->addWidget( pushButton4_4_2_3 );
	tab3Layout->addLayout( layout6 );
	tabWidget2->insertTab( tab3, QString::fromLatin1("") );

/////////////////////////////////////////////////
// tab 2.4 (headers)
/////////////////////////////////////////////////
	tab4 = new QWidget( tabWidget2, "tab4" );
	tab4Layout = new QVBoxLayout( tab4, 11, 6, "tab4Layout"); 

	groupBox22_2_2_3 = new QGroupBox( tab4, "groupBox22_2_2_3" );
	groupBox22_2_2_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox22_2_2_3->sizePolicy().hasHeightForWidth() ) );
	groupBox22_2_2_3->setColumnLayout(0, Qt::Vertical );
	groupBox22_2_2_3->layout()->setSpacing( 0 );
	groupBox22_2_2_3->layout()->setMargin( 11 );
	groupBox22_2_2_3Layout = new QGridLayout( groupBox22_2_2_3->layout() );
	groupBox22_2_2_3Layout->setAlignment( Qt::AlignTop );
	spacer3_2_2_2 = new QSpacerItem( 12, 25, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2_3Layout->addItem( spacer3_2_2_2, 3, 6 );
	spacer6_2_2_2 = new QSpacerItem( 83, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2_3Layout->addItem( spacer6_2_2_2, 3, 3 );

	kColorButton27_2_2_2_2 = new KColorButton( groupBox22_2_2_3, "kColorButton27_2_2_2_2" );
	kColorButton27_2_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, kColorButton27_2_2_2_2->sizePolicy().hasHeightForWidth() ) );
	kColorButton27_2_2_2_2->setMinimumSize( QSize( 75, 0 ) );

	groupBox22_2_2_3Layout->addWidget( kColorButton27_2_2_2_2, 3, 5 );
	spacer2_2_2_2 = new QSpacerItem( 82, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2_3Layout->addItem( spacer2_2_2_2, 3, 2 );
	spacer46_2_2_2 = new QSpacerItem( 20, 7, QSizePolicy::Minimum, QSizePolicy::Fixed );
	groupBox22_2_2_3Layout->addItem( spacer46_2_2_2, 2, 5 );

	textLabel1_2_2_2_2 = new QLabel( groupBox22_2_2_3, "textLabel1_2_2_2_2" );
	textLabel1_2_2_2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2_2_2_2->sizePolicy().hasHeightForWidth() ) );
	textLabel1_2_2_2_2->setTextFormat( QLabel::PlainText );

	groupBox22_2_2_3Layout->addWidget( textLabel1_2_2_2_2, 3, 4 );
	spacer43_2_2_2 = new QSpacerItem( 5, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2_3Layout->addItem( spacer43_2_2_2, 1, 1 );

	groupBox10_2_2_2_2_4_3 = new QGroupBox( groupBox22_2_2_3, "groupBox10_2_2_2_2_4_3" );
	groupBox10_2_2_2_2_4_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, groupBox10_2_2_2_2_4_3->sizePolicy().hasHeightForWidth() ) );
	groupBox10_2_2_2_2_4_3->setMinimumSize( QSize( 170, 0 ) );
	groupBox10_2_2_2_2_4_3->setColumnLayout(0, Qt::Vertical );
	groupBox10_2_2_2_2_4_3->layout()->setSpacing( 6 );
	groupBox10_2_2_2_2_4_3->layout()->setMargin( 11 );
	groupBox10_2_2_2_2_4_3Layout = new QVBoxLayout( groupBox10_2_2_2_2_4_3->layout() );
	groupBox10_2_2_2_2_4_3Layout->setAlignment( Qt::AlignTop );

	frame3 = new QFrame( groupBox10_2_2_2_2_4_3, "frame3" );
	frame3->setFrameShape( QFrame::StyledPanel );
	frame3->setFrameShadow( QFrame::Raised ); // ?^
	frame3->setLineWidth( 1 );
 //   frame3->setLayoutMargin(0);
	frame3Layout = new QVBoxLayout( frame3, 2, 6, "frame3Layout");
	
	layout1 = new QVBoxLayout( 0, 0, 0, "layout1");
	
	header1 = new PreviewHeader(frame3, "dominoPreviewWidget");
	if(strcmp(header1->style().name(), "domino"))
		header1->setStyle("domino");
	header1->setMovingEnabled(false);
	header1->setStretchEnabled(true);
	//header1->setResizeMode(QListView::LastColumn);
	header1->addLabel("Header");
	
	
	
	layout1->addWidget( header1);
	listBox1 = new QListBox( frame3, "listBox1" );
	listBox1->setLineWidth(0);
	//listBox1-> setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum/*, 0, 0, listBox1->sizePolicy().hasHeightForWidth()*/ ) );
	
	layout1->addWidget( listBox1 );
	frame3Layout->addLayout( layout1 );
	groupBox10_2_2_2_2_4_3Layout->addWidget( frame3 );

	//////////////////////////////////////
	groupBox22_2_2_3Layout->addMultiCellWidget( groupBox10_2_2_2_2_4_3, 0, 1, 0, 0 );

	groupBox8_2_2_2_2_4_3 = new QGroupBox( groupBox22_2_2_3, "groupBox8_2_2_2_2_4_3" );
	groupBox8_2_2_2_2_4_3->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox8_2_2_2_2_4_3->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(header1GradientChanged()));

	groupBox8_2_2_2_2_4_3->setColumnLayout(0, Qt::Vertical );
	groupBox8_2_2_2_2_4_3->layout()->setSpacing( 7 );
	groupBox8_2_2_2_2_4_3->layout()->setMargin( 4 );
	groupBox8_2_2_2_2_4_3Layout = new QGridLayout( groupBox8_2_2_2_2_4_3->layout() );
	groupBox8_2_2_2_2_4_3Layout->setAlignment( Qt::AlignTop );

	textLabel4_3_2_2_2_2_4_3 = new QLabel( groupBox8_2_2_2_2_4_3, "textLabel4_3_2_2_2_2_4_3" );
	textLabel4_3_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3Layout->addWidget( textLabel4_3_2_2_2_2_4_3, 1, 0 );

	spinBox2_2_2_2_2_2_4_3 = new DSpinBox( groupBox8_2_2_2_2_4_3, "spinBox2_2_2_2_2_2_4_3" );
	spinBox2_2_2_2_2_2_4_3->setAlignRight();
	spinBox2_2_2_2_2_2_4_3->setMaxValue( 100 );

	groupBox8_2_2_2_2_4_3Layout->addWidget( spinBox2_2_2_2_2_2_4_3, 1, 1 );

	textLabel2_2_2_2_2_2_4_3 = new QLabel( groupBox8_2_2_2_2_4_3, "textLabel2_2_2_2_2_2_4_3" );
	textLabel2_2_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3Layout->addWidget( textLabel2_2_2_2_2_2_4_3, 1, 2 );

	kColorButton1_2_2_2_2_2_4_3 = new KColorButton( groupBox8_2_2_2_2_4_3, "kColorButton1_2_2_2_2_2_4_3" );

	groupBox8_2_2_2_2_4_3Layout->addWidget( kColorButton1_2_2_2_2_2_4_3, 1, 3 );

	textLabel3_3_2_2_2_2_4_3 = new QLabel( groupBox8_2_2_2_2_4_3, "textLabel3_3_2_2_2_2_4_3" );
	textLabel3_3_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3Layout->addWidget( textLabel3_3_2_2_2_2_4_3, 0, 0 );

	spinBox1_2_2_2_2_2_4_3 = new DSpinBox( groupBox8_2_2_2_2_4_3, "spinBox1_2_2_2_2_2_4_3" );
	spinBox1_2_2_2_2_2_4_3->setAlignRight();
	spinBox1_2_2_2_2_2_4_3->setMaxValue( 100 );

	groupBox8_2_2_2_2_4_3Layout->addWidget( spinBox1_2_2_2_2_2_4_3, 0, 1 );

	textLabel1_4_2_2_2_2_4_3 = new QLabel( groupBox8_2_2_2_2_4_3, "textLabel1_4_2_2_2_2_4_3" );
	textLabel1_4_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3Layout->addWidget( textLabel1_4_2_2_2_2_4_3, 0, 2 );

	kColorButton2_4_2_2_2_2_4_3 = new KColorButton( groupBox8_2_2_2_2_4_3, "kColorButton2_4_2_2_2_2_4_3" );

	groupBox8_2_2_2_2_4_3Layout->addWidget( kColorButton2_4_2_2_2_2_4_3, 0, 3 );

	groupBox22_2_2_3Layout->addMultiCellWidget( groupBox8_2_2_2_2_4_3, 0, 0, 2, 6 );

	groupBox9_2_2_2_2_4_3 = new QGroupBox( groupBox22_2_2_3, "groupBox9_2_2_2_2_4_3" );
	groupBox9_2_2_2_2_4_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox9_2_2_2_2_4_3->sizePolicy().hasHeightForWidth() ) );
	groupBox9_2_2_2_2_4_3->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox9_2_2_2_2_4_3->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(header2GradientChanged()));
	groupBox9_2_2_2_2_4_3->setChecked( FALSE );
	groupBox9_2_2_2_2_4_3->setColumnLayout(0, Qt::Vertical );
	groupBox9_2_2_2_2_4_3->layout()->setSpacing( 7 );
	groupBox9_2_2_2_2_4_3->layout()->setMargin( 4 );
	groupBox9_2_2_2_2_4_3Layout = new QGridLayout( groupBox9_2_2_2_2_4_3->layout() );
	groupBox9_2_2_2_2_4_3Layout->setAlignment( Qt::AlignTop );

	textLabel4_4_2_2_2_2_4_3 = new QLabel( groupBox9_2_2_2_2_4_3, "textLabel4_4_2_2_2_2_4_3" );
	textLabel4_4_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3Layout->addWidget( textLabel4_4_2_2_2_2_4_3, 1, 0 );

	spinBox2_3_2_2_2_2_4_3 = new DSpinBox( groupBox9_2_2_2_2_4_3, "spinBox2_3_2_2_2_2_4_3" );
	spinBox2_3_2_2_2_2_4_3->setAlignRight();
	spinBox2_3_2_2_2_2_4_3->setMaxValue( 100 );

	groupBox9_2_2_2_2_4_3Layout->addWidget( spinBox2_3_2_2_2_2_4_3, 1, 1 );

	textLabel3_4_2_2_2_2_4_3 = new QLabel( groupBox9_2_2_2_2_4_3, "textLabel3_4_2_2_2_2_4_3" );
	textLabel3_4_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3Layout->addWidget( textLabel3_4_2_2_2_2_4_3, 0, 0 );

	spinBox1_3_2_2_2_2_4_3 = new DSpinBox( groupBox9_2_2_2_2_4_3, "spinBox1_3_2_2_2_2_4_3" );
	spinBox1_3_2_2_2_2_4_3->setAlignRight();
	spinBox1_3_2_2_2_2_4_3->setMaxValue( 100 );

	groupBox9_2_2_2_2_4_3Layout->addWidget( spinBox1_3_2_2_2_2_4_3, 0, 1 );

	kColorButton1_3_2_2_2_2_4_3 = new KColorButton( groupBox9_2_2_2_2_4_3, "kColorButton1_3_2_2_2_2_4_3" );
	kColorButton1_3_2_2_2_2_4_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, kColorButton1_3_2_2_2_2_4_3->sizePolicy().hasHeightForWidth() ) );

	groupBox9_2_2_2_2_4_3Layout->addWidget( kColorButton1_3_2_2_2_2_4_3, 1, 3 );

	kColorButton2_5_2_2_2_2_4_3 = new KColorButton( groupBox9_2_2_2_2_4_3, "kColorButton2_5_2_2_2_2_4_3" );

	groupBox9_2_2_2_2_4_3Layout->addWidget( kColorButton2_5_2_2_2_2_4_3, 0, 3 );

	textLabel1_5_2_2_2_2_4_3 = new QLabel( groupBox9_2_2_2_2_4_3, "textLabel1_5_2_2_2_2_4_3" );
	textLabel1_5_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3Layout->addWidget( textLabel1_5_2_2_2_2_4_3, 0, 2 );

	textLabel2_3_2_2_2_2_4_3 = new QLabel( groupBox9_2_2_2_2_4_3, "textLabel2_3_2_2_2_2_4_3" );
	textLabel2_3_2_2_2_2_4_3->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3Layout->addWidget( textLabel2_3_2_2_2_2_4_3, 1, 2 );

	groupBox22_2_2_3Layout->addMultiCellWidget( groupBox9_2_2_2_2_4_3, 1, 1, 2, 6 );
	tab4Layout->addWidget( groupBox22_2_2_3 );

	layout21 = new QHBoxLayout( 0, 0, 6, "layout21"); 
	spacer7_2_3_3 = new QSpacerItem( 354, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout21->addItem( spacer7_2_3_3 );

	textLabel7_4_2_3_3 = new QLabel( tab4, "textLabel7_4_2_3_3" );
	layout21->addWidget( textLabel7_4_2_3_3 );

	comboBox3_4_2_3_3 = new QComboBox( FALSE, tab4, "comboBox3_4_2_3_3" );
	layout21->addWidget( comboBox3_4_2_3_3 );

	pushButton4_4_2_3_3 = new QPushButton( tab4, "pushButton4_4_2_3_3" );
	connect(pushButton4_4_2_3_3, SIGNAL(clicked()), SLOT(copyColorsToHeader()));
	layout21->addWidget( pushButton4_4_2_3_3 );
	tab4Layout->addLayout( layout21 );
	tabWidget2->insertTab( tab4, QString::fromLatin1("") );

/////////////////////////////////////////////////
// tab 2.5 (radio/checkboxes)
/////////////////////////////////////////////////

	tab5 = new QWidget( tabWidget2, "tab5" );
	tab5Layout = new QVBoxLayout( tab5, 11, 6, "tab5Layout"); 

	groupBox22_2_2_3_2 = new QGroupBox( tab5, "groupBox22_2_2_3_2" );
	groupBox22_2_2_3_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox22_2_2_3_2->sizePolicy().hasHeightForWidth() ) );
	groupBox22_2_2_3_2->setColumnLayout(0, Qt::Vertical );
	groupBox22_2_2_3_2->layout()->setSpacing( 0 );
	groupBox22_2_2_3_2->layout()->setMargin( 11 );
	groupBox22_2_2_3_2Layout = new QGridLayout( groupBox22_2_2_3_2->layout() );
	groupBox22_2_2_3_2Layout->setAlignment( Qt::AlignTop );

	groupBox8_2_2_2_2_4_3_2 = new QGroupBox( groupBox22_2_2_3_2, "groupBox8_2_2_2_2_4_3_2" );
	groupBox8_2_2_2_2_4_3_2->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox8_2_2_2_2_4_3_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(checkItem1GradientChanged()));
	groupBox8_2_2_2_2_4_3_2->setChecked( FALSE );
	groupBox8_2_2_2_2_4_3_2->setColumnLayout(0, Qt::Vertical );
	groupBox8_2_2_2_2_4_3_2->layout()->setSpacing( 7 );
	groupBox8_2_2_2_2_4_3_2->layout()->setMargin( 4 );
	groupBox8_2_2_2_2_4_3_2Layout = new QGridLayout( groupBox8_2_2_2_2_4_3_2->layout() );
	groupBox8_2_2_2_2_4_3_2Layout->setAlignment( Qt::AlignTop );

	textLabel4_3_2_2_2_2_4_3_2 = new QLabel( groupBox8_2_2_2_2_4_3_2, "textLabel4_3_2_2_2_2_4_3_2" );
	textLabel4_3_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( textLabel4_3_2_2_2_2_4_3_2, 1, 0 );

	spinBox2_2_2_2_2_2_4_3_2 = new DSpinBox( groupBox8_2_2_2_2_4_3_2, "spinBox2_2_2_2_2_2_4_3_2" );
	spinBox2_2_2_2_2_2_4_3_2->setAlignRight();
	spinBox2_2_2_2_2_2_4_3_2->setMaxValue( 100 );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( spinBox2_2_2_2_2_2_4_3_2, 1, 1 );

	textLabel2_2_2_2_2_2_4_3_2 = new QLabel( groupBox8_2_2_2_2_4_3_2, "textLabel2_2_2_2_2_2_4_3_2" );
	textLabel2_2_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( textLabel2_2_2_2_2_2_4_3_2, 1, 2 );

	kColorButton1_2_2_2_2_2_4_3_2 = new KColorButton( groupBox8_2_2_2_2_4_3_2, "kColorButton1_2_2_2_2_2_4_3_2" );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( kColorButton1_2_2_2_2_2_4_3_2, 1, 3 );

	textLabel3_3_2_2_2_2_4_3_2 = new QLabel( groupBox8_2_2_2_2_4_3_2, "textLabel3_3_2_2_2_2_4_3_2" );
	textLabel3_3_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( textLabel3_3_2_2_2_2_4_3_2, 0, 0 );

	spinBox1_2_2_2_2_2_4_3_2 = new DSpinBox( groupBox8_2_2_2_2_4_3_2, "spinBox1_2_2_2_2_2_4_3_2" );
	spinBox1_2_2_2_2_2_4_3_2->setAlignRight();
	spinBox1_2_2_2_2_2_4_3_2->setMaxValue( 100 );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( spinBox1_2_2_2_2_2_4_3_2, 0, 1 );

	textLabel1_4_2_2_2_2_4_3_2 = new QLabel( groupBox8_2_2_2_2_4_3_2, "textLabel1_4_2_2_2_2_4_3_2" );
	textLabel1_4_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( textLabel1_4_2_2_2_2_4_3_2, 0, 2 );

	kColorButton2_4_2_2_2_2_4_3_2 = new KColorButton( groupBox8_2_2_2_2_4_3_2, "kColorButton2_4_2_2_2_2_4_3_2" );

	groupBox8_2_2_2_2_4_3_2Layout->addWidget( kColorButton2_4_2_2_2_2_4_3_2, 0, 3 );

	groupBox22_2_2_3_2Layout->addMultiCellWidget( groupBox8_2_2_2_2_4_3_2, 0, 0, 2, 6 );
	spacer3_2_2_3 = new QSpacerItem( 12, 25, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2_3_2Layout->addItem( spacer3_2_2_3, 3, 6 );
	spacer6_2_2_3 = new QSpacerItem( 83, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2_3_2Layout->addItem( spacer6_2_2_3, 3, 3 );

	kColorButton27_2_2_2_3 = new KColorButton( groupBox22_2_2_3_2, "kColorButton27_2_2_2_3" );
	kColorButton27_2_2_2_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)0, 0, 0, kColorButton27_2_2_2_3->sizePolicy().hasHeightForWidth() ) );
	kColorButton27_2_2_2_3->setMinimumSize( QSize( 75, 0 ) );

	groupBox22_2_2_3_2Layout->addWidget( kColorButton27_2_2_2_3, 3, 5 );
	spacer2_2_2_3 = new QSpacerItem( 82, 25, QSizePolicy::Expanding, QSizePolicy::Minimum );
	groupBox22_2_2_3_2Layout->addItem( spacer2_2_2_3, 3, 2 );
	spacer46_2_2_3 = new QSpacerItem( 20, 7, QSizePolicy::Minimum, QSizePolicy::Fixed );
	groupBox22_2_2_3_2Layout->addItem( spacer46_2_2_3, 2, 5 );

	textLabel1_2_2_2_3 = new QLabel( groupBox22_2_2_3_2, "textLabel1_2_2_2_3" );
	textLabel1_2_2_2_3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2_2_2_3->sizePolicy().hasHeightForWidth() ) );
	textLabel1_2_2_2_3->setTextFormat( QLabel::PlainText );

	groupBox22_2_2_3_2Layout->addWidget( textLabel1_2_2_2_3, 3, 4 );
	spacer43_2_2_2_2 = new QSpacerItem( 5, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	groupBox22_2_2_3_2Layout->addItem( spacer43_2_2_2_2, 1, 1 );

	groupBox10_2_2_2_2_4_3_2 = new QGroupBox( groupBox22_2_2_3_2, "groupBox10_2_2_2_2_4_3_2" );
	groupBox10_2_2_2_2_4_3_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, groupBox10_2_2_2_2_4_3_2->sizePolicy().hasHeightForWidth() ) );
	groupBox10_2_2_2_2_4_3_2->setMinimumSize( QSize( 170, 0 ) );
	groupBox10_2_2_2_2_4_3_2->setColumnLayout(0, Qt::Vertical );
	groupBox10_2_2_2_2_4_3_2->layout()->setSpacing( 6 );
	groupBox10_2_2_2_2_4_3_2->layout()->setMargin( 11 );
	groupBox10_2_2_2_2_4_3_2Layout = new QVBoxLayout( groupBox10_2_2_2_2_4_3_2->layout() );
	groupBox10_2_2_2_2_4_3_2Layout->setAlignment( Qt::AlignTop );

	radioButton2 = new PreviewRadioButton( groupBox10_2_2_2_2_4_3_2, "dominoPreviewWidget" );
	if(strcmp(radioButton2->style().name(), "domino"))
		radioButton2->setStyle("domino");
	groupBox10_2_2_2_2_4_3_2Layout->addWidget( radioButton2 );

	checkBox2 = new PreviewCheckBox( groupBox10_2_2_2_2_4_3_2, "dominoPreviewWidget" );
	if(strcmp(checkBox2->style().name(), "domino"))
		checkBox2->setStyle("domino");
	groupBox10_2_2_2_2_4_3_2Layout->addWidget( checkBox2 );

	groupBox22_2_2_3_2Layout->addMultiCellWidget( groupBox10_2_2_2_2_4_3_2, 0, 1, 0, 0 );

	groupBox9_2_2_2_2_4_3_2 = new QGroupBox( groupBox22_2_2_3_2, "groupBox9_2_2_2_2_4_3_2" );
	groupBox9_2_2_2_2_4_3_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, groupBox9_2_2_2_2_4_3_2->sizePolicy().hasHeightForWidth() ) );
	groupBox9_2_2_2_2_4_3_2->setCheckable( TRUE );
	connect(((QCheckBox*)groupBox9_2_2_2_2_4_3_2->child("qt_groupbox_checkbox", 0, false)), SIGNAL(clicked()), SLOT(checkItem2GradientChanged()));

	groupBox9_2_2_2_2_4_3_2->setChecked( FALSE );
	groupBox9_2_2_2_2_4_3_2->setColumnLayout(0, Qt::Vertical );
	groupBox9_2_2_2_2_4_3_2->layout()->setSpacing( 7 );
	groupBox9_2_2_2_2_4_3_2->layout()->setMargin( 4 );
	groupBox9_2_2_2_2_4_3_2Layout = new QGridLayout( groupBox9_2_2_2_2_4_3_2->layout() );
	groupBox9_2_2_2_2_4_3_2Layout->setAlignment( Qt::AlignTop );

	textLabel4_4_2_2_2_2_4_3_2 = new QLabel( groupBox9_2_2_2_2_4_3_2, "textLabel4_4_2_2_2_2_4_3_2" );
	textLabel4_4_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( textLabel4_4_2_2_2_2_4_3_2, 1, 0 );

	spinBox2_3_2_2_2_2_4_3_2 = new DSpinBox( groupBox9_2_2_2_2_4_3_2, "spinBox2_3_2_2_2_2_4_3_2" );
	spinBox2_3_2_2_2_2_4_3_2->setAlignRight();
	spinBox2_3_2_2_2_2_4_3_2->setMaxValue( 100 );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( spinBox2_3_2_2_2_2_4_3_2, 1, 1 );

	textLabel3_4_2_2_2_2_4_3_2 = new QLabel( groupBox9_2_2_2_2_4_3_2, "textLabel3_4_2_2_2_2_4_3_2" );
	textLabel3_4_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( textLabel3_4_2_2_2_2_4_3_2, 0, 0 );

	spinBox1_3_2_2_2_2_4_3_2 = new DSpinBox( groupBox9_2_2_2_2_4_3_2, "spinBox1_3_2_2_2_2_4_3_2" );
	spinBox1_3_2_2_2_2_4_3_2->setAlignRight();
	spinBox1_3_2_2_2_2_4_3_2->setMaxValue( 100 );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( spinBox1_3_2_2_2_2_4_3_2, 0, 1 );

	kColorButton1_3_2_2_2_2_4_3_2 = new KColorButton( groupBox9_2_2_2_2_4_3_2, "kColorButton1_3_2_2_2_2_4_3_2" );
	kColorButton1_3_2_2_2_2_4_3_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)0, 0, 0, kColorButton1_3_2_2_2_2_4_3_2->sizePolicy().hasHeightForWidth() ) );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( kColorButton1_3_2_2_2_2_4_3_2, 1, 3 );

	kColorButton2_5_2_2_2_2_4_3_2 = new KColorButton( groupBox9_2_2_2_2_4_3_2, "kColorButton2_5_2_2_2_2_4_3_2" );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( kColorButton2_5_2_2_2_2_4_3_2, 0, 3 );

	textLabel1_5_2_2_2_2_4_3_2 = new QLabel( groupBox9_2_2_2_2_4_3_2, "textLabel1_5_2_2_2_2_4_3_2" );
	textLabel1_5_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( textLabel1_5_2_2_2_2_4_3_2, 0, 2 );

	textLabel2_3_2_2_2_2_4_3_2 = new QLabel( groupBox9_2_2_2_2_4_3_2, "textLabel2_3_2_2_2_2_4_3_2" );
	textLabel2_3_2_2_2_2_4_3_2->setTextFormat( QLabel::PlainText );

	groupBox9_2_2_2_2_4_3_2Layout->addWidget( textLabel2_3_2_2_2_2_4_3_2, 1, 2 );

	groupBox22_2_2_3_2Layout->addMultiCellWidget( groupBox9_2_2_2_2_4_3_2, 1, 1, 2, 6 );
	tab5Layout->addWidget( groupBox22_2_2_3_2 );

	layout22 = new QHBoxLayout( 0, 0, 6, "layout22"); 
	spacer7_2_3_3_2 = new QSpacerItem( 354, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout22->addItem( spacer7_2_3_3_2 );

	textLabel7_4_2_3_3_2 = new QLabel( tab5, "textLabel7_4_2_3_3_2" );
	layout22->addWidget( textLabel7_4_2_3_3_2 );

	comboBox3_4_2_3_3_2 = new QComboBox( FALSE, tab5, "comboBox3_4_2_3_3_2" );
	layout22->addWidget( comboBox3_4_2_3_3_2 );

	pushButton4_4_2_3_3_2 = new QPushButton( tab5, "pushButton4_4_2_3_3_2" );
	connect(pushButton4_4_2_3_3_2, SIGNAL(clicked()), SLOT(copyColorsToCheckItem()));
	
	layout22->addWidget( pushButton4_4_2_3_3_2 );
	tab5Layout->addLayout( layout22 );
	tabWidget2->insertTab( tab5, QString::fromLatin1("") );
   /// resize( QSize(673, 465).expandedTo(minimumSizeHint()) );
  ///  clearWState( WState_Polished );

////////////////////////////////////////////////////////////////////////////
	// tab 3   load/save scheme
////////////////////////////////////////////////////////////////////////////


	QWidget *page3 = new QWidget(tabWidget);

	tab_lsLayout = new QVBoxLayout( page3, 11, 6, "tab_lsLayout");

	groupBox_ls = new QGroupBox( page3, "groupBox21" );
	groupBox_ls->setColumnLayout(0, Qt::Vertical );
	groupBox_ls->layout()->setSpacing( 6 );
	groupBox_ls->layout()->setMargin( 11 );
	groupBox_lsLayout = new QHBoxLayout( groupBox_ls->layout() );
	groupBox_lsLayout->setAlignment( Qt::AlignTop );

	listView_ls = new QListView( groupBox_ls, "listView_ls" );
	listView_ls->addColumn( tr( "Name" ) );
	listView_ls->header()->setMovingEnabled(false);
	listView_ls->setResizeMode(QListView::LastColumn);
	listView_ls->setShowSortIndicator(true);
	connect(listView_ls, SIGNAL(selectionChanged(QListViewItem*)), SLOT(slotDelPerm(QListViewItem*)));


	schemeList = new SchemeList();
	getConfigSchemes();
	
	groupBox_lsLayout->addWidget( listView_ls );

	layout8 = new QVBoxLayout( 0, 0, 6, "layout8");
	loadButton = new QPushButton( groupBox_ls, "loadButton" );
	connect(loadButton, SIGNAL(clicked()), SLOT(slotLoad()));
	layout8->addWidget( loadButton );

	saveButton = new QPushButton( groupBox_ls, "saveButton" );
	connect(saveButton, SIGNAL(clicked()), SLOT(slotSave()));
	layout8->addWidget( saveButton );

	deleteButton = new QPushButton( groupBox_ls, "deleteButton" );
	connect(deleteButton, SIGNAL(clicked()), SLOT(slotDelete()));
	layout8->addWidget( deleteButton );
	
	spacer32 = new QSpacerItem( 31, 141, QSizePolicy::Minimum, QSizePolicy::Expanding );
	layout8->addItem( spacer32 );
	groupBox_lsLayout->addLayout( layout8 );
	

	tab_lsLayout->addWidget( groupBox_ls );
	currentConfigLabel = new QLabel(page3);
	tab_lsLayout->addWidget(currentConfigLabel);
	tabWidget->addTab(page3, "Load/Save");
	
	
	resize( QSize(673, 465).expandedTo(minimumSizeHint()) );
	languageChange();

	////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	loadConfig("domino"); // dominorc, the current one
	configLoaded = false;
	updateChanged(); // enable or disable the KColorButtons
	
	// scrollbar buttons or groove
	connect(comboBox7, SIGNAL( activated(const QString &)), SLOT(userLoadScrollBarConfig(const QString & )));
	// btn
	connect(kColorButton2_4_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(btnPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(btnPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(btnPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(btnPv4(const QColor&)));
	connect(kColorButton27_2, SIGNAL(changed(const QColor&)),this,	SLOT(btnPv5(const QColor&)));
	connect(spinBox1_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(btnPv6(int)));
	connect(spinBox2_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(btnPv7(int)));
	connect(spinBox1_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(btnPv8(int)));
	connect(spinBox2_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(btnPv9(int)));
	// combo
	connect(kColorButton2_4_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(cbPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(cbPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(cbPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(cbPv4(const QColor&)));
	connect(kColorButton27_2, SIGNAL(changed(const QColor&)),this,	SLOT(cbPv5(const QColor&)));
	connect(spinBox1_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(cbPv6(int)));
	connect(spinBox2_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(cbPv7(int)));
	connect(spinBox1_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(cbPv8(int)));
	connect(spinBox2_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(cbPv9(int)));
	//spinbox
	connect(kColorButton2_4_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(spPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(spPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(spPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(spPv4(const QColor&)));
	connect(kColorButton27_2, SIGNAL(changed(const QColor&)),this,	SLOT(spPv5(const QColor&)));
	connect(spinBox1_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(spPv6(int)));
	connect(spinBox2_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(spPv7(int)));
	connect(spinBox1_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(spPv8(int)));
	connect(spinBox2_3_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(spPv9(int)));
	// header
	connect(kColorButton2_4_2_2_2_2_4_3, SIGNAL(changed(const QColor&)),this,  SLOT(hPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2_2_4_3, SIGNAL(changed(const QColor&)),this,  SLOT(hPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2_2_4_3, SIGNAL(changed(const QColor&)),this,  SLOT(hPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2_2_4_3, SIGNAL(changed(const QColor&)),this,  SLOT(hPv4(const QColor&)));
	connect(kColorButton27_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(hPv5(const QColor&)));
	connect(spinBox1_2_2_2_2_2_4_3, SIGNAL(valueChanged(int)), this, SLOT(hPv6(int)));
	connect(spinBox2_2_2_2_2_2_4_3, SIGNAL(valueChanged(int)), this, SLOT(hPv7(int)));
	connect(spinBox1_3_2_2_2_2_4_3, SIGNAL(valueChanged(int)), this, SLOT(hPv8(int)));
	connect(spinBox2_3_2_2_2_2_4_3, SIGNAL(valueChanged(int)), this, SLOT(hPv9(int)));
	// checkBox
	connect(kColorButton2_4_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(chbPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(chbPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(chbPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(chbPv4(const QColor&)));
	connect(kColorButton27_2_2_2_3, SIGNAL(changed(const QColor&)),this,  SLOT(chbPv5(const QColor&)));
	connect(spinBox1_2_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(chbPv6(int)));
	connect(spinBox2_2_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(chbPv7(int)));
	connect(spinBox1_3_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(chbPv8(int)));
	connect(spinBox2_3_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(chbPv9(int)));
	// radiobutton
	connect(kColorButton2_4_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(rPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(rPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(rPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2_2_4_3_2, SIGNAL(changed(const QColor&)),this,  SLOT(rPv4(const QColor&)));
	connect(kColorButton27_2_2_2_3, SIGNAL(changed(const QColor&)),this,  SLOT(rPv5(const QColor&)));
	connect(spinBox1_2_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(rPv6(int)));
	connect(spinBox2_2_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(rPv7(int)));
	connect(spinBox1_3_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(rPv8(int)));
	connect(spinBox2_3_2_2_2_2_4_3_2, SIGNAL(valueChanged(int)), this, SLOT(rPv9(int)));
	// scrollbar
	connect(kColorButton2_4_2_2_2_2_4, SIGNAL(changed(const QColor&)),this,  SLOT(sbPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2_2_4, SIGNAL(changed(const QColor&)),this,  SLOT(sbPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2_2_4, SIGNAL(changed(const QColor&)),this,  SLOT(sbPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2_2_4, SIGNAL(changed(const QColor&)),this,  SLOT(sbPv4(const QColor&)));
	connect(kColorButton27_2_2_2, SIGNAL(changed(const QColor&)),this,	SLOT(sbPv5(const QColor&)));
	connect(spinBox1_2_2_2_2_2_4, SIGNAL(valueChanged(int)), this, SLOT(sbPv6(int)));
	connect(spinBox2_2_2_2_2_2_4, SIGNAL(valueChanged(int)), this, SLOT(sbPv7(int)));
	connect(spinBox1_3_2_2_2_2_4, SIGNAL(valueChanged(int)), this, SLOT(sbPv8(int)));
	connect(spinBox2_3_2_2_2_2_4, SIGNAL(valueChanged(int)), this, SLOT(sbPv9(int)));
	// tab
	connect(kColorButton2_4_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(tabPv1(const QColor&)));
	connect(kColorButton1_2_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(tabPv2(const QColor&)));
	connect(kColorButton2_5_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(tabPv3(const QColor&)));
	connect(kColorButton1_3_2_2_2_2, SIGNAL(changed(const QColor&)),this,  SLOT(tabPv4(const QColor&)));
	connect(kColorButton27_2_2, SIGNAL(changed(const QColor&)),this,	SLOT(tabPv5(const QColor&)));
	connect(spinBox1_2_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(tabPv6(int)));
	connect(spinBox2_2_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(tabPv7(int)));
	connect(spinBox1_3_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(tabPv8(int)));
	connect(spinBox2_3_2_2_2_2, SIGNAL(valueChanged(int)), this, SLOT(tabPv9(int)));
	
	connect(tabPosCombo, SIGNAL( activated(int)), SLOT(userLoadTabConfig(int)));
	connect(tabStateCombo, SIGNAL( activated(int)), SLOT(userLoadTabConfig(int)));

	
	connect(customCheckMarkColor, SIGNAL(toggled(bool)), SLOT(updateChanged()));
	connect(customToolTipColor, SIGNAL(toggled(bool)), SLOT(updateChanged()));
	connect(customPopupMenuColor, SIGNAL(toggled(bool)), SLOT( updateChanged()));
	
	connect(indicatorColor, SIGNAL(changed(const QColor&)), indicatorPrevWidget, SLOT( update()));
	connect(indicatorBtnColor, SIGNAL(changed(const QColor&)), indicatorPrevButton, SLOT( update()));
	//languageChange();
	clearWState( WState_Polished );
	

}


// btn
void DominoStyleConfig::btnPv1(const QColor&col) {
	btnSurface.g1Color1 = col.name();
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv2(const QColor&col) {
	btnSurface.g1Color2 = col.name();
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv3(const QColor&col) {
	btnSurface.g2Color1 = col.name();
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv4(const QColor&col) {
	btnSurface.g2Color2 = col.name();
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv5(const QColor&col) {
	btnSurface.background = col.name();
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv6(int i) {
	btnSurface.g1Top = i;
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv7(int i) {
	btnSurface.g1Bottom = i;
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv8(int i) {
	btnSurface.g2Top = i;
	pushButton16->repaint(false);
}
void DominoStyleConfig::btnPv9(int i) {
	btnSurface.g2Bottom= i;
	pushButton16->repaint(false);
}
// combobox
void DominoStyleConfig::cbPv1(const QColor&) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv2(const QColor&) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv3(const QColor&) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv4(const QColor&) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv5(const QColor&) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv6(int ) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv7(int ) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv8(int ) {
	comboBox17->repaint(false);
}
void DominoStyleConfig::cbPv9(int ) {
	comboBox17->repaint(false);
}
// header
void DominoStyleConfig::hPv1(const QColor&col) {
	headerSurface.g1Color1 = col.name();
	header1->setBackgroundMode(Qt::PaletteBackground); // redo the backgroundPixmap
	header1->repaint(false);
}
void DominoStyleConfig::hPv2(const QColor&col) {
	headerSurface.g1Color2 = col.name();
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv3(const QColor&col) {
	headerSurface.g2Color1 = col.name();
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv4(const QColor&col) {
	headerSurface.g2Color2 = col.name();
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv5(const QColor&col) {
	headerSurface.background = col.name();
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv6(int i) {
	headerSurface.g1Top = i;
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv7(int i) {
	headerSurface.g1Bottom = i;
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv8(int i) {
	headerSurface.g2Top = i;
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
void DominoStyleConfig::hPv9(int i) {
	headerSurface.g2Bottom= i;
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}
// spinbox
void DominoStyleConfig::spPv1(const QColor& ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv2(const QColor& ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv3(const QColor& ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv4(const QColor& ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv5(const QColor& ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv6(int ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv7(int ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv8(int ) {
	spinBox44->repaint(false);
}
void DominoStyleConfig::spPv9(int ) {
	spinBox44->repaint(false);
}
// checkBox
void DominoStyleConfig::chbPv1(const QColor&col) {
	checkItemSurface.g1Color1 = col.name();
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv2(const QColor&col) {
	checkItemSurface.g1Color2 = col.name();
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv3(const QColor&col) {
	checkItemSurface.g2Color1 = col.name();
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv4(const QColor&col) {
	checkItemSurface.g2Color2 = col.name();
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv5(const QColor&col) {
	checkItemSurface.background = col.name();
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv6(int i) {
	checkItemSurface.g1Top = i;
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv7(int i) {
	checkItemSurface.g1Bottom = i;
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv8(int i) {
	checkItemSurface.g2Top = i;
	checkBox2->repaint(false);
}
void DominoStyleConfig::chbPv9(int i) {
	checkItemSurface.g2Bottom= i;
	checkBox2->repaint(false);
}
// radioButton
void DominoStyleConfig::rPv1(const QColor&col) {
	checkItemSurface.g1Color1 = col.name();
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv2(const QColor&col) {
	checkItemSurface.g1Color2 = col.name();
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv3(const QColor&col) {
	checkItemSurface.g2Color1 = col.name();
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv4(const QColor&col) {
	checkItemSurface.g2Color2 = col.name();
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv5(const QColor&col) {
	checkItemSurface.background = col.name();
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv6(int i) {
	checkItemSurface.g1Top = i;
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv7(int i) {
	checkItemSurface.g1Bottom = i;
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv8(int i) {
	checkItemSurface.g2Top = i;
	radioButton2->repaint(false);
}
void DominoStyleConfig::rPv9(int i) {
	checkItemSurface.g2Bottom= i;
	radioButton2->repaint(false);
}
// scrollBar
void DominoStyleConfig::sbPv1(const QColor&col) {
	currentScrollBarSurface->g1Color1 = col.name();
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv2(const QColor&col) {
	currentScrollBarSurface->g1Color2 = col.name();
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv3(const QColor&col) {
	currentScrollBarSurface->g2Color1 = col.name();
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv4(const QColor&col) {
	currentScrollBarSurface->g2Color2 = col.name();
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv5(const QColor&col) {
	currentScrollBarSurface->background = col.name();
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv6(int i) {
	currentScrollBarSurface->g1Top = i;
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv7(int i) {
	currentScrollBarSurface->g1Bottom = i;
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv8(int i) {
	currentScrollBarSurface->g2Top = i;
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}
void DominoStyleConfig::sbPv9(int i) {
	currentScrollBarSurface->g2Bottom = i;
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}


void DominoStyleConfig::tabPv1(const QColor&col) {
	currentTabSurface->g1Color1 = col.name();
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv2(const QColor&col) {
	currentTabSurface->g1Color2 = col.name();
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv3(const QColor&col) {
	currentTabSurface->g2Color1 = col.name();
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv4(const QColor&col) {
	currentTabSurface->g2Color2 = col.name();
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv5(const QColor&col) {
	currentTabSurface->background = col.name();
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv6(int i) {
	currentTabSurface->g1Top = i;
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv7(int i) {
	currentTabSurface->g1Bottom = i;
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv8(int i) {
	currentTabSurface->g2Top = i;
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}
void DominoStyleConfig::tabPv9(int i) {
	currentTabSurface->g2Bottom = i;
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}

void DominoStyleConfig::loadConfig(const char* name) {

	QColor background = qApp->palette().active().background();
	QColor button = qApp->palette().active().button();
	QString cName = name;
	QSettings settings;
	
	settings.beginGroup("/" + cName + "/Settings");
	currentConfig = settings.readEntry("/name", "unnamed");
//	origAnimProgressBar = settings.readBoolEntry("/animateProgressBar", false);
//	animateProgressBar->setChecked(origAnimProgressBar);
	origCenterTabs = settings.readBoolEntry("/centerTabs", false);
	centerTabs->setChecked(origCenterTabs);
	origDrawTriangularExpander = settings.readBoolEntry("/drawTriangularExpander", false);
	drawTriangularExpander->setChecked(origDrawTriangularExpander);

	origCustomCheckMarkColor = settings.readBoolEntry("/customCheckMarkColor", false);
	customCheckMarkColor->setChecked(origCustomCheckMarkColor);
	origCheckMarkColor = settings.readEntry("/checkMarkColor", "black");
	checkMarkColor->setColor(origCheckMarkColor);
	
	origTintGroupBoxBackground = settings.readBoolEntry("/tintGroupBoxBackground", true);
	tintGroupBoxBackground->setChecked(origTintGroupBoxBackground);
	origDrawGroupBoxDark = settings.readBoolEntry("/drawGroupBoxDark", true);
	drawGroupBoxDark->setChecked(origDrawGroupBoxDark);
	origDrawGroupBoxBright = settings.readBoolEntry("/drawGroupBoxBright", false);
	drawGroupBoxBright->setChecked(origDrawGroupBoxBright);

	origCustomToolTipColor = settings.readBoolEntry("/customToolTipColor", false);
	customToolTipColor->setChecked(origCustomToolTipColor);
	origToolTipColor = settings.readEntry("/toolTipColor", QColor(218,218,179).name());
	toolTipColor->setColor(origToolTipColor);
	
	origSmoothScrolling = settings.readBoolEntry("/smoothScrolling", true);
	smoothScrolling->setChecked(origSmoothScrolling);
	origButtonHeightAdjustment = settings.readNumEntry("/buttonHeightAdjustment", 0);
	buttonHeightAdjustment->setValue(origButtonHeightAdjustment);

	origCustomPopupMenuColor  = settings.readBoolEntry("/customPopupMenuColor", false);
	customPopupMenuColor->setChecked(origCustomPopupMenuColor);
	origPopupMenuColor = settings.readEntry("/popupMenuColor", background.name());
	popupMenuColor->setColor(origPopupMenuColor);
	
	origIndicateFocus = settings.readBoolEntry("/indicateFocus", false);
	indicateFocus->setChecked(origIndicateFocus);
	origIndicatorBtnColor = settings.readEntry("/indicatorBtnColor", button.light(120).name());
	indicatorBtnColor->setColor(origIndicatorBtnColor);
	origIndicatorColor = settings.readEntry("/indicatorColor", button.light(120).name());
	indicatorColor->setColor(origIndicatorColor);
	origDrawUnderline = settings.readBoolEntry("/drawFocusUnderline", true);
	drawUnderline->setChecked(origDrawUnderline);
	drawColoredOutline->setChecked(!origDrawUnderline);
	
	// buttons / comboBoxes / spinBoxes
	
	btnSurface.numGradients = settings.readNumEntry("/btnSurface_numGradients", 2);
	switch (btnSurface.numGradients) {
		case 2:
			groupBox8_2_2_2->setChecked(true);
			groupBox9_2_2_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2->setChecked(true);
			groupBox9_2_2_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2->setChecked(false);
			groupBox9_2_2_2->setChecked(false);
			break;
	}
	kColorButton2_4_2_2_2->setColor(settings.readEntry("/btnSurface_g1Color1", button.name()));
	kColorButton1_2_2_2_2->setColor(settings.readEntry("/btnSurface_g1Color2", button.dark(120).name()));
	spinBox1_2_2_2_2->setValue(settings.readNumEntry("/btnSurface_g1Top", 0));
	spinBox2_2_2_2_2->setValue(settings.readNumEntry("/btnSurface_g1Bottom", 50));
	kColorButton2_5_2_2_2->setColor(settings.readEntry("/btnSurface_g2Color1", button.dark(120).name()));
	kColorButton1_3_2_2_2->setColor(settings.readEntry("/btnSurface_g2Color2", button.dark(110).name()));
	spinBox1_3_2_2_2->setValue(settings.readNumEntry("/btnSurface_g2Top", 50));
	spinBox2_3_2_2_2->setValue(settings.readNumEntry("/btnSurface_g2Bottom", 100));
	kColorButton27_2->setColor(settings.readEntry("/btnSurface_background", button.name()));

	
	btnSurface.g1Color1 = kColorButton2_4_2_2_2->color();
	btnSurface.g1Color2 = kColorButton1_2_2_2_2->color();
	btnSurface.g2Color1 = kColorButton2_5_2_2_2->color();
	btnSurface.g2Color2 = kColorButton1_3_2_2_2->color();
	btnSurface.g1Top = spinBox1_2_2_2_2->value();
	btnSurface.g1Bottom = spinBox2_2_2_2_2->value();
	btnSurface.g2Top = spinBox1_3_2_2_2->value();
	btnSurface.g2Bottom = spinBox2_3_2_2_2->value();
	btnSurface.background = kColorButton27_2->color();
	
	origBtnSurface = btnSurface;
	
	// tabs
	
	activeTabTopSurface.numGradients = settings.readNumEntry("/activeTabTopSurface_numGradients", 2);
	switch (activeTabTopSurface.numGradients) {
		case 2:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2->setChecked(false);
			groupBox9_2_2_2_2->setChecked(false);
			break;
	}
	kColorButton2_4_2_2_2_2->setColor(settings.readEntry("/activeTabTopSurface_g1Color1", background.light(110).name()));
	kColorButton1_2_2_2_2_2->setColor(settings.readEntry("/activeTabTopSurface_g1Color2", background.name()));
	kColorButton2_5_2_2_2_2->setColor(settings.readEntry("/activeTabTopSurface_g2Color1", background.name()));
	kColorButton1_3_2_2_2_2->setColor(settings.readEntry("/activeTabTopSurface_g2Color2", background.name()));
	kColorButton27_2_2->setColor(settings.readEntry("/activeTabTopSurface_background", background.name()));
	spinBox1_2_2_2_2_2->setValue(settings.readNumEntry("/activeTabTopSurface_g1Top", 0));
	spinBox2_2_2_2_2_2->setValue(settings.readNumEntry("/activeTabTopSurface_g1Bottom", 50));
	spinBox1_3_2_2_2_2->setValue(settings.readNumEntry("/activeTabTopSurface_g2Top", 50));
	spinBox2_3_2_2_2_2->setValue(settings.readNumEntry("/activeTabTopSurface_g2Bottom", 100));
	
	activeTabTopSurface.g1Color1 = kColorButton2_4_2_2_2_2->color().name();
	activeTabTopSurface.g1Color2 = kColorButton1_2_2_2_2_2->color().name();
	activeTabTopSurface.g2Color1 = kColorButton2_5_2_2_2_2->color().name();
	activeTabTopSurface.g2Color2 = kColorButton1_3_2_2_2_2->color().name();
	activeTabTopSurface.background = kColorButton27_2_2->color().name();
	activeTabTopSurface.g1Top = spinBox1_2_2_2_2_2->value();
	activeTabTopSurface.g1Bottom = spinBox2_2_2_2_2_2->value();
	activeTabTopSurface.g2Top = spinBox1_3_2_2_2_2->value();
	activeTabTopSurface.g2Bottom = spinBox2_3_2_2_2_2->value();
	
	currentTabSurface = &activeTabTopSurface;
	
	origActiveTabTopSurface = activeTabTopSurface;
	
	tabTopSurface.numGradients = settings.readNumEntry("/tabTopSurface_numGradients", 2);
	tabTopSurface.g1Color1 = settings.readEntry("/tabTopSurface_g1Color1", background.light(110).name());
	tabTopSurface.g1Color2 = settings.readEntry("/tabTopSurface_g1Color2", background.dark(110).name());
	tabTopSurface.g2Color1 = settings.readEntry("/tabTopSurface_g2Color1", background.dark(110).name());
	tabTopSurface.g2Color2 = settings.readEntry("/tabTopSurface_g2Color2", background.dark(110).name());
	tabTopSurface.background = settings.readEntry("/tabTopSurface_background", background.dark(110).name());
	tabTopSurface.g1Top = settings.readNumEntry("/tabTopSurface_g1Top", 0);
	tabTopSurface.g1Bottom = settings.readNumEntry("/tabTopSurface_g1Bottom", 50);
	tabTopSurface.g2Top = settings.readNumEntry("/tabTopSurface_g2Top", 50);
	tabTopSurface.g2Bottom = settings.readNumEntry("/tabTopSurface_g2Bottom", 100);
	
	origTabTopSurface = tabTopSurface;
	
	activeTabBottomSurface.numGradients = settings.readNumEntry("/activeTabBottomSurface_numGradients", 2);
	activeTabBottomSurface.g1Color1 = settings.readEntry("/activeTabBottomSurface_g1Color1", background.name());
	activeTabBottomSurface.g1Color2 = settings.readEntry("/activeTabBottomSurface_g1Color2", background.name());
	activeTabBottomSurface.g2Color1 = settings.readEntry("/activeTabBottomSurface_g2Color1", background.name());
	activeTabBottomSurface.g2Color2 = settings.readEntry("/activeTabBottomSurface_g2Color2", background.dark(120).name());
	activeTabBottomSurface.background = settings.readEntry("/activeTabBottomSurface_background", background.name());
	activeTabBottomSurface.g1Top = settings.readNumEntry("/activeTabBottomSurface_g1Top", 0);
	activeTabBottomSurface.g1Bottom = settings.readNumEntry("/activeTabBottomSurface_g1Bottom", 50);
	activeTabBottomSurface.g2Top = settings.readNumEntry("/activeTabBottomSurface_g2Top", 50);
	activeTabBottomSurface.g2Bottom = settings.readNumEntry("/activeTabBottomSurface_g2Bottom", 100);
	
	origActiveTabBottomSurface = activeTabBottomSurface;
	
	tabBottomSurface.numGradients = settings.readNumEntry("/tabBottomSurface_numGradients", 2);
	tabBottomSurface.g1Color1 = settings.readEntry("/tabBottomSurface_g1Color1", background.dark(110).name());
	tabBottomSurface.g1Color2 = settings.readEntry("/tabBottomSurface_g1Color2", background.dark(110).name());
	tabBottomSurface.g2Color1 = settings.readEntry("/tabBottomSurface_g2Color1", background.dark(110).name());
	tabBottomSurface.g2Color2 = settings.readEntry("/tabBottomSurface_g2Color2", background.dark(120).name());
	tabBottomSurface.background = settings.readEntry("/tabBottomSurface_background", background.dark(110).name());
	tabBottomSurface.g1Top = settings.readNumEntry("/tabBottomSurface_g1Top", 0);
	tabBottomSurface.g1Bottom = settings.readNumEntry("/tabBottomSurface_g1Bottom", 50);
	tabBottomSurface.g2Top = settings.readNumEntry("/tabBottomSurface_g2Top", 50);
	tabBottomSurface.g2Bottom = settings.readNumEntry("/tabBottomSurface_g2Bottom", 100);
	
	origTabBottomSurface = tabBottomSurface;
	
	
	// headers

	headerSurface.numGradients = settings.readNumEntry("/headerSurface_numGradients", 2);
	switch (headerSurface.numGradients) {
		case 2:
			groupBox8_2_2_2_2_4_3->setChecked(true);
			groupBox9_2_2_2_2_4_3->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4_3->setChecked(true);
			groupBox9_2_2_2_2_4_3->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4_3->setChecked(false);
			groupBox9_2_2_2_2_4_3->setChecked(false);
			break;
	}
	kColorButton2_4_2_2_2_2_4_3->setColor(settings.readEntry("/headerSurface_g1Color1", background.name()));
	kColorButton1_2_2_2_2_2_4_3->setColor(settings.readEntry("/headerSurface_g1Color2", background.dark(120).name()));
	spinBox1_2_2_2_2_2_4_3->setValue(settings.readNumEntry("/headerSurface_g1Top", 0));
	spinBox2_2_2_2_2_2_4_3->setValue(settings.readNumEntry("/headerSurface_g1Bottom", 50));
	kColorButton2_5_2_2_2_2_4_3->setColor(settings.readEntry("/headerSurface_g2Color1", background.dark(120).name()));
	kColorButton1_3_2_2_2_2_4_3->setColor(settings.readEntry("/headerSurface_g2Color2", background.dark(110).name()));
	spinBox1_3_2_2_2_2_4_3->setValue(settings.readNumEntry("/headerSurface_g2Top", 50));
	spinBox2_3_2_2_2_2_4_3->setValue(settings.readNumEntry("/headerSurface_g2Bottom", 100));
	kColorButton27_2_2_2_2->setColor(settings.readEntry("/headerSurface_background", background.name()));

	headerSurface.g1Color1 = kColorButton2_4_2_2_2_2_4_3->color();
	headerSurface.g1Color2 = kColorButton1_2_2_2_2_2_4_3->color();
	headerSurface.g2Color1 = kColorButton2_5_2_2_2_2_4_3->color();
	headerSurface.g2Color2 = kColorButton1_3_2_2_2_2_4_3->color();
	headerSurface.g1Top = spinBox1_2_2_2_2_2_4_3->value();
	headerSurface.g1Bottom = spinBox2_2_2_2_2_2_4_3->value();
	headerSurface.g2Top = spinBox1_3_2_2_2_2_4_3->value();
	headerSurface.g2Bottom = spinBox2_3_2_2_2_2_4_3->value();
	headerSurface.background = kColorButton27_2_2_2_2->color();

	origHeaderSurface = headerSurface;
	
	
	// radio / checkboxes
	
	checkItemSurface.numGradients = settings.readNumEntry("/checkItemSurface_numGradients", 2);
	switch (checkItemSurface.numGradients) {
		case 2:
			groupBox8_2_2_2_2_4_3_2->setChecked(true);
			groupBox9_2_2_2_2_4_3_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4_3_2->setChecked(true);
			groupBox9_2_2_2_2_4_3_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4_3_2->setChecked(false);
			groupBox9_2_2_2_2_4_3_2->setChecked(false);
			break;
	}
	kColorButton2_4_2_2_2_2_4_3_2->setColor(settings.readEntry("/checkItemSurface_g1Color1", button.name()));
	kColorButton1_2_2_2_2_2_4_3_2->setColor(settings.readEntry("/checkItemSurface_g1Color2", button.dark(120).name()));
	spinBox1_2_2_2_2_2_4_3_2->setValue(settings.readNumEntry("/checkItemSurface_g1Top", 0));
	spinBox2_2_2_2_2_2_4_3_2->setValue(settings.readNumEntry("/checkItemSurface_g1Bottom", 50));
	kColorButton2_5_2_2_2_2_4_3_2->setColor(settings.readEntry("/checkItemSurface_g2Color1", button.dark(120).name()));
	kColorButton1_3_2_2_2_2_4_3_2->setColor(settings.readEntry("/checkItemSurface_g2Color2", button.dark(110).name()));
	spinBox1_3_2_2_2_2_4_3_2->setValue(settings.readNumEntry("/checkItemSurface_g2Top", 50));
	spinBox2_3_2_2_2_2_4_3_2->setValue(settings.readNumEntry("/checkItemSurface_g2Bottom", 100));
	kColorButton27_2_2_2_3->setColor(settings.readEntry("/checkItemSurface_background", button.name()));

	checkItemSurface.g1Color1 = kColorButton2_4_2_2_2_2_4_3_2->color();
	checkItemSurface.g1Color2 = kColorButton1_2_2_2_2_2_4_3_2->color();
	checkItemSurface.g2Color1 = kColorButton2_5_2_2_2_2_4_3_2->color();
	checkItemSurface.g2Color2 = kColorButton1_3_2_2_2_2_4_3_2->color();
	checkItemSurface.g1Top = spinBox1_2_2_2_2_2_4_3_2->value();
	checkItemSurface.g1Bottom = spinBox2_2_2_2_2_2_4_3_2->value();
	checkItemSurface.g2Top = spinBox1_3_2_2_2_2_4_3_2->value();
	checkItemSurface.g2Bottom = spinBox2_3_2_2_2_2_4_3_2->value();
	checkItemSurface.background = kColorButton27_2_2_2_3->color();

	origCheckItemSurface = checkItemSurface;
	
	
	// scrollBar Groove / Buttons
	
	scrollBarCapsColor = settings.readEntry("/scrollBarCapsColor", Qt::black.name());
	capsColorButton->setColor(scrollBarCapsColor);
	origScrollBarCapsColor = scrollBarCapsColor;
	
	
	// schrollBarButtons
	scrollBarSurface.numGradients = settings.readNumEntry("/scrollBarSurface_numGradients", 2);
	scrollBarSurface.g1Color1 = settings.readEntry("/scrollBarSurface_g1Color1", button.name());
	scrollBarSurface.g1Color2 = settings.readEntry("/scrollBarSurface_g1Color2", button.dark(120).name());
	scrollBarSurface.g2Color1 = settings.readEntry("/scrollBarSurface_g2Color1", button.dark(120).name());
	scrollBarSurface.g2Color2 = settings.readEntry("/scrollBarSurface_g2Color2", button.dark(110).name());
	scrollBarSurface.g1Top = settings.readNumEntry("/scrollBarSurface_g1Top", 0);
	scrollBarSurface.g1Bottom = settings.readNumEntry("/scrollBarSurface_g1Bottom", 50);
	scrollBarSurface.g2Top = settings.readNumEntry("/scrollBarSurface_g2Top", 50);
	scrollBarSurface.g2Bottom = settings.readNumEntry("/scrollBarSurface_g2Bottom", 100);
	scrollBarSurface.background = settings.readEntry("/scrollBarSurface_background", button.name());
	
	origScrollBarSurface = scrollBarSurface;
	
	// schrollBarGroove
	scrollBarGrooveSurface.numGradients = settings.readNumEntry("/scrollBarGrooveSurface_numGradients", 0);
	scrollBarGrooveSurface.g1Color1 = settings.readEntry("/scrollBarGrooveSurface_g1Color1", background.name());
	scrollBarGrooveSurface.g1Color2 = settings.readEntry("/scrollBarGrooveSurface_g1Color2", background.dark(120).name());
	scrollBarGrooveSurface.g2Color1 = settings.readEntry("/scrollBarGrooveSurface_g2Color1", background.dark(120).name());
	scrollBarGrooveSurface.g2Color2 = settings.readEntry("/scrollBarGrooveSurface_g2Color2", background.dark(110).name());
	scrollBarGrooveSurface.g1Top = settings.readNumEntry("/scrollBarGrooveSurface_g1Top", 0);
	scrollBarGrooveSurface.g1Bottom = settings.readNumEntry("/scrollBarGrooveSurface_g1Bottom", 50);
	scrollBarGrooveSurface.g2Top = settings.readNumEntry("/scrollBarGrooveSurface_g2Top", 50);
	scrollBarGrooveSurface.g2Bottom = settings.readNumEntry("/scrollBarGrooveSurface_g2Bottom", 100);
	scrollBarGrooveSurface.background = settings.readEntry("/scrollBarGrooveSurface_background", background.dark(150).name());
	
	origScrollBarGrooveSurface = scrollBarGrooveSurface;
	
	comboBox7->setCurrentItem(0);
	
	switch(scrollBarSurface.numGradients) {
		case 2:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4->setChecked(false);
			groupBox9_2_2_2_2_4->setChecked(false);
			break;
	}
	kColorButton2_4_2_2_2_2_4->setColor(scrollBarSurface.g1Color1);
	kColorButton1_2_2_2_2_2_4->setColor(scrollBarSurface.g1Color2);
	spinBox1_2_2_2_2_2_4->setValue(scrollBarSurface.g1Top);
	spinBox2_2_2_2_2_2_4->setValue(scrollBarSurface.g1Bottom);
	kColorButton2_5_2_2_2_2_4->setColor(scrollBarSurface.g2Color1);
	kColorButton1_3_2_2_2_2_4->setColor(scrollBarSurface.g2Color2);
	spinBox1_3_2_2_2_2_4->setValue(scrollBarSurface.g2Top);
	spinBox2_3_2_2_2_2_4->setValue(scrollBarSurface.g2Bottom);
	kColorButton27_2_2_2->setColor(scrollBarSurface.background);
	
	currentScrollBarSurface = &scrollBarSurface;
	
	settings.endGroup();
	
	currentConfigLabel->setText(i18n("Currently loaded config: ")+currentConfig);
}

void DominoStyleConfig::userLoadScrollBarConfig(const QString &) {

	scrollBar5->setUpdatesEnabled(false);
	scrollBar6->setUpdatesEnabled(false);
	
	// index: groove = 1, buttons/slider = 0
	if(comboBox7->currentItem())
		currentScrollBarSurface = &scrollBarGrooveSurface;
	else
		currentScrollBarSurface = &scrollBarSurface;

	kColorButton2_4_2_2_2_2_4->setColor(currentScrollBarSurface->g1Color1);
	kColorButton1_2_2_2_2_2_4->setColor(currentScrollBarSurface->g1Color2);
	spinBox1_2_2_2_2_2_4->setValue(currentScrollBarSurface->g1Top);
	spinBox2_2_2_2_2_2_4->setValue(currentScrollBarSurface->g1Bottom);
	kColorButton2_5_2_2_2_2_4->setColor(currentScrollBarSurface->g2Color1);
	kColorButton1_3_2_2_2_2_4->setColor(currentScrollBarSurface->g2Color2);
	spinBox1_3_2_2_2_2_4->setValue(currentScrollBarSurface->g2Top);
	spinBox2_3_2_2_2_2_4->setValue(currentScrollBarSurface->g2Bottom);
	kColorButton27_2_2_2->setColor(currentScrollBarSurface->background);
	
	
	switch(currentScrollBarSurface->numGradients) {
		case 2:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4->setChecked(false);
			groupBox9_2_2_2_2_4->setChecked(false);
			break;
	}
	scrollBar5->setUpdatesEnabled(true);
	scrollBar6->setUpdatesEnabled(true);
}

void DominoStyleConfig::userLoadTabConfig(int) {
	
	((DominoQTabWidget*)prevTopTabWidget)->tabBar()->setUpdatesEnabled(false);
	((DominoQTabWidget*)prevBottomTabWidget)->tabBar()->setUpdatesEnabled(false);
	
	bool topTabWidgetIsActive = false;
	// index: bottom = 1, top = 0
	if(tabPosCombo->currentItem())
		// index: inactive = 1, active = 0
		if(tabStateCombo->currentItem())
			currentTabSurface = &tabBottomSurface;
		else
			currentTabSurface = &activeTabBottomSurface;
	else {
		topTabWidgetIsActive = true;
		if(tabStateCombo->currentItem())
			currentTabSurface = &tabTopSurface;
		else
			currentTabSurface = &activeTabTopSurface;
	}
	
	QColor color1;
	QColor color2;
	
	if(topTabWidgetIsActive) {
		color1 = qApp->palette().active().background();
		color2 = prevTopTabWidget->parentWidget()->paletteBackgroundColor();
	}
	else {
		color2 = qApp->palette().active().background();
		color1 = prevTopTabWidget->parentWidget()->paletteBackgroundColor();
	}
	
	prevTopTabWidget->page(0)->parentWidget()->setPaletteBackgroundColor(color1);
	prevBottomTabWidget->page(0)->parentWidget()->setPaletteBackgroundColor(color2);
	
	kColorButton2_4_2_2_2_2->setColor(currentTabSurface->g1Color1);
	kColorButton1_2_2_2_2_2->setColor(currentTabSurface->g1Color2);
	kColorButton2_5_2_2_2_2->setColor(currentTabSurface->g2Color1);
	kColorButton1_3_2_2_2_2->setColor(currentTabSurface->g2Color2);
	kColorButton27_2_2->setColor(currentTabSurface->background);
	spinBox1_2_2_2_2_2->setValue(currentTabSurface->g1Top);
	spinBox2_2_2_2_2_2->setValue(currentTabSurface->g1Bottom);
	spinBox1_3_2_2_2_2->setValue(currentTabSurface->g2Top);
	spinBox2_3_2_2_2_2->setValue(currentTabSurface->g2Bottom);
	
	switch (currentTabSurface->numGradients) {
		case 2:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2->setChecked(false);
			groupBox9_2_2_2_2->setChecked(false);
			break;
	}
	((DominoQTabWidget*)prevTopTabWidget)->tabBar()->setUpdatesEnabled(true);
	((DominoQTabWidget*)prevBottomTabWidget)->tabBar()->setUpdatesEnabled(true);
}

void DominoStyleConfig::saveConfig(const char* fileName, const char* schemeName ) {
	
	QString fName;
	QSettings s;
	int c;

	if(!strcmp(fileName, "domino"))
		fName = fileName;
	else
		fName = QString("domino_").append(fileName);
	
	s.beginGroup("/" + fName + "/Settings");
	
	s.writeEntry("/name", schemeName);
//	s.writeEntry("/animateProgressBar", animateProgressBar->isChecked());
	s.writeEntry("/centerTabs", centerTabs->isChecked());
	s.writeEntry("/drawTriangularExpander", drawTriangularExpander->isChecked());
	s.writeEntry("/customCheckMarkColor", customCheckMarkColor->isChecked());
	s.writeEntry("/checkMarkColor", checkMarkColor->color().name());
	s.writeEntry("/tintGroupBoxBackground", tintGroupBoxBackground->isChecked());
	s.writeEntry("/drawGroupBoxDark", drawGroupBoxDark->isChecked());
	s.writeEntry("/drawGroupBoxBright", drawGroupBoxBright->isChecked());
	s.writeEntry("/customToolTipColor", customToolTipColor->isChecked());
	s.writeEntry("/toolTipColor", toolTipColor->color().name());
	s.writeEntry("/smoothScrolling", smoothScrolling->isChecked());
	s.writeEntry("/buttonHeightAdjustment", buttonHeightAdjustment->value());
	s.writeEntry("/customPopupMenuColor", customPopupMenuColor->isChecked());
	s.writeEntry("/popupMenuColor", popupMenuColor->color().name());
	s.writeEntry("/indicateFocus", indicateFocus->isChecked());
	s.writeEntry("/drawFocusUnderline", drawUnderline->isChecked());
	s.writeEntry("/indicatorBtnColor", indicatorBtnColor->color().name());
	s.writeEntry("/indicatorColor", indicatorColor->color().name());
	//////


	// buttons
	c = groupBox9_2_2_2->isChecked() ? 2 : groupBox8_2_2_2->isChecked() ? 1 : 0;
	
	s.writeEntry("/btnSurface_numGradients", c);
	s.writeEntry("/btnSurface_g1Color1", kColorButton2_4_2_2_2->color().name());
	s.writeEntry("/btnSurface_g1Color2", kColorButton1_2_2_2_2->color().name());
	s.writeEntry("/btnSurface_g2Color1", kColorButton2_5_2_2_2->color().name());
	s.writeEntry("/btnSurface_g2Color2", kColorButton1_3_2_2_2->color().name());
	s.writeEntry("/btnSurface_g1Top", spinBox1_2_2_2_2->value());
	s.writeEntry("/btnSurface_g1Bottom", spinBox2_2_2_2_2->value());
	s.writeEntry("/btnSurface_g2Top", spinBox1_3_2_2_2->value());
	s.writeEntry("/btnSurface_g2Bottom", spinBox2_3_2_2_2->value());
	s.writeEntry("/btnSurface_background", kColorButton27_2->color().name());
	
	// headers
	c = groupBox9_2_2_2_2_4_3->isChecked() ? 2 : groupBox8_2_2_2_2_4_3->isChecked() ? 1 : 0;
	
	s.writeEntry("/headerSurface_numGradients", c);
	s.writeEntry("/headerSurface_g1Color1", kColorButton2_4_2_2_2_2_4_3->color().name());
	s.writeEntry("/headerSurface_g1Color2", kColorButton1_2_2_2_2_2_4_3->color().name());
	s.writeEntry("/headerSurface_g2Color1", kColorButton2_5_2_2_2_2_4_3->color().name());
	s.writeEntry("/headerSurface_g2Color2", kColorButton1_3_2_2_2_2_4_3->color().name());
	s.writeEntry("/headerSurface_g1Top", spinBox1_2_2_2_2_2_4_3->value());
	s.writeEntry("/headerSurface_g1Bottom", spinBox2_2_2_2_2_2_4_3->value());
	s.writeEntry("/headerSurface_g2Top", spinBox1_3_2_2_2_2_4_3->value());
	s.writeEntry("/headerSurface_g2Bottom", spinBox2_3_2_2_2_2_4_3->value());
	s.writeEntry("/headerSurface_background", kColorButton27_2_2_2_2->color().name());
	
	
	// radio/checkboxes
	c = groupBox9_2_2_2_2_4_3_2->isChecked() ? 2 : groupBox8_2_2_2_2_4_3_2->isChecked() ? 1 : 0;
	
	s.writeEntry("/checkItemSurface_numGradients", c);
	s.writeEntry("/checkItemSurface_g1Color1", kColorButton2_4_2_2_2_2_4_3_2->color().name());
	s.writeEntry("/checkItemSurface_g1Color2", kColorButton1_2_2_2_2_2_4_3_2->color().name());
	s.writeEntry("/checkItemSurface_g2Color1", kColorButton2_5_2_2_2_2_4_3_2->color().name());
	s.writeEntry("/checkItemSurface_g2Color2", kColorButton1_3_2_2_2_2_4_3_2->color().name());
	s.writeEntry("/checkItemSurface_g1Top", spinBox1_2_2_2_2_2_4_3_2->value());
	s.writeEntry("/checkItemSurface_g1Bottom", spinBox2_2_2_2_2_2_4_3_2->value());
	s.writeEntry("/checkItemSurface_g2Top", spinBox1_3_2_2_2_2_4_3_2->value());
	s.writeEntry("/checkItemSurface_g2Bottom", spinBox2_3_2_2_2_2_4_3_2->value());
	s.writeEntry("/checkItemSurface_background", kColorButton27_2_2_2_3->color().name());
	
	
	// scrollbars

	// buttons

	s.writeEntry("/scrollBarSurface_numGradients", scrollBarSurface.numGradients);
	s.writeEntry("/scrollBarSurface_g1Color1", scrollBarSurface.g1Color1.name());
	s.writeEntry("/scrollBarSurface_g1Color2", scrollBarSurface.g1Color2.name());
	s.writeEntry("/scrollBarSurface_g2Color1", scrollBarSurface.g2Color1.name());
	s.writeEntry("/scrollBarSurface_g2Color2", scrollBarSurface.g2Color2.name());
	s.writeEntry("/scrollBarSurface_g1Top", scrollBarSurface.g1Top);
	s.writeEntry("/scrollBarSurface_g1Bottom", scrollBarSurface.g1Bottom);
	s.writeEntry("/scrollBarSurface_g2Top", scrollBarSurface.g2Top);
	s.writeEntry("/scrollBarSurface_g2Bottom", scrollBarSurface.g2Bottom);
	s.writeEntry("/scrollBarSurface_background", scrollBarSurface.background.name());
	
	// groove
	s.writeEntry("/scrollBarGrooveSurface_numGradients", scrollBarGrooveSurface.numGradients);
	s.writeEntry("/scrollBarGrooveSurface_g1Color1", scrollBarGrooveSurface.g1Color1.name());
	s.writeEntry("/scrollBarGrooveSurface_g1Color2", scrollBarGrooveSurface.g1Color2.name());
	s.writeEntry("/scrollBarGrooveSurface_g2Color1", scrollBarGrooveSurface.g2Color1.name());
	s.writeEntry("/scrollBarGrooveSurface_g2Color2", scrollBarGrooveSurface.g2Color2.name());
	s.writeEntry("/scrollBarGrooveSurface_g1Top", scrollBarGrooveSurface.g1Top);
	s.writeEntry("/scrollBarGrooveSurface_g1Bottom", scrollBarGrooveSurface.g1Bottom);
	s.writeEntry("/scrollBarGrooveSurface_g2Top", scrollBarGrooveSurface.g2Top);
	s.writeEntry("/scrollBarGrooveSurface_g2Bottom", scrollBarGrooveSurface.g2Bottom);
	s.writeEntry("/scrollBarGrooveSurface_background", scrollBarGrooveSurface.background.name());
	
	s.writeEntry("/scrollBarCapsColor", scrollBarCapsColor.name());
	
	// tabs
	s.writeEntry("/activeTabTopSurface_numGradients", activeTabTopSurface.numGradients);
	s.writeEntry("/activeTabTopSurface_g1Color1", activeTabTopSurface.g1Color1.name());
	s.writeEntry("/activeTabTopSurface_g1Color2", activeTabTopSurface.g1Color2.name());
	s.writeEntry("/activeTabTopSurface_g2Color1", activeTabTopSurface.g2Color1.name());
	s.writeEntry("/activeTabTopSurface_g2Color2", activeTabTopSurface.g2Color2.name());
	s.writeEntry("/activeTabTopSurface_g1Top", activeTabTopSurface.g1Top);
	s.writeEntry("/activeTabTopSurface_g1Bottom", activeTabTopSurface.g1Bottom);
	s.writeEntry("/activeTabTopSurface_g2Top", activeTabTopSurface.g2Top);
	s.writeEntry("/activeTabTopSurface_g2Bottom", activeTabTopSurface.g2Bottom);
	s.writeEntry("/activeTabTopSurface_background", activeTabTopSurface.background.name());
	
	s.writeEntry("/tabTopSurface_numGradients", tabTopSurface.numGradients);
	s.writeEntry("/tabTopSurface_g1Color1", tabTopSurface.g1Color1.name());
	s.writeEntry("/tabTopSurface_g1Color2", tabTopSurface.g1Color2.name());
	s.writeEntry("/tabTopSurface_g2Color1", tabTopSurface.g2Color1.name());
	s.writeEntry("/tabTopSurface_g2Color2", tabTopSurface.g2Color2.name());
	s.writeEntry("/tabTopSurface_g1Top", tabTopSurface.g1Top);
	s.writeEntry("/tabTopSurface_g1Bottom", tabTopSurface.g1Bottom);
	s.writeEntry("/tabTopSurface_g2Top", tabTopSurface.g2Top);
	s.writeEntry("/tabTopSurface_g2Bottom", tabTopSurface.g2Bottom);
	s.writeEntry("/tabTopSurface_background", tabTopSurface.background.name());
	
	s.writeEntry("/activeTabBottomSurface_numGradients", activeTabBottomSurface.numGradients);
	s.writeEntry("/activeTabBottomSurface_g1Color1", activeTabBottomSurface.g1Color1.name());
	s.writeEntry("/activeTabBottomSurface_g1Color2", activeTabBottomSurface.g1Color2.name());
	s.writeEntry("/activeTabBottomSurface_g2Color1", activeTabBottomSurface.g2Color1.name());
	s.writeEntry("/activeTabBottomSurface_g2Color2", activeTabBottomSurface.g2Color2.name());
	s.writeEntry("/activeTabBottomSurface_g1Top", activeTabBottomSurface.g1Top);
	s.writeEntry("/activeTabBottomSurface_g1Bottom", activeTabBottomSurface.g1Bottom);
	s.writeEntry("/activeTabBottomSurface_g2Top", activeTabBottomSurface.g2Top);
	s.writeEntry("/activeTabBottomSurface_g2Bottom", activeTabBottomSurface.g2Bottom);
	s.writeEntry("/activeTabBottomSurface_background", activeTabBottomSurface.background.name());
	
	s.writeEntry("/tabBottomSurface_numGradients", tabBottomSurface.numGradients);
	s.writeEntry("/tabBottomSurface_g1Color1", tabBottomSurface.g1Color1.name());
	s.writeEntry("/tabBottomSurface_g1Color2", tabBottomSurface.g1Color2.name());
	s.writeEntry("/tabBottomSurface_g2Color1", tabBottomSurface.g2Color1.name());
	s.writeEntry("/tabBottomSurface_g2Color2", tabBottomSurface.g2Color2.name());
	s.writeEntry("/tabBottomSurface_g1Top", tabBottomSurface.g1Top);
	s.writeEntry("/tabBottomSurface_g1Bottom", tabBottomSurface.g1Bottom);
	s.writeEntry("/tabBottomSurface_g2Top", tabBottomSurface.g2Top);
	s.writeEntry("/tabBottomSurface_g2Bottom", tabBottomSurface.g2Bottom);
	s.writeEntry("/tabBottomSurface_background", tabBottomSurface.background.name());
	
}

void PreviewPushButton::paintEvent(QPaintEvent* event) {

	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface btnSurface_save = dStyle->btnSurface;
	DominoStyle::DSurface previewSurface = btnSurface_save;
	previewSurface.numGradients = btnSurface.numGradients;
	previewSurface.g1Color1 = btnSurface.g1Color1;
	previewSurface.g1Color2 = btnSurface.g1Color2;
	previewSurface.g2Color1 = btnSurface.g2Color1;
	previewSurface.g2Color2 = btnSurface.g2Color2;
	previewSurface.background = btnSurface.background;
	previewSurface.g1Top = btnSurface.g1Top;
	previewSurface.g1Bottom = btnSurface.g1Bottom;
	previewSurface.g2Top = btnSurface.g2Top;
	previewSurface.g2Bottom = btnSurface.g2Bottom;
	
	dStyle->setBtnSurface(previewSurface);
	QPushButton::paintEvent( event );
	dStyle->setBtnSurface(btnSurface_save);
}

void PreviewComboBox::paintEvent(QPaintEvent* event) {

	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface btnSurface_save = dStyle->btnSurface;
	DominoStyle::DSurface previewSurface = btnSurface_save;
	previewSurface.numGradients = btnSurface.numGradients;
	previewSurface.g1Color1 = btnSurface.g1Color1;
	previewSurface.g1Color2 = btnSurface.g1Color2;
	previewSurface.g2Color1 = btnSurface.g2Color1;
	previewSurface.g2Color2 = btnSurface.g2Color2;
	previewSurface.background = btnSurface.background;
	previewSurface.g1Top = btnSurface.g1Top;
	previewSurface.g1Bottom = btnSurface.g1Bottom;
	previewSurface.g2Top = btnSurface.g2Top;
	previewSurface.g2Bottom = btnSurface.g2Bottom;
	
	dStyle->setBtnSurface(previewSurface);
	QComboBox::paintEvent( event );
	dStyle->setBtnSurface(btnSurface_save);
	
}

void PreviewSpinBox::paintEvent(QPaintEvent*) {
	sw->repaint(false);
}

bool PreviewSpinBox::eventFilter(QObject *obj, QEvent *ev) {
	
	if(dynamic_cast<QSpinWidget*>(obj)) {
		if(ev->type() == QEvent::Paint) {
			
			DominoStyle* dStyle = (DominoStyle*)&style();
			DominoStyle::DSurface btnSurface_save = dStyle->btnSurface;
			DominoStyle::DSurface previewSurface = btnSurface_save;
			previewSurface.numGradients = btnSurface.numGradients;
			previewSurface.g1Color1 = btnSurface.g1Color1;
			previewSurface.g1Color2 = btnSurface.g1Color2;
			previewSurface.g2Color1 = btnSurface.g2Color1;
			previewSurface.g2Color2 = btnSurface.g2Color2;
			previewSurface.background = btnSurface.background;
			previewSurface.g1Top = btnSurface.g1Top;
			previewSurface.g1Bottom = btnSurface.g1Bottom;
			previewSurface.g2Top = btnSurface.g2Top;
			previewSurface.g2Bottom = btnSurface.g2Bottom;
			dStyle->setBtnSurface(previewSurface);
			
			QSpinWidget* sw = dynamic_cast<QSpinWidget*>(obj);
			QPainter p(sw);

			QStyle::SFlags flags = QStyle::Style_Default;
			if (sw->isEnabled())
				flags |= QStyle::Style_Enabled;
			if (sw->hasFocus() || sw->focusProxy() && sw->focusProxy()->hasFocus())
				flags |= QStyle::Style_HasFocus;

			QRect fr = QStyle::visualRect(sw->style().querySubControlMetrics( QStyle::CC_SpinWidget, sw, QStyle::SC_SpinWidgetFrame ), sw );
			sw->style().drawComplexControl( QStyle::CC_SpinWidget, &p, sw, fr, colorGroup(), flags, (uint)QStyle::SC_All, QStyle::SC_None );
			
			dStyle->setBtnSurface(btnSurface_save);
			return true;
		}
		if(ev->type() == QEvent::Show) {
			QSpinWidget* sw = dynamic_cast<QSpinWidget*>(obj);
			if(strcmp(sw->style().name(), "domino")) {
				sw->setStyle("domino");
			}
			return true;
		}
			
		return false;
	}
	return false;
}

void PreviewHeader::paintEvent(QPaintEvent* event) {

	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface headerSurface_save = dStyle->headerSurface;
	DominoStyle::DSurface previewSurface = headerSurface_save;
	previewSurface.numGradients = headerSurface.numGradients;
	previewSurface.g1Color1 = headerSurface.g1Color1;
	previewSurface.g1Color2 = headerSurface.g1Color2;
	previewSurface.g2Color1 = headerSurface.g2Color1;
	previewSurface.g2Color2 = headerSurface.g2Color2;
	previewSurface.background = headerSurface.background;
	previewSurface.g1Top = headerSurface.g1Top;
	previewSurface.g1Bottom = headerSurface.g1Bottom;
	previewSurface.g2Top = headerSurface.g2Top;
	previewSurface.g2Bottom = headerSurface.g2Bottom;
	dStyle->setHeaderSurface(previewSurface);
	QHeader::paintEvent(event);
	dStyle->setHeaderSurface(headerSurface_save);
}

void PreviewCheckBox::paintEvent(QPaintEvent* ) {
	
	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface checkItemSurface_save = dStyle->checkItemSurface;
	DominoStyle::DSurface previewSurface = checkItemSurface_save;
	previewSurface.numGradients = checkItemSurface.numGradients;
	previewSurface.g1Color1 = checkItemSurface.g1Color1;
	previewSurface.g1Color2 = checkItemSurface.g1Color2;
	previewSurface.g2Color1 = checkItemSurface.g2Color1;
	previewSurface.g2Color2 = checkItemSurface.g2Color2;
	previewSurface.background = checkItemSurface.background;
	previewSurface.g1Top = checkItemSurface.g1Top;
	previewSurface.g1Bottom = checkItemSurface.g1Bottom;
	previewSurface.g2Top = checkItemSurface.g2Top;
	previewSurface.g2Bottom = checkItemSurface.g2Bottom;

	dStyle->setCheckItemSurface(previewSurface);
	QPainter p(this);
	drawButton(&p);
	dStyle->setCheckItemSurface(checkItemSurface_save);
}

// checkBoxes and radioButtons without pixmapCache
void PreviewCheckBox::drawButton(QPainter *p) {

	erase();
	QStyle::SFlags flags = QStyle::Style_Default;
	if ( isEnabled() )
		flags |= QStyle::Style_Enabled;
	if ( hasFocus() )
		flags |= QStyle::Style_HasFocus;
	if ( isDown() )
		flags |= QStyle::Style_Down;
	if ( hasMouse() )
		flags |= QStyle::Style_MouseOver;
	if ( state() == QButton::On )
		flags |= QStyle::Style_On;
	else if ( state() == QButton::Off )
		flags |= QStyle::Style_Off;
	else if ( state() == QButton::NoChange )
		flags |= QStyle::Style_NoChange;
	
	QRect irect = QStyle::visualRect( style().subRect(QStyle::SR_CheckBoxIndicator, this), this );
	style().drawControl(QStyle::CE_CheckBox, p, this, irect, colorGroup(), flags);
	drawButtonLabel(p);
}

void PreviewRadioButton::paintEvent(QPaintEvent* ) {

	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface checkItemSurface_save = dStyle->checkItemSurface;
	DominoStyle::DSurface previewSurface = checkItemSurface_save;
	previewSurface.numGradients = checkItemSurface.numGradients;
	previewSurface.g1Color1 = checkItemSurface.g1Color1;
	previewSurface.g1Color2 = checkItemSurface.g1Color2;
	previewSurface.g2Color1 = checkItemSurface.g2Color1;
	previewSurface.g2Color2 = checkItemSurface.g2Color2;
	previewSurface.background = checkItemSurface.background;
	previewSurface.g1Top = checkItemSurface.g1Top;
	previewSurface.g1Bottom = checkItemSurface.g1Bottom;
	previewSurface.g2Top = checkItemSurface.g2Top;
	previewSurface.g2Bottom = checkItemSurface.g2Bottom;
	
	dStyle->setCheckItemSurface(previewSurface);
	QPainter p(this);
	drawButton(&p);
	dStyle->setCheckItemSurface(checkItemSurface_save);
}

void PreviewRadioButton::drawButton(QPainter *p) {
	
	erase();
	QStyle::SFlags flags = QStyle::Style_Default;
	if ( isEnabled() )
		flags |= QStyle::Style_Enabled;
	if ( hasFocus() )
		flags |= QStyle::Style_HasFocus;
	if ( isDown() )
		flags |= QStyle::Style_Down;
	if ( hasMouse() )
		flags |= QStyle::Style_MouseOver;
	if ( state() == QButton::On )
		flags |= QStyle::Style_On;
	else if ( state() == QButton::Off )
		flags |= QStyle::Style_Off;
	
	QRect irect = QStyle::visualRect( style().subRect(QStyle::SR_RadioButtonIndicator, this), this );
	irect.addCoords(0,-1,0,0);
	style().drawControl(QStyle::CE_RadioButton, p, this, irect, colorGroup(), flags);
	drawButtonLabel( p );
}

void PreviewScrollBar::drawSbSurface(QEvent * event) {
	
	DominoStyle* dStyle = (DominoStyle*)&style();
	dStyle->removeCachedSbPix(colorGroup().button());
	
	DominoStyle::DSurface scrollBarSurface_save = dStyle->scrollBarSurface;
	DominoStyle::DSurface scrollBarGrooveSurface_save = dStyle->scrollBarGrooveSurface;
	
	DominoStyle::DSurface previewSurface = scrollBarSurface_save;
	previewSurface.numGradients = scrollBarSurface.numGradients;
	previewSurface.g1Color1 = scrollBarSurface.g1Color1;
	previewSurface.g1Color2 = scrollBarSurface.g1Color2;
	previewSurface.g2Color1 = scrollBarSurface.g2Color1;
	previewSurface.g2Color2 = scrollBarSurface.g2Color2;
	previewSurface.background = scrollBarSurface.background;
	previewSurface.g1Top = scrollBarSurface.g1Top;
	previewSurface.g1Bottom = scrollBarSurface.g1Bottom;
	previewSurface.g2Top = scrollBarSurface.g2Top;
	previewSurface.g2Bottom = scrollBarSurface.g2Bottom;

	DominoStyle::DSurface previewGrooveSurface = scrollBarGrooveSurface_save;
	previewGrooveSurface.numGradients = scrollBarGrooveSurface.numGradients;
	previewGrooveSurface.g1Color1 = scrollBarGrooveSurface.g1Color1;
	previewGrooveSurface.g1Color2 = scrollBarGrooveSurface.g1Color2;
	previewGrooveSurface.g2Color1 = scrollBarGrooveSurface.g2Color1;
	previewGrooveSurface.g2Color2 = scrollBarGrooveSurface.g2Color2;
	previewGrooveSurface.background = scrollBarGrooveSurface.background;
	previewGrooveSurface.g1Top = scrollBarGrooveSurface.g1Top;
	previewGrooveSurface.g1Bottom = scrollBarGrooveSurface.g1Bottom;
	previewGrooveSurface.g2Top = scrollBarGrooveSurface.g2Top;
	previewGrooveSurface.g2Bottom = scrollBarGrooveSurface.g2Bottom;
	
	QPixmap* scrollBarCap_top_save = dStyle->scrollBarCap_top;
	QPixmap* scrollBarCap_bottom_save = dStyle->scrollBarCap_bottom;
	QPixmap* scrollBarCap_left_save = dStyle->scrollBarCap_left;
	QPixmap* scrollBarCap_right_save = dStyle->scrollBarCap_right;


	dStyle->setScrollBarSurface(previewSurface);
	dStyle->setScrollBarGrooveSurface(previewGrooveSurface);
	dStyle->renderScrollBarCaps(scrollBarCapsColor);
	
	switch (event->type()) {
		case QEvent::Paint:
			QScrollBar::paintEvent((QPaintEvent*)event);
			break;
		case QEvent::MouseButtonPress:
			QScrollBar::mousePressEvent((QMouseEvent*)event);
			break;
		case QEvent::MouseButtonRelease:
			QScrollBar::mouseReleaseEvent((QMouseEvent*)event);
			break;
		case QEvent::MouseMove:
			QScrollBar::mouseMoveEvent((QMouseEvent*)event);
			break;
		case QEvent::Wheel:
			QScrollBar::wheelEvent((QWheelEvent*)event);
			break;
		default:
			break;
	}
	dStyle->setScrollBarSurface(scrollBarSurface_save);
	dStyle->setScrollBarGrooveSurface(scrollBarGrooveSurface_save);
	
	delete dStyle->scrollBarCap_top;
	dStyle->scrollBarCap_top = new QPixmap(*scrollBarCap_top_save);
	delete dStyle->scrollBarCap_bottom;
	dStyle->scrollBarCap_bottom = new QPixmap(*scrollBarCap_bottom_save);
	delete dStyle->scrollBarCap_left;
	dStyle->scrollBarCap_left = new QPixmap(*scrollBarCap_left_save);
	delete dStyle->scrollBarCap_right;
	dStyle->scrollBarCap_right = new QPixmap(*scrollBarCap_right_save);
	
}

void PreviewTabBar::paintEvent(QPaintEvent* event) {
	
	DominoStyle* dStyle = (DominoStyle*)&style();
	DominoStyle::DSurface tabTopSurface_save = dStyle->tabTopSurface;
	DominoStyle::DSurface activeTabTopSurface_save = dStyle->activeTabTopSurface;
	DominoStyle::DSurface tabBottomSurface_save = dStyle->tabBottomSurface;
	DominoStyle::DSurface activeTabBottomSurface_save = dStyle->activeTabBottomSurface;
	
	DominoStyle::DSurface previewTabTopSurface;
	previewTabTopSurface.numGradients = tabTopSurface.numGradients;
	previewTabTopSurface.g1Color1 = tabTopSurface.g1Color1;
	previewTabTopSurface.g1Color2 = tabTopSurface.g1Color2;
	previewTabTopSurface.g2Color1 = tabTopSurface.g2Color1;
	previewTabTopSurface.g2Color2 = tabTopSurface.g2Color2;
	previewTabTopSurface.background = tabTopSurface.background;
	previewTabTopSurface.g1Top = tabTopSurface.g1Top;
	previewTabTopSurface.g1Bottom = tabTopSurface.g1Bottom;
	previewTabTopSurface.g2Top = tabTopSurface.g2Top;
	previewTabTopSurface.g2Bottom = tabTopSurface.g2Bottom;
	
	DominoStyle::DSurface previewActiveTabTopSurface;
	previewActiveTabTopSurface.numGradients = activeTabTopSurface.numGradients;
	previewActiveTabTopSurface.g1Color1 = activeTabTopSurface.g1Color1;
	previewActiveTabTopSurface.g1Color2 = activeTabTopSurface.g1Color2;
	previewActiveTabTopSurface.g2Color1 = activeTabTopSurface.g2Color1;
	previewActiveTabTopSurface.g2Color2 = activeTabTopSurface.g2Color2;
	previewActiveTabTopSurface.background = activeTabTopSurface.background;
	previewActiveTabTopSurface.g1Top = activeTabTopSurface.g1Top;
	previewActiveTabTopSurface.g1Bottom = activeTabTopSurface.g1Bottom;
	previewActiveTabTopSurface.g2Top = activeTabTopSurface.g2Top;
	previewActiveTabTopSurface.g2Bottom = activeTabTopSurface.g2Bottom;
	
	DominoStyle::DSurface previewTabBottomSurface;
	previewTabBottomSurface.numGradients = tabBottomSurface.numGradients;
	previewTabBottomSurface.g1Color1 = tabBottomSurface.g1Color1;
	previewTabBottomSurface.g1Color2 = tabBottomSurface.g1Color2;
	previewTabBottomSurface.g2Color1 = tabBottomSurface.g2Color1;
	previewTabBottomSurface.g2Color2 = tabBottomSurface.g2Color2;
	previewTabBottomSurface.background = tabBottomSurface.background;
	previewTabBottomSurface.g1Top = tabBottomSurface.g1Top;
	previewTabBottomSurface.g1Bottom = tabBottomSurface.g1Bottom;
	previewTabBottomSurface.g2Top = tabBottomSurface.g2Top;
	previewTabBottomSurface.g2Bottom = tabBottomSurface.g2Bottom;
	
	DominoStyle::DSurface previewActiveTabBottomSurface;
	previewActiveTabBottomSurface.numGradients = activeTabBottomSurface.numGradients;
	previewActiveTabBottomSurface.g1Color1 = activeTabBottomSurface.g1Color1;
	previewActiveTabBottomSurface.g1Color2 = activeTabBottomSurface.g1Color2;
	previewActiveTabBottomSurface.g2Color1 = activeTabBottomSurface.g2Color1;
	previewActiveTabBottomSurface.g2Color2 = activeTabBottomSurface.g2Color2;
	previewActiveTabBottomSurface.background = activeTabBottomSurface.background;
	previewActiveTabBottomSurface.g1Top = activeTabBottomSurface.g1Top;
	previewActiveTabBottomSurface.g1Bottom = activeTabBottomSurface.g1Bottom;
	previewActiveTabBottomSurface.g2Top = activeTabBottomSurface.g2Top;
	previewActiveTabBottomSurface.g2Bottom = activeTabBottomSurface.g2Bottom;
	
	
	dStyle->setTabTopSurface(previewTabTopSurface);
	dStyle->setActiveTabTopSurface(previewActiveTabTopSurface);
	dStyle->setTabBottomSurface(previewTabBottomSurface);
	dStyle->setActiveTabBottomSurface(previewActiveTabBottomSurface);
	
	QTabBar::paintEvent(event);
	
	dStyle->setTabTopSurface(tabTopSurface_save);
	dStyle->setActiveTabTopSurface(activeTabTopSurface_save);
	dStyle->setTabBottomSurface(tabBottomSurface_save);
	dStyle->setActiveTabBottomSurface(activeTabBottomSurface_save);
}

void IndicatorPrevButton::paintEvent(QPaintEvent* /*event*/)
{

	QStyle::SFlags flags = QStyle::Style_Default;
	if (isEnabled())
		flags |= QStyle::Style_Enabled;
	if (isDown())
		flags |= QStyle::Style_Down;
	if (isOn())
		flags |= QStyle::Style_On;
	if (! isFlat() && ! isDown())
		flags |= QStyle::Style_Raised;
	if (isDefault())
		flags |= QStyle::Style_ButtonDefault;
	
	QPainter p(this);
	style().drawControl(QStyle::CE_PushButton, &p, this, rect(), colorGroup(), flags);
	
	if(isEnabled()) {
		QColor color = indicatorBtnColor->color();
		DominoStyle* dStyle = (DominoStyle*)&style();
		dStyle->drawFocusIndicator(&p, style().subRect(QStyle::SR_PushButtonFocusRect, this),  AlignHCenter| AlignVCenter |ShowPrefix, colorGroup(), true, 0, indicatorPreviewText, -1, &color, drawUnderline->isChecked());
		QColor fg = colorGroup().buttonText();
		
		style().drawItem(&p, rect(),  AlignHCenter| AlignVCenter |ShowPrefix, colorGroup(), true, 0, indicatorPreviewText, -1, &fg);
	}
}

void IndicatorPrevWidget::paintEvent(QPaintEvent* /*event*/)
{
	QPainter p(this);
	
	if(isEnabled()) {
		QColor color = indicatorColor->color();
		DominoStyle* dStyle = (DominoStyle*)&style();
		dStyle->drawFocusIndicator(&p, rect(),  AlignHCenter| AlignVCenter |ShowPrefix, colorGroup(), true, 0, indicatorPreviewText, -1, &color, drawUnderline->isChecked());
		QColor fg = colorGroup().foreground();
		style().drawItem(&p, style().subRect(QStyle::SR_PushButtonFocusRect, this),  AlignHCenter| AlignVCenter |ShowPrefix, colorGroup(), true, 0, indicatorPreviewText, -1, &fg);
	}
}

void DominoStyleConfig::languageChange()
{
	
	/////////////////////////////////////////////////
	// tab 1 (misc)
	/////////////////////////////////////////////////
	buttonHeightAdjustmentLabel->setText(tr("Button height adjustment:"));
	indicateFocus->setTitle(tr("Indicate focus"));
	labelIndicatorBtnColor->setText(tr("Indicator button color:"));
	labelIndicatorColor->setText(tr("Indicator color:"));
	indicatorPreviewText = tr("focused");
	int width = indicatorPrevButton->fontMetrics().width(indicatorPreviewText);
	indicatorPrevButton->setMinimumWidth(width);
	indicatorPrevButton->setMinimumWidth(width);
	
	/////////////////////////////////////////////////
	// tab 2.1 (buttons)
	/////////////////////////////////////////////////
	textLabel1_2->setText( tr( "Background Color:" ) );
	kColorButton27_2->setText( QString::null );
	groupBox10_2_2_2->setTitle( tr( "Preview" ) );
   // pushButton16->setText( tr( "PushButton" ) );
	groupBox8_2_2_2->setTitle( tr( "First Gradient" ) );
	textLabel4_3_2_2_2->setText( tr( "ends at:" ) );
	spinBox2_2_2_2_2->setSuffix( tr( " %" ) );
	textLabel2_2_2_2_2->setText( tr( "with color:" ) );
	kColorButton1_2_2_2_2->setText( QString::null );
	textLabel3_3_2_2_2->setText( tr( "starts at:" ) );
	spinBox1_2_2_2_2->setSuffix( tr( " %" ) );
	spinBox1_2_2_2_2->setSpecialValueText( QString::null );
	textLabel1_4_2_2_2->setText( tr( "with color:" ) );
	kColorButton2_4_2_2_2->setText( QString::null );
	groupBox9_2_2_2->setTitle( tr( "Second Gradient" ) );
	textLabel4_4_2_2_2->setText( tr( "ends at:" ) );
	spinBox2_3_2_2_2->setSuffix( tr( " %" ) );
	textLabel3_4_2_2_2->setText( tr( "starts at:" ) );
	spinBox1_3_2_2_2->setSuffix( tr( " %" ) );
	spinBox1_3_2_2_2->setSpecialValueText( QString::null );
	kColorButton1_3_2_2_2->setText( QString::null );
	kColorButton2_5_2_2_2->setText( QString::null );
	textLabel1_5_2_2_2->setText( tr( "with color:" ) );
	textLabel2_3_2_2_2->setText( tr( "with color:" ) );
	textLabel7_4->setText( tr( "Copy from:" ) );
	comboBox3_4->clear();
	comboBox3_4->insertItem( tr( "scrollbars" ) );
	comboBox3_4->insertItem( tr( "headers" ) );
	comboBox3_4->insertItem( tr( "radio/checkboxes" ) );
	pushButton4_4->setText( tr( "OK" ) );
	tabWidget2->changeTab( tab1, tr( "Buttons" ) );
	
	/////////////////////////////////////////////////
	// tab 2.2 (tabs)
	/////////////////////////////////////////////////
//	  groupBox22_2->setTitle( tr( "groupBox22" ) );
	   groupBox8_2_2_2_2->setTitle( tr( "First Gradient" ) );
	   textLabel4_3_2_2_2_2->setText( tr( "ends at:" ) );
	   spinBox2_2_2_2_2_2->setSuffix( tr( " %" ) );
	   textLabel2_2_2_2_2_2->setText( tr( "with color:" ) );
	   kColorButton1_2_2_2_2_2->setText( QString::null );
	   textLabel3_3_2_2_2_2->setText( tr( "starts at:" ) );
	   spinBox1_2_2_2_2_2->setSuffix( tr( " %" ) );
	   spinBox1_2_2_2_2_2->setSpecialValueText( QString::null );
	   textLabel1_4_2_2_2_2->setText( tr( "with color:" ) );
	   kColorButton2_4_2_2_2_2->setText( QString::null );
	   groupBox9_2_2_2_2->setTitle( tr( "Second Gradient" ) );
	   textLabel4_4_2_2_2_2->setText( tr( "ends at:" ) );
	   spinBox2_3_2_2_2_2->setSuffix( tr( " %" ) );
	   textLabel3_4_2_2_2_2->setText( tr( "starts at:" ) );
	   spinBox1_3_2_2_2_2->setSuffix( tr( " %" ) );
	   spinBox1_3_2_2_2_2->setSpecialValueText( QString::null );
	   kColorButton1_3_2_2_2_2->setText( QString::null );
	   kColorButton2_5_2_2_2_2->setText( QString::null );
	   textLabel1_5_2_2_2_2->setText( tr( "with color:" ) );
	   textLabel2_3_2_2_2_2->setText( tr( "with color:" ) );
	   textLabel1_2_2->setText( tr( "Background Color:" ) );
	   kColorButton27_2_2->setText( QString::null );
	   groupBox10_2_2_2_2->setTitle( tr( "Preview" ) );
	   textLabel7_4_2->setText( tr( "Copy from:" ) );
	   comboBox3_4_2->clear();
	   comboBox3_4_2->insertItem( tr( "top active tab" ) );
	   comboBox3_4_2->insertItem( tr( "top inactive tab" ) );
	   comboBox3_4_2->insertItem( tr( "bottom active tab" ) );
	   comboBox3_4_2->insertItem( tr( "bottom inactive tab" ) );
	   comboBox3_4_2->insertItem( tr( "buttons" ) );
	   comboBox3_4_2->insertItem( tr( "scrollbars" ) );
	   comboBox3_4_2->insertItem( tr( "headers" ) );
	   comboBox3_4_2->insertItem( tr( "radio/checkboxes" ) );
	   tabPosCombo->insertItem( tr( "top" ) );
	   tabPosCombo->insertItem( tr( "bottom" ) );
	   tabStateCombo->insertItem( tr( "active" ) );
	   tabStateCombo->insertItem( tr( "inactive" ) );
	   pushButton4_4_2->setText( tr( "OK" ) );
	   tabWidget2->changeTab( tab2, tr( "Tabs" ) );
	
	/////////////////////////////////////////////////
	// tab 2.3 (scrollBars)
	/////////////////////////////////////////////////
//	   groupBox22_2_2->setTitle( tr( "groupBox22" ) );
	groupBox8_2_2_2_2_4->setTitle( tr( "First Gradient" ) );
	textLabel4_3_2_2_2_2_4->setText( tr( "ends at:" ) );
	spinBox2_2_2_2_2_2_4->setSuffix( tr( " %" ) );
	textLabel2_2_2_2_2_2_4->setText( tr( "with color:" ) );
	kColorButton1_2_2_2_2_2_4->setText( QString::null );
	textLabel3_3_2_2_2_2_4->setText( tr( "starts at:" ) );
	spinBox1_2_2_2_2_2_4->setSuffix( tr( " %" ) );
	spinBox1_2_2_2_2_2_4->setSpecialValueText( QString::null );
	textLabel1_4_2_2_2_2_4->setText( tr( "with color:" ) );
	kColorButton2_4_2_2_2_2_4->setText( QString::null );
	groupBox9_2_2_2_2_4->setTitle( tr( "Second Gradient" ) );
	textLabel4_4_2_2_2_2_4->setText( tr( "ends at:" ) );
	spinBox2_3_2_2_2_2_4->setSuffix( tr( " %" ) );
	textLabel3_4_2_2_2_2_4->setText( tr( "starts at:" ) );
	spinBox1_3_2_2_2_2_4->setSuffix( tr( " %" ) );
	spinBox1_3_2_2_2_2_4->setSpecialValueText( QString::null );
	kColorButton1_3_2_2_2_2_4->setText( QString::null );
	kColorButton2_5_2_2_2_2_4->setText( QString::null );
	textLabel1_5_2_2_2_2_4->setText( tr( "with color:" ) );
	textLabel2_3_2_2_2_2_4->setText( tr( "with color:" ) );
	groupBox10_2_2_2_2_4->setTitle( tr( "Preview" ) );
	kColorButton27_2_2_2->setText( QString::null );
	textLabel1_2_2_2->setText( tr( "Background Color:" ) );
	comboBox7->clear();
	comboBox7->insertItem( tr( "Buttons/Slider" ) );
	comboBox7->insertItem( tr( "Groove" ) );
	textLabel7_4_2_3->setText( tr( "Copy from:" ) );
	comboBox3_4_2_3->clear();
	comboBox3_4_2_3->insertItem( tr( "buttons" ) );
	comboBox3_4_2_3->insertItem( tr( "headers" ) );
	comboBox3_4_2_3->insertItem( tr( "radio/checkboxes" ) );
	pushButton4_4_2_3->setText( tr( "OK" ) );
	tabWidget2->changeTab( tab3, tr( "ScrollBars" ) );
	
	/////////////////////////////////////////////////
	// tab 2.4 (headers)
	/////////////////////////////////////////////////
//	   groupBox22_2_2_3->setTitle( tr( "groupBox22" ) );
	kColorButton27_2_2_2_2->setText( QString::null );
	textLabel1_2_2_2_2->setText( tr( "Background Color:" ) );
	groupBox10_2_2_2_2_4_3->setTitle( tr( "Preview" ) );
	groupBox8_2_2_2_2_4_3->setTitle( tr( "First Gradient" ) );
	textLabel4_3_2_2_2_2_4_3->setText( tr( "ends at:" ) );
	spinBox2_2_2_2_2_2_4_3->setSuffix( tr( " %" ) );
	textLabel2_2_2_2_2_2_4_3->setText( tr( "with color:" ) );
	kColorButton1_2_2_2_2_2_4_3->setText( QString::null );
	textLabel3_3_2_2_2_2_4_3->setText( tr( "starts at:" ) );
	spinBox1_2_2_2_2_2_4_3->setSuffix( tr( " %" ) );
	spinBox1_2_2_2_2_2_4_3->setSpecialValueText( QString::null );
	textLabel1_4_2_2_2_2_4_3->setText( tr( "with color:" ) );
	kColorButton2_4_2_2_2_2_4_3->setText( QString::null );
	groupBox9_2_2_2_2_4_3->setTitle( tr( "Second Gradient" ) );
	textLabel4_4_2_2_2_2_4_3->setText( tr( "ends at:" ) );
	spinBox2_3_2_2_2_2_4_3->setSuffix( tr( " %" ) );
	textLabel3_4_2_2_2_2_4_3->setText( tr( "starts at:" ) );
	spinBox1_3_2_2_2_2_4_3->setSuffix( tr( " %" ) );
	spinBox1_3_2_2_2_2_4_3->setSpecialValueText( QString::null );
	kColorButton1_3_2_2_2_2_4_3->setText( QString::null );
	kColorButton2_5_2_2_2_2_4_3->setText( QString::null );
	textLabel1_5_2_2_2_2_4_3->setText( tr( "with color:" ) );
	textLabel2_3_2_2_2_2_4_3->setText( tr( "with color:" ) );
	textLabel7_4_2_3_3->setText( tr( "Copy from:" ) );
	comboBox3_4_2_3_3->clear();
	comboBox3_4_2_3_3->insertItem( tr( "buttons" ) );
	comboBox3_4_2_3_3->insertItem( tr( "scrollbars" ) );
	comboBox3_4_2_3_3->insertItem( tr( "radio/checkboxes" ) );
	pushButton4_4_2_3_3->setText( tr( "OK" ) );
	tabWidget2->changeTab( tab4, tr( "Headers" ) );
	
	/////////////////////////////////////////////////
	// tab 2.5 (radio/checkboxes)
	/////////////////////////////////////////////////
//	   groupBox22_2_2_3_2->setTitle( tr( "groupBox22" ) );
	groupBox8_2_2_2_2_4_3_2->setTitle( tr( "First Gradient" ) );
	textLabel4_3_2_2_2_2_4_3_2->setText( tr( "ends at:" ) );
	spinBox2_2_2_2_2_2_4_3_2->setSuffix( tr( " %" ) );
	textLabel2_2_2_2_2_2_4_3_2->setText( tr( "with color:" ) );
	kColorButton1_2_2_2_2_2_4_3_2->setText( QString::null );
	textLabel3_3_2_2_2_2_4_3_2->setText( tr( "starts at:" ) );
	spinBox1_2_2_2_2_2_4_3_2->setSuffix( tr( " %" ) );
	spinBox1_2_2_2_2_2_4_3_2->setSpecialValueText( QString::null );
	textLabel1_4_2_2_2_2_4_3_2->setText( tr( "with color:" ) );
	kColorButton2_4_2_2_2_2_4_3_2->setText( QString::null );
	kColorButton27_2_2_2_3->setText( QString::null );
	textLabel1_2_2_2_3->setText( tr( "Background Color:" ) );
	groupBox10_2_2_2_2_4_3_2->setTitle( tr( "Preview" ) );
	radioButton2->setText( tr( "RadioButton" ) );
	checkBox2->setText( tr( "CheckBox" ) );
	groupBox9_2_2_2_2_4_3_2->setTitle( tr( "Second Gradient" ) );
	textLabel4_4_2_2_2_2_4_3_2->setText( tr( "ends at:" ) );
	spinBox2_3_2_2_2_2_4_3_2->setSuffix( tr( " %" ) );
	textLabel3_4_2_2_2_2_4_3_2->setText( tr( "starts at:" ) );
	spinBox1_3_2_2_2_2_4_3_2->setSuffix( tr( " %" ) );
	spinBox1_3_2_2_2_2_4_3_2->setSpecialValueText( QString::null );
	kColorButton1_3_2_2_2_2_4_3_2->setText( QString::null );
	kColorButton2_5_2_2_2_2_4_3_2->setText( QString::null );
	textLabel1_5_2_2_2_2_4_3_2->setText( tr( "with color:" ) );
	textLabel2_3_2_2_2_2_4_3_2->setText( tr( "with color:" ) );
	textLabel7_4_2_3_3_2->setText( tr( "Copy from:" ) );
	comboBox3_4_2_3_3_2->clear();
	comboBox3_4_2_3_3_2->insertItem( tr( "buttons" ) );
	comboBox3_4_2_3_3_2->insertItem( tr( "scrollbars" ) );
	comboBox3_4_2_3_3_2->insertItem( tr( "headers" ) );
	pushButton4_4_2_3_3_2->setText( tr( "OK" ) );
	
	tabWidget2->changeTab( tab5, tr( "Radio/Checkboxes" ) );
	capsLabel->setText( tr( "Caps Color:" ) );
	
	/////////////////////////////////////////////////
	// tab 3 (load/save)
	/////////////////////////////////////////////////
	groupBox_ls->setTitle( QString::null );
	listView_ls->header()->setLabel( 0, tr( "Name" ) );
	loadButton->setText( tr( "Load Config" ) );
	saveButton->setText( tr( "Save Config" ) );
	deleteButton->setText( tr( "Delete Config" ) );
	


}


DominoStyleConfig::~DominoStyleConfig()
{
	delete schemeList;
}


void DominoStyleConfig::save()
{
	bool settingsChanged;
	if(
		  (centerTabs->isChecked() == origCenterTabs) &&
//		  (animateProgressBar->isChecked() == origAnimProgressBar) &&
		  (drawTriangularExpander->isChecked() == origDrawTriangularExpander) &&
		  (customCheckMarkColor->isChecked() == origCustomCheckMarkColor) &&
		  (checkMarkColor->color() == origCheckMarkColor) &&
		  (tintGroupBoxBackground->isChecked() == origTintGroupBoxBackground) &&
		  (drawGroupBoxDark->isChecked() == origDrawGroupBoxDark) &&
		  (drawGroupBoxBright->isChecked() == origDrawGroupBoxBright) &&
		  (customToolTipColor->isChecked() == origCustomToolTipColor) &&
		  (toolTipColor->color() == origToolTipColor) &&
		  (smoothScrolling->isChecked() == origSmoothScrolling) &&
		  (buttonHeightAdjustment->value() == origButtonHeightAdjustment) &&
		  (customPopupMenuColor->isChecked() == origCustomPopupMenuColor) &&
		  (popupMenuColor->color() == origPopupMenuColor) &&
		  (indicateFocus->isChecked() == origIndicateFocus) &&
		  (indicatorColor->color() == origIndicatorColor) &&
		  (indicatorBtnColor->color() == origIndicatorBtnColor) &&
		  (drawUnderline->isChecked() == origDrawUnderline) &&
		   
		  (kColorButton2_4_2_2_2->color() == origBtnSurface.g1Color1) &&
		  (kColorButton1_2_2_2_2->color() == origBtnSurface.g1Color2) &&
		  (kColorButton2_5_2_2_2->color() == origBtnSurface.g2Color1) &&
		  (kColorButton1_3_2_2_2->color() == origBtnSurface.g2Color2) &&
		  (spinBox1_2_2_2_2->value() == origBtnSurface.g1Top) &&
		  (spinBox2_2_2_2_2->value() == origBtnSurface.g1Bottom) &&
		  (spinBox1_3_2_2_2->value() == origBtnSurface.g2Top) &&
		  (spinBox2_3_2_2_2->value() == origBtnSurface.g2Bottom) &&
		  (kColorButton27_2->color() == origBtnSurface.background) &&
		  ((groupBox9_2_2_2->isChecked() ? 2 : groupBox8_2_2_2->isChecked() ? 1 : 0 ) == origBtnSurface.numGradients) &&
		  
		  (kColorButton2_4_2_2_2_2_4_3->color() == origHeaderSurface.g1Color1) &&
		  (kColorButton1_2_2_2_2_2_4_3->color() == origHeaderSurface.g1Color2) &&
		  (kColorButton2_5_2_2_2_2_4_3->color() == origHeaderSurface.g2Color1) &&
		  (kColorButton1_3_2_2_2_2_4_3->color() == origHeaderSurface.g2Color2) &&
		  (spinBox1_2_2_2_2_2_4_3->value() == origHeaderSurface.g1Top) &&
		  (spinBox2_2_2_2_2_2_4_3->value() == origHeaderSurface.g1Bottom) &&
		  (spinBox1_3_2_2_2_2_4_3->value() == origHeaderSurface.g2Top) &&
		  (spinBox2_3_2_2_2_2_4_3->value() == origHeaderSurface.g2Bottom) &&
		  (kColorButton27_2_2_2_2->color() == origHeaderSurface.background) &&
		  ((groupBox9_2_2_2_2_4_3->isChecked() ? 2 : groupBox8_2_2_2_2_4_3->isChecked() ? 1 : 0 ) == origHeaderSurface.numGradients) &&

		  (kColorButton2_4_2_2_2_2_4_3_2->color() == origCheckItemSurface.g1Color1) &&
		  (kColorButton1_2_2_2_2_2_4_3_2->color() == origCheckItemSurface.g1Color2) &&
		  (kColorButton2_5_2_2_2_2_4_3_2->color() == origCheckItemSurface.g2Color1) &&
		  (kColorButton1_3_2_2_2_2_4_3_2->color() == origCheckItemSurface.g2Color2) &&
		  (spinBox1_2_2_2_2_2_4_3_2->value() == origCheckItemSurface.g1Top) &&
		  (spinBox2_2_2_2_2_2_4_3_2->value() == origCheckItemSurface.g1Bottom) &&
		  (spinBox1_3_2_2_2_2_4_3_2->value() == origCheckItemSurface.g2Top) &&
		  (spinBox2_3_2_2_2_2_4_3_2->value() == origCheckItemSurface.g2Bottom) &&
		  (kColorButton27_2_2_2_3->color() == origCheckItemSurface.background) &&
		  ((groupBox9_2_2_2_2_4_3_2->isChecked() ? 2 : groupBox8_2_2_2_2_4_3_2->isChecked() ? 1 : 0 ) == origCheckItemSurface.numGradients) &&

		  (capsColorButton->color() == origScrollBarCapsColor) &&
		
		  (scrollBarSurface.g1Color1 == origScrollBarSurface.g1Color1) &&
		  (scrollBarSurface.g1Color2 == origScrollBarSurface.g1Color2) &&
		  (scrollBarSurface.g2Color1 == origScrollBarSurface.g2Color1) &&
		  (scrollBarSurface.g2Color2 == origScrollBarSurface.g2Color2) &&
		  (scrollBarSurface.g1Top == origScrollBarSurface.g1Top) &&
		  (scrollBarSurface.g1Bottom == origScrollBarSurface.g1Bottom) &&
		  (scrollBarSurface.g2Top == origScrollBarSurface.g2Top) &&
		  (scrollBarSurface.g2Bottom == origScrollBarSurface.g2Bottom) &&
		  (scrollBarSurface.background == origScrollBarSurface.background) &&
		  (scrollBarSurface.numGradients == origScrollBarSurface.numGradients) &&

		  (scrollBarGrooveSurface.g1Color1 == origScrollBarGrooveSurface.g1Color1) &&
		  (scrollBarGrooveSurface.g1Color2 == origScrollBarGrooveSurface.g1Color2) &&
		  (scrollBarGrooveSurface.g2Color1 == origScrollBarGrooveSurface.g2Color1) &&
		  (scrollBarGrooveSurface.g2Color2 == origScrollBarGrooveSurface.g2Color2) &&
		  (scrollBarGrooveSurface.g1Top == origScrollBarGrooveSurface.g1Top) &&
		  (scrollBarGrooveSurface.g1Bottom == origScrollBarGrooveSurface.g1Bottom) &&
		  (scrollBarGrooveSurface.g2Top == origScrollBarGrooveSurface.g2Top) &&
		  (scrollBarGrooveSurface.g2Bottom == origScrollBarGrooveSurface.g2Bottom) &&
		  (scrollBarGrooveSurface.background == origScrollBarGrooveSurface.background) &&
		  (scrollBarGrooveSurface.numGradients == origScrollBarGrooveSurface.numGradients) &&
		  
		  (activeTabTopSurface.g1Color1 == origActiveTabTopSurface.g1Color1) &&
		  (activeTabTopSurface.g1Color2 == origActiveTabTopSurface.g1Color2) &&
		  (activeTabTopSurface.g2Color1 == origActiveTabTopSurface.g2Color1) &&
		  (activeTabTopSurface.g2Color2 == origActiveTabTopSurface.g2Color2) &&
		  (activeTabTopSurface.background == origActiveTabTopSurface.background) &&
		  (activeTabTopSurface.g1Top == origActiveTabTopSurface.g1Top) &&
		  (activeTabTopSurface.g1Bottom == origActiveTabTopSurface.g1Bottom) &&
		  (activeTabTopSurface.g2Top == origActiveTabTopSurface.g2Top) &&
		  (activeTabTopSurface.g2Bottom == origActiveTabTopSurface.g2Bottom) &&
		  (activeTabTopSurface.numGradients == origActiveTabTopSurface.numGradients) &&
		  
		  (tabTopSurface.g1Color1 == origTabTopSurface.g1Color1) &&
		  (tabTopSurface.g1Color2 == origTabTopSurface.g1Color2) &&
		  (tabTopSurface.g2Color1 == origTabTopSurface.g2Color1) &&
		  (tabTopSurface.g2Color2 == origTabTopSurface.g2Color2) &&
		  (tabTopSurface.background == origTabTopSurface.background) &&
		  (tabTopSurface.g1Top == origTabTopSurface.g1Top) &&
		  (tabTopSurface.g1Bottom == origTabTopSurface.g1Bottom) &&
		  (tabTopSurface.g2Top == origTabTopSurface.g2Top) &&
		  (tabTopSurface.g2Bottom == origTabTopSurface.g2Bottom) &&
		  (tabTopSurface.numGradients == origTabTopSurface.numGradients) &&
		  
		  (activeTabBottomSurface.g1Color1 == origActiveTabBottomSurface.g1Color1) &&
		  (activeTabBottomSurface.g1Color2 == origActiveTabBottomSurface.g1Color2) &&
		  (activeTabBottomSurface.g2Color1 == origActiveTabBottomSurface.g2Color1) &&
		  (activeTabBottomSurface.g2Color2 == origActiveTabBottomSurface.g2Color2) &&
		  (activeTabBottomSurface.background == origActiveTabBottomSurface.background) &&
		  (activeTabBottomSurface.g1Top == origActiveTabBottomSurface.g1Top) &&
		  (activeTabBottomSurface.g1Bottom == origActiveTabBottomSurface.g1Bottom) &&
		  (activeTabBottomSurface.g2Top == origActiveTabBottomSurface.g2Top) &&
		  (activeTabBottomSurface.g2Bottom == origActiveTabBottomSurface.g2Bottom) &&
		  (activeTabBottomSurface.numGradients == origActiveTabBottomSurface.numGradients) &&
		  
		  (tabBottomSurface.g1Color1 == origTabBottomSurface.g1Color1) &&
		  (tabBottomSurface.g1Color2 == origTabBottomSurface.g1Color2) &&
		  (tabBottomSurface.g2Color1 == origTabBottomSurface.g2Color1) &&
		  (tabBottomSurface.g2Color2 == origTabBottomSurface.g2Color2) &&
		  (tabBottomSurface.background == origTabBottomSurface.background) &&
		  (tabBottomSurface.g1Top == origTabBottomSurface.g1Top) &&
		  (tabBottomSurface.g1Bottom == origTabBottomSurface.g1Bottom) &&
		  (tabBottomSurface.g2Top == origTabBottomSurface.g2Top) &&
		  (tabBottomSurface.g2Bottom == origTabBottomSurface.g2Bottom) &&
		  (tabBottomSurface.numGradients == origTabBottomSurface.numGradients)
	  )
		settingsChanged = false;
	else
		settingsChanged = true;

	
	if(settingsChanged) {
		if(!QString(currentConfig).endsWith(" (" + i18n("modified") + ")")) {
			currentConfig = QString(currentConfig).append(" (" + i18n("modified") + ")");
		}
	  }
	  
	  if(configLoaded)
		  settingsChanged = true;
	  if(settingsChanged)
		  emit changed(true);
	  else
		  emit changed(false);
	  
	saveConfig("domino", currentConfig);
}

void DominoStyleConfig::defaults()
{
	QColor background = qApp->palette().active().background();
	QColor button = qApp->palette().active().button();
	centerTabs->setChecked(false);
	drawTriangularExpander->setChecked(false);
	customCheckMarkColor->setChecked(false);
	checkMarkColor->setColor(QColor(0,0,0));
	tintGroupBoxBackground->setChecked(true);
	drawGroupBoxDark->setChecked(true);
	drawGroupBoxBright->setChecked(false);
	customToolTipColor->setChecked(false);
	toolTipColor->setColor(QColor(218,218,179));
	smoothScrolling->setChecked(true);
	buttonHeightAdjustment->setValue(0);
	customPopupMenuColor->setChecked(false);
	popupMenuColor->setColor(background.name());
	indicateFocus->setChecked(false);
	indicatorColor->setColor(button.light(120).name());
	indicatorBtnColor->setColor(button.light(120).name());
	drawUnderline->setChecked(true);

// buttons / comboBoxes / spinBoxes
	groupBox9_2_2_2->setChecked(true);
	kColorButton2_5_2_2_2->setColor(button.dark(120).name());
	kColorButton1_3_2_2_2->setColor(button.dark(110).name());
	spinBox1_3_2_2_2->setValue(50);
	spinBox2_3_2_2_2->setValue(100);
	groupBox8_2_2_2->setChecked(true);
	kColorButton2_4_2_2_2->setColor(button.name());
	kColorButton1_2_2_2_2->setColor(button.dark(120).name());
	spinBox1_2_2_2_2->setValue(0);
	spinBox2_2_2_2_2->setValue(50);
	kColorButton27_2->setColor(button.name());

// headers
	groupBox9_2_2_2_2_4_3->setChecked(true);
	kColorButton2_5_2_2_2_2_4_3->setColor(background.dark(120).name());
	kColorButton1_3_2_2_2_2_4_3->setColor(background.dark(110).name());
	spinBox1_3_2_2_2_2_4_3->setValue(50);
	spinBox2_3_2_2_2_2_4_3->setValue(100);
	groupBox8_2_2_2_2_4_3->setChecked(true);
	kColorButton2_4_2_2_2_2_4_3->setColor(background.name());
	kColorButton1_2_2_2_2_2_4_3->setColor(background.dark(120).name());
	spinBox1_2_2_2_2_2_4_3->setValue(0);
	spinBox2_2_2_2_2_2_4_3->setValue(50);
	kColorButton27_2_2_2_2->setColor(background.name());

// radio / checkboxes
	groupBox9_2_2_2_2_4_3_2->setChecked(true);
	kColorButton2_5_2_2_2_2_4_3_2->setColor(button.dark(120).name());
	kColorButton1_3_2_2_2_2_4_3_2->setColor(button.dark(110).name());
	spinBox1_3_2_2_2_2_4_3_2->setValue(50);
	spinBox2_3_2_2_2_2_4_3_2->setValue(100);
	groupBox8_2_2_2_2_4_3_2->setChecked(true);
	kColorButton2_4_2_2_2_2_4_3_2->setColor(button.name());
	kColorButton1_2_2_2_2_2_4_3_2->setColor(button.dark(120).name());
	spinBox1_2_2_2_2_2_4_3_2->setValue(0);
	spinBox2_2_2_2_2_2_4_3_2->setValue(50);
	kColorButton27_2_2_2_3->setColor(button.name());

// scrollBar Groove / Buttons

	scrollBarGrooveSurface.numGradients = 0;
	scrollBarGrooveSurface.g1Color1 = background.name();
	scrollBarGrooveSurface.g1Color2 = background.dark(120).name();
	scrollBarGrooveSurface.g2Color1 = background.dark(120).name();
	scrollBarGrooveSurface.g2Color2 = background.dark(110).name();
	scrollBarGrooveSurface.g1Top = 0;
	scrollBarGrooveSurface.g1Bottom = 50;
	scrollBarGrooveSurface.g2Top = 50;
	scrollBarGrooveSurface.g2Bottom = 100;
	scrollBarGrooveSurface.background = background.dark(150).name();

	scrollBarSurface.numGradients = 2;
	scrollBarSurface.g1Color1 = button.name();
	scrollBarSurface.g1Color2 = button.dark(120).name();
	scrollBarSurface.g2Color1 = button.dark(120).name();
	scrollBarSurface.g2Color2 = button.dark(110).name();
	scrollBarSurface.g1Top = 0;
	scrollBarSurface.g1Bottom = 50;
	scrollBarSurface.g2Top = 50;
	scrollBarSurface.g2Bottom = 100;
	scrollBarSurface.background = button.name();

	comboBox7->setCurrentItem(0);
	groupBox8_2_2_2_2_4->setChecked(true);
	groupBox9_2_2_2_2_4->setChecked(true);
	kColorButton2_5_2_2_2_2_4->setColor(scrollBarSurface.g2Color1);
	kColorButton1_3_2_2_2_2_4->setColor(scrollBarSurface.g2Color2);
	spinBox1_3_2_2_2_2_4->setValue(scrollBarSurface.g2Top);
	spinBox2_3_2_2_2_2_4->setValue(scrollBarSurface.g2Bottom);
	kColorButton2_4_2_2_2_2_4->setColor(scrollBarSurface.g1Color1);
	kColorButton1_2_2_2_2_2_4->setColor(scrollBarSurface.g1Color2);
	spinBox1_2_2_2_2_2_4->setValue(scrollBarSurface.g1Top);
	spinBox2_2_2_2_2_2_4->setValue(scrollBarSurface.g1Bottom);
	kColorButton27_2_2_2->setColor(scrollBarSurface.background);
	
	// tabs
	activeTabTopSurface.numGradients = 2;
	activeTabTopSurface.g1Color1 = background.light(110).name();
	activeTabTopSurface.g1Color2 = background.name();
	activeTabTopSurface.g2Color1 = background.name();
	activeTabTopSurface.g2Color2 = background.name();
	activeTabTopSurface.background = background.name();
	activeTabTopSurface.g1Top = 0;
	activeTabTopSurface.g1Bottom = 50;
	activeTabTopSurface.g2Top = 50;
	activeTabTopSurface.g2Bottom = 100;
	
	tabTopSurface.numGradients = 2;
	tabTopSurface.g1Color1 = background.light(110).name();
	tabTopSurface.g1Color2 = background.dark(110).name();
	tabTopSurface.g2Color1 = background.dark(110).name();
	tabTopSurface.g2Color2 = background.dark(110).name();
	tabTopSurface.background = background.dark(110).name();
	tabTopSurface.g1Top = 0;
	tabTopSurface.g1Bottom = 50;
	tabTopSurface.g2Top = 50;
	tabTopSurface.g2Bottom = 100;
	
	activeTabBottomSurface.numGradients = 2;
	activeTabBottomSurface.g1Color1 = background.name();
	activeTabBottomSurface.g1Color2 = background.name();
	activeTabBottomSurface.g2Color1 = background.name();
	activeTabBottomSurface.g2Color2 = background.dark(120).name();
	activeTabBottomSurface.background = background.name();
	activeTabBottomSurface.g1Top = 0;
	activeTabBottomSurface.g1Bottom = 50;
	activeTabBottomSurface.g2Top = 50;
	activeTabBottomSurface.g2Bottom = 100;
	
	tabBottomSurface.numGradients = 2;
	tabBottomSurface.g1Color1 = background.dark(110).name();
	tabBottomSurface.g1Color2 = background.dark(110).name();
	tabBottomSurface.g2Color1 = background.dark(110).name();
	tabBottomSurface.g2Color2 = background.dark(120).name();
	tabBottomSurface.background = background.dark(110).name();
	tabBottomSurface.g1Top = 0;
	tabBottomSurface.g1Bottom = 50;
	tabBottomSurface.g2Top = 50;
	tabBottomSurface.g2Bottom = 100;
	
	groupBox8_2_2_2_2->setChecked(true);
	groupBox9_2_2_2_2->setChecked(true);
	kColorButton2_4_2_2_2_2->setColor(background.light(110).name());
	kColorButton1_2_2_2_2_2->setColor(background.name());
	kColorButton2_5_2_2_2_2->setColor(background.name());
	kColorButton1_3_2_2_2_2->setColor(background.name());
	kColorButton27_2_2->setColor(background.name());
	spinBox1_2_2_2_2_2->setValue(0);
	spinBox2_2_2_2_2_2->setValue(50);
	spinBox1_3_2_2_2_2->setValue(50);
	spinBox2_3_2_2_2_2->setValue(100);
	
	tabPosCombo->setCurrentItem(0);
	tabStateCombo->setCurrentItem(0);
}

void DominoStyleConfig::updateChanged()
{
	checkMarkColor->setEnabled(customCheckMarkColor->isChecked());
	toolTipColor->setEnabled(customToolTipColor->isChecked());
	popupMenuColor->setEnabled(customPopupMenuColor->isChecked());
}

void DominoStyleConfig::copyColorsToButton() {
	
	const DSurface* ds;
	switch(comboBox3_4->currentItem()) {
		case 0:
			ds = &scrollBarSurface;
			break;
		case 1:
			ds = &headerSurface;
			break;
		case 2:
			ds = &checkItemSurface;
			break;
		default:
			return;
	}
	
	kColorButton2_4_2_2_2->setColor(ds->g1Color1);
	kColorButton1_2_2_2_2->setColor(ds->g1Color2);
	kColorButton2_5_2_2_2->setColor(ds->g2Color1);
	kColorButton1_3_2_2_2->setColor(ds->g2Color2);
	kColorButton27_2->setColor(ds->background);
	spinBox1_2_2_2_2->setValue(ds->g1Top);
	spinBox2_2_2_2_2->setValue(ds->g1Bottom);
	spinBox1_3_2_2_2->setValue(ds->g2Top);
	spinBox2_3_2_2_2->setValue(ds->g2Bottom);
	switch(ds->numGradients) {
		case 2:
			groupBox8_2_2_2->setChecked(true);
			groupBox9_2_2_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2->setChecked(true);
			groupBox9_2_2_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2->setChecked(false);
			groupBox9_2_2_2->setChecked(false);
	}
	btnNumGradientsChanged();
}

void DominoStyleConfig::copyColorsToTab() {
	
	const DSurface* ds;
	switch(comboBox3_4_2->currentItem()) {
		case 0:
			ds = &activeTabTopSurface;
			break;
		case 1:
			ds = &tabTopSurface;
			break;
		case 2:
			ds = &activeTabBottomSurface;
			break;
		case 3:
			ds = &tabBottomSurface;
			break;
		case 4:
			ds = &btnSurface;
			break;
		case 5:
			ds = &scrollBarSurface;
			break;
		case 6:
			ds = &headerSurface;
			break;
		case 7:
			ds = &checkItemSurface;
			break;
		default:
			return;
	}
	
	kColorButton2_4_2_2_2_2->setColor(ds->g1Color1);
	kColorButton1_2_2_2_2_2->setColor(ds->g1Color2);
	kColorButton2_5_2_2_2_2->setColor(ds->g2Color1);
	kColorButton1_3_2_2_2_2->setColor(ds->g2Color2);
	kColorButton27_2_2->setColor(ds->background);
	spinBox1_2_2_2_2_2->setValue(ds->g1Top);
	spinBox2_2_2_2_2_2->setValue(ds->g1Bottom);
	spinBox1_3_2_2_2_2->setValue(ds->g2Top);
	spinBox2_3_2_2_2_2->setValue(ds->g2Bottom);
	switch(ds->numGradients) {
		case 2:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2->setChecked(true);
			groupBox9_2_2_2_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2->setChecked(false);
			groupBox9_2_2_2_2->setChecked(false);
	}
	tabNumGradientsChanged();
}

void DominoStyleConfig::copyColorsToScrollBar() {
	
	const DSurface* ds;
	switch(comboBox3_4_2_3->currentItem()) {
		case 0:
			ds = &btnSurface;
			break;
		case 1:
			ds = &headerSurface;
			break;
		case 2:
			ds = &checkItemSurface;
			break;
		default:
			return;
	}
	
	kColorButton2_4_2_2_2_2_4->setColor(ds->g1Color1);
	kColorButton1_2_2_2_2_2_4->setColor(ds->g1Color2);
	kColorButton2_5_2_2_2_2_4->setColor(ds->g2Color1);
	kColorButton1_3_2_2_2_2_4->setColor(ds->g2Color2);
	kColorButton27_2_2_2->setColor(ds->background);
	spinBox1_2_2_2_2_2_4->setValue(ds->g1Top);
	spinBox2_2_2_2_2_2_4->setValue(ds->g1Bottom);
	spinBox1_3_2_2_2_2_4->setValue(ds->g2Top);
	spinBox2_3_2_2_2_2_4->setValue(ds->g2Bottom);
	switch(ds->numGradients) {
		case 2:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4->setChecked(true);
			groupBox9_2_2_2_2_4->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4->setChecked(false);
			groupBox9_2_2_2_2_4->setChecked(false);
	}
	scrollBarNumGradientsChanged();
}

void DominoStyleConfig::copyColorsToHeader() {
	
	const DSurface* ds;
	switch(comboBox3_4_2_3_3->currentItem()) {
		case 0:
			ds = &btnSurface;
			break;
		case 1:
			ds = &scrollBarSurface;
			break;
		case 2:
			ds = &checkItemSurface;
			break;
		default:
			return;
	}
	
	kColorButton2_4_2_2_2_2_4_3->setColor(ds->g1Color1);
	kColorButton1_2_2_2_2_2_4_3->setColor(ds->g1Color2);
	kColorButton2_5_2_2_2_2_4_3->setColor(ds->g2Color1);
	kColorButton1_3_2_2_2_2_4_3->setColor(ds->g2Color2);
	kColorButton27_2_2_2_2->setColor(ds->background);
	spinBox1_2_2_2_2_2_4_3->setValue(ds->g1Top);
	spinBox2_2_2_2_2_2_4_3->setValue(ds->g1Bottom);
	spinBox1_3_2_2_2_2_4_3->setValue(ds->g2Top);
	spinBox2_3_2_2_2_2_4_3->setValue(ds->g2Bottom);
	switch(ds->numGradients) {
		case 2:
			groupBox8_2_2_2_2_4_3->setChecked(true);
			groupBox9_2_2_2_2_4_3->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4_3->setChecked(true);
			groupBox9_2_2_2_2_4_3->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4_3->setChecked(false);
			groupBox9_2_2_2_2_4_3->setChecked(false);
	}
	headerNumGradientsChanged();
}

void DominoStyleConfig::copyColorsToCheckItem() {
	
	const DSurface* ds;
	switch(comboBox3_4_2_3_3_2->currentItem()) {
		case 0:
			ds = &btnSurface;
			break;
		case 1:
			ds = &scrollBarSurface;
			break;
		case 2:
			ds = &headerSurface;
			break;
		default:
			return;
	}
	
	kColorButton2_4_2_2_2_2_4_3_2->setColor(ds->g1Color1);
	kColorButton1_2_2_2_2_2_4_3_2->setColor(ds->g1Color2);
	kColorButton2_5_2_2_2_2_4_3_2->setColor(ds->g2Color1);
	kColorButton1_3_2_2_2_2_4_3_2->setColor(ds->g2Color2);
	kColorButton27_2_2_2_3->setColor(ds->background);
	spinBox1_2_2_2_2_2_4_3_2->setValue(ds->g1Top);
	spinBox2_2_2_2_2_2_4_3_2->setValue(ds->g1Bottom);
	spinBox1_3_2_2_2_2_4_3_2->setValue(ds->g2Top);
	spinBox2_3_2_2_2_2_4_3_2->setValue(ds->g2Bottom);
	switch(ds->numGradients) {
		case 2:
			groupBox8_2_2_2_2_4_3_2->setChecked(true);
			groupBox9_2_2_2_2_4_3_2->setChecked(true);
			break;
		case 1:
			groupBox8_2_2_2_2_4_3_2->setChecked(true);
			groupBox9_2_2_2_2_4_3_2->setChecked(false);
			break;
		default:
			groupBox8_2_2_2_2_4_3_2->setChecked(false);
			groupBox9_2_2_2_2_4_3_2->setChecked(false);
	}
	checkItemNumGradientsChanged();
}

void DominoStyleConfig::slotLoad() {
	
	QString sName;
	if(listView_ls->selectedItem()) {
		sName = listView_ls->currentItem()->text(0);
	}
	else
		return;

	SchemeEntry * entry;
	QString fName;
	for (entry = schemeList->first(); entry; entry = schemeList->next()) {
		if(!strcmp(entry->getSchemeName(), sName)) {
			fName = entry->getFileName();
			break;
		}
	}

	loadConfig(fName.left(fName.length()-2));
	configLoaded = true;
}

void DominoStyleConfig::slotSave() {
	
	bool ok;
	QString cName;
	if(listView_ls->selectedItem()) {
		cName = listView_ls->currentItem()->text(0);
	}
	
	cName = KInputDialog::getText(i18n("Save Config Scheme"), i18n("Enter a name for the config scheme:"), cName, &ok, this);
	if (!ok)
		return;
	
	if (!listView_ls->findItem(cName, 0)) {
		new QListViewItem(listView_ls, cName);
		schemeList->append( new SchemeEntry("domino_" + cName.simplifyWhiteSpace() + "rc", cName, true ));
	}
	else {
		int result = KMessageBox::warningContinueCancel(0,	i18n("A config scheme with the name '%1' already exists.\n"
				"Do you want to overwrite it?\n").arg(cName),
				i18n("Save Config Scheme"), i18n("Overwrite"));
		if (result == KMessageBox::Cancel)
			return;
	}
	
	cName = cName.simplifyWhiteSpace();
	saveConfig(cName, cName);
	currentConfig = cName;
	currentConfigLabel->setText(i18n("Currently loaded config: ")+currentConfig);

}

void DominoStyleConfig::slotDelete() {
	
	
	QString sName;
	if(listView_ls->selectedItem())
		sName = listView_ls->currentItem()->text(0);
	else
		return;
	
	SchemeEntry * entry;
	QString fName;
	for (entry = schemeList->first(); entry; entry = schemeList->next()) {
		if(!strcmp(entry->getSchemeName(), sName)) {
			fName = entry->getFileName();
			schemeList->remove(entry);
			break;
		}
	}
	
	delete listView_ls->currentItem();
	QDir qDir(QDir::homeDirPath() + ".qt/");
	qDir.remove(fName);
	qDir.remove("." + fName + ".lock");
}


void DominoStyleConfig::getConfigSchemes() {
	
	// only local
	QString qtHome(QDir::homeDirPath() + "/.qt");
	QSettings s;
	s.insertSearchPath( QSettings::Unix, qtHome);
	
	QDir schemeFile(qtHome, "domino_*rc");
	for(uint i = 0; i < schemeFile.count(); i++) {
		bool deletable = QFileInfo(qtHome, schemeFile[i]).isWritable();
		s.beginGroup("/" + schemeFile[i].left(schemeFile[i].length()-2) + "/Settings");
		QString origName = s.readEntry("/name", "unnamied");
		QString sName = origName;
		if(listView_ls->findItem(sName, 0)) {
			for(int i = 2; listView_ls->findItem(sName, 0); i++) {
				QString num = QString().setNum(i);
				sName = QString(origName).append(" (" + num + ")" );
			}
		}
		new QListViewItem(listView_ls, sName);
		schemeList->append( new SchemeEntry(schemeFile[i], sName, deletable ));
		
		s.endGroup();
	}
}

void DominoStyleConfig::slotDelPerm(QListViewItem* li) {
	
	if(!li)
		return;
	
	SchemeEntry * entry;
	QString selected = li->text(0);

	for ( entry = schemeList->first(); entry; entry = schemeList->next() ) {
		if(!strcmp(entry->getSchemeName(), selected)) {
			deleteButton->setEnabled(entry->isDeletable());
			return;
		}
	}
}

void DominoStyleConfig::scrollBar1GradientChanged() {
	scrollBarNumGradientsChanged(1);
}

void DominoStyleConfig::scrollBar2GradientChanged() {
	scrollBarNumGradientsChanged(2);
}

void DominoStyleConfig::scrollBarNumGradientsChanged(int g) {
	
	switch(g) {
		case 2:
			if(groupBox9_2_2_2_2_4->isChecked() && !groupBox8_2_2_2_2_4->isChecked())
				groupBox8_2_2_2_2_4->setChecked(true);
			break;
		case 1:
			if(groupBox9_2_2_2_2_4->isChecked())
				groupBox9_2_2_2_2_4->setChecked(false);
		default:
			break;
	}
	
	int c = groupBox9_2_2_2_2_4->isChecked() ? 2 : groupBox8_2_2_2_2_4->isChecked() ? 1 : 0;
	currentScrollBarSurface->numGradients = c;
	
	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
}

void DominoStyleConfig::btn1GradientChanged() {
	btnNumGradientsChanged(1);
}

void DominoStyleConfig::btn2GradientChanged() {
	btnNumGradientsChanged(2);
}

void DominoStyleConfig::btnNumGradientsChanged(int g) {
	
	switch(g) {
		case 2:
			if(groupBox9_2_2_2->isChecked() && !groupBox8_2_2_2->isChecked())
				groupBox8_2_2_2->setChecked(true);
			break;
		case 1:
			if(groupBox9_2_2_2->isChecked())
				groupBox9_2_2_2->setChecked(false);
		default:
			break;
	}
	
	int c = groupBox9_2_2_2->isChecked() ? 2 : groupBox8_2_2_2->isChecked() ? 1 : 0;
	btnSurface.numGradients = c;
	
	spinBox44->repaint(false);
	comboBox17->repaint(false);
	pushButton16->repaint(false);
}

void DominoStyleConfig::header1GradientChanged() {
	headerNumGradientsChanged(1);
}

void DominoStyleConfig::header2GradientChanged() {
	headerNumGradientsChanged(2);
}

void DominoStyleConfig::headerNumGradientsChanged(int g) {
	
	switch(g) {
		case 2:
			if(groupBox9_2_2_2_2_4_3->isChecked() && !groupBox8_2_2_2_2_4_3->isChecked())
				groupBox8_2_2_2_2_4_3->setChecked(true);
			break;
		case 1:
			if(groupBox9_2_2_2_2_4_3->isChecked())
				groupBox9_2_2_2_2_4_3->setChecked(false);
		default:
			break;
	}
	
	int c = groupBox9_2_2_2_2_4_3->isChecked() ? 2 : groupBox8_2_2_2_2_4_3->isChecked() ? 1 : 0;
	headerSurface.numGradients = c;
	
	header1->setBackgroundMode(Qt::PaletteBackground);
	header1->repaint(false);
}

void DominoStyleConfig::checkItem1GradientChanged() {
	checkItemNumGradientsChanged(1);
}

void DominoStyleConfig::checkItem2GradientChanged() {
	checkItemNumGradientsChanged(2);
}

void DominoStyleConfig::checkItemNumGradientsChanged(int g) {
	
	switch(g) {
		case 2:
			if(groupBox9_2_2_2_2_4_3_2->isChecked() && !groupBox8_2_2_2_2_4_3_2->isChecked())
				groupBox8_2_2_2_2_4_3_2->setChecked(true);
			break;
		case 1:
			if(groupBox9_2_2_2_2_4_3_2->isChecked())
				groupBox9_2_2_2_2_4_3_2->setChecked(false);
		default:
			break;
	}
	
	int c = groupBox9_2_2_2_2_4_3_2->isChecked() ? 2 : groupBox8_2_2_2_2_4_3_2->isChecked() ? 1 : 0;
	checkItemSurface.numGradients = c;
	
	checkBox2->repaint(false);
	radioButton2->repaint(false);
}

void DominoStyleConfig::tab1GradientChanged() {
	tabNumGradientsChanged(1);
}

void DominoStyleConfig::tab2GradientChanged() {
	tabNumGradientsChanged(2);
}

void DominoStyleConfig::tabNumGradientsChanged(int g) {
	
	switch(g) {
		case 2:
			if(groupBox9_2_2_2_2->isChecked() && !groupBox8_2_2_2_2->isChecked())
				groupBox8_2_2_2_2->setChecked(true);
			break;
		case 1:
			if(groupBox9_2_2_2_2->isChecked())
				groupBox9_2_2_2_2->setChecked(false);
		default:
			break;
	}
	int c = groupBox9_2_2_2_2->isChecked() ? 2 : groupBox8_2_2_2_2->isChecked() ? 1 : 0;
	currentTabSurface->numGradients = c;
	
	tabBarTop->repaint(false);
	tabBarBottom->repaint(false);
}



void DominoStyleConfig::capsColorChanged(const QColor &capsColor) {
	scrollBarCapsColor = capsColor;

	scrollBar5->repaint(false);
	scrollBar6->repaint(false);
	
}







#include "dominoconf.moc"
