/* Domino widget style for KDE 3
   Copyright (C) 2006 Michael Lentner <michaell@gmx.net>

   based on the KDE style "Plastik":
   Copyright (C) 2003 Sandro Giessl <ceebx@users.sourceforge.net>

   based on the KDE style "dotNET":
   Copyright (C) 2001-2002, Chris Lee <clee@kde.org>
					Carsten Pfeiffer <pfeiffer@kde.org>
					Karol Szwed <gallium@kde.org>
   Drawing routines completely reimplemented from KDE3 HighColor, which was
   originally based on some stuff from the KDE2 HighColor.

   based on drawing routines of the style "Keramik":
   Copyright (c) 2002 Malte Starostik <malte@kde.org>
			 (c) 2002,2003 Maksim Orlovich <mo002j@mail.rochester.edu>
   based on the KDE3 HighColor Style
   Copyright (C) 2001-2002 Karol Szwed		<gallium@kde.org>
			 (C) 2001-2002 Fredrik Hglund	<fredrik@kde.org>
   Drawing routines adapted from the KDE2 HCStyle,
   Copyright (C) 2000 Daniel M. Duley		<mosfet@kde.org>
			 (C) 2000 Dirk Mueller			<mueller@kde.org>
			 (C) 2001 Martijn Klingens		<klingens@kde.org>
   Progressbar code based on KStyle,
   Copyright (C) 2001-2002 Karol Szwed <gallium@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


#include <kglobalsettings.h>
#include <qpainter.h>
#include <qtabbar.h>
#include <qprogressbar.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qstyleplugin.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qmenubar.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qsettings.h>
#include <kpixmap.h>
#include <qgroupbox.h>
#include <qmenudata.h>
#include <qdockarea.h>
#include <qobjectlist.h>
#include <qheader.h>
#include <qlistview.h>
#include <qstatusbar.h>
#include <qcombobox.h>
#include <qtabwidget.h>
#include <ktoolbar.h>
#include <qlabel.h>
#include <qsimplerichtext.h>
#include <qiconview.h>
#include <kconfig.h>
#include <qapplication.h>
#include <qtoolbox.h>
#include <qstylesheet.h>
#include <qsplitter.h>
#include <qwidgetstack.h>
#include <qtimer.h>

#include "domino.h"



// popupmenu item constants...
static const int itemHMargin = 6;
static const int itemFrame = 2;
static const int arrowHMargin = 6;
static const int rightBorder = 12;




// -- Style Plugin Interface -------------------------
class DominoStylePlugin : public QStylePlugin
{
	public:
		DominoStylePlugin() {}
		~DominoStylePlugin() {}

		QStringList keys() const {
			return QStringList() << "Domino";
		}

		QStyle* create( const QString& key ) {
			if (key.lower() == "domino")
				return new DominoStyle;
			return 0;
		}
};

KDE_Q_EXPORT_PLUGIN( DominoStylePlugin )
// -- end --

DominoStyle::DominoStyle() : KStyle( AllowMenuTransparency, ThreeButtonScrollBar ),
	kickerMode(false),
	designerMode(false),
	konquerorMode(false),
	flatMode(false)
{
	
	QColor background = qApp->palette().active().background();
	QColor buttonColor = qApp->palette().active().button();
	QSettings settings;
	_contrast = settings.readNumEntry("/Qt/KDE/contrast", 0);
	settings.beginGroup("/domino/Settings");

	_animateProgressBar = settings.readBoolEntry("/animateProgressBar", false);
	_centerTabs = settings.readBoolEntry("/centerTabs", false);
	_drawTriangularExpander = settings.readBoolEntry("/drawTriangularExpander", false);
	_customCheckMarkColor = settings.readBoolEntry("/customCheckMarkColor", false);
	_checkMarkColor.setNamedColor( settings.readEntry("/checkMarkColor", "black") );
	_scrollBarCapsColor = settings.readEntry("/scrollBarCapsColor", Qt::black.name());
	_tintGroupBoxBackground = settings.readBoolEntry("/tintGroupBoxBackground", true);
	_drawGroupBoxDark = settings.readBoolEntry("/drawGroupBoxDark", true);
	_drawGroupBoxBright = settings.readBoolEntry("/drawGroupBoxBright", false);
	_customToolTipColor = settings.readBoolEntry("/customToolTipColor", false);
	_toolTipColor = _customToolTipColor? settings.readEntry("/toolTipColor", QColor(218,218,179).name()) : QColor(218,218,179);
	//_toolTipColor = QColor(255,255,220); // yellow
	_smoothScrolling = settings.readBoolEntry("/smoothScrolling", true);
	_buttonHeightAdjustment = settings.readNumEntry("/buttonHeightAdjustment", 0);
	_customPopupMenuColor = settings.readBoolEntry("/customPopupMenuColor", false);
	_popupMenuColor = _customPopupMenuColor ? settings.readEntry("/popupMenuColor", background.name()) : background.name();
	_indicateFocus = settings.readBoolEntry("/indicateFocus", false);
	_focusIndicatorColor = settings.readEntry("/indicatorColor", qApp->palette().active().button().light(120).name());
	_focusIndicatorBtnColor = settings.readEntry("/indicatorBtnColor", qApp->palette().active().button().light(120).name());
	_drawUnderline = settings.readBoolEntry("/drawFocusUnderline", true);
	
	btnSurface.numGradients = settings.readNumEntry("/btnSurface_numGradients", 2);
	btnSurface.g1Color1 = settings.readEntry("/btnSurface_g1Color1", buttonColor.name());
	btnSurface.g1Color2 = settings.readEntry("/btnSurface_g1Color2", buttonColor.dark(120).name());
	btnSurface.g2Color1 = settings.readEntry("/btnSurface_g2Color1", buttonColor.dark(120).name());
	btnSurface.g2Color2 = settings.readEntry("/btnSurface_g2Color2", buttonColor.dark(110).name());
	btnSurface.background = settings.readEntry("/btnSurface_background", buttonColor.name());
	btnSurface.g1Top = settings.readNumEntry("/btnSurface_g1Top", 0);
	btnSurface.g1Bottom = settings.readNumEntry("/btnSurface_g1Bottom", 50);
	btnSurface.g2Top = settings.readNumEntry("/btnSurface_g2Top", 50);
	btnSurface.g2Bottom = settings.readNumEntry("/btnSurface_g2Bottom", 100);
	
	activeTabTopSurface.numGradients = settings.readNumEntry("/activeTabTopSurface_numGradients", 2);
	activeTabTopSurface.g1Color1 = settings.readEntry("/activeTabTopSurface_g1Color1", background.light(110).name());
	activeTabTopSurface.g1Color2 = settings.readEntry("/activeTabTopSurface_g1Color2", background.name());
	activeTabTopSurface.g2Color1 = settings.readEntry("/activeTabTopSurface_g2Color1", background.name());
	activeTabTopSurface.g2Color2 = settings.readEntry("/activeTabTopSurface_g2Color2", background.name());
	activeTabTopSurface.background = settings.readEntry("/activeTabTopSurface_background", background.name());
	activeTabTopSurface.g1Top = settings.readNumEntry("/activeTabTopSurface_g1Top", 0);
	activeTabTopSurface.g1Bottom = settings.readNumEntry("/activeTabTopSurface_g1Bottom", 50);
	activeTabTopSurface.g2Top = settings.readNumEntry("/activeTabTopSurface_g2Top", 50);
	activeTabTopSurface.g2Bottom = settings.readNumEntry("/activeTabTopSurface_g2Bottom", 100);
	
	tabTopSurface.numGradients = settings.readNumEntry("/tabTopSurface_numGradients", 2);
	tabTopSurface.g1Color1 = settings.readEntry("/tabTopSurface_g1Color1", background.light(110).name());
	tabTopSurface.g1Color2 = settings.readEntry("/tabTopSurface_g1Color2", background.dark(110).name());
	tabTopSurface.g2Color1 = settings.readEntry("/tabTopSurface_g2Color1", background.dark(110).name());
	tabTopSurface.g2Color2 = settings.readEntry("/tabTopSurface_g2Color2", background.dark(110).name());
	tabTopSurface.background = settings.readEntry("/tabTopSurface_background", background.dark(110).name());
	tabTopSurface.g1Top = settings.readNumEntry("/tabTopSurface_g1Top", 0);
	tabTopSurface.g1Bottom = settings.readNumEntry("/tabTopSurface_g1Bottom", 50);
	tabTopSurface.g2Top = settings.readNumEntry("/tabTopSurface_g2Top", 50);
	tabTopSurface.g2Bottom = settings.readNumEntry("/tabTopSurface_g2Bottom", 100);
	
	activeTabBottomSurface.numGradients = settings.readNumEntry("/activeTabBottomSurface_numGradients", 2);
	activeTabBottomSurface.g1Color1 = settings.readEntry("/activeTabBottomSurface_g1Color1", background.name());
	activeTabBottomSurface.g1Color2 = settings.readEntry("/activeTabBottomSurface_g1Color2", background.name());
	activeTabBottomSurface.g2Color1 = settings.readEntry("/activeTabBottomSurface_g2Color1", background.name());
	activeTabBottomSurface.g2Color2 = settings.readEntry("/activeTabBottomSurface_g2Color2", background.dark(120).name());
	activeTabBottomSurface.background = settings.readEntry("/activeTabBottomSurface_background", background.name());
	activeTabBottomSurface.g1Top = settings.readNumEntry("/activeTabBottomSurface_g1Top", 0);
	activeTabBottomSurface.g1Bottom = settings.readNumEntry("/activeTabBottomSurface_g1Bottom", 50);
	activeTabBottomSurface.g2Top = settings.readNumEntry("/activeTabBottomSurface_g2Top", 50);
	activeTabBottomSurface.g2Bottom = settings.readNumEntry("/activeTabBottomSurface_g2Bottom", 100);
	
	tabBottomSurface.numGradients = settings.readNumEntry("/tabBottomSurface_numGradients", 2);
	tabBottomSurface.g1Color1 = settings.readEntry("/tabBottomSurface_g1Color1", background.dark(110).name());
	tabBottomSurface.g1Color2 = settings.readEntry("/tabBottomSurface_g1Color2", background.dark(110).name());
	tabBottomSurface.g2Color1 = settings.readEntry("/tabBottomSurface_g2Color1", background.dark(110).name());
	tabBottomSurface.g2Color2 = settings.readEntry("/tabBottomSurface_g2Color2", background.dark(120).name());
	tabBottomSurface.background = settings.readEntry("/tabBottomSurface_background", background.dark(110).name());
	tabBottomSurface.g1Top = settings.readNumEntry("/tabBottomSurface_g1Top", 0);
	tabBottomSurface.g1Bottom = settings.readNumEntry("/tabBottomSurface_g1Bottom", 50);
	tabBottomSurface.g2Top = settings.readNumEntry("/tabBottomSurface_g2Top", 50);
	tabBottomSurface.g2Bottom = settings.readNumEntry("/tabBottomSurface_g2Bottom", 100);
	
	scrollBarSurface.numGradients = settings.readNumEntry("/scrollBarSurface_numGradients", 2);
	scrollBarSurface.g1Color1 = settings.readEntry("/scrollBarSurface_g1Color1", buttonColor.name());
	scrollBarSurface.g1Color2 = settings.readEntry("/scrollBarSurface_g1Color2", buttonColor.dark(120).name());
	scrollBarSurface.g2Color1 = settings.readEntry("/scrollBarSurface_g2Color1", buttonColor.dark(120).name());
	scrollBarSurface.g2Color2 = settings.readEntry("/scrollBarSurface_g2Color2", buttonColor.dark(110).name());
	scrollBarSurface.background = settings.readEntry("/scrollBarSurface_background", buttonColor.name());
	scrollBarSurface.g1Top = settings.readNumEntry("/scrollBarSurface_g1Top", 0);
	scrollBarSurface.g1Bottom = settings.readNumEntry("/scrollBarSurface_g1Bottom", 50);
	scrollBarSurface.g2Top = settings.readNumEntry("/scrollBarSurface_g2Top", 50);
	scrollBarSurface.g2Bottom = settings.readNumEntry("/scrollBarSurface_g2Bottom", 100);
	
	scrollBarGrooveSurface.numGradients = settings.readNumEntry("/scrollBarGrooveSurface_numGradients", 0);
	scrollBarGrooveSurface.g1Color1 = settings.readEntry("/scrollBarGrooveSurface_g1Color1", background.name());
	scrollBarGrooveSurface.g1Color2 = settings.readEntry("/scrollBarGrooveSurface_g1Color2", background.dark(120).name());
	scrollBarGrooveSurface.g2Color1 = settings.readEntry("/scrollBarGrooveSurface_g2Color1", background.dark(120).name());
	scrollBarGrooveSurface.g2Color2 = settings.readEntry("/scrollBarGrooveSurface_g2Color2", background.dark(110).name());
	scrollBarGrooveSurface.background = settings.readEntry("/scrollBarGrooveSurface_background", background.dark(150).name());
	scrollBarGrooveSurface.g1Top = settings.readNumEntry("/scrollBarGrooveSurface_g1Top", 0);
	scrollBarGrooveSurface.g1Bottom = settings.readNumEntry("/scrollBarGrooveSurface_g1Bottom", 50);
	scrollBarGrooveSurface.g2Top = settings.readNumEntry("/scrollBarGrooveSurface_g2Top", 50);
	scrollBarGrooveSurface.g2Bottom = settings.readNumEntry("/scrollBarGrooveSurface_g2Bottom", 100);

	headerSurface.numGradients = settings.readNumEntry("/headerSurface_numGradients", 2);
	headerSurface.g1Color1 = settings.readEntry("/headerSurface_g1Color1", background.name());
	headerSurface.g1Color2 = settings.readEntry("/headerSurface_g1Color2", background.dark(120).name());
	headerSurface.g2Color1 = settings.readEntry("/headerSurface_g2Color1", background.dark(120).name());
	headerSurface.g2Color2 = settings.readEntry("/headerSurface_g2Color2", background.dark(110).name());
	headerSurface.background = settings.readEntry("/headerSurface_background", background.name());
	headerSurface.g1Top = settings.readNumEntry("/headerSurface_g1Top", 0);
	headerSurface.g1Bottom = settings.readNumEntry("/headerSurface_g1Bottom", 50);
	headerSurface.g2Top = settings.readNumEntry("/headerSurface_g2Top", 50);
	headerSurface.g2Bottom = settings.readNumEntry("/headerSurface_g2Bottom", 100);

	// checkBoxes + radioButtons
	checkItemSurface.numGradients = settings.readNumEntry("/checkItemSurface_numGradients", 2);
	checkItemSurface.g1Color1 = settings.readEntry("/checkItemSurface_g1Color1", buttonColor.name());
	checkItemSurface.g1Color2 = settings.readEntry("/checkItemSurface_g1Color2", buttonColor.dark(120).name());
	checkItemSurface.g2Color1 = settings.readEntry("/checkItemSurface_g2Color1", buttonColor.dark(120).name());
	checkItemSurface.g2Color2 = settings.readEntry("/checkItemSurface_g2Color2", buttonColor.dark(110).name());
	checkItemSurface.background = settings.readEntry("/checkItemSurface_background", buttonColor.name());
	checkItemSurface.g1Top = settings.readNumEntry("/checkItemSurface_g1Top", 0);
	checkItemSurface.g1Bottom = settings.readNumEntry("/checkItemSurface_g1Bottom", 50);
	checkItemSurface.g2Top = settings.readNumEntry("/checkItemSurface_g2Top", 50);
	checkItemSurface.g2Bottom = settings.readNumEntry("/checkItemSurface_g2Bottom", 100);

	settings.endGroup();
	
	if(!strcmp(qApp->argv()[0], "konqueror")) {
		KGlobal::config()->setGroup("General");
		minTabLength = KGlobal::config()->readNumEntry("MinimumTabLength", 3);
	}
	else
		minTabLength = 0;
	
	tabBarContrast = 150; // remove later
	hoverWidget = 0;
	horizontalLine = 0;
	verticalLine = 0;
	sideRepaint = false;
	designerGroupBox = 0;
	hoveredToolBoxTab = false;


	checkMark = new QPixmap(tintImage(uic_findImage("checkmark"), _customCheckMarkColor? _checkMarkColor : qApp->palette().active().foreground()));
	radioIndicator = new QPixmap(tintImage(uic_findImage("rb-dot"), _customCheckMarkColor? _checkMarkColor : qApp->palette().active().foreground()));
	QPainter p;
	p.begin(radioIndicator);
	p.drawPixmap(0,0, uic_findImage("rb-dot-shadow"));
	p.end();
	
	checkbox = new QPixmap(uic_findImage("checkbox"));
	radiobutton = new QPixmap(uic_findImage("radiobutton"));
	
	tabContour = qApp->palette().active().background().dark(tabBarContrast);
	popupFrame = new QPixmap(tintImage(uic_findImage("popup5"), _popupMenuColor.dark(150) ));
	
	// setup pixmap cache...
	pixmapCache = new QIntCache<CacheEntry>(150000, 499);
	pixmapCache->setAutoDelete(true);

	if ( _animateProgressBar )
	{
		animationTimer = new QTimer( this );
		connect( animationTimer, SIGNAL(timeout()), this, SLOT(updateProgressPos()) );
	}

	scrollDistance = 0;
	scrollWidget = 0;
	scrollTimer = new QTimer(this);
	connect( scrollTimer, SIGNAL(timeout()), this, SLOT(updateScrollPos()) );
	

	button = new QPixmap(uic_findImage("btn"));
	buttonVlines = new QPixmap(uic_findImage("btn-v"));
	buttonHlines = new QPixmap(uic_findImage("btn-h"));
	
	buttonShadowSquare = new QPixmap(uic_findImage("btn-pressed"));
	buttonShadowRound = new QPixmap(uic_findImage("btn-pressedRound"));
	buttonShadowHlines = new QPixmap(uic_findImage("btn-pressedHLines"));
	buttonShadowVlines = new QPixmap(uic_findImage("btn-pressedVLines"));
	
	
	renderScrollBarCaps(_scrollBarCapsColor);

	QColor tabContour2 = qApp->palette().active().background().dark(150);
	border1= new QPixmap(tintImage(uic_findImage("border1"), tabContour2));
}

void DominoStyle::updateProgressPos()
{
	QProgressBar* pb;
	//Update the registered progressbars.
	QMap<QWidget*, int>::iterator iter;
	bool visible = false;
	for (iter = progAnimWidgets.begin(); iter != progAnimWidgets.end(); iter++)
	{	
		if ( !dynamic_cast<QProgressBar*>(iter.key()) )
			continue;
		
		pb = dynamic_cast<QProgressBar*>(iter.key());
		if ( iter.key() -> isEnabled() && 
			 pb -> progress() != pb->totalSteps() )
		{
			// update animation Offset of the current Widget
			iter.data() = (iter.data() + 1) % 20;
			iter.key()->update();
		}
		if (iter.key()->isVisible())
			visible = true;
	}
	if (!visible)
		animationTimer->stop();
}

void DominoStyle::updateScrollPos()
{

	if(dynamic_cast<QScrollView*>(scrollWidget) && scrollDistance != 0) {
		int scroll;
		if(scrollDistance < 0) {
			scroll = +15;
			scrollDistance += scroll;
			scrollDistance = QMIN(scrollDistance, 0);
		}
		else {
			scroll = -15;
			scrollDistance += scroll;
			scrollDistance = QMAX(scrollDistance, 0);
		}
		scrollWidget->scrollBy(0, scroll);
	}
	else if(scrollTimer->isActive()) {
		scrollTimer->stop();
	}
}


DominoStyle::~DominoStyle()
{
	delete pixmapCache;
	delete horizontalLine;
	delete verticalLine;
	
	delete checkMark;
	delete radioIndicator;
	delete scrollBarCap_top;
	delete scrollBarCap_bottom;
	delete scrollBarCap_left;
	delete scrollBarCap_right;

	delete border1;
	delete popupFrame;
	delete checkbox;
	delete radiobutton;
	
	delete button;
	delete buttonVlines;
	delete buttonHlines;
	delete buttonShadowSquare;
	delete buttonShadowRound;
	delete buttonShadowHlines;
	delete buttonShadowVlines;
	
}

void DominoStyle::polish(QApplication* app)
{
	if(!qstrcmp(app->argv()[0], "konqueror"))
		konquerorMode = true;
	else if(!qstrcmp(app->argv()[0], "kicker"))
		kickerMode = true;
	else if(!qstrcmp(app->argv()[0], "designer"))
		designerMode = true;
	
	QPixmapCache::clear();
}

void DominoStyle::polish(QPalette &p) {
	
	p.setBrush(QColorGroup::Button, p.active().background());
	
	if(!_customPopupMenuColor)
		_popupMenuColor = qApp->palette().active().background();
	
	QColor tabContour2 = qApp->palette().active().background().dark(150);
	delete border1;
	border1= new QPixmap(tintImage(uic_findImage("border1"), tabContour2));
	delete popupFrame;
	popupFrame = new QPixmap(tintImage(uic_findImage("popup5"), _popupMenuColor.dark(150)));
	if(!_customCheckMarkColor) {
		delete checkMark;
		checkMark = new QPixmap(tintImage(uic_findImage("checkmark"), qApp->palette().active().foreground()));
		delete radioIndicator;
		radioIndicator = new QPixmap(tintImage(uic_findImage("rb-dot"), qApp->palette().active().foreground()));
		QPainter p(radioIndicator);
		p.drawPixmap(0,0, uic_findImage("rb-dot-shadow"));
	}
}


void DominoStyle::polish(QWidget* widget)
{
	
	if(!strcmp(widget->name(), "__khtml")) { // is it a khtml widget...?
		khtmlWidgets[widget] = true;
		connect(widget, SIGNAL(destroyed(QObject*)), this, SLOT(khtmlWidgetDestroyed(QObject*)));
	}
	else if(!strcmp(widget->name(), "kfile button") && !strcmp(widget->parentWidget()->name(), "__khtml")) {
		widget->setBackgroundMode(Qt::NoBackground);
		widget->parentWidget()->setBackgroundMode(Qt::NoBackground);
		khtmlWidgets[widget] = true;
		connect(widget, SIGNAL(destroyed(QObject*)), this, SLOT(khtmlWidgetDestroyed(QObject*)));
	}

	if(dynamic_cast<QPushButton*>(widget)) {
		if (widget->isA("KDockButton_Private")) {
			widget->setFixedSize(13, 13); // 12x12 pixmap + 1px top and right spacing
			widget->installEventFilter(this);
		}
		else
			((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
	}
	else if(dynamic_cast<QCheckBox*>(widget)) {
		((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<KToolBarButton*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QRadioButton*>(widget)) {
		((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QComboBox*>(widget)) {
		((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
		QComboBox* cb = static_cast<QComboBox*>(widget);
		if(cb->listBox())
			cb->listBox()->setFrameStyle(QFrame::Panel|QFrame::Sunken);
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QGroupBox*>(widget)) {
		QGroupBox* gb = dynamic_cast<QGroupBox*>(widget);
		gb->setAlignment(Qt::AlignHCenter);
		if(_tintGroupBoxBackground && gb->frameShape() == QFrame::GroupBoxPanel && !gb->isFlat() && !gb->lineWidth() <= 0) {
			if(_drawGroupBoxDark)
				gb->setPaletteBackgroundColor(darkenColor(gb->parentWidget()->paletteBackgroundColor(), 10));
			else if(_drawGroupBoxBright)
				gb->setPaletteBackgroundColor(lightenColor(gb->parentWidget()->paletteBackgroundColor(), 10));
			else
				gb->setPaletteBackgroundColor(gb->parentWidget()->paletteBackgroundColor());
			if(gb->isCheckable()) {
				((QCheckBox*)gb->child("qt_groupbox_checkbox", 0, false))->setPaletteBackgroundColor(gb->parentWidget()->paletteBackgroundColor());
			}
		}
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QSpinWidget*>(widget)) {
		((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QLineEdit*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QScrollBar*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QStatusBar*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QHeader*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QPopupMenu*>(widget)) {
		if(!widget->isA("PrettyPopupMenu")) { // amarok
			widget->setBackgroundMode( NoBackground );
			widget->installEventFilter(this);
		}
	}
	else if(dynamic_cast<QTabWidget*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QIconView*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QListView*>(widget)) {
		if(widget->isA("KonqTreeViewWidget")) { // konqi frames
			QFrame* f = dynamic_cast<QFrame*>(widget);
			f->setFrameStyle(QFrame::Box|QFrame::Plain);
			f->setLineWidth(1);
		}
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QScrollView*>(widget)) {
		QFrame* f = dynamic_cast<QFrame*>(widget);
		if((konquerorMode && widget->isA("KHTMLView")) || f->parentWidget() && !strcmp(f->parentWidget()->name(), "KonqFrame")) { // konqi frames
			f->setFrameStyle(QFrame::Box|QFrame::Plain);
			f->setLineWidth(1);
			widget->installEventFilter(this);
		}
		else if(_smoothScrolling)
			widget->installEventFilter(this);
	}
	else if(dynamic_cast<QSlider*>(widget)) {
		((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
	}
	else if(dynamic_cast<QListBox*>(widget)) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<KToolBarSeparator*>(widget) || dynamic_cast<QToolBar*>(widget)) {
		widget->setBackgroundMode(Qt::PaletteBackground);
	}
	else if(widget->isA("KonqCheckBox")) {
		widget->installEventFilter(this);
	}
	else if(widget->isA("QTipLabel")) {
		widget->installEventFilter(this);
	}
	else if(widget->isA("QWhatsThat")) {
		widget->installEventFilter(this);
	}
	else if(widget->isA("KonqFileTip")) {
		widget->installEventFilter(this);
	}
	else if(widget->isA("KTextEdit") || widget->isA("KActiveLabel")) { // koffice (< version 1.5) template startDialogs + some kmail labels.
		if(dynamic_cast<QGroupBox*>(widget->parentWidget()))
			if(widget->paletteBackgroundColor() ==  qApp->palette().active().background())
				widget->setPaletteBackgroundColor(widget->parentWidget()->paletteBackgroundColor());
	}
	else if(dynamic_cast<QButton*>(widget) && widget->parentWidget() && widget->parentWidget()->isA("QToolBox")) {
		widget->installEventFilter(this);
	}
	else if(dynamic_cast<QFrame*>(widget)) {
		QFrame* f = dynamic_cast<QFrame*>(widget);

		switch (f->frameShape()) {
			case QFrame::Box:
				f->setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
				f->setLineWidth(2);
				break;
			case QFrame::Panel:
				if(kickerMode && f->parentWidget() &&  f->parentWidget()->isA("ClockApplet")) {
					f->setLineWidth(0);
					break;
				}
			case QFrame::WinPanel:
				f->setLineWidth(1);
				widget->installEventFilter(this);
				break;
			case QFrame::LineEditPanel: {
				widget->installEventFilter(this);
				break;
			}
			case QFrame::TabWidgetPanel:
				if(dynamic_cast<QWidgetStack*>(widget)) {
					((DominoQWidget*)widget)->setWFlags(Qt::WNoAutoErase);
					if(f->parentWidget() && f->parentWidget()->isA("KonqFrameTabs"))
						break;
				}
			case QFrame::HLine:
			case QFrame::VLine: {
				widget->installEventFilter(this);
				break;
				}
			case QFrame::StyledPanel:
				if(dynamic_cast<QDockWindow*>(widget)) {
					widget->installEventFilter(this);
				}
				else if(widget->isA("QWorkspaceChild")) {
					f->setFrameStyle(QFrame::Panel|QFrame::Sunken);
					f->setLineWidth(1);
					widget->installEventFilter(this);
				}
				else if(widget->isA("KAnimWidget"))
					f->setLineWidth(0);
			default:
				break;
		}
	}
// 	else if (dynamic_cast<KDockSplitter*>(widget)) {
// 		widget->installEventFilter(this);
// 	} else if (dynamic_cast<KDockWidgetAbstractHeaderDrag*>(widget)) {
// 		widget->installEventFilter(this);
// 	} else if ( dynamic_cast<QToolBox*>(widget)) {
// 		widget->installEventFilter(this);
	else if(widget->inherits("QDockWindowHandle") || widget->isA("QDockWindowResizeHandle") ||
		        widget->isA("AppletHandleDrag")) { // !qstrcmp(widget->name(), "qt_dockwidget_internal")
		widget->installEventFilter(this);
	}
	else if(widget->isA("KDockWidget") && widget->parentWidget() && widget->parentWidget()->parentWidget()
		       && widget->parentWidget()->parentWidget()->isA("Sidebar_Widget")) {
		widget->setEraseColor(widget->palette().active().base());
	}

// 	if( _animateProgressBar && dynamic_cast<QProgressBar*>(widget) )
// 	{
// 		widget->installEventFilter(this);
// 		progAnimWidgets[widget] = 0;
// 		connect(widget, SIGNAL(destroyed(QObject*)), this, SLOT(progressBarDestroyed(QObject*)));
// 		if (!animationTimer->isActive())
// 			animationTimer->start( 50, false );
// 	}


	
	//widget->installEventFilter(this);
	//qDebug("className %s, name: %s", widget->className(), widget->name());
	
}


void DominoStyle::unPolish(QWidget* widget)
{
	
	if( !strcmp(widget->name(), "__khtml") || (!strcmp(widget->name(), "kfile button") && !strcmp(widget->parentWidget()->name(), "__khtml"))) { // is it a khtml widget...?
		khtmlWidgets.remove(widget);
	}

	if (dynamic_cast<QPushButton*>(widget) || 
		   dynamic_cast<QSpinWidget*>(widget) || dynamic_cast<QSlider*>(widget) ||
		   dynamic_cast<QRadioButton*>(widget) ||
		   dynamic_cast<QListBox*>(widget) || dynamic_cast<QListView*>(widget) ||
		   dynamic_cast<QStatusBar*>(widget) || widget->isA("AppletHandleDrag") ||
		   widget->isA("QDockWindowHandle"))
	{
		widget->removeEventFilter(this);
	}
	else if(widget->isA("KonqCheckBox") || widget->isA("QWhatsThat") || widget->isA("QTipLabel")) {
		widget->removeEventFilter(this);
		widget->setPaletteBackgroundColor(qApp->palette().active().background());
		widget->setBackgroundMode(Qt::PaletteBackground);
	}
	else if(dynamic_cast<QTabWidget*>(widget) || dynamic_cast<QTabBar*>(widget)) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteBackground);
	}
	else if(dynamic_cast<QCheckBox*>(widget)) {
		widget->removeEventFilter(this);
	}
	else if(dynamic_cast<QComboBox*>(widget)) { // after KonqCheckBox
		widget->removeEventFilter(this);
	}
	else if(dynamic_cast<QHeader*>(widget)) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteButton);
	}
	else if(dynamic_cast<QPopupMenu*>(widget)) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteBackground);
		static_cast<QFrame*>(widget)->setLineWidth(2);
		widget->clearMask();
	}
	else if(dynamic_cast<KToolBarButton*>(widget)) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteBackground);
	}
	else if(dynamic_cast<QGroupBox*>(widget)) {
		widget->removeEventFilter(this);
		QGroupBox* gb = dynamic_cast<QGroupBox*>(widget);
		gb->setBackgroundMode(Qt::PaletteBackground);
		gb->setPaletteBackgroundColor(qApp->palette().active().background());
		gb->setAlignment(Qt::AlignLeft|Qt::AlignBottom);
	}
	else if(dynamic_cast<QLineEdit*>(widget)) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteBase);
		widget->clearMask();
	}
	else if(dynamic_cast<QProgressBar*>(widget) ) {
		progAnimWidgets.remove(widget);
	}
	else if(dynamic_cast<KToolBarSeparator*>(widget) || dynamic_cast<QToolBar*>(widget) ) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode(Qt::PaletteButton);
	}
	if(widget->isA("KonqFileTip")) {
		widget->removeEventFilter(this);
		widget->setPaletteBackgroundColor(widget->palette().active().background());
		QFrame* f = dynamic_cast<QFrame*>(widget);
		f->setMargin(1);
		f->setFrameStyle(QFrame::Plain|QFrame::Box);

		QObjectList *list = const_cast<QObjectList *>(widget->children());
		QObjectListIt it( *list );
		QObject * o;
		while ( (o = it.current()) != 0 ) {
			++it;
			if(dynamic_cast<QLabel*>(o)) {
				QLabel* l = dynamic_cast<QLabel*>(o);
				l->setBackgroundOrigin(QWidget::WidgetOrigin);
				l->setBackgroundMode(Qt::PaletteBackground);
				l->setPaletteBackgroundColor(l->palette().active().background());
			}
		}
	}
	else if(dynamic_cast<QScrollView*>(widget)) {
		QFrame* f = dynamic_cast<QFrame*>(widget);
		if((konquerorMode && widget->isA("KHTMLView")) || f->parentWidget() && !strcmp(f->parentWidget()->name(), "KonqFrame")) { // konqi frames
			widget->removeEventFilter(this);
			f->setFrameStyle(QFrame::NoFrame|QFrame::Plain);
			f->setLineWidth(0);
		}
	}
	else if(dynamic_cast<QFrame*>(widget)) {
		widget->removeEventFilter(this);
		QFrame* f = dynamic_cast<QFrame*>(widget);
		switch(f->frameShape()) {
			case QFrame::Panel:
				if(widget->isA("QWorkspaceChild")) {
					f->setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
					f->setLineWidth(2);
				}
				break;
			case QFrame::WinPanel:
				f->setLineWidth(2);
				break;
			case QFrame::TabWidgetPanel:
				f->setLineWidth(2);
				f->setMargin(2);
				f->setFrameRect(QRect(0, 0, 0, 0));
				break;
			case QFrame::StyledPanel:
				if(widget->isA("KAnimWidget"))
					f->setLineWidth(2);
				break;
			default:
				break;
		}
	}

	
	KStyle::unPolish(widget);
}

void DominoStyle::khtmlWidgetDestroyed(QObject* obj)
{
	khtmlWidgets.remove(static_cast<QWidget*>(obj));
}

void DominoStyle::progressBarDestroyed(QObject* obj)
{
	progAnimWidgets.remove(static_cast<QWidget*>(obj));
}


void DominoStyle::renderContour(QPainter *p, const QRect &r, const QColor &backgroundColor, const QColor &contour, const uint flags) const {
	
	if((r.width() <= 0)||(r.height() <= 0))
		return;

	const bool drawLeft = flags&Draw_Left;
	const bool drawRight = flags&Draw_Right;
	const bool drawTop = flags&Draw_Top;
	const bool drawBottom = flags&Draw_Bottom;
//	const bool disabled = flags&Is_Disabled;
	const bool alphaBlend = flags&Draw_AlphaBlend;

	const bool roundUpperLeft = flags&Round_UpperLeft;
	const bool roundUpperRight = flags&Round_UpperRight;
	const bool roundBottomLeft = flags&Round_BottomLeft;
	const bool roundBottomRight = flags&Round_BottomRight;
	const bool isButton = flags&Is_Button;
	const bool isHorizontal = flags&Is_Horizontal;
	const bool isScrollBarBtn = flags&Is_ScrollBarBtn;
	QColor bgColor;
	QColor contourColor = contour;

	/////contour = cg.button

	
	QColor g1Color1;
	QColor g1Color2;
	// darker inside colors
	QColor sideTopColor;
	QColor sideBottomColor;
	QColor sideBackground;
	QColor hTopColor;
	QColor hBottomColor;
	QColor hTopColor2;
	QColor hBottomColor2;
	
	sideBackground = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.background, 127) : btnSurface.background;
	sideBackground = alphaBlendColors(QColor(0, 0, 0), sideBackground, 65);
	
	switch (btnSurface.numGradients) {
		case 2:{
			
			if(btnSurface.g1Top == 0)
				g1Color1 = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color1, 127) : btnSurface.g1Color1;
			else
				g1Color1 = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.background, 127) : btnSurface.background;
			sideTopColor = alphaBlendColors(QColor(0, 0, 0), g1Color1, 65);
			hTopColor = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color1, 127) : btnSurface.g1Color1)
							, 65);
			hTopColor2 = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g2Color1, 127) : btnSurface.g2Color1)
							, 65);
			
			if(btnSurface.g2Bottom == 100)
				g1Color2 = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g2Color2, 127) : btnSurface.g2Color2;
			else
				g1Color2 = (contour != qApp->palette().active().button() )? alphaBlendColors(contour, btnSurface.background, 127) : btnSurface.background;
			hBottomColor = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color2, 127) : btnSurface.g1Color2)
							, 65);
			hBottomColor2 = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g2Color2, 127) : btnSurface.g2Color2)
							, 65);
			sideBottomColor = alphaBlendColors(QColor(0, 0, 0), g1Color2, 65);
			
			break;
		}
		case 1: {
			
			if(btnSurface.g1Top == 0)
				g1Color1 =  (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color1, 127) : btnSurface.g1Color1;
			else
				g1Color1 =  (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.background, 127) : btnSurface.background;
			sideTopColor = alphaBlendColors(QColor(0, 0, 0), g1Color1, 65);
			hTopColor = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color1, 127) : btnSurface.g1Color1)
							, 65);
			
			if(btnSurface.g1Bottom == 100)
				g1Color2 = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color2, 127)  :btnSurface.g1Color2;
			else
				g1Color2 = (contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.background, 127) : btnSurface.background;
			hBottomColor = alphaBlendColors(QColor(0, 0, 0),
					((contour != qApp->palette().active().button()) ? alphaBlendColors(contour, btnSurface.g1Color2, 127) : btnSurface.g1Color2)
							, 65);
			sideBottomColor = alphaBlendColors(QColor(0, 0, 0), g1Color2, 65);
			break;
		}
		default: {
			sideTopColor = sideBackground;
			sideBottomColor = sideBackground;
			sideBackground = sideBackground;
			hTopColor = sideBackground;
			hBottomColor = sideBackground;
			hTopColor2 = sideBackground;
			hBottomColor2 = sideBackground;
			break;
		}
	}
	
	if(isButton) {

		const QColor color1 = alphaBlendColors(QColor(0, 0, 0), backgroundColor, 179);
		const QColor color3 = alphaBlendColors(QColor(218, 218, 218), backgroundColor, 35);
		const QColor color4 = alphaBlendColors(QColor(0, 0, 0), backgroundColor, 28);
		const QColor color5 = alphaBlendColors(QColor(254, 254, 254), backgroundColor, 79);
		const QColor color6 = alphaBlendColors(QColor(255, 255, 255), backgroundColor, 20);
		
		int top = r.y()+r.height()*btnSurface.g1Top/100;
		int bottom = r.height()*btnSurface.g1Bottom/100;
		int top2 = r.y()+r.height()*btnSurface.g2Top/100;
		int bottom2 = r.y()+r.height()*btnSurface.g2Bottom/100-top2;

		int left = r.x()+r.width()*btnSurface.g1Top/100;
		int width = r.width()*btnSurface.g1Bottom/100+1;
		int left2 = r.x()+r.width()*btnSurface.g2Top/100+1;
		int width2 = r.width()*btnSurface.g2Bottom/100-left2;

		if(roundUpperLeft)
			bitBlt(p->device(), r.left(), r.top(), button, 0, 0, 5, 5, Qt::CopyROP);
		
		if(roundUpperRight)
			bitBlt(p->device(), r.right()-4, r.top(), button, 5, 0, 5, 5, Qt::CopyROP);
		
		if(roundBottomLeft)
			bitBlt(p->device(), r.left(), r.bottom()-5, button, 0, 5, 5, 6, Qt::CopyROP);
		
		if(roundBottomRight)
			bitBlt(p->device(), r.right()-4, r.bottom()-5, button, 5, 5, 5, 6, Qt::CopyROP);
		
		if(drawTop) {
			if(!alphaBlend) {
				p->setPen(color4);
				p->drawLine(r.left()+5, r.top(), r.right()-5, r.top());
				p->setPen(color1);
				p->drawLine(r.left()+5, r.top()+1, r.right()-5, r.top()+1);
				
				if(isHorizontal) {
					p->setPen(sideTopColor);
					p->drawLine(r.left()+5, r.top()+2, r.right()-5, r.top()+2);
				}
				else {
					p->setClipRegion(QRegion(5, 2, r.width()-10, 1));
					switch(btnSurface.numGradients) {
						case 2:
							renderGradient(p, QRect(left2, r.top()+2, width2, 1), hTopColor2.rgb(),hBottomColor2.rgb(), false);
							p->setClipRegion(p->clipRegion()-QRegion(left2, r.top()+2, width2, 1));
						case 1:
							renderGradient(p, QRect(left, r.top()+2, width, 1), hTopColor.rgb(),hBottomColor.rgb(), false);
							p->setClipRegion(p->clipRegion()-QRegion(left, r.top()+2, width, 1));
						default:
							break;
					}
					// the remaining
					p->setPen(sideBackground);
					p->drawLine(r.left()+5, r.top()+2, r.right()-5, r.top()+2);
					p->setClipping(false);
				}
			}
			else {
				p->drawTiledPixmap(r.left()+5, r.top(), r.width()-10, 3, *buttonHlines, 0, 0);
			}
		}
		
		if(drawBottom) {
			if(!alphaBlend) {
				if(isHorizontal) {
					p->setPen(sideBottomColor);
					p->drawLine(r.left()+5, r.bottom()-3, r.right()-5, r.bottom()-3);
				}
				else {
					p->setClipRegion(QRegion(5, r.bottom()-3, r.width()-10, 1));
					switch(btnSurface.numGradients) {
						case 2:
							renderGradient(p, QRect(left2, r.bottom()-3, width2, 1), hTopColor2.rgb(), hBottomColor2.rgb(), false);
							p->setClipRegion(p->clipRegion()-QRegion(left2, r.bottom()-3, width2, 1));
						case 1:
							renderGradient(p, QRect(left, r.bottom()-3, width, 1), hTopColor.rgb(), hBottomColor.rgb(), false);
							p->setClipRegion(p->clipRegion()-QRegion(left, r.bottom()-3, width, 1));
						default:
							break;
					}
					// the remaining
					p->setPen(sideBackground);
					p->drawLine(r.left()+5, r.bottom()-3, r.right()-5, r.bottom()-3);
					p->setClipping(false);
				}
				p->setPen(color1);
				p->drawLine(r.left()+5, r.bottom()-2, r.right()-5, r.bottom()-2);
				p->setPen(color5);
				p->drawLine(r.left()+5, r.bottom()-1, r.right()-5, r.bottom()-1);
				p->setPen(color6);
				p->drawLine(r.left()+5, r.bottom(), r.right()-5, r.bottom());
			}
			else {
				p->drawTiledPixmap(r.left()+5, r.bottom()-3, r.width()-10, 4, *buttonHlines, 0, 3);
			}
		}
		
		if(drawLeft) {
			if(!alphaBlend) {
				p->setPen(color3);
				p->drawLine(r.left(), r.top()+5, r.left(), r.bottom()-6);
				p->setPen(color1);
				p->drawLine(r.left()+1, r.top()+5, r.left()+1, r.bottom()-6);
				if(isHorizontal) {
					p->setClipRegion(QRegion(r.left()+2, r.top()+5, 1, r.height()-11));
					switch(btnSurface.numGradients) {
						case 2:
							renderGradient(p, QRect(r.left(), top2, r.width(), bottom2), hTopColor2.rgb(), hBottomColor2.rgb(), true);
							p->setClipRegion(p->clipRegion()-QRegion(r.left()+2, top2, 1, bottom2));
						case 1:
							renderGradient(p, QRect(r.left(), top, r.width(), bottom), hTopColor.rgb(), hBottomColor.rgb(), true);
							p->setClipRegion(p->clipRegion()-QRegion(r.left()+2, top, 1, bottom));
						default:
							break;
					}
					// the remaining
					p->setPen(sideBackground);
					p->drawLine(r.left()+2, r.top()+5, r.left()+2, r.bottom()-6);
					p->setClipping(false);
				}
				else {
					p->setPen(sideTopColor);
					p->drawLine(r.left()+2, r.top()+5, r.left()+2, r.bottom()-6);
				}
			}
			else {
				p->drawTiledPixmap(r.left(), r.top()+5, 3, r.height()-11, *buttonVlines, 0, 0);
			}
		}
		
		if(drawRight) {
			if(!alphaBlend) {
				if(isHorizontal) {
					p->setClipRegion(QRegion(r.right()-2, r.top()+5, 1, r.height()-11));
					switch(btnSurface.numGradients) {
						case 2:
							renderGradient(p, QRect(r.left(), top2, r.width(), bottom2), hTopColor2.rgb(), hBottomColor2.rgb(), true);
							p->setClipRegion(p->clipRegion()-QRegion(r.right()-2, top2, 1, bottom2));
						case 1:
							renderGradient(p, QRect(r.left(), top, r.width(), bottom), hTopColor.rgb(), hBottomColor.rgb(), true);
							p->setClipRegion(p->clipRegion()-QRegion(r.right()-2, top, 1, bottom));
						default:
							break;
					}
					// the remaining
					p->setPen(sideBackground);
					p->drawLine(r.right()-2, r.top()+5, r.right()-2, r.bottom()-6);
					p->setClipping(false);
				}
				else {
					p->setPen(sideBottomColor);
					p->drawLine(r.right()-2, r.top()+5, r.right()-2, r.bottom()-6);
				}
				p->setPen(color1);
				p->drawLine(r.right()-1, r.top()+5, r.right()-1, r.bottom()-6);
				p->setPen(color3);
				p->drawLine(r.right(), r.top()+5, r.right(), r.bottom()-6);
			}
			else {
				p->drawTiledPixmap(r.right()-2, r.top()+5, 3, r.height()-11, *buttonVlines, 3, 0);
			}
		}
	}
	else if(isScrollBarBtn) { // this needs a cleanup
		
		QColor color1 = alphaBlendColors(QColor(0, 0, 0), backgroundColor, 179);
		const QColor color2 = alphaBlendColors(QColor(0, 0, 0), backgroundColor, 75);
		bool isComboPopup = false;
		QColor color5;
		if(dynamic_cast<QWidget*>(p->device())) {
			QWidget* w = dynamic_cast<QWidget*>(p->device());
			if(w->parentWidget() && dynamic_cast<QListBox*>(w->parentWidget()) && w->parentWidget()->parentWidget() && dynamic_cast<QComboBox*>(w->parentWidget()->parentWidget())) {
				isComboPopup = true;
				color5 = w->parentWidget()->parentWidget()->palette().active().base();
			}
			
		}
		
		QColor sbTopContour = getColor(scrollBarSurface, TopBorder);
		QColor sbBottomContour = getColor(scrollBarSurface, BottomBorder);
		if(backgroundColor != qApp->palette().active().button()) {
			sbTopContour = alphaBlendColors(backgroundColor, sbTopContour, 127);
			sbBottomContour = alphaBlendColors(backgroundColor, sbBottomContour, 127);
		}
		const QColor color3 = alphaBlendColors(QColor(0,0,0), sbTopContour, 120);
		const QColor color4 = alphaBlendColors(QColor(0,0,0), sbBottomContour, 120);
		
		sbTopContour = alphaBlendColors(QColor(41,41,47), sbTopContour, 100);
		sbBottomContour = alphaBlendColors(QColor(41,41,47), sbBottomContour, 100);

		if(drawTop) {
			p->setPen(color1);
			p->drawLine(r.left()+2, r.top(), r.right()-2, r.top()); // top
			p->setPen(sbTopContour);
			p->drawLine(r.left()+2, r.top()+1, r.right()-2, r.top()+1); // top
		}
		if(drawBottom) {
			p->setPen(sbBottomContour);
			p->drawLine(r.left()+2, r.bottom()-1, r.right()-2, r.bottom()-1); // bottom
			p->setPen(color1);
			p->drawLine(r.left()+2, r.bottom(), r.right()-2, r.bottom()); // bottom
		}
	
		if(drawLeft) {
			p->setPen(color1);
			p->drawLine(r.left(), r.top()+2, r.left(), r.bottom()-2); // left
			p->setPen(sbTopContour);
			p->drawLine(r.left()+1, r.top()+2, r.left()+1, r.bottom()-2); // left
		}
		if(drawRight) {
			p->setPen(sbBottomContour);
			p->drawLine(r.right()-1, r.top()+2, r.right()-1, r.bottom()-2); // right
			p->setPen(color1);
			p->drawLine(r.right(), r.top()+2, r.right(), r.bottom()-2); // right
		}
		if(roundUpperLeft) {

			//p->eraseRect(r.x(), r.y(), 2, 2);
			
			p->setPen(color3);
			p->drawPoint(r.left()+1, r.top()+1);
			int alphaAA;
			if(isComboPopup) {
				alphaAA = 100;
				renderPixel(p,QPoint(r.x(), r.y()),alphaAA, color1, color5, false);
				alphaAA = 200;
				renderPixel(p,QPoint(r.x()+1, r.y()),alphaAA, color1, color5, false);
				renderPixel(p,QPoint(r.x(), r.y()+1),alphaAA, color1, color5, false);
			}
			else {
				alphaAA = 100;
				renderPixel(p,QPoint(r.x(), r.y()),alphaAA, color1, color1, true);
				alphaAA = 200;
				renderPixel(p,QPoint(r.x()+1, r.y()),alphaAA, color1, color1, true);
				renderPixel(p,QPoint(r.x(), r.y()+1),alphaAA, color1, color1, true);
			}
			
		}
		if(roundUpperRight) {
			
			//p->eraseRect(r.width()-2, r.y(), 2, 2);
			
			p->setPen(color4);
			p->drawPoint(r.right()-1, r.top()+1);
			int alphaAA;
			if(isComboPopup) {
				p->setPen(color5);
				p->drawPoint(r.right(), r.y());
				p->drawPoint(r.right(), r.y()+1);
				p->drawPoint(r.right()-1, r.y());
				
				alphaAA = 120;
				renderPixel(p,QPoint(r.right(), r.y()),alphaAA, color1, color1, true);
				alphaAA = 200;
				renderPixel(p,QPoint(r.right()-1, r.y()),alphaAA, color1, color1, true);
				renderPixel(p,QPoint(r.right(), r.y()+1),alphaAA, color1, color1, true);
			}
			else {
				alphaAA = 120;
				renderPixel(p,QPoint(r.right(), r.y()),alphaAA, color1, color1, true);
				alphaAA = 200;
				renderPixel(p,QPoint(r.right()-1, r.y()),alphaAA, color1, color1, true);
				renderPixel(p,QPoint(r.right(), r.y()+1),alphaAA, color1, color1, true);
			}
			
		}
		if(roundBottomLeft) {
			p->setPen(color3);
			p->drawPoint(r.x()+1, r.bottom()-1);
			int alphaAA;
			if(isComboPopup) {
				alphaAA = 120;
				renderPixel(p,QPoint(r.x(), r.bottom()),alphaAA, color1, color5, false);
				alphaAA = 170;
				renderPixel(p,QPoint(r.x()+1, r.bottom()),alphaAA, color1, color5, false);
				renderPixel(p,QPoint(r.x(), r.bottom()-1),alphaAA, color1, color5, false);
			}
			else {
				alphaAA = 120;
				renderPixel(p,QPoint(r.x(), r.bottom()),alphaAA, color1, color1, true);
				alphaAA = 200;
				renderPixel(p,QPoint(r.x()+1, r.bottom()),alphaAA, color1, color1, true);
				renderPixel(p,QPoint(r.x(), r.bottom()-1),alphaAA, color1, color1, true);
			}

			
		}
		if(roundBottomRight) {
			p->setPen(color4);
			p->drawPoint(r.right()-1, r.bottom()-1);
			int alphaAA;
			if(isComboPopup) {
				alphaAA = 120;
				renderPixel(p,QPoint(r.right(), r.bottom()),alphaAA, color1, color5, false);
				alphaAA = 170;
				renderPixel(p,QPoint(r.right()-1, r.bottom()),alphaAA, color1, color5, false);
				renderPixel(p,QPoint(r.right(), r.bottom()-1),alphaAA, color1, color5, false);
			}
			else {
				alphaAA = 120;
				renderPixel(p,QPoint(r.right(), r.bottom()),alphaAA, color1, color1, true);
				alphaAA = 200;
				renderPixel(p,QPoint(r.right()-1, r.bottom()),alphaAA, color1, color1, true);
				renderPixel(p,QPoint(r.right(), r.bottom()-1),alphaAA, color1, color1, true);
			}
		}
	}
	return;
}

void DominoStyle::renderSurface(QPainter *p,
								 const QRect &r,
								 const QColor &backgroundColor,
								 const QColor &buttonColor,
								 const QColor &/*highlightColor*/,
								 const uint flags) const
{
	
	if((r.width() <= 0)||(r.height() <= 0))
		return;
	
	const DSurface* ds = 0;
	const bool isButton = flags&Is_Button;
	const bool isScrollBarBtn = flags&Is_ScrollBarBtn;
	const bool isScrollBarGroove = flags&Is_ScrollBarGroove;
	const bool isCheckItem = flags&Is_CheckItem;
	const bool isHeader = flags&Is_Header;
	const bool isTabTop = flags&Is_TabTop;
	const bool isActiveTabTop = flags&Is_ActiveTabTop;
	const bool isTabBottom = flags&Is_TabBottom;
	const bool isActiveTabBottom = flags&Is_ActiveTabBottom;
	const bool horizontal = flags&Is_Horizontal;
	
	if(isButton)
		ds = &btnSurface;
	else if(isScrollBarBtn)
		ds = &scrollBarSurface;
	else if(isScrollBarGroove)
		ds = &scrollBarGrooveSurface;
	else if(isCheckItem)
		ds = &checkItemSurface;
	else if(isHeader)
		ds = &headerSurface;
	else if(isTabTop)
		ds = &tabTopSurface;
	else if(isActiveTabTop)
		ds = &activeTabTopSurface;
	else if(isTabBottom)
		ds = &tabBottomSurface;
	else if(isActiveTabBottom)
		ds = &activeTabBottomSurface;
	
	QColor g1Color1;
	QColor g1Color2;
	QColor g2Color1;
	QColor g2Color2;
	QColor background;

	if(buttonColor != qApp->palette().active().button()) {
		g2Color1 = alphaBlendColors(buttonColor, ds->g2Color1, 127);
		g2Color2 = alphaBlendColors(buttonColor, ds->g2Color2, 127);
		g1Color1 = alphaBlendColors(buttonColor, ds->g1Color1, 127);
		g1Color2 = alphaBlendColors(buttonColor, ds->g1Color2, 127);
		background = alphaBlendColors(buttonColor, ds->background, 127);
	}
	else {
		g1Color1 = ds->g1Color1;
		g1Color2 = ds->g1Color2;
		g2Color1 = ds->g2Color1;
		g2Color2 = ds->g2Color2;
		background = ds->background;
	}
	if(flags&Is_Down) {
		g1Color1 = darkenColor(g1Color1, 10);
		g1Color2 = darkenColor(g1Color2, 10);
		g2Color1 = darkenColor(g2Color1, 10);
		g2Color2 = darkenColor(g2Color2, 10);
		background = darkenColor(background, 10);
	}
	
	
	if(isButton) { // button / toolButton / comboBox / spinWidget / slider / progressBar
		
		if(!flatMode) {
			QRegion mask(r.x()+3, r.y()+2, r.width()-6, r.height()-5);
			mask += QRegion(r.x()+2, r.y()+3, r.width()-4, r.height()-7);
			mask -= p->clipRegion();
			p->setClipRegion(mask, QPainter::CoordPainter);
		}
		
		
		QRect re = r;
		if(re.x() < 0 ) {
			re.setWidth(re.width()+QABS(re.x()));
			re.setX(0);
		}
		else if(re.y() < 0 ) {
			re.setHeight(re.height()+QABS(re.y()));
			re.setY(0);
		}

		QPixmap pix(re.width(), re.height());
		pix.fill(background);
		QPainter painter(&pix);

		int top = re.height()*btnSurface.g1Top/100;
		int bottom = re.height()*btnSurface.g1Bottom/100;
		int top2 = re.height()*btnSurface.g2Top/100;
		int bottom2 = re.height()*btnSurface.g2Bottom/100-top2;

		int left = re.width()*btnSurface.g1Top/100;
		int width = re.width()*btnSurface.g1Bottom/100+1;
		int left2 = re.width()*btnSurface.g2Top/100+1;
		int width2 = re.width()*btnSurface.g2Bottom/100-left2;

		switch (btnSurface.numGradients) {
			case 1:{
				if(horizontal) {
					renderGradient(&painter, QRect(0, top, re.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				}
				else {
					renderGradient(&painter, QRect(left, re.top(), width, re.height()), g1Color1.rgb(), g1Color2.rgb(), false);
				}
				break;
			}
			case 2: {
				if(horizontal) {
					renderGradient(&painter, QRect(0, top, re.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
					renderGradient(&painter, QRect(0, top2, re.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
				}
				else {
					renderGradient(&painter, QRect(left, re.top(), width, re.height()), g1Color1.rgb(), g1Color2.rgb(), false);
					renderGradient(&painter, QRect(left2, re.top(), width2, re.height()), g2Color1.rgb(), g2Color2.rgb(), false);
				}
			}
			default: break;
		}
		
		p->drawPixmap(r.x(), r.y(), pix);
		p->setClipping(false);

	}
	else if(isTabTop || isActiveTabTop || isTabBottom || isActiveTabBottom) {
		int top = r.y()+r.height()*ds->g1Top/100;
		int bottom = r.height()*ds->g1Bottom/100;
		int top2 = r.y()+r.height()*ds->g2Top/100;
		int bottom2 = r.height()*ds->g2Bottom/100-top2;
		
		p->fillRect(r, background);
		switch (ds->numGradients) {
			case 1:{
				renderGradient(p, QRect(r.x(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				break;
			}
			case 2: {
				renderGradient(p, QRect(r.x(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				renderGradient(p, QRect(r.x(), top2, r.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
			}
			default: break;
		}
		
	}
	else if(isScrollBarBtn) {

		int top = r.height()*scrollBarSurface.g1Top/100;
		int bottom = r.height()*scrollBarSurface.g1Bottom/100+1;
		int top2 = r.height()*scrollBarSurface.g2Top/100+1;
		int bottom2 = r.height()*scrollBarSurface.g2Bottom/100-top2;
		int left = r.width()*scrollBarSurface.g1Top/100;
		int width = r.width()*scrollBarSurface.g1Bottom/100+1;
		int left2 = r.width()*scrollBarSurface.g2Top/100+1;
		int width2 = r.width()*scrollBarSurface.g2Bottom/100-left2;
		
		switch (scrollBarSurface.numGradients) {
			case 1: {
				if(horizontal) {
					renderGradient(p, QRect(r.left(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
					if(p->hasClipping())
						p->setClipRegion(p->clipRegion()-QRegion(r.left(), top, r.width(), bottom));
					else
						p->setClipRegion(QRegion(r)-QRegion(r.left(), top, r.width(), bottom));
				}
				else {
					renderGradient(p, QRect(left, r.top(), width, r.height()), g1Color1.rgb(), g1Color2.rgb(), false);
					if(p->hasClipping())
						p->setClipRegion(p->clipRegion()-QRegion(left, r.top(), width, r.height()));
					else
						p->setClipRegion(QRegion(r)-QRegion(left, r.top(), width, r.height()));
				}
				break;
			}
			case 2: {
				if(horizontal) {
					renderGradient(p, QRect(r.left(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
					renderGradient(p, QRect(r.left(),  top2, r.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
					if(p->hasClipping())
						p->setClipRegion(p->clipRegion()-(QRegion(r.left(), top, r.width(), bottom)+QRegion(r.left(), top2, r.width(), bottom2)));
					else
						p->setClipRegion(QRegion(r)-(QRegion(r.left(), top, r.width(), bottom)+QRegion(r.left(), top2, r.width(), bottom2)));
				}
				else {
					renderGradient(p, QRect(left, r.top(), width, r.height()), g1Color1.rgb(), g1Color2.rgb(), false);
					renderGradient(p, QRect(left2, r.top(), width2, r.height()), g2Color1.rgb(), g2Color2.rgb(), false);
					if(p->hasClipping())
						p->setClipRegion(p->clipRegion()-(QRegion(left, r.top(), width, r.height())+QRegion(left2, r.top(), width2, r.height())));
					else
						p->setClipRegion(QRegion(r)-(QRegion(left, r.top(), width, r.height())+QRegion(left2, r.top(), width2, r.height())));
				}
			}
			default:
				break;
		}
		p->fillRect(r, background);
		p->setClipping(false);
	}
	else if(isScrollBarGroove) {

		int top = r.height()*scrollBarGrooveSurface.g1Top/100;
		int bottom = r.height()*scrollBarGrooveSurface.g1Bottom/100+1;
		int top2 = r.height()*scrollBarGrooveSurface.g2Top/100+1;
		int bottom2 = r.height()*scrollBarGrooveSurface.g2Bottom/100-top2;
		int left = r.width()*scrollBarGrooveSurface.g1Top/100;
		int width = r.width()*scrollBarGrooveSurface.g1Bottom/100+1;
		int left2 = r.width()*scrollBarGrooveSurface.g2Top/100+1;
		int width2 = r.width()*scrollBarGrooveSurface.g2Bottom/100-left2;
		
		switch (scrollBarGrooveSurface.numGradients) {
			case 1: {
				if(horizontal) {
					renderGradient(p, QRect(r.left(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
					p->setClipRegion(QRegion(r)-QRegion(r.left(), top, r.width(), bottom));
				}
				else {
					renderGradient(p, QRect(left, r.top(), width, r.height()), g1Color1.rgb(), g1Color2.rgb(), false);
					p->setClipRegion(QRegion(r)-QRegion(left, r.top(), width, r.height()));
				}
				break;
			}
			case 2: {
				if(horizontal) {
					renderGradient(p, QRect(r.left(), top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
					renderGradient(p, QRect(r.left(),  top2, r.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
					p->setClipRegion(QRegion(r)-(QRegion(r.left(), top, r.width(), bottom)+QRegion(r.left(), top2, r.width(), bottom2)));
				}
				else {
					renderGradient(p, QRect(left, r.top(), width, r.height()), g1Color1.rgb(), g1Color2.rgb(), false);
					renderGradient(p, QRect(left2, r.top(), width2, r.height()), g2Color1.rgb(), g2Color2.rgb(), false);
					p->setClipRegion(QRegion(r)-(QRegion(left, r.top(), width, r.height())+QRegion(left2, r.top(), width2, r.height())));
				}
			}
			default:
				break;
		}
		p->fillRect(r, background);
		p->setClipping(false);

	}
	else if(isCheckItem) {
		
		QPixmap pix(r.width(), r.height());
		pix.fill(background);
		QPainter painter(&pix);

		int top = r.height()*checkItemSurface.g1Top/100;
		int bottom = r.height()*checkItemSurface.g1Bottom/100+1;
		int top2 = r.height()*checkItemSurface.g2Top/100+1;
		int bottom2 = r.height()*checkItemSurface.g2Bottom/100-top2;
		
		switch(checkItemSurface.numGradients) {
			case 1:{
				renderGradient(&painter, QRect(0, top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				break;
			}
			case 2: {
				renderGradient(&painter, QRect(0, top, r.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				renderGradient(&painter, QRect(0, top2, r.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
				break;
			}
			default: break;
		}
		painter.end();
		p->drawPixmap(QRect(r.x(), r.y()+3, r.width(), r.height()-7), pix);
		
	}
	else if(isHeader) {
		
		QHeader* hdr = dynamic_cast<QHeader*>(p->device());
		bool needErasePixmap = false;
		QRect re = r;
		bool horizontal = false;
		if(hdr && hdr->orientation() == Qt::Horizontal) {
			horizontal = true;
			if(hdr->erasePixmap()) {
				hdr->erase(r);
				return;
			}
			else {
				re = QRect(0, 0, 10, r.height());
				needErasePixmap = true;
			}
		}
		QPixmap pix(re.width(), re.height());
		pix.fill(background);
		QPainter painter(&pix);
		
		int top = re.height()*headerSurface.g1Top/100;
		int bottom = re.height()*headerSurface.g1Bottom/100;
		int top2 = re.height()*headerSurface.g2Top/100;
		int bottom2 = re.height()*headerSurface.g2Bottom/100-top2;
		
		switch (headerSurface.numGradients) {
			case 1:{
				renderGradient(&painter, QRect(0, top, re.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				break;
			}
			case 2: {
				renderGradient(&painter, QRect(0, top, re.width(), bottom), g1Color1.rgb(), g1Color2.rgb(), true);
				renderGradient(&painter, QRect(0, top2, re.width(), bottom2), g2Color1.rgb(), g2Color2.rgb(), true);
				break;
			}
			default: break;
		}
		

		painter.setPen(backgroundColor);
		if(horizontal) painter.drawLine(re.x(), re.y(), re.width(), re.y());
		painter.drawLine(re.x(), re.height()-1, re.width(), re.height()-1);
		
		if(needErasePixmap) {
			hdr->setUpdatesEnabled(false);
			hdr->setErasePixmap(pix);
			hdr->setUpdatesEnabled(true);
			hdr->erase(r);
		}
		else {
			p->drawPixmap(r.x(), r.y(), pix);
		}
	}
}


void DominoStyle::renderPixel(QPainter *p,
			const QPoint &pos,
			const int alpha,
			const QColor &color,
			const QColor &background,
			bool fullAlphaBlend) const
{	 
	if(fullAlphaBlend)
	// full alpha blend: paint into an image with alpha buffer and convert to a pixmap ...
	{
		QRgb rgb = color.rgb();
		// generate a quite unique key -- use the unused width field to store the alpha value.
		CacheEntry search(cAlphaDot, alpha, 0, rgb);
		int key = search.key();

		CacheEntry *cacheEntry;
		if( (cacheEntry = pixmapCache->find(key)) ) {
			if( search == *cacheEntry ) { // match! we can draw now...
				if(cacheEntry->pixmap)
					p->drawPixmap(pos, *(cacheEntry->pixmap) );
				return;
			} else { //Remove old entry in case of a conflict!
				pixmapCache->remove( key );
			}
		}


		QImage aImg(1,1,32); // 1x1
		aImg.setAlphaBuffer(true);
		aImg.setPixel(0,0,qRgba(qRed(rgb),qGreen(rgb),qBlue(rgb),alpha));
		QPixmap *result = new QPixmap(aImg);

		p->drawPixmap(pos, *result);

		// add to the cache...
		CacheEntry *toAdd = new CacheEntry(search);
		toAdd->pixmap = result;
		bool insertOk = pixmapCache->insert( key, toAdd, result->depth()/8);
		if(!insertOk)
			delete result;
	} else
	// don't use an alpha buffer: calculate the resulting color from the alpha value, the fg- and the bg-color.
	{
		QRgb rgb_a = color.rgb();
		QRgb rgb_b = background.rgb();
		int a = alpha;
		if(a>255) a = 255;
		if(a<0) a = 0;
		int a_inv = 255 - a;

		QColor res	= QColor( qRgb(qRed(rgb_b)*a_inv/255 + qRed(rgb_a)*a/255,
							  qGreen(rgb_b)*a_inv/255 + qGreen(rgb_a)*a/255,
							  qBlue(rgb_b)*a_inv/255 + qBlue(rgb_a)*a/255) );
		p->setPen(res);
		p->drawPoint(pos);
	} 
}

void DominoStyle::renderButtonShadow(QPainter *p, const QRect &r, const uint flags) const {

	if(flags&Round_UpperLeft) {
		bitBlt(p->device(), r.left(), r.top(), buttonShadowRound, 0, 0, 3, 4, Qt::CopyROP);
	}
	else if(flags&Square_UpperLeft){
		bitBlt(p->device(), r.left(), r.top(), buttonShadowSquare, 0, 0, 3, 4, Qt::CopyROP);
	}
	if(flags&Round_UpperRight) {
		bitBlt(p->device(), r.right()-4, r.top(), buttonShadowRound, 3, 0, 3, 4, Qt::CopyROP);
	}
	else if(flags&Square_UpperRight){
		bitBlt(p->device(), r.right()-4, r.top(), buttonShadowSquare, 3, 0, 3, 4, Qt::CopyROP);
	}
	if(flags&Round_BottomLeft) {
		bitBlt(p->device(), r.left(), r.bottom()-4, buttonShadowRound, 0, 4, 3, 3, Qt::CopyROP);
	}
	else if(flags&Square_BottomLeft)
		bitBlt(p->device(), r.left(), r.bottom()-4, buttonShadowSquare, 0, 4, 3, 3, Qt::CopyROP);
	if(flags&Round_BottomRight) {
		bitBlt(p->device(), r.right()-4, r.bottom()-4, buttonShadowRound, 3, 4, 3, 3, Qt::CopyROP);
	}
	else if(flags&Square_BottomRight)
		bitBlt(p->device(), r.right()-4, r.bottom()-4, buttonShadowSquare, 3, 4, 3, 3, Qt::CopyROP);
	if(flags&Draw_Top)
		p->drawTiledPixmap(r.left()+3, r.top(), r.width()-8, 4, *buttonShadowHlines, 0, 0);
	if(flags&Draw_Left)
		p->drawTiledPixmap(r.left(), r.top()+4, 3, r.height()-9, *buttonShadowVlines, 0, 0);
	if(flags&Draw_Right)
		p->drawTiledPixmap(r.right()-4, r.top()+4, 3, r.height()-9, *buttonShadowVlines, 3, 0);
	if(flags&Draw_Bottom)
		p->drawTiledPixmap(r.left()+3, r.bottom()-4, r.width()-8, 3, *buttonShadowHlines, 0, 4);
	
}

void DominoStyle::renderButton(QPainter *p,
							   const QRect &r,
							   const QColorGroup &g,
							   bool sunken,
							   bool /*mouseOver*/,
							   bool horizontal,
							   bool enabled,
							   bool khtmlMode) const
{

	const QPen oldPen( p->pen() );

	uint contourFlags = Draw_Left|Draw_Right|Draw_Top|Draw_Bottom;

	if(!enabled) contourFlags|=Is_Disabled;
	
	if(khtmlMode || kickerMode)
		contourFlags|=Draw_AlphaBlend;

	uint surfaceFlags = Is_Button;
	if(horizontal) {
		 surfaceFlags|=Is_Horizontal;
		 contourFlags|=Is_Horizontal;
	}
	if(!enabled) surfaceFlags|=Is_Disabled;
	else {
		if(sunken) surfaceFlags|=Is_Down;
	}

	QColor buttonColor = khtmlMode ? g.button() : qApp->palette().active().background();
	
	if (!flatMode) {
		contourFlags |= Round_UpperLeft|Round_UpperRight|Round_BottomLeft|Round_BottomRight|Is_Button;
		renderSurface(p, QRect(r.x(), r.y(), r.width(), r.height()), g.background(), buttonColor, g.button(), surfaceFlags);
		renderContour(p, r, g.background(), buttonColor, contourFlags);
		if(sunken) {
			renderButtonShadow(p, QRect(r.x()+2, r.y()+2, r.width()-2, r.height()-3), contourFlags);
		}
	}
	else {
		renderSurface(p, QRect(r.x()-3, r.y()-1, r.width()+6, r.height()+2), g.background(), buttonColor, g.button(), surfaceFlags);

		if(sunken) {
			contourFlags |= Square_UpperLeft|Square_UpperRight|Square_BottomLeft|Square_BottomRight;
			renderButtonShadow(p, QRect(r.x()-2, r.y()-2, r.width()+4, r.height()+4), contourFlags);
		}
	}
	p->setPen(oldPen);
}

void DominoStyle::renderHeader(QPainter *p,
								const QRect &r,
								const QColorGroup &g,
								bool /*sunken*/,
								bool /*mouseOver*/,
								bool /*horizontal*/,
								bool /*enabled*/) const
{
	QHeader* hd = dynamic_cast<QHeader*>(p->device());
	int section = hd ? hd->sectionAt(r.left()) : 0;

	uint surfaceFlags = Is_Header|Is_Horizontal;
	renderSurface(p, QRect(r.left(), r.top(), r.width(), r.height()), g.button(), qApp->palette().active().background(), g.button(), surfaceFlags);

	p->setPen(g.background());

	if(section == 0)
		p->drawLine(r.left(), r.top(), r.left(), r.bottom());
	p->drawLine(r.right(), r.top(), r.right(), r.bottom());
	
}


void DominoStyle::renderGradient(QPainter *painter,
								  const QRect &rect,
								  const QColor &c1,
								  const QColor &c2,
								  bool horizontal) const
{
	if((rect.width() <= 0)||(rect.height() <= 0))
		return;

	// generate a quite unique key for this surface.
	CacheEntry search(cGradientTile,
					  horizontal ? 0 : rect.width(),
					  horizontal ? rect.height() : 0,
					  c1.rgb(), c2.rgb(), horizontal );
	int key = search.key();

	CacheEntry *cacheEntry;
	if( (cacheEntry = pixmapCache->find(key)) ) {
		if( search == *cacheEntry ) { // match! we can draw now...
			if(cacheEntry->pixmap) {
				painter->drawTiledPixmap(rect, *(cacheEntry->pixmap) );
			}
			return;
		} else {
			// Remove old entry in case of a conflict!
			// This shouldn't happen very often, see comment in CacheEntry.
			pixmapCache->remove(key);
		}
	}

	// there wasn't anything matching in the cache, create the pixmap now...
	QPixmap *result = new QPixmap(horizontal ? 10 : rect.width(),
								  horizontal ? rect.height() : 10);
	QPainter p(result);

	int r_w = result->rect().width();
	int r_h = result->rect().height();
	int r_x, r_y, r_x2, r_y2;
	result->rect().coords(&r_x, &r_y, &r_x2, &r_y2);

	int rDiff, gDiff, bDiff;
	int rc, gc, bc;

	register int x, y;

	rDiff = ( c2.red())	  - (rc = c1.red());
	gDiff = ( c2.green()) - (gc = c1.green());
	bDiff = ( c2.blue())  - (bc = c1.blue());

	register int rl = rc << 16;
	register int gl = gc << 16;
	register int bl = bc << 16;

	int rdelta = ((1<<16) / (horizontal ? r_h : r_w)) * rDiff;
	int gdelta = ((1<<16) / (horizontal ? r_h : r_w)) * gDiff;
	int bdelta = ((1<<16) / (horizontal ? r_h : r_w)) * bDiff;

	// these for-loops could be merged, but the if's in the inner loop
	// would make it slow
	if(horizontal) {
		for ( y = 0; y < r_h; y++ ) {
			rl += rdelta;
			gl += gdelta;
			bl += bdelta;

			p.setPen(QColor(rl>>16, gl>>16, bl>>16));
			p.drawLine(r_x, r_y+y, r_x2, r_y+y);
		}
	} else {
		for( x = 0; x < r_w; x++) {
			rl += rdelta;
			gl += gdelta;
			bl += bdelta;

			p.setPen(QColor(rl>>16, gl>>16, bl>>16));
			p.drawLine(r_x+x, r_y, r_x+x, r_y2);
		}
	}

	p.end();

	// draw the result...
	painter->drawTiledPixmap(rect, *result);

	// insert into cache using the previously created key.
	CacheEntry *toAdd = new CacheEntry(search);
	toAdd->pixmap = result;
	bool insertOk = pixmapCache->insert( key, toAdd, result->width()*result->height()*result->depth()/8 );

	if(!insertOk)
		delete result;
}

void DominoStyle::renderPanel(QPainter *p,
							  const QRect &r,
							  const QColorGroup &g,
							  const bool pseudo3d,
							  const bool sunken) const
{
	int x, x2, y, y2, w, h;
	r.rect(&x,&y,&w,&h);
	r.coords(&x, &y, &x2, &y2);

	if (kickerMode &&
			p->device() && p->device()->devType() == QInternal::Widget &&
			QCString(static_cast<QWidget*>(p->device())->className()) == "FittsLawFrame") {
	//	Stolen wholesale from Keramik. I don't like it, but oh well.
		if (sunken) {
			const QCOORD corners[] = { x2, y, x2, y2, x, y2, x, y };
			p->setPen(g.background().dark());
			p->drawConvexPolygon(QPointArray(4, corners));
			p->setPen(g.background().light());
			p->drawPolyline(QPointArray(4, corners), 0, 3);
		} else {
			const QCOORD corners[] = { x, y2, x, y, x2, y, x2, y2 };
			p->setPen(g.background().dark());
			p->drawPolygon(QPointArray(4, corners));
			p->setPen(g.background().light());
			p->drawPolyline(QPointArray(4, corners), 0, 3);
		}
	} else {
		renderContour(p, r, g.background(), getColor(g, PanelContour) );

		if(pseudo3d) {
			if (sunken) {
				p->setPen(getColor(g, PanelDark) );
			} else {
				p->setPen(getColor(g, PanelLight) );
			}
			p->drawLine(r.left()+2, r.top()+1, r.right()-2, r.top()+1);
			p->drawLine(r.left()+1, r.top()+2, r.left()+1, r.bottom()-2);
			if (sunken) {
				p->setPen(getColor(g, PanelLight) );
			} else {
				p->setPen(getColor(g, PanelDark) );
			}

			p->setPen(g.background().dark(128) );
			p->drawLine(r.right()-1, r.top()+2, r.right()-1, r.bottom()-2);
			p->drawLine(r.x()+1, r.top()+2, r.x()+1, r.bottom()-2);
			p->fillRect(r.x()-10, r.top()+2, 10, r.bottom()-2, black);
			//p->drawLine(10, r.top()+2, 15, r.bottom()-2);
			
		}
	}
}


void DominoStyle::renderTab(QPainter *p,
							const QRect &r,
							const QColorGroup &g,
							bool /*mouseOver*/,
							const bool selected,
							const bool bottom,
							const TabPosition pos,
							const bool /*triangular*/,
							const bool /*cornerWidget*/,
							const bool konqTab) const
{
	
	const bool isFirst = (pos == First);
	const bool isLast = (pos == Last);

// 	int x = 0;
	int y = 0;
	int w = r.width();
	int h = r.height();
	
	QPixmap buffer(r.width(), r.height());
	QPainter painter;
	painter.begin(&buffer);
	
	QColor tabContour2 = g.background().dark(150);
	QPixmap* tabEdges;
	if((tabEdges = QPixmapCache::find(tabContour2.name()+"tabEdges"))) {
	}
	else {
		tabEdges = new QPixmap(tintImage(uic_findImage("border1"), tabContour2));
		QPixmapCache::insert(tabContour2.name()+"tabEdges", tabEdges);
	}
	QColor buttonColor = qApp->palette().active().background();
	
	if(konqTab) {
		if (selected) { // tabBarContrast = 150
			
			QRegion mask(0,0,w,h-2);
			mask -= QRegion(1,h-5,1,1);
			mask -= QRegion(1,h-4,2,1);
			mask -= QRegion(1,h-3,3,1);
			mask -= QRegion(w-2,h-5,1,3);
			mask -= QRegion(w-3,h-4,1,1);
			mask -= QRegion(w-4,h-3,3,1);
			
			painter.fillRect(QRect(0,r.bottom()-7, 7, 7), g.background().dark(tabBarContrast-20));
			painter.fillRect(QRect(r.width()-7, r.bottom()-7, 7, 7), g.background().dark(tabBarContrast-20));
			painter.fillRect(0, 0, r.width(), r.height()-9, g.background()); // rect above gradient
			
			painter.setClipRegion(mask);
			painter.translate(0, 2);
			renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-2), g.background(), buttonColor, g.button(), Is_ActiveTabBottom);
			painter.translate(0, -2);
			painter.setClipping(false);

			bitBlt(&buffer, 0, r.bottom()-7, tabEdges, 0, 7, 7, 7, Qt::CopyROP);
			bitBlt(&buffer, r.width()-7, r.bottom()-7, tabEdges, 7, 7, 7, 7, Qt::CopyROP);
			QPixmap inside = uic_findImage("tabInsideWhite");
			bitBlt(&buffer, 0, r.bottom()-7, &inside, 0, 7, 7, 7, Qt::CopyROP);
			bitBlt(&buffer, r.width()-7, r.bottom()-7, &inside, 7, 7, 7, 7, Qt::CopyROP);
			
			painter.setPen(g.background().dark(110)); // . helle linie 104
			painter.drawPoint(0, 1);
			painter.drawPoint(r.width()-1, 1);
			
			painter.setPen(g.background().dark(tabBarContrast-10)); // 2. helle linie 120
			painter.drawPoint(0, 2);
			painter.drawPoint(r.width()-1, 2);
			QColor topColor = (activeTabBottomSurface.g1Top == 0 && activeTabBottomSurface.numGradients > 0) ? activeTabBottomSurface.g1Color1 : activeTabBottomSurface.background;
			painter.setPen(alphaBlendColors(buttonColor, topColor, 127));
			painter.drawLine(1, 1, r.width()-2, 1); // top line
			
			painter.setPen(g.background().dark(tabBarContrast)); //QColor(110,110,110)) aka dunkel
			painter.drawLine(6, r.bottom()-1, r.width()-7, r.bottom()-1); // bottom-line
			painter.drawLine(0, 3, 0, r.bottom()-7); // left-line
			painter.drawPoint(r.width(), 2);
			painter.drawLine(r.width()-1, 3, r.width()-1, r.bottom()-7); // right-line
			painter.setPen(g.background().dark(tabBarContrast-20)); // g.background().dark(125)
			painter.drawLine(0, r.bottom(), r.width(), r.bottom());
			
			QPixmap insideL = uic_findImage("tabInsideWhiteLines");
			painter.drawTiledPixmap(1, 2, 1, r.bottom()-7-2, insideL, 0, 0);
			painter.drawTiledPixmap(r.width()-2, 2, 1, r.bottom()-7-2, insideL, 0, 0);
			
			
			p->drawPixmap(r.left(), r.top(), buffer);
			
		}
		else {
			painter.fillRect(QRect(0, 3, r.width(), r.height()-7), g.background().dark(tabBarContrast-40));
			painter.fillRect(QRect(0, r.height()-10, r.width(), r.height()), g.background().dark(tabBarContrast-20));
			
			painter.setPen(g.background());// 0. tabbarline
			painter.drawLine(0, 0, r.width(), 0);
			painter.setPen(g.background().dark(120));//QColor(110,110,110) // 1. tabbarline
			painter.drawLine(0, 1, r.width(), 1);
			painter.setPen(g.background().dark(tabBarContrast-30)); //QColor(110,110,110)
			if(isFirst) {
				
				QRegion mask(0,0,w,h);
				mask -= QRegion(1,h-8,1,1);
				mask -= QRegion(1,h-7,2,1);
				mask -= QRegion(1,h-6,3,1);
				
				
				painter.setPen(g.background().dark(tabBarContrast)); // QColor(80,80,80)
				painter.drawLine(0, 2, r.width(), 2);
				painter.setPen(g.background().dark(tabBarContrast)); //QColor(110,110,110)
				painter.drawLine(6, r.height()-5, r.width(), r.height()-5); // bottom
				painter.drawLine(0, 3, 0, r.height()-11); // left
				painter.drawLine(r.width()-1, r.top()+3, r.width()-1, r.height()-5); // right
				
				painter.setClipRegion(mask);
				painter.translate(0, 3);
				renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5-3), g.background(), buttonColor, g.button(), Is_TabBottom);
				painter.translate(0, -3);
				painter.setClipping(false);
				
				bitBlt(&buffer, 0, r.height()-11, tabEdges, 0, 7, 7, 7, Qt::CopyROP);
				QPixmap inside = uic_findImage("tabInsideWhite");
				bitBlt(&buffer, 0, r.height()-11, &inside, 0, 7, 7, 7, Qt::CopyROP);
				QPixmap insideL = uic_findImage("tabInsideWhiteLines");
				painter.drawTiledPixmap(1, 3, 1, r.height()-12-2, insideL, 0, 0);
				
				p->drawPixmap(r.left(), r.top(), buffer);
			}
			else if(isLast) {
				
				QRegion mask(0,0,w,h);
				mask -= QRegion(w-2,h-8,1,3);
				mask -= QRegion(w-3,h-7,1,1);
				mask -= QRegion(w-4,h-6,3,1);
				
				painter.setPen(g.background().dark(tabBarContrast)); // QColor(80,80,80) - rahmen oben
				painter.drawLine(0, 2, r.width(), 2);
				painter.setPen(g.background().dark(tabBarContrast)); //QColor(110,110,110) // dark(120) - rest
				painter.drawLine(0, r.height()-5, r.width()-7, r.height()-5); // unten
				painter.drawLine(0, 3, 0, r.height()-6); // l
				painter.drawLine(r.width()-1, 3, r.width()-1, r.height()-11);// r
				
				painter.setClipRegion(mask);
				painter.translate(0, 3);
				renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5-3), g.background(), buttonColor, g.button(), Is_TabBottom);
				painter.translate(0, -3);
				painter.setClipping(false);
				
				bitBlt(&buffer, r.width()-7, r.height()-11, tabEdges, 7, 7, 7, 7, Qt::CopyROP);
				
				QPixmap insideL = uic_findImage("tabInsideWhiteLines");
				painter.drawTiledPixmap(r.width()-2, 3, 1, r.height()-12-2, insideL, 0, 0);
				QPixmap inside = uic_findImage("tabInsideWhite");
				bitBlt(&buffer, r.width()-7, r.height()-11, &inside, 7, 7, 7, 7, Qt::CopyROP);
				
				p->drawPixmap(r.left(), r.top(), buffer);
			}
			else {
				painter.setPen(g.background().dark(tabBarContrast)); // QColor(80,80,80)
				painter.drawLine(0, 2, r.width(), 2);
				painter.setPen(g.background().dark(tabBarContrast)); //QColor(110,110,110) // dark(120)
				painter.drawLine(0, r.height()-5, r.width(), r.height()-5); // bottom
				painter.drawLine(0, 3, 0, r.height()-5); // left
				painter.drawLine(r.width()-1, 3, r.width()-1, r.height()-5);// right
				painter.translate(0, 3);
				renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5-3), g.background(), buttonColor, g.button(), Is_TabBottom);
				painter.translate(0, -3);
				p->drawPixmap(r.left(), r.top(), buffer);
				
			}
		}
	}
	else { // standard tabs
		if(bottom) {
			if (selected) {
	
				QRegion mask(0,0,w,h-2);
				mask -= QRegion(1,h-5,1,3);
				mask -= QRegion(2,h-4,1,1);
				mask -= QRegion(1,h-3,3,1);
				mask -= QRegion(w-2,h-5,1,3);
				mask -= QRegion(w-3,h-4,1,1);
				mask -= QRegion(w-4,h-3,3,1);
					
				painter.fillRect(0, r.height()-7, r.width(), 7, g.background()); // rect under the tab
				painter.setClipRegion(mask);
				renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()), g.background(), buttonColor, g.button(), Is_ActiveTabBottom);
				painter.setClipping(false);
	
				bitBlt(&buffer, 0, r.bottom()-7, tabEdges, 0, 7, 7, 7, Qt::CopyROP);
				bitBlt(&buffer, r.width()-7, r.bottom()-7, tabEdges, 7, 7, 7, 7, Qt::CopyROP);
				QPixmap inside = uic_findImage("tabInsideWhite");
				bitBlt(&buffer, 0, r.bottom()-7, &inside, 0, 7, 7, 7, Qt::CopyROP);
				bitBlt(&buffer, r.width()-7, r.bottom()-7, &inside, 7, 7, 7, 7, Qt::CopyROP);
				
				painter.setPen(g.background().dark(150));
				painter.drawLine(0, 0, 0, r.bottom()-7); // left-line
				painter.drawLine(r.width()-1, 0, r.width()-1, r.bottom()-7); // right-line
				painter.drawLine(6, r.bottom()-1, r.width()-7, r.bottom()-1); // bottom-line
				
				QPixmap insideL = uic_findImage("tabInsideWhiteLines");
				painter.drawTiledPixmap(1, 0, 1, r.bottom()-7, insideL, 0, 0);
				painter.drawTiledPixmap(r.width()-2, 0, 1, r.bottom()-7, insideL, 0, 0);
				
				painter.setPen(g.background().dark(105));
				painter.drawLine(1, 0, r.width()-2, 0); // top
					
				p->drawPixmap(r.left(), r.top(), buffer);
				
			}
			else {
				painter.fillRect(QRect(0, r.height()-10, r.width(), 10), g.background());
				if(isFirst) {
					QRegion mask(0,0,w,h);
					mask -= QRegion(1,h-8,1,3);
					mask -= QRegion(2,h-7,1,1);
					mask -= QRegion(1,h-6,3,1);
					
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5), g.background(), buttonColor, g.button(), Is_TabBottom);
					painter.setClipping(false);
						
					painter.setPen(g.background().dark(125));
					painter.drawLine(0, 0, r.width()-1, 0); // top
					painter.setPen(g.background().dark(150));
					painter.drawLine(6, r.height()-5, r.width(), r.height()-5); // bottom
					painter.drawLine(0, 0, 0, r.height()-11); // left
					painter.drawLine(r.width()-1, 0, r.width()-1, r.height()-5); // right
					
					QPixmap insideL = uic_findImage("tabInsideWhiteLines");
					painter.drawTiledPixmap(1, 1, 1, r.height()-12, insideL, 0, 0);
					
					bitBlt(&buffer, 0, r.height()-11, tabEdges, 0, 7, 7, 7, Qt::CopyROP);
					QPixmap inside = uic_findImage("tabInsideWhite");
					bitBlt(&buffer, 0, r.height()-11, &inside, 0, 7, 7, 7, Qt::CopyROP);
					
					p->drawPixmap(r.left(), r.top(), buffer);
				}
				else if(isLast) {
					QRegion mask(0,0,w,h);
					mask -= QRegion(w-2,h-8,1,3);
					mask -= QRegion(w-3,h-7,1,1);
					mask -= QRegion(w-4,h-6,3,1);
						
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5), g.background(), buttonColor, g.button(), Is_TabBottom);
					painter.setClipping(false);
						
					painter.setPen(g.background().dark(125));
					painter.drawLine(0, 0, r.width()-1, 0); // top
					painter.setPen(g.background().dark(150));
					painter.drawLine(0, r.height()-5, r.width()-7, r.height()-5); // bottom
					painter.drawLine(0, 0, 0, r.height()-6); // l
					painter.drawLine(r.width()-1, 0, r.width()-1, r.height()-11);// r
					
					QPixmap insideL = uic_findImage("tabInsideWhiteLines");
					painter.drawTiledPixmap(r.width()-2, 1, 1, r.height()-12, insideL, 0, 0);
					
					bitBlt(&buffer, r.width()-7, r.height()-11, tabEdges, 7, 7, 7, 7, Qt::CopyROP);
					QPixmap inside = uic_findImage("tabInsideWhite");
					bitBlt(&buffer, r.width()-7, r.height()-11, &inside, 7, 7, 7, 7, Qt::CopyROP);
					
					p->drawPixmap(r.left(), r.top(), buffer);
				}
				else {
					painter.setPen(g.background().dark(150));
					painter.drawLine(0, r.height()-5, r.width(), r.height()-5); // bottom
					painter.drawLine(0, 0, 0, r.height()-5); // left
					painter.drawLine(r.width()-1, 0, r.width()-1, r.height()-5);// right
					
					renderSurface(&painter, QRect(1, 0, r.width()-2, r.height()-5), g.background(), buttonColor, g.button(), Is_TabBottom);
					painter.setPen(g.background().dark(125));
					painter.drawLine(0, 0, r.width()-1, 0); // top
					
					p->drawPixmap(r.left(), r.top(), buffer);
					
				}
			}
		}
		else { // above
			if (selected) {
				QRegion mask(0,0,w,h);
				mask -= QRegion(w-4,y+1,3,1);
				mask -= QRegion(w-3,y+2,1,1);
				mask -= QRegion(w-2,y+1,1,3);
				mask -= QRegion(1,y+1,1,3);
				mask -= QRegion(2,y+2,1,1);
				mask -= QRegion(1,y+1,3,1);
		
				painter.fillRect(0, 0, r.width(), 7, g.background());
				painter.setClipRegion(mask);
				renderSurface(&painter, QRect(1, 1, r.width()-2, r.height()), g.background(), buttonColor, g.button(), Is_ActiveTabTop);
				painter.setClipping(false);
					
				bitBlt(&buffer, 0, 0, tabEdges, 0, 0, 7, 7, Qt::CopyROP);
				bitBlt(&buffer, r.width()-7, 0, tabEdges, 7, 0, 7, 7, Qt::CopyROP);
				QPixmap inside = uic_findImage("tabInsideWhite");
				bitBlt(&buffer, 0, 0, &inside, 0, 0, 7, 7, Qt::CopyROP);
				bitBlt(&buffer, r.width()-7, 0, &inside, 7, 0, 7, 7, Qt::CopyROP);
				
				painter.setPen(g.background().dark(150));
				painter.drawLine(7, 0, r.width()-8, 0); // top
				painter.drawLine(0, 7, 0, r.height()-1); // left
				painter.drawLine(r.width()-1, 7, r.width()-1, r.height()-1);
				QPixmap insideL = uic_findImage("tabInsideWhiteLines");
				painter.drawTiledPixmap(7, 1, r.width()-14, 1, insideL, 0, 0);
				painter.drawTiledPixmap(1, 7, 1, r.height()-1, insideL, 0, 0);
				painter.drawTiledPixmap(r.width()-2, 7, 1, r.height()-1, insideL, 0, 0);
				p->drawPixmap(r.left(), r.top(), buffer);
			}
			else {
				painter.fillRect(0, 0, r.width(), 10, g.background());
				painter.setPen(g.background().dark(150));
				if(isFirst) {
	
					y=3;
					QRegion mask(0,0,w,h);
					mask -= QRegion(1,y+1,1,3);
					mask -= QRegion(2,y+2,1,1);
					mask -= QRegion(1,y+1,3,1);
						
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(1, 4, r.width()-2, r.height()), g.background(), buttonColor, g.button(), Is_TabTop);
					painter.setClipping(false);
						
					bitBlt(&buffer, 0, 3, tabEdges, 0, 0, 7, 7, Qt::CopyROP);
					QPixmap inside = uic_findImage("tabInsideWhite");
					bitBlt(&buffer, 0, 3, &inside, 0, 0, 7, 7, Qt::CopyROP);
					
					QPixmap insideL = uic_findImage("tabInsideWhiteLines");
					painter.drawTiledPixmap(7, 4, r.width()-1, 1, insideL, 0, 0);
					painter.drawTiledPixmap(1, 10, 1, r.height()-2, insideL, 0, 0);
					
					painter.drawLine(7, 3, r.width()-1, 3); // top
					painter.drawLine(0, 10, 0, r.height()-2); // left
					painter.drawLine(r.width()-1, 4, r.width()-1, r.height()-2);
						
					painter.setPen(g.background().dark(120));
					painter.drawLine(0, r.height()-1, r.width()-1, r.height()-1); // bottom
						
					p->drawPixmap(r.left(), r.top(), buffer);
				}
				else if(isLast) {
	
					y=3;
					QRegion mask(0,0,w,h);
					mask -= QRegion(w-4,y+1,3,1);
					mask -= QRegion(w-3,y+2,1,1);
					mask -= QRegion(w-2,y+1,1,3);
						
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(1, 4, r.width()-2, r.height()), g.background(), buttonColor, g.button(), Is_TabTop);
					painter.setClipping(false);
						
					bitBlt(&buffer, r.width()-7, 3, tabEdges, 7, 0, 7, 7, Qt::CopyROP);
					QPixmap inside = uic_findImage("tabInsideWhite");
					bitBlt(&buffer, r.width()-7, 3, &inside, 7, 0, 7, 7, Qt::CopyROP);
					
					QPixmap insideL = uic_findImage("tabInsideWhiteLines");
					painter.drawTiledPixmap(0, 4, r.width()-7, 1, insideL, 0, 0);
					painter.drawTiledPixmap(r.width()-2, 10, r.width()-2, r.height()-2, insideL, 0, 0);
					
					painter.drawLine(0, 3, r.width()-8, 3); // top
					painter.drawLine(0, 4, 0, r.height()-2); // left
					painter.drawLine(r.width()-1, 10, r.width()-1, r.height()-2); // right
					
					painter.setPen(g.background().dark(120));
					painter.drawLine(0, r.height()-1, r.width()-1, r.height()-1); // bottom
						
					p->drawPixmap(r.left(), r.top(), buffer);
				}
				else {
					renderSurface(&painter, QRect(1, 4, r.width()-2, r.height()), g.background(), buttonColor, g.button(), Is_TabTop);
					
					QPixmap insideL = uic_findImage("tabInsideWhiteLines");
					painter.drawTiledPixmap(0, 4, r.width()-1, 1, insideL, 0, 0);
					
					painter.drawLine(0, 3, r.width()-1, 3); // top
					painter.drawLine(0, 4, 0, r.height()-2); // left
					painter.drawLine(r.width()-1, 4, r.width()-1, r.height()-2); // right
						
					painter.setPen(g.background().dark(120));
					painter.drawLine(0, r.height()-1, r.width()-1, r.height()-1); // bottom
						
					p->drawPixmap(r.left(), r.top(), buffer);
				}
			}
		}
	}
}

void DominoStyle::drawKStylePrimitive(KStylePrimitive kpe,
									  QPainter *p,
									  const QWidget* widget,
									  const QRect &r,
									  const QColorGroup &cg,
									  SFlags flags,
									  const QStyleOption& opt) const
{
	//return KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);

//	SLIDER
//	------
	switch( kpe ) {
		case KPE_SliderGroove: {
			
			bool horizontal = ((const QSlider*)widget)->orientation() == Horizontal;
			
			QRect re;
			if(horizontal)
				re.setRect(r.left(), r.y()+r.height()/2-3, r.width(), 4);
			else
				re.setRect(r.x()+r.width()/2-3, r.top(), 4, r.height());
			
			QString hori = QString().setNum(horizontal);
			QPixmap* sliderGroove;
			if((sliderGroove = QPixmapCache::find(cg.background().name()+"sliderGroove"+hori))) {
			}
			else {
				const QColor c1 = alphaBlendColors(QColor(0,0,0), cg.background(), 11);
				const QColor c2 = alphaBlendColors(QColor(53,53,53), cg.background(), 155);
				const QColor c3 = alphaBlendColors(QColor(67,67,67), cg.background(), 146);
				const QColor c4 = alphaBlendColors(QColor(83,83,83), cg.background(), 140);
				const QColor c5 = alphaBlendColors(QColor(101,101,101), cg.background(), 134);
				const QColor c6 = alphaBlendColors(QColor(254,254,254), cg.background(), 43);
				const QColor c7 = alphaBlendColors(QColor(255,255,255), cg.background(), 9);
				sliderGroove = new QPixmap(7, 7);
				sliderGroove->fill(red);
				QPainter painter(sliderGroove);
				QRect rr = sliderGroove->rect();
				
				if (horizontal) {
					painter.setPen(c1);
					painter.drawLine(0, 0, 7, 0);
					painter.setPen(c2);
					painter.drawLine(0, 1, 7, 1);
					painter.setPen(c3);
					painter.drawLine(0, 2, 7, 2);
					painter.setPen(c4);
					painter.drawLine(0, 3, 7, 3);
					painter.setPen(c5);
					painter.drawLine(0, 4, 7, 4);
					painter.setPen(c6);
					painter.drawLine(0, 5, 7, 5);
					painter.setPen(c7);
					painter.drawLine(0, 6, 7, 6);
				}
				else {
					painter.setPen(c1);
					painter.drawLine(0, 0, 0, 7);
					painter.setPen(c2);
					painter.drawLine(1, 0, 1 , 7);
					painter.setPen(c3);
					painter.drawLine(2, 0, 2 , 7);
					painter.setPen(c4);
					painter.drawLine(3, 0, 3, 7);
					painter.setPen(c5);
					painter.drawLine(4, 0, 4, 7);
					painter.setPen(c6);
					painter.drawLine(5, 0, 5, 7);
					painter.setPen(c7);
					painter.drawLine(6, 0, 6, 7);
				}
				
				QPixmapCache::insert(cg.background().name()+"sliderGroove"+hori, sliderGroove);
			}
			if (horizontal) {
				QPixmap* slGh;
				if((slGh = QPixmapCache::find(cg.background().name()+"slGh"))) {
				}
				else {
					slGh = new QPixmap(6, 7);
					slGh->fill(cg.background());
					QPixmap pix = uic_findImage("slGh");
					bitBlt(slGh, 0, 0, &pix, 0, 0, -1, -1, Qt::CopyROP);
					QPixmapCache::insert(cg.background().name()+"slGh",slGh );
				}
				bitBlt(p->device(), re.x(), re.y(), slGh, 0, 0, 3, 7, Qt::CopyROP);
				bitBlt(p->device(), r.width()-3, re.y(), slGh, 3, 0, 3, 7, Qt::CopyROP);
				p->drawTiledPixmap(re.x()+3, re.y(), r.width()-6, 7, *sliderGroove);
			}
			else {
				QPixmap* slGv;
				if((slGv = QPixmapCache::find(cg.background().name()+"slGv"))) {
				}
				else {
					slGv = new QPixmap(7, 6);
					slGv->fill(cg.background());
					QPixmap pix = uic_findImage("slGv");
					bitBlt(slGv, 0, 0, &pix, 0, 0, -1, -1, Qt::CopyROP);
					QPixmapCache::insert(cg.background().name()+"slGv",slGv );
				}
				bitBlt(p->device(), re.x(), re.y(), slGv, 0, 0, 7, 3, Qt::CopyROP);
				bitBlt(p->device(), re.x(), re.height()-3, slGv, 0, 3, 7, 3, Qt::CopyROP);
				p->drawTiledPixmap(re.x(), re.y()+3, 7, r.height()-6, *sliderGroove);
			}
			
			break;
		}

		case KPE_SliderHandle: {
			const QSlider* slider = (const QSlider*)widget;
			bool horizontal = slider->orientation() == Horizontal;
			
			int y, x;
			if(horizontal) {
				x = r.x();
				y =r.y()+r.height()/2-8+1;
			}
			else {
				y = r.y()+1;
				x = r.x()+r.width()/2-8;
			}
			
			QRegion reg(x+4, y+1, 7, 1);
			reg += QRegion(x+3, y+2, 9, 1);
			reg += QRegion(x+2, y+3, 11, 7);
			reg += QRegion(x+3, y+10, 9, 1);
			reg += QRegion(x+4, y+11, 7, 1);
			
			p->setClipRegion(reg);
			flatMode=true;
			uint surfaceFlags = Is_Button|Is_Horizontal;
			renderSurface(p, QRect(r.left()+1, r.top(), r.width(), r.height()), cg.background(), qApp->palette().active().background(), cg.button(), surfaceFlags);
			flatMode=false;

			QPixmap sliderHandle =  uic_findImage("sliderHandle");
			bitBlt(p->device(), x, y, &sliderHandle, 0, 0, -1, -1, Qt::CopyROP);
			break;
		}

		case KPE_ListViewExpander: {
			
			p->setPen( cg.text() );
			if(!_drawTriangularExpander) {
				
				QWidget* w = dynamic_cast<QWidget*>(p->device());
				QColor bg = w ? w->paletteBackgroundColor() : cg.background();
				QString hasBgPix = QString().setNum(w && w->paletteBackgroundPixmap());
				QPixmap* lve;
				if((lve = QPixmapCache::find(bg.name()+cg.text().name()+"lve"+hasBgPix))) {
				}
				else {
					if(w && !w->paletteBackgroundPixmap()) {
						lve = new QPixmap(11,11);
						lve->fill(bg);
						QPainter painter(lve);
						painter.drawPixmap(0, 0, tintImage(uic_findImage("lve"), cg.text()));
					}
					else {
						lve = new QPixmap(tintImage(uic_findImage("lve"), cg.text()));
					}
					QPixmapCache::insert(bg.name()+cg.text().name()+"lve"+hasBgPix, lve);
				}
				p->drawPixmap(r.x()-1,r.y()-1, *lve);
				
				int radius = (r.width() - 4) / 2;
				int centerx = r.x() + r.width()/2;
				int centery = r.y() + r.height()/2;
				
				// plus or minus
				p->drawLine( centerx - radius, centery, centerx + radius, centery );
				if ( flags & Style_On ) // Collapsed = On
					p->drawLine( centerx, centery - radius, centerx, centery + radius );
			}
			else
				drawPrimitive((flags & Style_On)?PE_ArrowRight:PE_ArrowDown, p, QRect(r.x()+1,r.y()+1,r.width(),r.height()), cg, ButtonContour, flags);

			break;
		}

		// copied and slightly modified from KStyle.
		case KPE_ListViewBranch: {
		// Typical Windows style listview branch element (dotted line).

		// Create the dotline pixmaps if not already created
		if ( !verticalLine )
		{
			// make 128*1 and 1*128 bitmaps that can be used for
			// drawing the right sort of lines.
			verticalLine   = new QBitmap( 1, 129, true );
			horizontalLine = new QBitmap( 128, 1, true );
			QPointArray a( 64 );
			QPainter p2;
			p2.begin( verticalLine );

			int i;
			for( i=0; i < 64; i++ )
				a.setPoint( i, 0, i*2+1 );
			p2.setPen( color1 );
			p2.drawPoints( a );
			p2.end();
			QApplication::flushX();
			verticalLine->setMask( *verticalLine );

			p2.begin( horizontalLine );
			for( i=0; i < 64; i++ )
				a.setPoint( i, i*2+1, 0 );
			p2.setPen( color1 );
			p2.drawPoints( a );
			p2.end();
			QApplication::flushX();
			horizontalLine->setMask( *horizontalLine );
		}

		p->setPen( cg.mid() );

		if (flags & Style_Horizontal)
		{
			int point = r.x();
			int other = r.y();
			int end = r.x()+r.width();
			int thickness = r.height();

			while( point < end )
			{
				int i = 128;
				if ( i+point > end )
					i = end-point;
				p->drawPixmap( point, other, *horizontalLine, 0, 0, i, thickness );
				point += i;
			}

		} else {
			int point = r.y();
			int other = r.x();
			int end = r.y()+r.height();
			int thickness = r.width();
			int pixmapoffset = (flags & Style_NoChange) ? 0 : 1;	// ### Hackish

			while( point < end )
			{
				int i = 128;
				if ( i+point > end )
					i = end-point;
				p->drawPixmap( other, point, *verticalLine, 0, pixmapoffset, thickness, i );
				point += i;
			}
		}
		
		break;
	}

		default:
			KStyle::drawKStylePrimitive(kpe, p, widget, r, cg, flags, opt);
	}
}


void DominoStyle::drawPrimitive(PrimitiveElement pe,
								QPainter *p,
								const QRect &r,
								const QColorGroup &cg,
								SFlags flags,
								const QStyleOption &opt ) const
{
	bool down	= flags & Style_Down;
	bool on		= flags & Style_On;
	bool sunken = flags & Style_Sunken;
	bool horiz	= flags & Style_Horizontal;
	const bool enabled = flags & Style_Enabled;
	const bool mouseOver = flags & Style_MouseOver;

//	bool hasFocus = flags & Style_HasFocus;

	int x = r.x();
	int y = r.y();
	int w = r.width();
	int h = r.height();

	int x2, y2;
	r.coords(&x, &y, &x2, &y2);
	
	//return KStyle::drawPrimitive(pe, p, r, cg, flags, opt);
	
	switch(pe) {

		case PE_FocusRect: {
			p->drawWinFocusRect( r );
			break;
		}
		
		case PE_HeaderSection: {
			renderHeader(p, r, cg, (on||down), mouseOver, true, enabled );
			break;
		}
		
	// BUTTONS
	// -------
		 // MenuBar buttons
		case PE_ButtonTool:
		case PE_ButtonDropDown:
		case PE_ButtonBevel:
		case PE_ButtonCommand: {
			
			bool khtmlMode = khtmlWidgets.contains(opt.widget());
			renderButton(p, r, cg, (on||down), mouseOver, true, enabled, khtmlMode );
			break;
		}
		
		case PE_ButtonDefault: { // ### put someting here
			break;
		}

		case PE_SpinWidgetPlus:
		case PE_SpinWidgetMinus: {
			p->setPen( cg.buttonText() );

			int l = QMIN( w-2, h-2 );
			// make the length even so that we get a nice symmetric plus...
			if(l%2 != 0)
				--l;
			QPoint c = r.center();

			p->drawLine( c.x()-l/2, c.y(), c.x()+l/2, c.y() );
			if ( pe == PE_SpinWidgetPlus ) {
				p->drawLine( c.x(), c.y()-l/2, c.x(), c.y()+l/2 );
			}
			break;
		}

		case PE_ScrollBarSlider: {
			
			QColor buttonColor = cg.button();
			
			QPixmap buffer;
			if(horiz)
				buffer.resize(r.width()-14, r.height());
			else
				buffer.resize(r.width(), r.height()-14);
			if(buffer.isNull()) break;
			
			QRect br = buffer.rect();
			QPainter painter(&buffer);

			if(horiz) {
				uint surfaceFlags = Is_ScrollBarBtn|Is_Horizontal;
				renderSurface(&painter, br, cg.button(), cg.button(), buttonColor, surfaceFlags);

				uint contourFlags = Is_ScrollBarBtn|Draw_Top|Draw_Bottom;
				renderContour(&painter, QRect(-2, 0, br.width()+4, br.height()), cg.button(), buttonColor, contourFlags);

				p->drawPixmap(r.left()+7, r.y(), buffer);
			}
			else {
				uint surfaceFlags = Is_ScrollBarBtn;
				renderSurface(&painter, br, cg.button(), cg.button(), buttonColor, surfaceFlags);

				uint contourFlags = Is_ScrollBarBtn|Draw_Left|Draw_Right;
				renderContour(&painter, QRect(0, -2, br.width(), br.height()+4), cg.button(), buttonColor, contourFlags);

				bitBlt(p->device(), r.x(), r.y()+7, &buffer, 0, 0, -1, -1, Qt::CopyROP);
			}
			break;
		}

		case PE_ScrollBarAddPage: { // bottom / right
			QColor buttonColor = cg.button();
			QPixmap* addPageBtn;
			QPixmap buffer;
			
			uint surfaceFlags = Is_ScrollBarBtn;
			if(horiz)
				surfaceFlags |= Is_Horizontal;
			
			QColor sbTopContour = getColor(scrollBarSurface, TopBorder);
			QColor sbBottomContour = getColor(scrollBarSurface, BottomBorder);
			if(buttonColor != qApp->palette().active().button()) {
				sbTopContour = alphaBlendColors(buttonColor, sbTopContour, 127);
				sbBottomContour = alphaBlendColors(buttonColor, sbBottomContour, 127);
			}
			sbTopContour = alphaBlendColors(QColor(41,41,47), sbTopContour, 100);
			sbBottomContour = alphaBlendColors(QColor(41,41,47), sbBottomContour, 100);
			
			int sl = 6;
			if(sliderlen < 14)
				sl = sliderlen/2-1;
			if(horiz) {
				buffer.resize(r.width()+sl, r.height());
				if(buffer.isNull()) break;
				
				if((addPageBtn = QPixmapCache::find(buttonColor.name()+"ah" ))) {
				}
				else {
					QRegion mask(6, 1, 3, 13);
					mask -= QRegion(6, 3, 1, 9);
					mask -= QRegion(7,5,1,5);
					
					addPageBtn = new QPixmap(uic_findImage("roundScrollBtn_left_mask"));
					QRect re = addPageBtn->rect();

					QPainter painter(addPageBtn);
					
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(0, 0, re.width(), re.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
					
					painter.setClipping(false);
					painter.drawPixmap(0,0, uic_findImage("roundScrollBtn_left"));
					painter.setPen(sbTopContour);
					painter.drawLine(6,1,8,1);
					painter.setPen(sbBottomContour);
					painter.drawLine(6,13,8,13);
					painter.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 198));
					painter.drawLine(3,0,8,0);
					painter.drawLine(3,14,8,14);

					QPixmapCache::insert(buttonColor.name()+"ah", addPageBtn);
				}
			}
			else {
				buffer.resize(r.width(), r.height()+sl);
				if(buffer.isNull()) break;
				if((addPageBtn = QPixmapCache::find(buttonColor.name()+"av" ))) {
				}
				else {
					QRegion mask(1, 6, 13, 3);
					mask -= QRegion(3, 6, 9, 1);
					mask -= QRegion(5,7,5,1);

					addPageBtn = new QPixmap(uic_findImage("roundScrollBtn_top_mask"));
					QRect re = addPageBtn->rect();

					QPainter painter(addPageBtn);
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(0, 0, re.width(), re.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
					
					painter.setClipping(false);
					painter.drawPixmap(0,0, uic_findImage("roundScrollBtn_top"));
					painter.setPen(sbTopContour);
					painter.drawLine(1,6,1,8);
					painter.setPen(sbBottomContour);
					painter.drawLine(13,6,13,8);
					painter.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 198));
					painter.drawLine(0,3,0,8);
					painter.drawLine(14,3,14,8);

					QPixmapCache::insert(buttonColor.name()+"av", addPageBtn);
				}
			}

			QRect br(buffer.rect() );
			QPainter bp(&buffer);

			if(sliderlen >= 10) {
				surfaceFlags = Is_ScrollBarGroove;
				if(horiz)
					surfaceFlags |= Is_Horizontal;
				renderSurface(&bp, QRect(0, 0, br.width(), br.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
			}
			else {
				renderSurface(&bp, QRect(0, 0, br.width(), br.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
				uint contourFlags = Is_ScrollBarBtn;
				contourFlags |= (horiz)? Draw_Top|Draw_Bottom : Draw_Left|Draw_Right;
				renderContour(&bp, horiz?QRect(-2, 0, br.width()+4, br.height()):QRect(0, -2, br.width(), br.height()+4), buttonColor, buttonColor, contourFlags);
			}

			bp.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 120));
			if(horiz) {
				if(sliderlen >= 10) {
					bp.drawLine(0, 0, br.right(), 0);
					bp.drawLine(0, br.bottom(), br.right(), br.bottom());
					bp.drawPixmap(br.right()-8, 0, *addPageBtn);
					bp.drawPixmap(-6+sl, 0, *scrollBarCap_right);
				}
				bitBlt(p->device(), r.x()-sl, r.y(), &buffer, 0, 0, br.width(), br.height(), Qt::CopyROP);
			}
			else {
				if(sliderlen >= 10) {
					bp.drawLine(0, 0, 0, br.bottom());
					bp.drawLine(br.right(), 0, br.right(), br.bottom());
					bp.drawPixmap(0, br.bottom()-8, *addPageBtn);
					bp.drawPixmap(0, (br.height() <= 6)? br.bottom()-6 :br.top(), *scrollBarCap_bottom);
				}
				bitBlt(p->device(), r.x(), r.y()-sl, &buffer, 0, 0, br.width(), br.height(), Qt::CopyROP);
			}

			break;
		}

		case PE_ScrollBarSubPage: { // top / left
			QColor buttonColor = cg.button();
			QPixmap* subPageBtn;
			QPixmap buffer;

			uint surfaceFlags = Is_ScrollBarBtn;
			if(horiz)
				surfaceFlags |= Is_Horizontal;

			QColor sbTopContour = getColor(scrollBarSurface, TopBorder);
			QColor sbBottomContour = getColor(scrollBarSurface, BottomBorder);
			if(buttonColor != qApp->palette().active().button()) {
				sbTopContour = alphaBlendColors(buttonColor, sbTopContour, 127);
				sbBottomContour = alphaBlendColors(buttonColor, sbBottomContour, 127);
			}
			sbTopContour = alphaBlendColors(QColor(41,41,47), sbTopContour, 100);
			sbBottomContour = alphaBlendColors(QColor(41,41,47), sbBottomContour, 100);

			int sl = 6;
			if(sliderlen < 14)
				sl = sliderlen/2;
			if(horiz) {
				buffer.resize(r.width()+sl, r.height());
				if(buffer.isNull()) break;
				
				if((subPageBtn = QPixmapCache::find(buttonColor.name()+"sh" ))) {
				}
				else {
					QRegion mask(0, 1, 3, 13);
					mask -= QRegion(2, 3, 1, 9);
					mask -= QRegion(1,5,1,5);
					
					subPageBtn = new QPixmap(uic_findImage("roundScrollBtn_right_mask"));
					QRect re = subPageBtn->rect();

					QPainter painter(subPageBtn);
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(0, 0, re.width(), re.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
					
					painter.setClipping(false);
					painter.drawPixmap(0,0, uic_findImage("roundScrollBtn_right"));
					painter.setPen(sbTopContour);
					painter.drawLine(0,1,2,1);
					painter.setPen(sbBottomContour);
					painter.drawLine(0,13,2,13);
					painter.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 198));
					painter.drawLine(0,0,5,0);
					painter.drawLine(0,14,5,14);

					QPixmapCache::insert(buttonColor.name()+"sh", subPageBtn);
				}
			}
			else {
				buffer.resize(r.width(), r.height()+sl);
				if(buffer.isNull()) break;
				
				if((subPageBtn = QPixmapCache::find(buttonColor.name()+"sv" ))) {
				}
				else {
					QRegion mask(1, 0, 13, 3);
					mask -= QRegion(3, 2, 9, 1);
					mask -= QRegion(5,1,5,1);
					
					subPageBtn = new QPixmap(uic_findImage("roundScrollBtn_bottom_mask"));
					QRect re = subPageBtn->rect();

					QPainter painter(subPageBtn);
					painter.setClipRegion(mask);
					renderSurface(&painter, QRect(0, 0, re.width(), re.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
					
					painter.setClipping(false);
					painter.drawPixmap(0,0, uic_findImage("roundScrollBtn_bottom"));
					painter.setPen(sbTopContour);
					painter.drawLine(1,0,1,2);
					painter.setPen(sbBottomContour);
					painter.drawLine(13,0,13,2);
					painter.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 198));
					painter.drawLine(0,0,0,5);
					painter.drawLine(14,0,14,5);

					QPixmapCache::insert(buttonColor.name()+"sv", subPageBtn);
				}
			}

			QRect br(buffer.rect());
			QPainter bp(&buffer);

			if(sliderlen >= 10) {
				surfaceFlags = Is_ScrollBarGroove;
				if(horiz)
					surfaceFlags |= Is_Horizontal;
				renderSurface(&bp, QRect(0, 0, br.width(), br.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
			}
			else {
				renderSurface(&bp, QRect(0, 0, br.width(), br.height()), buttonColor, buttonColor, buttonColor, surfaceFlags);
				uint contourFlags = Is_ScrollBarBtn;
				contourFlags |= (horiz)? Draw_Top|Draw_Bottom : Draw_Left|Draw_Right;
				renderContour(&bp, horiz?QRect(-2, 0, br.width()+4, br.height()):QRect(0, -2, br.width(), br.height()+4), buttonColor, buttonColor, contourFlags);
			}
			
			bp.setPen(alphaBlendColors(QColor(0, 0, 0), buttonColor, 120));
			if(horiz) {
				if(sliderlen >= 10) {
					bp.drawLine(0, 0, br.right(), 0);
					bp.drawLine(0, br.bottom(), br.right(), br.bottom());
					bp.drawPixmap(0, 0, *subPageBtn);
					bp.drawPixmap((br.width() <= 7)? -2 : br.width()-9, 0, *scrollBarCap_left);
				}
			}
			else {
				if(sliderlen >= 10) {
					bp.drawLine(0, 0, 0, br.bottom());
					bp.drawLine(br.right(), 0, br.right(), br.bottom());
					bp.drawPixmap(0, 0, *subPageBtn);
					bp.drawPixmap(0, br.bottom()-sl-2, *scrollBarCap_top);
				}
			}
			bitBlt(p->device(), r.x(), r.y(), &buffer, 0, 0, br.width(), br.height(), Qt::CopyROP);
			break;
		}

	// SCROLLBAR BUTTONS
	// -----------------

		case PE_ScrollBarSubLine: { // top / left button
			
			uint contourFlags = Is_ScrollBarBtn;
			uint surfaceFlags = Is_ScrollBarBtn;

			QRect cr = r;
			QRect ar = r;
			QRect sr = r;

			if(horiz) {
				ar.setY(1);
				surfaceFlags |= Is_Horizontal;
				contourFlags |= Draw_Top|Draw_Bottom;
				if(r.x() == 0) { // = single Button
					cr.setWidth(17);
					sr.setX(2);
					sr.setWidth(13);
					ar.setX(3);
					contourFlags |= Draw_Left|Round_BottomLeft|Round_UpperLeft;
				}
				else {
					cr.setWidth(17);
					cr.setX(cr.x()-2);
					ar.setX(r.x()+1);
				}
			}
			else { // vertical
				ar.setX(1);
				contourFlags |= Draw_Left|Draw_Right;
				if(r.y() == 0) {
					cr.setHeight(17);
					sr.setY(2);
					sr.setHeight(13);
					ar.setY(3);
					contourFlags |= Draw_Top|Round_UpperLeft|Round_UpperRight;
				}
				else {
					cr.setHeight(17);
					cr.setY(cr.y()-2);
				}
			}
			
			if(horiz) {
				renderSurface(p, QRect(sr.x(), sr.y(), sr.width(), sr.height()), cg.button(), cg.button(), cg.button(), surfaceFlags);
				renderContour(p, QRect(cr.x(), cr.y(), cr.width(), cr.height()), cg.button(), cg.button(), contourFlags);
			}
			else {
				p->setClipRegion(QRect(sr.x()+2, sr.y(), sr.width()-4, sr.height()));
				renderSurface(p, QRect(sr.x(), sr.y(), sr.width(), sr.height()), cg.button(), cg.button(), cg.button(), surfaceFlags);
				p->setClipping(false);
				renderContour(p, QRect(cr.x(), cr.y(), cr.width(), cr.height()), cg.button(), cg.button(), contourFlags);
			}

			p->setPen(cg.buttonText());
			drawPrimitive((horiz ? PE_ArrowLeft : PE_ArrowUp), p, ar, cg, flags);
			break;
		}

		case PE_ScrollBarAddLine: { // bottom / right button
			
			uint contourFlags = Is_ScrollBarBtn;
			uint surfaceFlags = Is_ScrollBarBtn;

// 			if(down) surfaceFlags|=Is_Sunken;
			
			QRect cr = r;
			QRect ar = r;
			QRect sr = r;
			if(horiz) {
				surfaceFlags |= Is_Horizontal;
				contourFlags |= Draw_Top|Draw_Bottom|Draw_Right|Round_UpperRight|Round_BottomRight;
				cr.setX(cr.x()-2);
				cr.setWidth(17);
				sr.setWidth(13);
				ar.setY(r.y()+1);
				ar.setX(r.x()-2);
			}
			else {
				contourFlags |= Draw_Bottom|Draw_Left|Draw_Right|Round_BottomLeft|Round_BottomRight;
				cr.setY(cr.y()-2);
				cr.setHeight(17);
				sr.setHeight(13);
				ar.setY(r.y()-2);
				ar.setX(r.x()+1);
			}

			if(horiz) {
				renderSurface(p, QRect(sr.x(), sr.y(), sr.width(), sr.height()), cg.button(), cg.button(), cg.button(), surfaceFlags);
				renderContour(p, QRect(cr.x(), cr.y(), cr.width(), cr.height()), cg.button(), cg.button(), contourFlags);
			}
			else {
				p->setClipRegion(QRect(sr.x()+2, sr.y(), sr.width()-4, sr.height()));
				renderSurface(p, QRect(sr.x(), sr.y(), sr.width(), sr.height()), cg.button(), cg.button(), cg.button(), surfaceFlags);
				p->setClipping(false);
				renderContour(p, QRect(cr.x(), cr.y(), cr.width(), cr.height()), cg.button(), cg.button(), contourFlags);
			}
			
			p->setPen(cg.buttonText());
			drawPrimitive((horiz ? PE_ArrowRight : PE_ArrowDown), p, ar, cg, flags);
			break;
		}

	// CHECKBOXES
	// ----------
		case PE_Indicator: {
			renderSurface(p, QRect(r.x()+3, r.y(), r.width()-6, r.height()), cg.background(), cg.button(), cg.button(), Is_CheckItem);
			p->drawPixmap(r.left(), r.top(), *checkbox);
			break;
		}

		case PE_IndicatorMask: {
			p->fillRect (r.x()+3, r.y()+3, r.width()-6, r.height()-7, color1);
			break;
		}

	// RADIOBUTTONS
	// ------------
		case PE_ExclusiveIndicator: {

			QPointArray pa(8);
			pa.setPoint(0, 3,6);
			pa.setPoint(1, 6,3);
			pa.setPoint(2, 11,3);
			pa.setPoint(3, 14,6);
			pa.setPoint(4, 14,10);
			pa.setPoint(5, 10,14);
			pa.setPoint(6, 6,14);
			pa.setPoint(7, 3,10);

			p->setClipRegion(pa);
			renderSurface(p, QRect(r.left(), r.top(), r.width(), r.height()+1), cg.background(), cg.button(), cg.background(), Is_CheckItem);
			p->setClipping(false);

			bitBlt(p->device(), r.left(), r.top(), radiobutton, 0, 0, -1, -1, Qt::CopyROP);
			break;
		}

		case PE_ExclusiveIndicatorMask: {

			QPointArray pa(8);
			pa.setPoint(0, 3,6);
			pa.setPoint(1, 6,3);
			pa.setPoint(2, 10,3);
			pa.setPoint(3, 13,6);
			pa.setPoint(4, 13,10);
			pa.setPoint(5, 10,13);
			pa.setPoint(6, 6,13);
			pa.setPoint(7, 3,10);

			p->setBrush(color1);
			p->setPen(color1);
			p->drawPolygon(pa);
			break;
		}

	// GENERAL PANELS
	// --------------
		case PE_Splitter: {
			
			p->fillRect(r, cg.background());
			int ch, cs, cv;
			cg.background().getHsv(ch, cs, cv);
			if(cv < 100)
				p->setPen(alphaBlendColors(cg.background(), white, 120));
			else
				p->setPen(alphaBlendColors(cg.background(), black, 180));

			p->setPen(dGetColor(cg.background(), 55));
			if (w > h) {
				int ycenter = r.height()/2;
				for(int k =r.width()/2-5; k < r.width()/2+5; k+=3) {
					p->drawRect(k, ycenter-1, 2,2);
				}
			}
			else {
				int xcenter = r.width()/2;
				for(int k =r.height()/2-5; k < r.height()/2+5; k+=3) {
					p->drawRect( xcenter-1, k, 2,2);
				}
			}
			
			break;
		}

		case PE_GroupBoxFrame:
		case PE_PanelGroupBox: { // QFrame::GroupBoxPanel
			break;
		}

		case PE_WindowFrame: {
			if ( opt.isDefault() || opt.lineWidth() <= 0 )
				break;
			renderPanel(p, r, cg, true, sunken);
			break;
		}

		case PE_Panel: { // QFrame::StyledPanel

			p->setPen(cg.background().dark(120));
			// innen
			p->drawLine(r.x()+1, r.y()+1, r.right()-1, r.y()+1); //top
			p->drawLine(r.x()+1, r.bottom()-1, r.right()-1, r.bottom()-1);
			p->drawLine(r.x()+1, r.y()+1, r.x()+1, r.bottom()-1);
			p->drawLine(r.right()-1, r.y()+1, r.right()-1, r.bottom()-1);
			
			p->setPen(cg.background());
			p->drawLine(r.x(), r.y(), r.right(), r.y()); //top
			p->setPen(alphaBlendColors(white, cg.background(), 30));
			p->drawLine(r.x(), r.y()+1, r.x(), r.bottom());
			p->drawLine(r.right(), r.y()+1, r.right(), r.bottom());
			
			p->setPen(alphaBlendColors(white, cg.background(), 50));
			p->drawLine(r.x()+1, r.bottom(), r.right()-1, r.bottom());
			break;
		}

		case PE_PanelLineEdit: { // QFrame::LineEditPanel
			break;
		}

		case PE_PanelTabWidget: { // QFrame::TabWidgetPanel
			QRect re = QRect(r.x()+2, r.y(), r.width()-4, r.height());
			renderGradient(p, QRect(r.x()+1, r.top(), r.width()-2, 6), lightenColor(cg.background(), 23).rgb(), cg.background().rgb(), true);
			renderGradient(p, QRect(r.x()+1, r.bottom()-5, r.width()-2, 6), cg.background().rgb(), darkenColor(cg.background(), 18).rgb(), true);

			p->setPen(cg.background().dark(150));
			p->drawLine(re.x(), re.top(), re.x(), re.bottom());
			p->drawLine(re.right(), re.top(), re.right(), re.bottom());
			p->setPen(cg.background().dark(130));
			p->drawLine(re.left(), re.bottom(), re.right(), re.bottom());
			break;
		}

		case PE_PanelPopup: { // QFrame::PopupPanel

			QPopupMenu* pm = dynamic_cast<QPopupMenu*>(p->device());
			QColor background = dynamic_cast<QListBox*>(p->device()) ? cg.base() : _popupMenuColor;
			if(pm && pm->isA("PrettyPopupMenu")) { // amarok
				p->setPen(background.dark(150));
				p->drawRect(r);
				break;
			}
			
			QRegion mask(r);
			mask -= QRegion(x,y,1,5);
			mask -= QRegion(x+1,y+1,1,1);
			mask -= QRegion(x,y,5,1);
			mask -= QRegion(x,h-5,1,5); // left bottom
			mask -= QRegion(x+1,h-2,1,1);
			mask -= QRegion(x,h-1,5,1);
			mask -= QRegion(w-1,y,1,5); // right
			mask -= QRegion(w-2,y+1,1,1);
			mask -= QRegion(w-5,y,5,1);
			mask -= QRegion(w-1,h-5,1,5); // right bottom
			mask -= QRegion(w-2,h-2,1,1);
			mask -= QRegion(w-5,h-1,5,1);
			
			
			QPixmap buffer(r.width(), r.height());
			QPainter bp(&buffer);
			renderGradient(&bp, r, lightenColor(background, 18), darkenColor(background, 12), true);
			
			if(pm) {
				pm->setUpdatesEnabled(false);
				pm->setPaletteBackgroundPixmap(buffer);
				pm->setUpdatesEnabled(true);
			}
			p->setClipRegion(mask);
			p->drawPixmap(r, buffer);
			p->setClipping(false);
			
			bitBlt(p->device(), r.x(), r.y(), popupFrame, 0, 0, 5, 5, Qt::CopyROP);
			bitBlt(p->device(), r.width()-5, r.y(), popupFrame, 5, 0, 5, 5, Qt::CopyROP);
			bitBlt(p->device(), r.x(), r.bottom()-4, popupFrame, 0, 5, 5, 5, Qt::CopyROP);
			bitBlt(p->device(), r.width()-5, r.bottom()-4, popupFrame, 5, 5, 5, 5, Qt::CopyROP);
			
			p->setPen(background.dark(150));
			p->drawLine(r.x()+5, r.y(), r.width()-6, r.y());
			p->drawLine(r.x()+5, r.bottom(), r.width()-6, r.bottom());
			p->drawLine(r.x(), r.y()+5, r.x(), r.bottom()-5);
			p->drawLine(r.width()-1, r.y()+5, r.width()-1, r.bottom()-5);

			break;
		}

	// MENU / TOOLBAR PANEL
	// --------------------
		case PE_PanelMenuBar: // QFrame::MenuBarPanel
		case PE_PanelDockWindow: { // QFrame::ToolBarPanel

			p->setPen(cg.background());
			// overpainting the menus leads to flicker so we draw only the frame here.
			if(dynamic_cast<QWidget*>(p->device()))
				p->drawRect(r);
			else
				p->fillRect(r, cg.brush(QColorGroup::Background)); // colorpreview in kcontrol
			break;
		}

		case PE_StatusBarSection: {
			break;
		}

		case PE_TabBarBase: {
			break;
		}
		
	// TOOLBAR/DOCK WINDOW HANDLE
	// --------------------------
		case PE_DockWindowResizeHandle: {
			QWidget* w = dynamic_cast<QWidget*>(p->device());
			if(w && !w->erasePixmap()) {
				QPixmap pix(r.size());
				QPainter painter(&pix);
				renderGradient(&painter, r, lightenColor(cg.background(), 40), darkenColor(cg.background(), 30), horiz);
				w->setErasePixmap(pix);
			}
			break;
		}

		case PE_DockWindowHandle: {
			QWidget* widget = dynamic_cast<QWidget*>(p->device()); //qt_dockwidget_internal - QDockWindowHandle
			
			int ch, cs, cv;
			cg.background().getHsv(ch, cs, cv);
			if(cv < 100)
				p->setPen(alphaBlendColors(cg.background(), white, 120));
			else
				p->setPen(alphaBlendColors(cg.background(), black, 180));
			
			if(widget && widget == hoverWidget) {
				if (w > h) {
					int ycenter = r.height()/2;
					for(int k = r.width()/2-5; k < r.width()/2+5; k+=3) {
						p->drawRect(k, ycenter-1, 2, 2);
					}
				}
				else {
					int xcenter = r.width()/2;
					for(int k = r.height()/2-5; k < r.height()/2+5; k+=3) {
						p->drawRect(xcenter-1, k, 2, 2);
					}
				}
			}
			break;
		}

	// TOOLBAR SEPARATOR
	// -----------------
		case PE_DockWindowSeparator: {
			break;
		}

		case PE_CheckMark: {
			if( flags&Style_On ) {
				p->drawPixmap(r.x()+1, r.y()+1, *checkMark);
			}
			else if ( flags & Style_Off ) {
			}
			else {// tristate
				p->setPen(_customCheckMarkColor ? _checkMarkColor : qApp->palette().active().foreground());
				p->drawLine(6,6,11,6);
				p->drawLine(6,8,11,8);
				p->drawLine(6,10,11,10);
			}
			
			break;
		}

		case PE_SpinWidgetUp:
		case PE_SpinWidgetDown:
		case PE_HeaderArrow:
		case PE_ArrowUp:
		case PE_ArrowDown:
		case PE_ArrowLeft:
		case PE_ArrowRight: {
			QPointArray a;

			switch (pe) {
				case PE_SpinWidgetUp:
				case PE_ArrowUp: {
					a.setPoints(7, u_arrow);
					break;
				}
				case PE_SpinWidgetDown:
				case PE_ArrowDown: {
					a.setPoints(7, d_arrow);
					break;
				}
				case PE_ArrowLeft: {
					a.setPoints(7, l_arrow);
					break;
				}
				case PE_ArrowRight: {
					a.setPoints(7, r_arrow);
					break;
				}
				default: {
					if (flags & Style_Up) {
						a.setPoints(7, u_arrow);
					} else {
						a.setPoints(7, d_arrow);
					}
				}
			}

			const QWMatrix oldMatrix( p->worldMatrix() );

		//	if (flags & Style_Down) {
		//		p->translate(pixelMetric(PM_ButtonShiftHorizontal),
		//						pixelMetric(PM_ButtonShiftVertical));
		//	}

			a.translate((r.x()+r.width()/2), (r.y()+r.height()/2));
			// extra-pixel-shift, correcting some visual tics...
			switch(pe) {
				case PE_ArrowLeft:
				case PE_ArrowRight:
					a.translate(0, -1);
					break;
				case PE_SpinWidgetUp:
				case PE_SpinWidgetDown:
					a.translate(+1, 0);
					break;
				default:
					a.translate(0, 0);
			}

			if (p->pen() == QPen::NoPen) {
				if (flags & Style_Enabled) {
					p->setPen(cg.buttonText());
				} else {
					p->setPen(cg.highlightedText());
				}
			}
			p->drawLineSegments(a, 0, 3);
			p->drawPoint(a[6]);

			p->setWorldMatrix( oldMatrix );
			break;
		}

		case PE_CheckListIndicator: {
			QCheckListItem *item = opt.checkListItem();
			QListView *lv = item->listView();
			if(!item)
				return;
			
			QString on = QString().setNum(flags& Style_On);
			QString selected = QString().setNum(flags& Style_Selected);
			QString noChange = QString().setNum(flags& Style_NoChange);
			
			QColor background = flags& Style_Selected ? cg.highlight() : cg.base();
			QColor foreground = cg.text();
			
			QPixmap* pix;
			if((pix = QPixmapCache::find(background.name()+foreground.name()+"checkListIndicator"+on+selected+noChange))) {
			}
			else {
				pix = new QPixmap(r.x()+lv->itemMargin()+r.width()+4, item->height());
				pix->fill(background);
				QPainter painter(pix);
				
				painter.setPen(alphaBlendColors(foreground, background, 170));
				painter.drawLine(r.x()+1, r.y(), r.x()+12, r.y()); // t
				painter.drawLine(r.x()+1, r.y()+13, r.x()+12, r.y()+13); // b
				painter.drawLine(r.x(), r.y()+1, r.x(), r.y()+12); // l
				painter.drawLine(r.x()+13, r.y()+1, r.x()+13, r.y()+12); // r
				
				painter.setPen(alphaBlendColors(foreground, background, 120));
				painter.drawPoint(r.x(), r.y());
				painter.drawPoint(r.x()+13, r.y());
				painter.drawPoint(r.x(), r.y()+13);
				painter.drawPoint(r.x()+13, r.y()+13);
				
				if(flags& Style_NoChange) {
					painter.fillRect(r.x()+1, r.y()+1, 12, 12, dGetColor(background, 30));
					painter.drawPixmap(r.x()-1, r.y()-1, tintImage(uic_findImage("checkmark"), foreground));
				}
				if(flags& Style_On)
					painter.drawPixmap(r.x()-1, r.y()-1, tintImage(uic_findImage("checkmark"), foreground));
				
				QPixmapCache::insert(background.name()+foreground.name()+"checkListIndicator"+on+selected+noChange, pix);
			}
			
			p->drawPixmap(0, 0, *pix);
			break;
		}
		
		case PE_CheckListExclusiveIndicator: {
			
			p->drawPixmap(r.left()+3, r.top()+1, tintImage(uic_findImage("checkListExclusiveIndicator"), cg.text()));

			if(flags&Style_On)
				p->drawPixmap(r.x()+7, r.y()+5, tintImage(uic_findImage("rb-dot"),cg.text()));
			break;
		}
		
		case PE_SizeGrip: {
			QPixmap* grip;
			if((grip = QPixmapCache::find(cg.background().name()+"sizeGrip"))) {
			}
			else {
				grip = new QPixmap(4, 4);
				grip->fill(cg.background());
				QPixmap tmpPix = uic_findImage("sizeGrip");
				bitBlt(grip, 0, 0, &tmpPix, 0, 0, -1, -1, Qt::CopyROP);
				QPixmapCache::insert(cg.background().name()+"sizeGrip", grip);
			}
			
			bitBlt(p->device(), r.width()-(QApplication::reverseLayout() ? 12 : 6), r.height()-12, grip, 0, 0, -1, -1, Qt::CopyROP);
			bitBlt(p->device(), r.width()-6, r.height()-6, grip, 0, 0, -1, -1, Qt::CopyROP);
			bitBlt(p->device(), r.width()-12, r.height()-6, grip, 0, 0, -1, -1, Qt::CopyROP);
			break;
		}

		default: {
			return KStyle::drawPrimitive(pe, p, r, cg, flags, opt);
		}
	}
}


void DominoStyle::drawControl(ControlElement element,
							  QPainter *p,
							  const QWidget *widget,
							  const QRect &r,
							  const QColorGroup &cg,
							  SFlags flags,
							  const QStyleOption& opt) const
{
	const bool reverseLayout = QApplication::reverseLayout();

	const bool enabled = (flags & Style_Enabled);

	//return KStyle::drawControl(element, p, widget, r, cg, flags, opt);
	
	switch (element) {

		case CE_ToolButtonLabel: {
			
			if(!widget->isA("KToolBarButton")) {
				QCommonStyle::drawControl(CE_ToolButtonLabel, p, widget, QRect(r.x(), r.y(), r.width(), r.height()-1), cg, flags, opt);
				break;
			}
			
			DominoKToolBarButton* tb= (DominoKToolBarButton *) widget;
			// reimplemented from ktoolbarbutton to keep the icons under the button contour.

			QStyle::SFlags flags   = QStyle::Style_Default;
			QStyle::SCFlags active = QStyle::SC_None;

			if (tb->isDown()) {
				flags  |= QStyle::Style_Down;
				active |= QStyle::SC_ToolButton;
			}
			if (tb->isEnabled()) 	flags |= QStyle::Style_Enabled;
			if (tb->isOn()) 		flags |= QStyle::Style_On;
			if (tb->isEnabled() && tb->hasMouse())	flags |= QStyle::Style_Raised;
			if (tb->hasFocus())	flags |= QStyle::Style_HasFocus;
			
			int dx, dy;
			QFont tmp_font(KGlobalSettings::toolBarFont());
			QFontMetrics fm(tmp_font);
			QRect textRect;
			int textFlags = 0;

			if (tb->iconTextMode() == KToolBar::IconOnly) // icon only
			{
				QPixmap pixmap =tb->iconSet().pixmap( QIconSet::Automatic,
					tb->isEnabled() ? (tb->isActive() ? QIconSet::Active : QIconSet::Normal) :
						QIconSet::Disabled,
					tb->isOn() ? QIconSet::On : QIconSet::Off );
				
				if( !pixmap.isNull())
				{
					dx = ( tb->width() - pixmap.width() ) / 2;
					dy = ( tb->height()-1 - pixmap.height() ) / 2;

					p->drawPixmap( dx, dy, pixmap );
				}
			}
			
			else if (tb->iconTextMode() == KToolBar::IconTextRight) // icon and text (if any)
			{
				QPixmap pixmap = tb->iconSet().pixmap( QIconSet::Automatic,
					tb->isEnabled() ? (tb->isActive() ? QIconSet::Active : QIconSet::Normal) :
						QIconSet::Disabled,
					tb->isOn() ? QIconSet::On : QIconSet::Off );
				if( !pixmap.isNull())
				{
					dx = 4;
					dy = ( tb->height()-1 - pixmap.height() ) / 2;

					p->drawPixmap( dx, dy, pixmap );
				}
				if (!tb->textLabel().isNull())
				{
					textFlags = AlignVCenter|AlignLeft;
					if (!pixmap.isNull())
						dx = 4 + pixmap.width() + 2;
					else
						dx = 4;
					dy = 0;

					textRect = QRect(dx, dy, tb->width()-dx, tb->height()-1);
				}
			}
			else if (tb->iconTextMode() == KToolBar::TextOnly)
			{
				if (!tb->textLabel().isNull())
				{
					textFlags = AlignVCenter|AlignLeft;
					dx = (tb->width() - fm.width(tb->textLabel())) / 2;
					dy = (tb->height()-1 - fm.lineSpacing()) / 2;

					textRect = QRect( dx, dy, fm.width(tb->textLabel()), fm.lineSpacing() );
				}
			}
			else if (tb->iconTextMode() == KToolBar::IconTextBottom)
			{
				QPixmap pixmap = tb->iconSet().pixmap( QIconSet::Automatic,
					tb->isEnabled() ? (tb->isActive() ? QIconSet::Active : QIconSet::Normal) :
						QIconSet::Disabled,
					tb->isOn() ? QIconSet::On : QIconSet::Off );
				if( !pixmap.isNull())
				{
					dx = (tb->width() - pixmap.width()) / 2;
					dy = (tb->height()-1 - fm.lineSpacing() - pixmap.height()) / 2;

					p->drawPixmap( dx, dy, pixmap );
				}

				if (!tb->textLabel().isNull())
				{
					textFlags = AlignBottom|AlignHCenter;
					dx = (tb->width() - fm.width(tb->textLabel())) / 2;
					dy = tb->height()-1 - fm.lineSpacing() - 4;

					textRect = QRect( dx, dy, fm.width(tb->textLabel()), fm.lineSpacing() );
				}
			}
			if (!tb->textLabel().isNull() && !textRect.isNull())
			{
				p->setFont(KGlobalSettings::toolBarFont());
				if (!tb->isEnabled())
					p->setPen(tb->palette().disabled().dark());
				else if(tb->isRaised())
					p->setPen(KGlobalSettings::toolBarHighlightColor());
				else
					p->setPen( tb->colorGroup().buttonText() );
				p->drawText(textRect, textFlags, tb->textLabel());
			}

			break;
		}

	// PROGRESSBAR
	// -----------
		case CE_ProgressBarGroove: {
			drawPrimitive(PE_Panel, p, r, cg, flags, opt);
			break;
		}

		case CE_ProgressBarContents: {
			const QProgressBar *pb = dynamic_cast<const QProgressBar*>(widget);
			int steps = pb->totalSteps();
			
			const QColor bg = enabled? cg.base(): cg.background(); // background
			const QColor fg = cg.background();
			
			QPixmap* prBgShadow;
			
			int h, s, v, bv;
			bg.getHsv(&h, &s, &v);
			cg.background().getHsv(&h, &s, &bv);
			int diff = KMAX(v-bv, 0);
			
			// if we don't adjust the shadow to the background, it will look strange
			QColor tintColor = alphaBlendColors(Qt::black, bg, diff+30);
			
			if((prBgShadow = QPixmapCache::find(bg.name()+cg.background().name()+"prBgShadow"))) {
			}
			else {
				prBgShadow = new QPixmap(4,4);
				prBgShadow->fill(bg);
				QPainter painter(prBgShadow);
				painter.drawPixmap(0, 0, tintImage(uic_findImage("progressShadow2"), tintColor));
				QPixmapCache::insert(bg.name()+cg.background().name()+"prBgShadow", prBgShadow);
			}
			
			QRect surface;
			QRect rightBg;
			QRect leftBg;
			p->setClipRegion(r);
			
			if( steps == 0 ) { // Busy indicator
				static const int barWidth = 10;
				int progress = pb->progress() % (2*(r.width()-barWidth));
				if( progress < 0)
					progress = 0;
				if( progress > r.width()-barWidth )
					progress = (r.width()-barWidth)-(progress-(r.width()-barWidth));
				
				surface  = QRect(reverseLayout? r.right()-progress+1: r.x()+progress+1, r.top()-1, barWidth-2, r.height()+4 );
				rightBg = reverseLayout ? QRect(r.x(), r.y(), r.width()-(progress-1), r.height()) :
						QRect(r.x()+progress+barWidth-1, r.y(), r.width()-(progress+barWidth-1), r.height());
				
				if(rightBg.width()+surface.width() < r.width()) {
					leftBg = reverseLayout ? QRect(rightBg.width()+surface.width(), r.y(), r.width()-(rightBg.width()+surface.width()-2), r.height()) :
							QRect(r.x(), r.y(), r.x()+progress+1, r.height());
				}
				flatMode=true;
				renderSurface(p, surface, fg,  qApp->palette().active().background(), fg , Is_Button|Is_Horizontal);
				flatMode=false;
				
				
			}
			else {
				double percent = static_cast<double>(pb->progress()) / static_cast<double>(steps);
				int w = static_cast<int>(r.width() * percent);
				if(w < 0)
					w = 0;
				int w2 = r.width()-(r.width()-w);
				rightBg = QRect(reverseLayout? r.left(): r.left()+w, r.top(), r.width()-w, r.height());
				surface = QRect(reverseLayout? r.right()-w2+1: r.left(), r.top()-1, w2, r.height()+4 );
				flatMode=true;
				renderSurface(p, surface, fg,  qApp->palette().active().background(), fg , Is_Button|Is_Horizontal);
				flatMode=false;
				
			}
			// empty area
			
			QColor color_d = alphaBlendColors(tintColor, bg, 170);
			QColor color_m = alphaBlendColors(tintColor, bg, 85);
			QColor color_l = alphaBlendColors(tintColor, bg, 25);
			
			if(leftBg.isValid()) {
				p->setClipRegion(leftBg);
				p->fillRect(leftBg, bg);
				
				p->setPen(color_d);
				p->drawLine(leftBg.x()+2, leftBg.y(), leftBg.right()-2, leftBg.y());
				p->setPen(color_m);
				p->drawLine(leftBg.x()+2, leftBg.y()+1, leftBg.right()-2, leftBg.y()+1);
				p->setPen(color_l);
				p->drawLine(leftBg.x()+2, leftBg.y()+2, leftBg.right()-2, leftBg.y()+2);
				p->drawLine(leftBg.x()+2, leftBg.bottom(), leftBg.right()-2, leftBg.bottom());

				//p->setPen(color_l);
				p->drawLine(leftBg.x()+1, leftBg.y()+3, leftBg.x()+1, leftBg.bottom()-1); // l
				p->drawLine(leftBg.right()-1, leftBg.y()+3, leftBg.right()-1, leftBg.bottom()-1); // r
			
				p->setPen(color_m);
				p->drawLine(leftBg.x(), leftBg.y()+3, leftBg.x(), leftBg.bottom()-1); // l
				p->drawLine(leftBg.right(), leftBg.y()+3, leftBg.right(), leftBg.bottom()-1); // r
			
				p->drawPixmap(leftBg.right()-1, leftBg.y(), *prBgShadow, 2, 0, 2, 3); // tr
				p->drawPixmap(leftBg.x(), leftBg.y(), *prBgShadow, 0, 0, 2, 3);
				p->drawPixmap(leftBg.right()-1, leftBg.bottom(), *prBgShadow, 2, 3, 2, 1); // br
				p->drawPixmap(leftBg.x(), leftBg.bottom(), *prBgShadow, 0, 3, 2, 1);
			}
			
			p->setClipRegion(rightBg);
			p->fillRect(rightBg, bg);
			
			p->setPen(color_d);
			p->drawLine(rightBg.x()+2, rightBg.y(), rightBg.right()-2, rightBg.y());
			p->setPen(color_m);
			p->drawLine(rightBg.x()+2, rightBg.y()+1, rightBg.right()-2, rightBg.y()+1);
			p->setPen(color_l);
			p->drawLine(rightBg.x()+2, rightBg.y()+2, rightBg.right()-2, rightBg.y()+2);
			p->drawLine(rightBg.x()+2, rightBg.bottom(), rightBg.right()-2, rightBg.bottom());

			//p->setPen(color_l);
			p->drawLine(rightBg.x()+1, rightBg.y()+3, rightBg.x()+1, rightBg.bottom()-1); // l
			p->drawLine(rightBg.right()-1, rightBg.y()+3, rightBg.right()-1, rightBg.bottom()-1); // r
			
			p->setPen(color_m);
			p->drawLine(rightBg.x(), rightBg.y()+3, rightBg.x(), rightBg.bottom()-1); // l
			p->drawLine(rightBg.right(), rightBg.y()+3, rightBg.right(), rightBg.bottom()-1); // r
			
			
			p->drawPixmap(rightBg.right()-1, rightBg.y(), *prBgShadow, 2, 0, 2, 3); // tr
			p->drawPixmap(rightBg.x(), rightBg.y(), *prBgShadow, 0, 0, 2, 3);
			p->drawPixmap(rightBg.right()-1, rightBg.bottom(), *prBgShadow, 2, 3, 2, 1); // br
			p->drawPixmap(rightBg.x(), rightBg.bottom(), *prBgShadow, 0, 3, 2, 1);
			
			if(rightBg.width() == 2) { // must overpaint two lighter pixel
				p->drawPixmap(rightBg.right(), rightBg.y()+2, *prBgShadow, 0, 3, 1, 1); // br
				p->drawPixmap(rightBg.right(), rightBg.bottom(), *prBgShadow, 0, 3, 1, 1); // br
			}
			
			p->setClipping(false);

			break;
		}
		
		case CE_CheckBox: {
			QColorGroup g = cg;
			if(!khtmlWidgets.contains(widget))
				g.setColor(QColorGroup::Button, qApp->palette().active().background());
			drawPrimitive(PE_Indicator, p, r, g, flags);

			if(flags&Style_Down) 
				p->drawPixmap(r.x(), r.y(), uic_findImage("checkbox-pressed"));
			
			if(flags&Style_On)
				drawPrimitive(PE_CheckMark, p, r, cg, flags);
			break;
		}
		
		case CE_CheckBoxLabel: {
			const QCheckBox *cb = (const QCheckBox *) widget;

			int alignment = QApplication::reverseLayout() ? AlignRight : AlignLeft;
			if (!styleHint(SH_UnderlineAccelerator, widget, QStyleOption::Default, 0))
				alignment |= NoAccel;
			
			QRect re = _drawUnderline ? subRect(SR_CheckBoxFocusRect, cb) : r;
			
			if (flags& Style_HasFocus && _indicateFocus)
				drawFocusIndicator(p, re, alignment | AlignVCenter |ShowPrefix, cg, flags & Style_Enabled, cb->pixmap(), cb->text(), -1, &_focusIndicatorColor, _drawUnderline);

			drawItem(p, r, alignment | AlignVCenter | ShowPrefix, cg,
				flags & Style_Enabled, cb->pixmap(), cb->text());
			break;
		}

	// RADIOBUTTONS
	// ------------
		case CE_RadioButton: {
			QColorGroup g = cg;
			if(!khtmlWidgets.contains(widget))
				g.setColor(QColorGroup::Button, qApp->palette().active().background());
			drawPrimitive(PE_ExclusiveIndicator, p, r, g, flags);

			if(flags&Style_Down) {
				p->drawPixmap(r.x(), r.y(), uic_findImage("radioButton-pressed"));
			}
			
			if(flags&Style_On)
				p->drawPixmap(r.x()+6, r.y()+6, *radioIndicator);
			
			break;
		}
		
		case CE_RadioButtonLabel: {
			const QRadioButton *radiobutton = (const QRadioButton *) widget;

			int alignment = QApplication::reverseLayout() ? AlignRight : AlignLeft;
			if (!styleHint(SH_UnderlineAccelerator, widget, QStyleOption::Default, 0))
			alignment |= NoAccel;
			
			QRect re = _drawUnderline ? subRect(SR_RadioButtonFocusRect, radiobutton) : r;
			
			if (flags& Style_HasFocus && _indicateFocus)
				drawFocusIndicator(p, re, alignment | AlignVCenter |ShowPrefix, cg, flags & Style_Enabled, radiobutton->pixmap(), radiobutton->text(), -1, &_focusIndicatorColor, _drawUnderline);
			
			drawItem(p, r, alignment | AlignVCenter | ShowPrefix, cg,
				flags & Style_Enabled, radiobutton->pixmap(), radiobutton->text());
			
			break;
		}

	// TABS
	// ----
		case CE_TabBarTab: {
			
			//const QTabBar * tb = (const QTabBar *) widget;
			QTabBar * tb = (QTabBar *) widget;

			bool cornerWidget = false;
			if( dynamic_cast<QTabWidget*>(tb->parent()) ) {
				const QTabWidget *tw = (const QTabWidget*)tb->parent();
				// is there a corner widget in the (top) left edge?
				QWidget *cw = tw->cornerWidget(Qt::TopLeft);
				if(cw)
					cornerWidget = true;
			}
			bool konqTab = false;
			if(tb->parent() && tb->parent()->inherits("KonqFrameTabs")) {
				konqTab = true;
				tb->setShape(QTabBar::RoundedBelow);
					if(tb->eraseColor() != cg.background().dark(tabBarContrast-20))
						tb->setEraseColor(cg.background().dark(tabBarContrast-20));
			}

			QTabBar::Shape tbs = tb->shape();
			bool selected = false;
			bool reverseLayout = QApplication::reverseLayout();
			if (flags & Style_Selected) selected = true;
			TabPosition pos;
			
			if ((tb->indexOf(opt.tab()->identifier()) == 0)) {
				pos = reverseLayout?Last:First;
			} else if (tb->indexOf(opt.tab()->identifier()) == tb->count() - 1) {
				pos = reverseLayout?First:Last;
			} else {
				pos = Middle;
			}
			
			switch (tbs) {
				case QTabBar::TriangularAbove:
					renderTab(p, r, cg, (flags & Style_MouseOver), selected, false, pos, true, cornerWidget, konqTab);
					break;
				case QTabBar::RoundedAbove:
					renderTab(p, r, cg, (flags & Style_MouseOver), selected, false, pos, false, cornerWidget, konqTab);
					break;
				case QTabBar::TriangularBelow:
					renderTab(p, r, cg, (flags & Style_MouseOver), selected, true, pos, true, cornerWidget, konqTab);
					break;
				case QTabBar::RoundedBelow:
					renderTab(p, r, cg, (flags & Style_MouseOver), selected, true, pos, false, cornerWidget, konqTab);
					break;
				default:
					KStyle::drawControl(element, p, widget, r, cg, flags, opt);
			}
			
			break;
		}

		case CE_PushButton: {
			
			QPushButton *button = (QPushButton *)widget;
			bool khtmlMode = khtmlWidgets.contains(button);
			const bool isDefault = enabled && button->isDefault();

			if (button->isFlat() ) {
				flatMode = true;
			}
			else if(!khtmlMode) {
				QColor bg;
				if(dynamic_cast<QGroupBox*>(button->parentWidget()))
					bg = button->parentWidget()->paletteBackgroundColor();
				else
					bg = cg.background();
				
				p->fillRect(QRect(r.x(), r.y(), 5, 5), bg);
				p->fillRect(QRect(r.width()-5, r.y(), 5, 5), bg);
				p->fillRect(QRect(r.width()-5, r.height()-6, 5, 6), bg);
				p->fillRect(QRect(r.x(), r.height()-6, 5, 6), bg);
			}
			
			drawPrimitive(PE_ButtonBevel, p, r, cg, flags, QStyleOption(button) );
			
			if(flatMode) {
				flatMode = false;
				if(button->isA("KMultiTabBarTab")) {
					const QWidget* parent = button->parentWidget();
					
					p->setPen(cg.background().dark(150));
					
					if(parent->width() >= parent->height()) {
						if(button->x() == parent->x())
							p->drawLine(r.x(), r.y(), r.x(), r.bottom());
						p->drawLine(r.right(), r.y(), r.right(), r.bottom());
					}
					else {
						if(button->y() == parent->y())
							p->drawLine(r.right(), r.y(), r.right(), r.bottom());
						p->drawLine(r.x(), r.y(), r.x(), r.bottom());
					}
					p->drawLine(r.x(), r.y(), r.right(), r.y());
					p->drawLine(r.x(), r.bottom(), r.right(), r.bottom());
				}
				else {
					p->setPen(cg.background().dark(150));
					p->drawRect(r);
				}
			}
			
			if (isDefault )
				drawPrimitive(PE_ButtonDefault, p, r, cg, flags);
			break;
		}
		
		case CE_PushButtonLabel:
		{
			
			int x, y, w, h;
			r.rect( &x, &y, &w, &h );
			
			const QPushButton* button = static_cast<const QPushButton *>( widget );
//			bool active = button->isOn() || button->isDown();
			bool cornArrow = false;
			bool hasFocus = button->hasFocus();
			// Shift button contents if pushed.
		//	if ( active )
		//	{
		//		x += pixelMetric(PM_ButtonShiftHorizontal, widget);
		//		y += pixelMetric(PM_ButtonShiftVertical, widget);
		//		flags |= Style_Sunken;
		//	}

			// Does the button have a popup menu?
			if ( button->isMenuButton() )
			{
				int dx = pixelMetric( PM_MenuButtonIndicator, widget );
				if ( button->iconSet() && !button->iconSet()->isNull()	&&
					(dx + button->iconSet()->pixmap (QIconSet::Small, QIconSet::Normal, QIconSet::Off ).width()) >= w )
				{
					cornArrow = true; //To little room. Draw the arrow in the corner, don't adjust the widget
				}
				else
				{
					drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - dx - 8, y + 2, dx, h - 4), r ),
								cg, flags, opt );
					w -= dx;
				}
			}

			// Draw the icon if there is one
			if ( button->iconSet() && !button->iconSet()->isNull() )
			{
				QIconSet::Mode	mode  = QIconSet::Disabled;
				QIconSet::State state = QIconSet::Off;

				if (button->isEnabled())
					mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
				if (button->isToggleButton() && button->isOn())
					state = QIconSet::On;

				QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );

				
				if(hasFocus && _indicateFocus) {
					QPixmap shadowPixmap = tintImage(pixmap.convertToImage(), _focusIndicatorBtnColor);
					if (button->text().isEmpty() && !button->pixmap())
						p->drawPixmap( x + w/2 - shadowPixmap.width()/2 + 1, y + h / 2 - shadowPixmap.height() / 2 + 1,
							shadowPixmap );
					else
						p->drawPixmap( x + 4 + 1, y + h / 2 - shadowPixmap.height() / 2 + 1, shadowPixmap );
				}
				
				if (button->text().isEmpty() && !button->pixmap())
					p->drawPixmap( x + w/2 - pixmap.width()/2, y + h / 2 - pixmap.height() / 2,
									pixmap );
				else
					p->drawPixmap( x + 4, y + h / 2 - pixmap.height() / 2, pixmap );

				if (cornArrow) //Draw over the icon
					drawPrimitive( PE_ArrowDown, p, visualRect( QRect(x + w - 6, x + h - 6, 7, 7), r ),
								cg, flags, opt );


				int	 pw = pixmap.width();
				x += pw + 4;
				w -= pw + 4;
			}
			
			QRect re = _drawUnderline ? subRect(SR_PushButtonFocusRect, button) : QRect(x, y, w, h);
			
			if(hasFocus && _indicateFocus && !button->text().isNull())
				drawFocusIndicator(p, re, AlignCenter|ShowPrefix, cg, button->isEnabled(), 0, button->text(), -1, &_focusIndicatorBtnColor, _drawUnderline);
			
			drawItem( p, QRect(x, y, w, h), AlignCenter|ShowPrefix, button->colorGroup(),
						button->isEnabled(), button->pixmap(), button->text(), -1,
						&button->colorGroup().buttonText() );

			break;
		}
		case CE_TabBarLabel: {
			if ( opt.isDefault() )
				break;
			const QTabBar * tb = (const QTabBar *) widget;
			QTab * t = opt.tab();
			int shift, cshift;
			switch (tb->shape()) {
				case QTabBar::RoundedAbove:
					shift = 4;
					cshift = 3;
					break;
				case QTabBar::RoundedBelow:
					if(tb->parent() && tb->parent()->inherits("KonqFrameTabs")) {
						shift = 0;
						cshift = 0;
					}
					else {
						shift = -3;
						cshift = -1;
					}
					break;
				case QTabBar::TriangularAbove:
					shift = 4;
					cshift = 3;
					break;
				case QTabBar::TriangularBelow:
					shift = -5;
					cshift = -3;
					break;
				default:
					shift = 0;
					cshift = 0;
			}

			QRect tr = r;
			//p->drawLine(r.left(), r.top()+r.height()/2, r.width(), r.top()+r.height()/2 );
// 			if ( t->identifier() == tb->currentTab() )
// 				tr.setBottom( tr.bottom() -
// 						pixelMetric( QStyle::PM_DefaultFrameWidth, tb ) );
			if ( t->identifier() != tb->currentTab() )
				tr.setTop( tr.top() +shift );
			else
				tr.setTop( tr.top() +cshift );

			int alignment = AlignCenter | ShowPrefix;
			
			if (flags& Style_HasFocus && _indicateFocus && !t->text().isEmpty())
				drawFocusIndicator(p, QRect(tr.x(), tr.y(), tr.width(), tr.height()), alignment, cg, flags & Style_Enabled, 0, t->text(), -1, &_focusIndicatorBtnColor, _drawUnderline);
			
			drawItem( p, tr, alignment, cg, flags & Style_Enabled, 0, t->text() );

			break;
		}

		case CE_MenuBarItem: {
			QMenuItem *mi = opt.menuItem();
			bool active	 = flags & Style_Active;
			bool focused = flags & Style_HasFocus;
			bool down = flags & Style_Down;
			const int text_flags =
				AlignVCenter | AlignHCenter | ShowPrefix | DontClip | SingleLine;
			
			if (active && focused) {
				p->fillRect(r, cg.background()); // gtk-qt engine fix
				
				QColor c = down ? dGetColor(cg.background(), 15) : dGetColor(cg.background(), 25);
				
				p->fillRect(QRect(r.left()+3,r.y()+1, r.width()-7, r.height()), c);
				p->fillRect(QRect(r.left(),r.top()+4, 3, r.height()-7), c);
				p->fillRect(QRect(r.right()-3, r.y()+4, 3, r.height()-7), c);
		
				QPixmap pix = tintImage(uic_findImage("menuItem"), c);
				p->drawPixmap(r.left(),r.y()+1, pix, 0, 0, 3, 3);
				p->drawPixmap(r.left(),r.bottom()-2, pix, 0, 3, 3, 3);
				p->drawPixmap(r.right()-3,r.y()+1, pix, 3, 0, 3, 3);
				p->drawPixmap(r.right()-3,r.bottom()-2, pix, 3, 3, 3, 3);
			}
			
			p->setPen(cg.foreground() );
			p->drawText(r, text_flags, mi->text());
			break;
		}

	// POPUPMENU ITEM (highlighted on mouseover)
	// ------------------------------------------
		case CE_PopupMenuItem: {

			const QPopupMenu *popupmenu = static_cast< const QPopupMenu * >( widget );
			QMenuItem *mi = opt.menuItem();
			
			if ( !mi )
				break;
			
			int tab = opt.tabWidth();
			int checkcol = mi->iconSet() ? opt.maxIconWidth() : 0;
			bool enabled = mi->isEnabled();
			bool checkable = popupmenu->isCheckable();
			bool active = flags & Style_Active;
			bool reverse = QApplication::reverseLayout();
			int cmwidth = 13;

			// Draw the menu item background
			if (active) {
				if (enabled) {
					QRect rect = r;
					
					int ch, cs, cv;
					cg.background().getHsv(ch, cs, cv);
					bool light = cv < 127;
					QColor sel = light ? lightenColor(_popupMenuColor, 30) : darkenColor(_popupMenuColor, 30);
					float val = 0;
					int ph = popupmenu->rect().height()/2;
					int teil = ph/r.height();
					
					QColor c ;
					if(r.y()+r.height()/2 < ph) {
						for( int i=r.y(); i < ph; i +=teil) {
							val += 0.5;
						}
						c = lightenColor(sel, (int)val);
					}
					else {
						for( int i=0; i < r.y()-ph; i +=teil) {
							val += 0.5;
						}
						c = darkenColor(sel, (int)val);
					}
					
					QColor color1 = alphaBlendColors(c, cg.highlight(), 128);
					QColor color2 = alphaBlendColors(c, cg.highlight(), 76);
					
					QPixmap hPix(3, r.height());
					QPainter painter(&hPix);
					painter.setPen(cg.highlight());
					painter.drawLine(1, 1, 1, r.height()-2);
					painter.setPen(color2);
					painter.drawLine(0, 1, 0, r.height()-2);
					painter.drawLine(2, 1, 2, r.height()-2);
					painter.drawPoint(1, 0);
					painter.drawPoint(1, r.height()-1);
					painter.setPen(color1);
					painter.drawPoint(0, 0);
					painter.drawPoint(2, 0);
					painter.drawPoint(0, r.height()-1);
					painter.drawPoint(2, r.height()-1);
					bitBlt(p->device(), r.x()+3, r.y(), &hPix, 0, 0, -1, -1, Qt::CopyROP);
					bitBlt(p->device(), r.x()+r.width()-6, r.y(), &hPix, 0, 0, -1, -1, Qt::CopyROP);
					
					
					
					p->fillRect(QRect(r.x()+6, r.y(), r.width()-12, r.height()), c);
					p->fillRect(QRect(r.x(), r.y()+3, 3, r.height()-6), c);
					p->fillRect(QRect(r.x()+r.width()-3, r.y()+3, 3, r.height()-6), c);
					
					QPixmap pix = tintImage(uic_findImage("menuItem"), c);
					bitBlt(p->device(), r.x(), r.y(), &pix, 0, 0, 3, 3, Qt::CopyROP);
					bitBlt(p->device(), r.x(), r.bottom()-2, &pix, 0, 3, 3, 3, Qt::CopyROP);
					bitBlt(p->device(), r.x()+r.width()-3, r.y(), &pix, 3, 0, 3, 3, Qt::CopyROP);
					bitBlt(p->device(), r.x()+r.width()-3, r.bottom()-2, &pix, 3, 3, 3, 3, Qt::CopyROP);
				}
			}

			// Are we a menu item separator?
			if ( mi->isSeparator() )
			{
				p->setPen(dGetColor(_popupMenuColor, 20));
				p->drawLine(r.x(), r.y()+2, r.right(), r.y()+2);
				break;
			}
		
			// Do we have an icon?
			if ( mi->iconSet() )
			{
				QRect cr = visualRect( QRect( r.x() + 2 + cmwidth+ 2 , r.y() + 1, checkcol -1 , r.height() - 2 ), r );
				QIconSet::Mode mode;

				// Select the correct icon from the iconset
				if (active)
					mode = enabled?QIconSet::Active:QIconSet::Disabled;
				else
					mode = enabled?QIconSet::Normal:QIconSet::Disabled;

				// Draw the icon
				QPixmap pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
// 				QRect pmr( 0, 0, pixmap.width(), pixmap.height() );
// 				pmr.moveCenter( cr.center() );
// 				p->drawPixmap( pmr.x()+cmwidth, pmr.y(), pixmap );
				p->drawPixmap( cr.x(), cr.y(), pixmap );
			}

			// Are we checked? (This time without an icon)
			if ( checkable && mi->isChecked() )
			{
				
				// We only have to draw the background if the menu item is inactive -
				// if it's active the "pressed" background is already drawn

				QRect cmr = visualRect( QRect( r.x()-1 , r.y() + 1, cmwidth +8, r.height() - 2 ), r ); // 8??
				// Draw the checkmark
				SFlags cflags = Style_On;
				if (enabled)
					cflags |= Style_Enabled;
				drawPrimitive( PE_CheckMark, p, QRect(cmr.x(), cmr.y(), cmr.width(), cmr.height()), cg, cflags );
			}

			// Time to draw the menu item label...
			int xm = cmwidth +2 + checkcol + 2; // X position margin

			int xp = reverse ? // X position
					r.x() + tab + rightBorder + itemHMargin + itemFrame /*- 1*/ +2: // augenmass
					r.x() + xm;

			// Label width (minus the width of the accelerator portion)
			int tw = r.width() - xm - tab - arrowHMargin - itemHMargin * 3 - itemFrame + 1;

			// Set the color for enabled and disabled text
			// (used for both active and inactive menu items)
			p->setPen( enabled ? cg.buttonText() : cg.mid() );

			// This color will be used instead of the above if the menu item
			// is active and disabled at the same time. (etched text)
			QColor discol = cg.mid();

			// Does the menu item draw it's own label?
			if ( mi->custom() ) {
				int m = 2;
				// Save the painter state in case the custom
				// paint method changes it in some way
				mi->custom()->paint( p, cg, active, enabled, xp, r.y()+m, tw, r.height()-2*m );
			}
			else {
				// The menu item doesn't draw it's own label
				QString s = mi->text();
				// Does the menu item have a text label?
				if ( !s.isNull() ) {
					int t = s.find( '\t' );
					int m = 2;
					int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;
					text_flags |= reverse ? AlignRight : AlignLeft;

					QColor draw = (active && enabled) ? cg.highlightedText() : cg.foreground();
					p->setPen(draw);

					// Does the menu item have a tabstop? (for the accelerator text)
					if ( t >= 0 ) {
						int tabx = reverse ? r.x() + rightBorder + itemHMargin + itemFrame :
							r.x() + r.width() - tab - rightBorder - itemHMargin - itemFrame;

						// Draw the right part of the label (accelerator text)
						p->drawText( tabx, r.y()+m, tab, r.height()-2*m, text_flags, s.mid( t+1 ) );
						s = s.left( t );
					}

					// Draw the left part of the label (or the whole label
					// if there's no accelerator)
					p->drawText( xp, r.y()+m, tw, r.height()-2*m, text_flags, s, t );
					p->setPen(cg.text());
				}

				// The menu item doesn't have a text label
				// Check if it has a pixmap instead
				else if ( mi->pixmap() ) {
					QPixmap *pixmap = mi->pixmap();

					// Draw the pixmap
					if ( pixmap->depth() == 1 )
						p->setBackgroundMode( OpaqueMode );

					int diffw = ( ( r.width() - pixmap->width() ) / 2 )
									+ ( ( r.width() - pixmap->width() ) % 2 );
					p->drawPixmap( r.x()+diffw, r.y()+1, *pixmap );

					if ( pixmap->depth() == 1 )
						p->setBackgroundMode( TransparentMode );
				}
			}

			// Does the menu item have a submenu?
			if ( mi->popup() ) {
				PrimitiveElement arrow = reverse ? PE_ArrowLeft : PE_ArrowRight;
				int dim = pixelMetric(PM_MenuButtonIndicator) - 1;
				QRect vr = visualRect( QRect( r.x() + r.width() - 5 - 1 - dim,
							r.y() + r.height() / 2 - dim / 2+1, dim, dim), r );

				if(enabled) {
					p->setPen(active? cg.highlightedText(): cg.foreground());
					drawPrimitive( arrow, p, vr, cg, Style_Enabled);
				}
				else {
					p->setPen(cg.mid());
					drawPrimitive( arrow, p, vr, cg, Style_Default);
				}
			}
			break;
		}

		case CE_ToolBoxTab: {
			
			bool selected= (flags & Style_Selected && ((QToolBox*)widget)->currentItem());
			
			QPixmap pix(widget->size());
			QPainter painter(&pix);

			painter.fillRect(QRect(5, 0, r.width()-10, r.height()), cg.background());
			renderGradient(&painter, QRect(r.x(), r.y(), 5, r.height()), cg.background().light(120).rgb(), cg.background().rgb(), false);
			renderGradient(&painter, QRect(r.width()-5, r.y(), 5, r.height()), cg.background().rgb(), cg.background().light(115).rgb(), false);
			painter.setPen(cg.background().dark(120));
			painter.drawLine(r.x(), r.bottom(), r.width(), r.bottom());

			painter.setPen((hoveredToolBoxTab && !selected)? dGetColor(cg.background(), 95): dGetColor(cg.background(), 45));
			drawPrimitive(selected?PE_ArrowDown:PE_ArrowRight, &painter, QRect(r.x()+6,r.y()+1,r.x()+10,r.height()), cg);
			painter.end();
			
			p->drawPixmap(r.x(), r.y(), pix);
			break;
		}
			
		default:
		  KStyle::drawControl(element, p, widget, r, cg, flags, opt);
	}
}

void DominoStyle::drawControlMask(ControlElement element,
								  QPainter *p,
								  const QWidget *w,
								  const QRect &r,
								  const QStyleOption &opt) const
{

	switch (element) {
		case CE_PushButton: {
			break;
		}
		case CE_CheckBox: {
			drawPrimitive(PE_IndicatorMask, p, r, w->colorGroup(), Style_Default, opt);
			break;
		}
		case CE_RadioButton : {
			drawPrimitive(PE_ExclusiveIndicatorMask, p, r, w->colorGroup(), Style_Default, opt);
			break;
		}
		default: {
			break;
			//KStyle::drawControlMask (element, p, w, r, opt);
		}
	}
}

void DominoStyle::drawComplexControlMask(ComplexControl c,
										 QPainter *p,
										 const QWidget *w,
										 const QRect &r,
										 const QStyleOption &o) const
{
// 	KStyle::drawComplexControlMask (c, p, w, r, o);
	
	switch (c) {
// 		case CC_SpinWidget:
// 		case CC_ListView:
// 		case CC_ComboBox: {
// 			break;
// 		}
		default: {
			KStyle::drawComplexControlMask (c, p, w, r, o);
		}
	}
}


void DominoStyle::drawComplexControl(ComplexControl control,
									 QPainter *p,
									 const QWidget *widget,
									 const QRect &r,
									 const QColorGroup &cg,
									 SFlags flags,
									 SCFlags controls,
									 SCFlags active,
									 const QStyleOption& opt) const
{
	const bool reverseLayout = QApplication::reverseLayout();

	const bool enabled = (flags & Style_Enabled);

	//return KStyle::drawComplexControl(control, p, widget, r, cg, flags, controls, active, opt);
	
	switch(control) {

	// COMBOBOX
	// --------
		case CC_ComboBox: {
			static const unsigned int handleWidth = 15;
			const QComboBox *cb = dynamic_cast<const QComboBox*>(widget);
			bool editable = cb && cb->editable();
			bool isDown = cb && cb->listBox() && cb->listBox()->isVisible();
			bool hasFocus =(flags&Style_HasFocus);
			const QColor inputColor = enabled ? (editable ? cg.base() : cg.button()) : cg.background();
			const QColor bg = (cb && cb->parentWidget()) ?  cb->parentWidget()->paletteBackgroundColor() : cg.background();
			bool hasPixmap = cb && cb->count() > 0 && cb->pixmap(cb->currentItem());
			uint contourFlags = 0;
			bool isKhtmlWidget = khtmlWidgets.contains(cb);
			if(isKhtmlWidget)
				contourFlags |= Draw_AlphaBlend;
			
			else {
				p->fillRect(QRect(r.x(), r.y(), 5, 5), bg);
				p->fillRect(QRect(r.width()-5, r.y(), 5, 5), bg);
				p->fillRect(QRect(r.width()-5, r.height()-6, 5, 6), bg);
				p->fillRect(QRect(r.x(), r.height()-6, 5, 6), bg);
			}
			
			// don't flicker (so much)
			if(hasPixmap && editable && r.width() > 40) {
				int x = reverseLayout ? handleWidth+3 : 5;
				QRegion mask(x, 0, 18, r.height());
				if(!reverseLayout)
					mask += QRegion(x-2, 4, 2, r.height()-9);
				p->setClipRegion(mask);
			}
			
			contourFlags |= Round_UpperLeft|Round_BottomLeft|Round_UpperRight|Round_BottomRight|Draw_Right|Draw_Top|
					Draw_Bottom|Draw_Left|Is_Button|Is_Horizontal;
			uint surfaceFlags = Is_Button|Is_Horizontal;
			if(isDown)
				surfaceFlags |= Is_Down;
			QColor buttonColor = isKhtmlWidget ? cg.button() : qApp->palette().active().background();
			renderSurface(p, QRect(r.left(), r.top(), r.width(), r.height()), btnSurface.background, buttonColor, cg.background(), surfaceFlags);
			renderContour(p, r, bg, buttonColor, contourFlags);
			
			if(editable) {
				QColor contourColor = enabled ? cg.base() : cg.background();
				QString ena = QString().setNum(enabled);
				QPixmap* lineEditInnerEdges;
				if((lineEditInnerEdges = QPixmapCache::find(contourColor.name()+"lineEditInnerEdges"+ena))) {
				}
				else {
					lineEditInnerEdges = new QPixmap(enabled ? tintImage(uic_findImage("lineEditEdges"), cg.base()) :  tintImage(uic_findImage("lineEditEdges"), cg.background()));
					if(enabled) {
						QColor tintColor = alphaBlendColors(Qt::black, cg.base(), 30+35);
						QPixmap pix = tintImage(uic_findImage("progressShadow2"), tintColor);
						bitBlt(lineEditInnerEdges, 0, 0, &pix, 0, 0, -1, -1);
					}
					QPixmapCache::insert(contourColor.name()+"lineEditInnerEdges"+ena, lineEditInnerEdges);
				}
					
				if(reverseLayout) {
					bitBlt(p->device(), r.width()-5, r.y()+3, lineEditInnerEdges, 2, 0, 2, 2);
					bitBlt(p->device(), r.width()-5, r.height()-6, lineEditInnerEdges, 2, 2, 2, 2);
				}
				else {
					bitBlt(p->device(), r.x()+3, r.y()+3, lineEditInnerEdges, 0, 0, 2, 2);
					bitBlt(p->device(), r.x()+3, r.height()-6, lineEditInnerEdges, 0, 2, 2, 2);
				}
				
			}
			
			
			QRect ar(reverseLayout?r.left()+2: r.width()-handleWidth-2, r.height()/2-3, handleWidth, 6);
			if(hasFocus && _indicateFocus && enabled && cb) {
				QRect editField = querySubControlMetrics(control, widget, SC_ComboBoxEditField);
				int pw = hasPixmap ? (cb->pixmap(cb->currentItem()))->width()+2 : 0;
				if(!editable) {
					int fmheight = p->fontMetrics().height();
					if(reverseLayout)
						editField.addCoords(pw+17, editField.height()/2-fmheight/2-1,0 , 0);
					else
						editField.addCoords(2+pw, editField.height()/2-fmheight/2-1,0 , 0);
					QRect re = _drawUnderline ? subRect(SR_ComboBoxFocusRect, cb) : editField;
					drawFocusIndicator(p, re, AlignLeft, cg, true, 0, cb->currentText(), -1, &_focusIndicatorBtnColor, _drawUnderline);
				}
				if(!_drawUnderline) {
					p->setPen(_focusIndicatorBtnColor);
					// one bigger arrow would be probably better ...
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()+1, ar.y(), ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()-1, ar.y(), ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x(), ar.y()+1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x(), ar.y()-1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()+1, ar.y()+1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()-1, ar.y()+1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()+1, ar.y()-1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
					drawPrimitive(PE_SpinWidgetDown, p, QRect(ar.x()-1, ar.y()-1, ar.width(), ar.height()), cg, Style_Default|Style_Enabled|Style_Raised);
				}
			}
			
			if(isDown) {
				uint shadowFlags = Draw_Top|Draw_Right|Draw_Bottom|Draw_Left;
				QRect sr;
				if(editable) {
					shadowFlags |= reverseLayout? Round_UpperLeft|Round_BottomLeft|Square_UpperRight|Square_BottomRight :
							Square_UpperLeft|Square_BottomLeft|Round_UpperRight|Round_BottomRight;
					sr = QRect(reverseLayout ? r.left()+2 : r.width()-handleWidth-3, r.y()+2, handleWidth+3, r.height()-3);
				}
				else {
					shadowFlags |= Round_UpperLeft|Round_BottomLeft|Round_UpperRight|Round_BottomRight;
					sr = QRect(r.x()+2, r.y()+2, r.width()-2, r.height()-3);
				}
				
				renderButtonShadow(p, sr, shadowFlags);
			}

			p->setPen(cg.foreground());
			drawPrimitive(PE_SpinWidgetDown, p, ar, cg, Style_Default|Style_Enabled|Style_Raised);

			break;
		}
		
	// TOOLBUTTON
	// ----------
		case CC_ToolButton: {
			
			
			const QToolButton *tb = (const QToolButton *) widget;
			
			if (::qt_cast<KToolBarButton*>(tb)) {
			
				const bool reverse = QApplication::reverseLayout();
				KToolBar *bar = dynamic_cast<KToolBar*>(tb->parent());
				KToolBarButton* btn = ::qt_cast<KToolBarButton*>(tb);
				KToolBarButton* prevBtn = 0;
				KToolBarButton* nextBtn = 0;
				bool prevBtnIsDown = false;
				bool nextBtnIsDown = false;
				const bool horizontal = bar->orientation() == Qt::Horizontal;
				
				QRect cr = r , sr = r, bsr = r;
				int index = bar->itemIndex(btn->id());
				
				if(bar->getButton(bar->idAt(index-1))) {
					prevBtn = bar->getButton(bar->idAt(index-1));
					if((!reverse) ? (horizontal) ? prevBtn->x() + prevBtn->width() != btn->x() : prevBtn->y() + prevBtn->height() != btn->y() :
							(horizontal) ? prevBtn->x() != btn->x()+btn->width() : prevBtn->y() + prevBtn->height() != btn->y()
					  )
						prevBtn = 0;
					else
						prevBtnIsDown = prevBtn->isOn();
				}
				if(bar->getButton(bar->idAt(index+1))) {
					nextBtn= bar->getButton(bar->idAt(index+1));
					if((!reverse) ? (horizontal) ? nextBtn->x() != btn->x() + btn->width() : nextBtn->y() != btn->y() + btn->height() :
							(horizontal) ? nextBtn->x() + nextBtn->width() != btn->x() : nextBtn->y() != btn->y() + btn->height()
					   )
						nextBtn = 0;
					else
						nextBtnIsDown = nextBtn->isOn();
				}

				QPixmap buffer(btn->size());
				buffer.fill(cg.background());
				QPainter painter(&buffer);
				
				bool drawSeparator = false;
				uint contourFlags = Is_Button;
				uint surfaceFlags = Is_Button;
				if(horizontal) {
					surfaceFlags |=Is_Horizontal;
					contourFlags |=Is_Horizontal;
				}
				
				if(flags &Style_On || flags &Style_Down)
					surfaceFlags|=Is_Down;
				
					if(prevBtn) {
						if(nextBtn) {
							// middle
							if(horizontal) {
								contourFlags |= Draw_Top|Draw_Bottom;
								cr.setLeft(r.left()-5);
								cr.setWidth(r.width()+10);
								sr.setLeft(r.left()-7);
								sr.setWidth(r.width()+13);
								bsr.setLeft(r.left()-2);
								bsr.setWidth(r.width()+3);
								drawSeparator = true;
							}
							else {
								contourFlags |= Draw_Left|Draw_Right;
								cr.setY(r.y()-5);
								sr.setY(r.y()-5); //-2
								cr.setHeight(r.height()+11);
								sr.setHeight(r.height()+10); //5
								bsr.addCoords(2, 0, 0, 1);
								drawSeparator = true;
							}
						}
						else {
							// right / bottom
							if(!horizontal) {
								contourFlags |= Draw_Bottom|Draw_Right|Draw_Left|Round_BottomRight|Round_BottomLeft;
								cr.setY(r.y()-5);
								sr.setY(r.y()-5);//-2
								cr.setHeight(r.height()+5);
								sr.setHeight(r.height()+5);
								bsr.addCoords(2, 0, 0, -1);
							}
							else if(!reverse) {
								contourFlags |= Draw_Top|Draw_Bottom|Draw_Right|Round_UpperRight|Round_BottomRight;
								cr.setLeft(r.left()-5);
								cr.setWidth(r.width()+4);
								sr.setLeft(r.left()-7);
								sr.setWidth(r.width()+6);
								bsr.setLeft(r.left()-2);
								bsr.setWidth(r.width()+1);
							}
							else { // reverse
								contourFlags |= Draw_Top|Draw_Bottom|Draw_Left|Round_UpperLeft|Round_BottomLeft;
								cr.setWidth(r.width()+5);
								sr.setWidth(r.width()+6);
								bsr.setWidth(r.width()+1);
								drawSeparator = true;
							}
						}
					}
					else {
						if(nextBtn) {
							// left / top
							if(!horizontal) {
								contourFlags |= Draw_Top|Draw_Left|Draw_Right|Round_UpperLeft|Round_UpperRight;
								cr.setHeight(r.height()+6);
								sr.setHeight(r.height()+5);//+3
								bsr.addCoords(2, 2, 0, +1);
								drawSeparator = true;
							}
							else if(!reverse) {
								contourFlags |= Draw_Top|Draw_Bottom|Draw_Left|Round_UpperLeft|Round_BottomLeft;
								cr.setWidth(r.width()+5);
								sr.setWidth(r.width()+6);
								bsr.setWidth(r.width()+1);
								drawSeparator = true;
							}
							else { //reverse
								contourFlags |= Draw_Top|Draw_Bottom|Draw_Right|Round_UpperRight|Round_BottomRight;
								cr.setLeft(r.left()-5);
								cr.setWidth(r.width()+4);
								sr.setLeft(r.left()-7);
								sr.setWidth(r.width()+6);
								bsr.setLeft(r.left()-2);
								bsr.setWidth(r.width()+1);
							}
						}
						else {
							// full
							if(horizontal) {
								cr.setWidth(r.width()-1);
								sr.setWidth(r.width()-1);
								bsr.setWidth(r.width()-1);
							}
							contourFlags |= Draw_Top|Draw_Bottom|Draw_Left|Draw_Right|Round_UpperLeft|Round_BottomLeft|
									Round_UpperRight|Round_BottomRight;
							
						}
					}
					
					renderSurface(&painter, sr, cg.background(), qApp->palette().active().background(), cg.background(), surfaceFlags);
					drawControl(CE_ToolButtonLabel, &painter, widget, r, cg, flags,opt);
					renderContour(&painter, cr, cg.background(), qApp->palette().active().background(), contourFlags);
					
					/////////////////////////////////////////// drop down indicator
					
					if(tb->popup()) {
						
						QPixmap* arrow;
						if((arrow = QPixmapCache::find(cg.buttonText().name()+"arrow_down"))) {
						}
						else {
							arrow = new QPixmap(tintImage(uic_findImage("arrow_down"), cg.buttonText()));
							QPixmapCache::insert(cg.buttonText().name()+"arrow_down", arrow);
						}
						
						
						
						if(prevBtn)
							if(nextBtn)
								if(horizontal)
									bitBlt(&buffer, cr.width()-17, cr.bottom()-8, arrow);
								else
									bitBlt(&buffer, cr.width()-8, cr.bottom()-17, arrow);
							else
								if(horizontal)
									bitBlt(&buffer, cr.width()-13, cr.bottom()-8, arrow);
								else
									bitBlt(&buffer, cr.width()-8, cr.bottom()-13, arrow);
						else
							if(nextBtn)
								if(horizontal)
									bitBlt(&buffer, cr.width()-12, cr.bottom()-8, arrow);
								else
									bitBlt(&buffer, cr.width()-8, cr.bottom()-12, arrow);
							else
								if(horizontal)
									bitBlt(&buffer, cr.width()-9, cr.bottom()-8, arrow);
								else
									bitBlt(&buffer, cr.width()-8, cr.bottom()-9, arrow);
					}
					
					/////////////////////////////////////////// separator
					
					if(drawSeparator) {
						painter.setPen(alphaBlendColors(QColor(30,30,30), cg.background(), 200));
						if(horizontal)
							painter.drawLine(r.width()-1, r.top()+2, r.width()-1, r.bottom()-3);
						else
							painter.drawLine(r.x()+2, r.bottom(), r.width()-3, r.bottom());
							
					}
					
					/////////////////////////////////////////// button shadow
					
					if (flags & (Style_Down | Style_On)) {
						uint shadowFlags;
						if(prevBtnIsDown) { // or above is down
							if(nextBtnIsDown) { // or below is down
								if(horizontal) {
									shadowFlags = Draw_Top|Draw_Bottom;
									bsr.addCoords(-1, 2, +3, -1);
								}
								else {
									shadowFlags = Draw_Left|Draw_Right;
									bsr.addCoords(0, -4, 0, +3);
								}
							}
							else {
								if(horizontal) {
									if(!reverse) {
										shadowFlags = Draw_Top|Draw_Bottom|Draw_Right;
										shadowFlags |= contourFlags& Round_UpperRight ? Round_UpperRight : Square_UpperRight;
										shadowFlags |= contourFlags& Round_BottomRight ? Round_BottomRight : Square_BottomRight;
										bsr.addCoords(-1, 2, 0, -1);
									}
									else {
										shadowFlags = Draw_Top|Draw_Bottom|Draw_Left;
										shadowFlags |= contourFlags& Round_UpperLeft ? Round_UpperLeft : Square_UpperLeft;
										shadowFlags |= contourFlags& Round_BottomLeft ? Round_BottomLeft : Square_BottomLeft;
										bsr.addCoords(2, 2, 3, -1);
									}
								}
								else {
									shadowFlags = Draw_Bottom|Draw_Left|Draw_Right;
									shadowFlags |= contourFlags& Round_BottomLeft ? Round_BottomLeft : Square_BottomLeft;
									shadowFlags |= contourFlags& Round_BottomRight ? Round_BottomRight : Square_BottomRight;
									bsr.setY(bsr.y()-8);
								}
							}
						}
						else if(nextBtnIsDown) {
							if(horizontal) {
								if(!reverse) {
									shadowFlags = Draw_Top|Draw_Bottom|Draw_Left;
									shadowFlags |= (contourFlags& Round_UpperLeft) ? Round_UpperLeft : Square_UpperLeft;
									shadowFlags |= (contourFlags& Round_BottomLeft) ? Round_BottomLeft : Square_BottomLeft;
									bsr.addCoords(2, 2, 3, -1);
								}
								else {
									shadowFlags = Draw_Top|Draw_Bottom|Draw_Right;
									shadowFlags |= contourFlags& Round_UpperRight ? Round_UpperRight : Square_UpperRight;
									shadowFlags |= contourFlags& Round_BottomRight ? Round_BottomRight : Square_BottomRight;
									bsr.addCoords(-1, 2, 0, -1);
								}
							}
							else {
								shadowFlags = Draw_Top|Draw_Left|Draw_Right;
								shadowFlags |= contourFlags& Round_UpperLeft ? Round_UpperLeft : Square_UpperLeft;
								shadowFlags |= contourFlags& Round_UpperRight ? Round_UpperRight : Square_UpperRight;
								bsr.setHeight(bsr.height()+3);
							}
						}
						else {
							if(horizontal) {
								shadowFlags = Draw_Top|Draw_Bottom|Draw_Left|Draw_Right;
								shadowFlags |= contourFlags& Round_UpperLeft ? Round_UpperLeft : Square_UpperLeft;
								shadowFlags |= contourFlags& Round_UpperRight ? Round_UpperRight : Square_UpperRight;
								shadowFlags |= contourFlags& Round_BottomRight ? Round_BottomRight : Square_BottomRight;
								shadowFlags |= contourFlags& Round_BottomLeft ? Round_BottomLeft : Square_BottomLeft;
								bsr.addCoords(2, 2, 0, -1);
							}
							else {
								shadowFlags = Draw_Top|Draw_Bottom|Draw_Left|Draw_Right;
								shadowFlags |= contourFlags& Round_UpperLeft ? Round_UpperLeft : Square_UpperLeft;
								shadowFlags |= contourFlags& Round_UpperRight ? Round_UpperRight : Square_UpperRight;
								shadowFlags |= contourFlags& Round_BottomRight ? Round_BottomRight : Square_BottomRight;
								shadowFlags |= contourFlags& Round_BottomLeft ? Round_BottomLeft : Square_BottomLeft;
							}
						}
						renderButtonShadow(&painter, bsr, shadowFlags);
					}
					
					btn->setUpdatesEnabled(false);
					btn->setErasePixmap(buffer);
					btn->setUpdatesEnabled(true);
					btn->erase();
					
			}
			else { // not a KToolBarButton
				
				const QRect br = querySubControlMetrics(control, widget, SC_ToolButton, opt);
				if(br.width() < 16 || br.height()< 16 )
					break;
				
				QColorGroup g = widget->colorGroup();
				QColor bg;
				if(widget->parentWidget() && widget->paletteBackgroundColor() != widget->parentWidget()->paletteBackgroundColor()) {
					bg = widget->parentWidget()->paletteBackgroundColor();
					g.setColor(QColorGroup::Background, bg);
				}
				else
					bg = widget->paletteBackgroundColor();
				
				p->fillRect(QRect(br.x(), br.y(), 5, 5), bg);
				p->fillRect(QRect(br.width()-5, br.y(), 5, 5), bg);
				p->fillRect(QRect(br.width()-5, br.height()-6, 5, 6), bg);
				p->fillRect(QRect(br.x(), br.height()-6, 5, 6), bg);
				
				drawPrimitive(PE_ButtonTool, p, br, g, flags, opt);
				
			}

			p->setPen(cg.buttonText() );
			break;
		}

	// SPINWIDGETS
	// -----------
		case CC_SpinWidget: {
			
			static const unsigned int handleWidth = 15;
			const QSpinWidget *sw = dynamic_cast<const QSpinWidget *>(widget);
			const QColor bg = cg.background();
			SFlags sflags = flags;
			PrimitiveElement pe;

			// bool hasFocus = false; // doesn't work !?
			bool isEnabled = false;
			if (sw) {
// 				hasFocus = sw->hasFocus();
				isEnabled = sw->isEnabled();
			}
			
			const QColor buttonColor = enabled ? cg.button() : cg.background();
			const QColor inputColor = enabled ? cg.base() : cg.background();

			// fill edges
			p->fillRect(QRect(r.x(), r.y(), 5, 5), bg);
			p->fillRect(QRect(r.width()-5, r.y(), 5, 5), bg);
			p->fillRect(QRect(r.width()-5, r.height()-6, 5, 6), bg);
			p->fillRect(QRect(r.x(), r.height()-6, 5, 6), bg);
			
			uint surfaceFlags = Is_Button|Is_Horizontal;
			renderSurface(p, r, cg.background(), qApp->palette().active().background(), cg.background(), surfaceFlags);
			uint contourFlags = Round_UpperLeft|Round_BottomLeft|Round_UpperRight|Round_BottomRight|Draw_Right|Draw_Top|
					Draw_Bottom|Draw_Left|Is_Button|Is_Horizontal;
			renderContour(p, r, cg.background(), qApp->palette().active().background(), contourFlags);
			
			
			QString ena = QString().setNum(isEnabled);
			QPixmap* lineEditInnerEdges;
			if((lineEditInnerEdges = QPixmapCache::find(inputColor.name()+"lineEditInnerEdges"+ena))) {
			}
			else {
				lineEditInnerEdges = new QPixmap(isEnabled ? tintImage(uic_findImage("lineEditEdges"), cg.base()) :  tintImage(uic_findImage("lineEditEdges"), cg.background()));
				if(isEnabled) {
					QColor tintColor = alphaBlendColors(Qt::black, cg.base(), 30+35);
					QPixmap pix = tintImage(uic_findImage("progressShadow2"), tintColor);
					bitBlt(lineEditInnerEdges, 0, 0, &pix, 0, 0, -1, -1);
				}
				QPixmapCache::insert(inputColor.name()+"lineEditInnerEdges"+ena, lineEditInnerEdges);
			}
			
			if(reverseLayout) {
				bitBlt(p->device(), r.width()-5, r.y()+3, lineEditInnerEdges, 2, 0, 2, 2);
				bitBlt(p->device(), r.width()-5, r.height()-6, lineEditInnerEdges, 2, 2, 2, 2);
			}
			else {
				bitBlt(p->device(), r.x()+3, r.y()+3, lineEditInnerEdges, 0, 0, 2, 2);
				bitBlt(p->device(), r.x()+3, r.height()-6, lineEditInnerEdges, 0, 2, 2, 2);
			}

			QRect upRect = QRect(reverseLayout?r.left()+2:r.right()-handleWidth-1, r.height()/2-7, handleWidth, 6);
			QRect downRect = QRect(reverseLayout?r.left()+2:r.right()-handleWidth-1, r.height()/2, handleWidth, 6);

			// icons...
			p->setPen(cg.foreground());
			sflags = Style_Default | Style_Enabled;
			if (active == SC_SpinWidgetUp) {
				sflags |= Style_On;
				sflags |= Style_Sunken;
			} else
				sflags |= Style_Raised;
			if (sw->buttonSymbols() == QSpinWidget::PlusMinus) {
				pe = PE_SpinWidgetPlus;
				upRect.moveTop(upRect.y()+1);
			}
			else
				pe = PE_SpinWidgetUp;
			drawPrimitive(pe, p, upRect, cg, sflags);

			sflags = Style_Default | Style_Enabled;
			if (active == SC_SpinWidgetDown) {
				sflags |= Style_On;
				sflags |= Style_Sunken;
			} else
				sflags |= Style_Raised;
			if (sw->buttonSymbols() == QSpinWidget::PlusMinus)
				pe = PE_SpinWidgetMinus;
			else
				pe = PE_SpinWidgetDown;
			drawPrimitive(pe, p, downRect, cg, sflags);
			
			break;
		}

		case CC_TitleBar: {
			QRect ir;
			bool down;
			QColorGroup g = widget->colorGroup();
			
			if ( controls & SC_TitleBarLabel ) {
				QPixmap pix(10, r.height());
				pix.fill(g.background());
				
				QPainter painter(&pix);
				renderGradient(&painter, QRect(0, 0, 10, r.height()/2), g.background().light(120).rgb(), g.background().rgb(), true);
				painter.setPen(g.background().dark(105));
				painter.drawLine(0, r.bottom(), 10, r.bottom());
				p->drawTiledPixmap(0,0, r.width(), r.height(), pix);
				
				ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarLabel ), widget );
				p->setPen( g.foreground() );
				p->drawText(ir.x()+2, ir.y(), ir.width()-2, ir.height(), AlignCenter | AlignVCenter | SingleLine, widget->caption() );
			}
			if ( controls & SC_TitleBarCloseButton ) {
				ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarCloseButton ), widget );
				down = active & SC_TitleBarCloseButton;
				p->drawPixmap(ir.x(), ir.y(), uic_findImage("DockCloseButton"), 0, down ? 12 : 0, 12, 12);
			}
			if ( controls & SC_TitleBarMaxButton ) {
				ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarMaxButton ), widget );
				down = active & SC_TitleBarMaxButton;
				p->drawPixmap(ir.x(), ir.y(), uic_findImage("OverlapButton"), 0, down ? 12 : 0, 12, 12);
			}
			if ( controls & SC_TitleBarNormalButton) {
				ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarNormalButton ), widget );
				down = active & SC_TitleBarNormalButton;
				p->drawPixmap(ir.x(), ir.y(), uic_findImage("ToDesktopButton"), 0, down ? 12 : 0, 12, 12);
			}
			if ( controls & SC_TitleBarMinButton) {
				ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarMinButton ), widget );
				down = active & SC_TitleBarMinButton;
				p->drawPixmap(ir.x(), ir.y(), uic_findImage("DockbackButton"), 0, down ? 12 : 0, 12, 12);
			}
			if ( controls & SC_TitleBarSysMenu ) {
				if ( widget->icon() ) {
					ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarSysMenu ), widget );
					drawItem( p, ir, AlignCenter, widget->colorGroup(), TRUE, widget->icon(), QString::null );
				}
			}
			
			break;
		}
		default:
			KStyle::drawComplexControl(control, p, widget,
										r, cg, flags, controls,
										active, opt);
			break;
	}
}

QRect DominoStyle::subRect(SubRect r, const QWidget *widget) const
{
	switch (r) {
		case SR_PushButtonContents: {
			const QPushButton* btn = static_cast<const QPushButton*>(widget);
			QRect rw = widget->rect();
			if(btn->text().isEmpty())
				return QRect(rw.left()+2, rw.top()+2, rw.width()-4, rw.height()-4);
			return QRect(rw.left()+2, rw.top()+3, rw.width()-4, rw.height()-7);
		}
		case SR_PushButtonFocusRect:
		{
			QRect rect, wrect(widget->rect());
			const QPushButton *button = (const QPushButton *) widget;
			int dbw1 = 0, dbw2 = 0;
			if (button->isDefault() || button->autoDefault()) {
				dbw1 = pixelMetric(PM_ButtonDefaultIndicator, widget);
				dbw2 = dbw1 * 2;
			}

			int dfw1 = pixelMetric(PM_DefaultFrameWidth, widget) * 2,
			dfw2 = dfw1 * 2;

			rect.setRect(wrect.x()      + dfw1 + dbw1,
			wrect.y()      + dfw1 + dbw1,
			wrect.width()  - dfw2 - dbw2,
			wrect.height() - dfw2 - dbw2);
			return rect;
		}
		case SR_ComboBoxFocusRect: {
			return QRect(4, 3, widget->width()-8, widget->height()-7);
// 			return querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxEditField );
		}
		case SR_CheckBoxFocusRect:
		{
			QRect cr = subRect(SR_CheckBoxContents, widget);
	    		QRect fr = KStyle::subRect(r, widget);
			if(cr.width() <= 0) // no text
				fr.setHeight(fr.height()+2);
			else
				fr.addCoords(+2, 0, -2, 0);
	    		return fr;
		}
		case SR_RadioButtonFocusRect:
		{
			QRect cr = subRect(SR_CheckBoxContents, widget);
			QRect fr = KStyle::subRect(r, widget);
			if(cr.width() <= 0) // no text
				fr.setHeight(fr.height()+2);
			else
				fr.addCoords(+2, 0, -2, 0);
			return fr;
		}
		case SR_ProgressBarGroove: {
			return QRect(widget->rect());
		}
		case SR_ProgressBarContents:
		case SR_ProgressBarLabel: {
			QRect rw = widget->rect();
			return QRect(rw.left()+2, rw.top()+2, rw.width()-4, rw.height()-4 );
		}
		case SR_ToolBoxTabContents: {
			QRect rw = widget->rect();
			return QRect(rw.left()+18, rw.top(), rw.width()-18, rw.height());
		}
		default: {
			return KStyle::subRect(r, widget);
		}
	}
}

QRect DominoStyle::querySubControlMetrics(ComplexControl control,
										  const QWidget *widget,
										  SubControl subcontrol,
										  const QStyleOption &opt) const
{
	if (!widget) {
		return QRect();
	}

	QRect r(widget->rect());
	switch (control) {
		case CC_ComboBox: {
			switch (subcontrol) {
				case SC_ComboBoxEditField: {
					const QComboBox* cb = dynamic_cast<const QComboBox *>(widget);
					if(!cb) // maybe kdevelop's KComboView
						return QRect(r.left()+4, r.top()+3, r.width()-4-15-3, r.height()-7);
					if(!cb->editable())
						return QRect(r.left()+3, r.top()+3, r.width()-4-15-2, r.height()-6);
					
					return QRect(r.left()+3, r.top()+3, r.width()-4-15-2, r.height()-7);
				}
				default: {
					return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
				}
			}
			break;
		}
		case CC_SpinWidget: {
			const int fw = 2; // Frame width...
			const int buttonsWidth = 15;
			const bool heightDividable = ((r.height()%2) == 0);
			
			const int buttonsLeft = r.right()-buttonsWidth+1-fw;
			switch (subcontrol) {
				case SC_SpinWidgetUp: { // rect without 2px frame
					return QRect(buttonsLeft, r.top()+3, buttonsWidth+1, r.height()/2-3 );
				}
				case SC_SpinWidgetDown: {
					if(heightDividable) {
						return QRect(buttonsLeft, r.top()+r.height()/2+1, buttonsWidth+1, r.height()/2-3);
					} else {
						return QRect(buttonsLeft, r.top()+r.height()/2+1, buttonsWidth+1, r.height()/2-2);
					}
				}
				case SC_SpinWidgetFrame: {
					return QRect(r.left(), r.top(), r.width(), r.height() );
				}
				case SC_SpinWidgetEditField: {
					return QRect(r.left()+1+fw, r.top()+1+fw, r.width()-(buttonsWidth+2+fw*2), r.height()-3-fw*2);
				}
				case SC_SpinWidgetButtonField: {
					return QRect(buttonsLeft, r.top()+2, buttonsWidth, r.height()-5);
				}
				default: {
					return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
				}
			}
			break;
		}

		case CC_ScrollBar: {
			
			const QScrollBar *scrollbar = (const QScrollBar *) widget;
			int sliderstart = scrollbar->sliderStart();
			const int buttonw = 15 /*- (fw * 2)*/;
			const int buttonh = 15 /*- (fw * 2)*/;

			int maxlen = ((scrollbar->orientation() == Qt::Horizontal) ?
					scrollbar->width() : scrollbar->height()) - buttonh * 3;

			// calculate slider length
			if (scrollbar->maxValue() != scrollbar->minValue()) {
				uint range = scrollbar->maxValue() - scrollbar->minValue();
				sliderlen = (scrollbar->pageStep() * maxlen) /
						(range + scrollbar->pageStep());

				if ( sliderlen < 32 || range > INT_MAX/2 )
					sliderlen = 32;
				if ( sliderlen > maxlen )
					sliderlen = maxlen;
			} else
				sliderlen = maxlen;
			switch(subcontrol) {
				case SC_ScrollBarAddLine: { // bottom/right button
					if (scrollbar->orientation() == Qt::Horizontal) {
						return QRect(scrollbar->width() - buttonw , 0, buttonw, buttonh);
					} else {
						return QRect(0, scrollbar->height() - buttonh, buttonw, buttonh);
					}
				}
				case SC_ScrollBarSubLine: { // top/left button
					if (scrollbar->orientation() == Qt::Horizontal) {
						return QRect(0, 0, buttonw, buttonh);
					} else {
						return QRect(0, 0, buttonw, buttonh);
					}
				}
				case SC_ScrollBarSubPage: // top/left
					if (scrollbar->orientation() == Qt::Horizontal)
						return QRect(buttonw, 0, sliderstart-buttonw+1, buttonw);
					return QRect(0, buttonh, buttonw, sliderstart-buttonw+1);

				case SC_ScrollBarAddPage: // bottom/left
					if (scrollbar->orientation() == Qt::Horizontal)
						return QRect(sliderstart + sliderlen-1, 0, maxlen - sliderstart - sliderlen + buttonw+1, buttonw);
					return QRect(0, sliderstart + sliderlen-1, buttonw, maxlen - sliderstart - sliderlen + buttonw+1);

				case SC_ScrollBarGroove:
					if (scrollbar->orientation() == Qt::Horizontal)
						return QRect(buttonw , 0, maxlen, buttonw);
					return QRect(0, buttonw, buttonw, maxlen);

				case SC_ScrollBarSlider:
					if (scrollbar->orientation() == Qt::Horizontal)
						return QRect(sliderstart, 0, sliderlen, buttonw);
					return QRect(0, sliderstart, buttonw, sliderlen);
				default:
					break;
			}
		}
		default: {
			return KStyle::querySubControlMetrics(control, widget, subcontrol, opt);
		}
	}
}

int DominoStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
	//return KStyle::pixelMetric(m, widget);
	
	switch(m) {
	// TABS
	// ----
		case PM_TabBarTabVSpace: {
			const QTabBar * tb = (const QTabBar *) widget;
			return (tb->shape() == QTabBar::RoundedAbove || tb->shape() == QTabBar::RoundedBelow)? 12 : 4;
		}
		case PM_TabBarTabShiftVertical: {
			
			const QTabBar * tb = (const QTabBar *) widget;
			switch (tb->shape()) {
				case QTabBar::RoundedAbove:
					return 1;
				case QTabBar::RoundedBelow:
					return (tb->parent() && tb->parent()->inherits("KonqFrameTabs"))? 0 : -1;
				default:
					return 0;
			}
		}
		case PM_TabBarTabShiftHorizontal: {
			return 0;
		}
		case PM_TabBarTabOverlap: {
			return 1;
		}
		case PM_TabBarBaseOverlap:
			return widget->isA("KonqFrameTabs")? 0 : -3;

#if (QT_VERSION >= 0x030300) // requires Qt 3.3
	// extra space between menubar items
		case PM_MenuBarItemSpacing: {
			return 6;
		}
#endif

// #if (QT_VERSION >= 0x030300) // requires Qt 3.3
//	   // extra space between toolbar items
//		   case PM_ToolBarItemSpacing: {
//			   return 4;
//		   }
// #endif

	// SCROLL BAR
// 		case PM_ScrollBarSliderMin: {
// 			return 21;
// 		}

		case PM_ScrollBarExtent: {
			return 15;
		}

		case PM_DockWindowSeparatorExtent:
			return 6;

	// SPLITTERS
	// ---------
		case PM_SplitterWidth: {
			return 6;
		}

	// PROGRESSBARS
	// ------------
		case PM_ProgressBarChunkWidth:
			return 10;

	// SLIDER
	// ------
		case PM_SliderLength:
		case PM_SliderThickness:
			return 15;

	// MENU INDICATOR
	// --------------
		case PM_MenuButtonIndicator:
			return 8;

	// CHECKBOXES / RADIO BUTTONS
	// --------------------------
		case PM_ExclusiveIndicatorWidth:	// Radiobutton 17x17
		case PM_ExclusiveIndicatorHeight:
			return 17;
		case PM_IndicatorWidth:		// Checkbox 18x19
			return 18;
		case PM_IndicatorHeight:
			return 19;

	// FRAMES
	// ------
		case PM_SpinBoxFrameWidth:
			return 2;
			
		case PM_MenuBarFrameWidth:
			return 1;

		case PM_DefaultFrameWidth: {
			
			if(widget) {
				if(dynamic_cast<const QPopupMenu*>(widget))
					return 3;
				
				// stolen from baghira
				if (widget->parentWidget() && dynamic_cast<QTabWidget*>(widget->parentWidget()) && widget->parentWidget()->parentWidget() && (widget->parentWidget()->parentWidget()->inherits("KonqMainWindow") || widget->parentWidget()->parentWidget()->inherits("KonqFrameContainer")))
					return 0;
				
			}
			return 2;
		}
		case PM_ButtonDefaultIndicator: {
			return 0;
		}

		case PM_ButtonMargin: {
			//return 2;
			return 4;
		}
		case PM_HeaderMargin:
			return 2;
		case PM_ButtonShiftVertical:
		case PM_ButtonShiftHorizontal: {
			return 0;
		}
 		case PM_CheckListControllerSize:
		case PM_CheckListButtonSize: {
			return 16;
		}
// 		case PM_PopupMenuFrameVerticalExtra:
// 		case PM_PopupMenuFrameHorizontalExtra:
// 			return 1;
		default:
			return KStyle::pixelMetric(m, widget);
	}
}


QSize DominoStyle::sizeFromContents(ContentsType t,
									const QWidget *widget,
									const QSize &s,
									const QStyleOption &opt) const
{
	//return KStyle::sizeFromContents (t, widget, s, opt);
	
	switch (t) {
		case CT_PopupMenuItem: {
			if (!widget || opt.isDefault())
				return s;
// 			static const int itemHMargin = 6;
// 			static const int itemFrame = 2;
// 			static const int arrowHMargin = 6;
// 			static const int rightBorder = 12;
			
			const QPopupMenu *popup = dynamic_cast<const QPopupMenu *>(widget);
			QMenuItem *mi = opt.menuItem();
			int maxpmw = opt.maxIconWidth();
			int w = s.width(), h = s.height();
			bool checkable = popup->isCheckable();
			
			if (mi->custom()) {
				w = mi->custom()->sizeHint().width();
				h = mi->custom()->sizeHint().height();
				if (!mi->custom()->fullSpan() )
					h += 4;
			} else if (mi->widget()) {
			// don't change the size in this case.
			} else if (mi->isSeparator()) {
				w = 20;
				h = 5;
			} else {
				if (mi->pixmap()) {
					h = QMAX(h, mi->pixmap()->height() + 2);
				} else {
					h = QMAX(h, 16 + 2 );
					h = QMAX(h, popup->fontMetrics().height() + 4 );
				}

				if (mi->iconSet()) {
					h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal).height() + 2);
				}
			}

			if (!mi->text().isNull() && (mi->text().find('\t') >= 0)) {
				w += itemHMargin + itemFrame*2 + 7;
			} else if (mi->popup()) {
				w += 2 * arrowHMargin;
			}

			if (maxpmw) {
				w += maxpmw + 6;
			}
			if (checkable && maxpmw < 20) {
				w += 20 - maxpmw;
			}
			if (checkable || maxpmw > 0) {
				w += 12;
			}
			
			return QSize(w, h);
		}

		case CT_PushButton:
		{
			const QPushButton* btn = static_cast<const QPushButton*>(widget);

			int w = s.width();
			int h = s.height();
			if ( btn->text().isEmpty() && s.width() < 32 ) {
				w += pixelMetric(PM_ButtonMargin, widget)*2;
				h += pixelMetric(PM_ButtonMargin, widget)*2;
				return QSize(w, h);
			}
			
			int bm = pixelMetric(PM_ButtonMargin, widget);
			int fw = pixelMetric(PM_DefaultFrameWidth, widget) * 2;
			w += bm + fw + 22;
			h += bm + fw;
			return QSize(w, h < 26 ? 26+_buttonHeightAdjustment : h+4 );
		}
		
		case CT_ToolButton: {
			if(dynamic_cast<const QToolButton*>(widget)) {
				if(dynamic_cast<QTabWidget*>(widget->parentWidget())) // smaller cornerWidget
					return QSize( s.width()+9, s.height()+7 );
				return QSize( s.width()+11, s.height()+7 );
			}
			return KStyle::sizeFromContents (t, widget, s, opt); // KMultiTabBar buttons
		}
		
		case CT_TabBarTab: {
			// konqi = tabbar
			// other = tab control
			// this is total crap but it helps to reduce the flicker in konqi.
			/////////////////////
			QTabBar* tb = 0;
			if(::qt_cast<QTabBar*>(widget))
				tb = ::qt_cast<QTabBar*>(widget);
			else
				return QSize(s.width(), s.height());
				
			if(!strcmp(widget->name(), "tabbar") && widget->parentWidget()->isA("KonqFrameTabs")) {
				
				QTab* tab = opt.tab();
				int count = tb->count();

				int fm = tb->fontMetrics().width("about:blank");
				if(tb->parentWidget()->width() <= opt.tab()->rect().x()+44+fm)
					return QSize(s.width(), s.height());

				int dots = tb->fontMetrics().width(".");
				if((dots*minTabLength+44)*count >= tb->width())  //  44 = frame+iconwidth
					return QSize(s.width(), s.height());

				QSize sz = KStyle::sizeFromContents( t, widget, s, opt );
	
				QString str;
				str.fill(' ', minTabLength);
				if( tab->text().isEmpty() || tab->text() == str) {
					tab->setText("about:blank");
					//return QSize(  fm+44, 26 );
					return QSize(  fm+44, s.height());
				}
				return QSize(s.width(), s.height());
			}
			else {
				switch (tb->shape()) {
					case QTabBar::RoundedAbove:
					case QTabBar::RoundedBelow:
						return QSize( s.width(), s.height()-2 );
					default:
						return QSize( s.width(), s.height() ); // triangular
				}
			}
		}

		case CT_ComboBox: {
			int bm = pixelMetric(PM_ButtonMargin, widget);
			int fw = pixelMetric(PM_DefaultFrameWidth, widget) * 2;
			int w = s.width();
			int h = s.height();
			w += bm + fw + 22;
			h += bm + fw-3;
			return QSize(w, h < 26 ? 26+_buttonHeightAdjustment : h+4 );
		}
		
		case CT_SpinBox: {
			QSize sz = KStyle::sizeFromContents( t, widget, s, opt );
			int h = sz.height()%2 != 0 ? sz.height()+1 : sz.height();
			return ( QSize( sz.width(), h < 24 ? 24 : h) );
		}
		
		case CT_Header: {
			QSize sz = KStyle::sizeFromContents( t, widget, s, opt );
			return QSize(sz.width(), sz.height()-2);
		}
		
		default:
			return KStyle::sizeFromContents (t, widget, s, opt);
	}

	return KStyle::sizeFromContents (t, widget, s, opt);
}

int DominoStyle::styleHint( StyleHint stylehint,
								   const QWidget *widget,
								   const QStyleOption &option,
								   QStyleHintReturn* returnData ) const
{
	switch (stylehint) {
		case SH_PopupMenu_SubMenuPopupDelay:
			//return 96; // Motif-like delay...
			return 250;
		case SH_GroupBox_TextLabelVerticalAlignment: {
			return Qt::AlignTop;
		}
		case SH_ScrollBar_BackgroundMode: {
			return Qt::PaletteBackground;
		}
		case SH_TabBar_Alignment: {
			return _centerTabs ? AlignHCenter : AlignLeft;
		}
		default:
			return KStyle::styleHint(stylehint, widget, option, returnData);
	}
}

QPixmap DominoStyle::stylePixmap(StylePixmap stylepixmap, const QWidget* widget, const QStyleOption& opt) const {
	return KStyle::stylePixmap(stylepixmap, widget, opt);
}

QColor DominoStyle::getColor(const DSurface &ds, const SurfaceBorder border)const
{
	QColor color;
	switch (ds.numGradients) {
		case 2: {
			switch(border) {
				case TopBorder: // topColor
					color = (ds.g1Top == 0)? ds.g1Color1 : ds.background;
					break;
				case BottomBorder: // bottomColor
					color = (ds.g2Bottom == 100)? ds.g2Color2 : ds.background;
				default:
					break;
			}
			break;
		}
		case 1: {
			switch(border) {
				case TopBorder:
					color = (ds.g1Top == 0)? ds.g1Color1 : ds.background;
					break;
				case BottomBorder:
					color = (ds.g1Bottom == 100)? ds.g1Color2 : ds.background;
				default:
					break;
			}
			break;
		}
		default: {
			color = ds.background;
		}
	}
	return color;
}


QColor DominoStyle::lightenColor(const QColor &color, const int value) const
{
	int r, g, b;
	color.getRgb(&r, &g, &b);
	r += value;
	if(r > 255)
		r =255;
	g += value;
	if(g > 255)
		g =255;
	b += value;
	if(b > 255)
		b =255;
	return QColor(r, g, b);
}

QColor DominoStyle::darkenColor(const QColor &color, const int value) const
{
	int r, g, b;
	color.getRgb(&r, &g, &b);
	r -= value;
	if(r < 0)
		r = 0;
	g -= value;
	if(g < 0)
		g = 0;
	b -= value;
	if(b < 0)
		b = 0;
	return QColor(r, g, b);
}

QColor DominoStyle::dGetColor(const QColor &color, const int value) const
{
	int h, s, v;
	color.getHsv(&h, &s, &v);
	return (v<127)? lightenColor(color, value): darkenColor(color, value);
}

QColor DominoStyle::getColor(const QColorGroup &cg, const ColorType t, const bool enabled)const
{
	return getColor(cg, t, enabled?IsEnabled:IsDisabled);
}

QColor DominoStyle::getColor(const QColorGroup &cg, const ColorType t, const WidgetState s)const
{
	const bool enabled = (s != IsDisabled) &&
			((s == IsEnabled) || (s == IsPressed) || (s == IsHighlighted));
	const bool pressed = (s == IsPressed);
	const bool highlighted = (s == IsHighlighted);
	switch(t) {
		case ButtonContour:
			return enabled ? cg.button().dark(130+_contrast*8)
						   : cg.background().dark(120+_contrast*8);
		case DragButtonContour: {
			if(enabled) {
				if(pressed)
					return cg.button().dark(130+_contrast*6); // bright
				else if(highlighted)
					return cg.button().dark(130+_contrast*9); // dark
				else
					return cg.button().dark(130+_contrast*8); // normal
			} else {
				return cg.background().dark(120+_contrast*8);
			}
		}
		case DragButtonSurface: {
			if(enabled) {
				if(pressed)
					return cg.button().dark(100-_contrast);	 // bright
				else if(highlighted)
					return cg.button().light(100+_contrast); // dark
				else
					return cg.button();						 // normal
			} else {
				return cg.background();
			}
		}
		case PanelContour:
			return cg.background().dark(160+_contrast*8);
		case PanelDark:
			return alphaBlendColors(cg.background(), cg.background().dark(120+_contrast*5), 110);
		case PanelDark2:
			return alphaBlendColors(cg.background(), cg.background().dark(110+_contrast*5), 110);
		case PanelLight:
			return alphaBlendColors(cg.background(), cg.background().light(120+_contrast*5), 110);
		case PanelLight2:
			return alphaBlendColors(cg.background(), cg.background().light(110+_contrast*5), 110);
		case CheckMark:
			if( _customCheckMarkColor )
				return _checkMarkColor;
			else
				return cg.foreground();
		default:
			return cg.background();
	}
}

void DominoStyle::renderScrollBarCaps(const QColor &capsColor) {
	
	// for the contour we need to blend the top and bottom color with a darker caps color
	QColor sbTopContour = alphaBlendColors(QColor(41,41,47), alphaBlendColors(capsColor, getColor(scrollBarSurface, TopBorder), 127), 100);
	QColor sbBottomContour = alphaBlendColors(QColor(41,41,47), alphaBlendColors(capsColor, getColor(scrollBarSurface, BottomBorder), 127), 100);
	
	
	const QColor background = black; // unused highlight color
	QRect r;
	QPainter p;
	
	// top
	QRegion stop1(5, 3, 5, 1);
	QRegion stop2(3, 4, 9, 1);
	QRegion stop3(2, 5, 11, 2);
	QRegion stop4(1, 7, 13, 2);

	scrollBarCap_top = new QPixmap(uic_findImage("roundScrollSlider_top_mask"));
	r = scrollBarCap_top->rect();
	p.begin(scrollBarCap_top);
	p.setClipRegion(stop1^stop2^stop3^stop4);
	uint surfaceFlags = Is_ScrollBarBtn;
	renderSurface(&p, QRect(r.x()+1, r.y(), r.width()-2, r.height()), capsColor.rgb(), capsColor.rgb(), background.rgb(), surfaceFlags);
	p.setClipping(false);
	p.drawPixmap(0,0, uic_findImage("roundScrollSlider_top"));
	p.setPen(sbTopContour);
	p.drawLine(1,7,1,8);
	p.setPen(sbBottomContour);
	p.drawLine(13,7,13,8);
	p.end();


	
	// bottom
	QRegion sbottom1(1, 0, 13, 2);
	QRegion sbottom2(2, 2, 11, 2);
	QRegion sbottom3(3, 4, 9, 1);
	QRegion sbottom4(5,5,5,1);
	
	scrollBarCap_bottom = new QPixmap(uic_findImage("roundScrollSlider_bottom_mask"));
	r = scrollBarCap_bottom->rect();
	p.begin(scrollBarCap_bottom);
	p.setClipRegion(sbottom1^sbottom2^sbottom3^sbottom4);
	renderSurface(&p, QRect(r.x()+1, r.y(), r.width()-2, r.height()), capsColor.rgb(), capsColor.rgb(), background.rgb(), surfaceFlags);
	p.setClipping(false);
	p.drawPixmap(0,0, uic_findImage("roundScrollSlider_bottom"));
	p.setPen(sbTopContour);
	p.drawLine(1,0,1,1);
	p.setPen(sbBottomContour);
	p.drawLine(13,0,13,1);
	p.end();


	
	surfaceFlags |= Is_Horizontal;

	// left
	QRegion sleft4(3,5,1,5);
	QRegion sleft3(4,3,1,9);
	QRegion sleft2(5, 2, 2, 11);
	QRegion sleft1(7, 1, 2, 13);
	
	scrollBarCap_left = new QPixmap(uic_findImage("roundScrollSlider_left_mask"));
	r = scrollBarCap_left->rect();
	p.begin(scrollBarCap_left);
	p.setClipRegion(sleft1^sleft2^sleft3^sleft4);
	renderSurface(&p, QRect(r.x(), r.y()+1, r.width(), r.height()-2), capsColor.rgb(), capsColor.rgb(), capsColor.rgb(), surfaceFlags);
	p.setClipping(false);
	p.drawPixmap(0,0, uic_findImage("roundScrollSlider_left"));
	p.setPen(sbTopContour);
	p.drawLine(7,1,8,1);
	p.setPen(sbBottomContour);
	p.drawLine(7,13,8,13);
	p.end();


	
	// right
	QRegion sright1(0, 1, 2, 13);
	QRegion sright2(2, 2, 2, 11);
	QRegion sright3(4, 3, 1, 9);
	QRegion sright4(5,5,1,5);
	
	scrollBarCap_right = new QPixmap(uic_findImage("roundScrollSlider_right_mask"));
	r = scrollBarCap_right->rect();
	p.begin(scrollBarCap_right);
	p.setClipRegion(sright1^sright2^sright3^sright4);
	renderSurface(&p, QRect(r.x(), r.y()+1, r.width(), r.height()-2), capsColor.rgb(), capsColor.rgb(), capsColor.rgb(), surfaceFlags);
	p.setClipping(false);
	p.drawPixmap(0,0, uic_findImage("roundScrollSlider_right"));
	p.setPen(sbTopContour);
	p.drawLine(0,1,1,1);
	p.setPen(sbBottomContour);
	p.drawLine(0,13,1,13);
	p.end();

}

void DominoStyle::delayedGroupBoxUpdate() {
	QEvent e(QEvent::PaletteChange);
	QApplication::sendEvent(designerGroupBox, &e);
}

QPixmap* DominoStyle::renderLineEditShadow(QWidget* widget, QRect r, QColor bg, uint flags, bool /*reverse*/) {
	
	bool drawLeft = flags& Draw_Left;
	bool drawRight = flags& Draw_Right;
	
	int x, y, w, h;
	r.rect(&x, &y, &w, &h);
	if(!drawLeft) {
		x = -3;
		w += 3;
	}
	else if(!drawRight)
		w += 3;
	QRect rect(x, y, w, h);
	
	
	QColor tintColor;
	if(!widget->isEnabled())
		tintColor =bg;
	else
		tintColor = alphaBlendColors(Qt::black, bg, 30+35);
	
	QPixmap* prBgShadow;
	if((prBgShadow = QPixmapCache::find(bg.name()+qApp->palette().active().background().name()+"leBgShadow"))) {
	}
	else {
		prBgShadow = new QPixmap(4,4);
		prBgShadow->fill(bg);
		QPixmap pix = tintImage(uic_findImage("progressShadow2"), tintColor);
		bitBlt(prBgShadow, 0, 0, &pix, 0, 0, -1, -1, Qt::CopyROP);
		QPixmapCache::insert(bg.name()+qApp->palette().active().background().name()+"leBgShadow", prBgShadow);
	}
	
	
	QColor color_d = alphaBlendColors(tintColor, bg, 170);
	QColor color_m = alphaBlendColors(tintColor, bg, 85);
	QColor color_l = alphaBlendColors(tintColor, bg, 25);
	
	QPixmap* pix = new QPixmap(rect.size());
	pix->fill(bg);
	QPainter p(pix);
	
	p.setPen(color_d);
	p.drawLine(rect.x()+2, rect.y(), rect.right()-2, rect.y());
	p.setPen(color_m);
	p.drawLine(rect.x()+2, rect.y()+1, rect.right()-2, rect.y()+1);
	p.setPen(color_l);
	p.drawLine(rect.x()+2, rect.y()+2, rect.right()-2, rect.y()+2);
	p.drawLine(rect.x()+2, rect.bottom(), rect.right()-2, rect.bottom());

	//p.setPen(color_l);
	p.drawLine(rect.x()+1, rect.y()+3, rect.x()+1, rect.bottom()-1); // l
	p.drawLine(rect.right()-1, rect.y()+3, rect.right()-1, rect.bottom()-1); // r
			
	p.setPen(color_m);
	p.drawLine(rect.x(), rect.y()+3, rect.x(), rect.bottom()-1); // l
	p.drawLine(rect.right(), rect.y()+3, rect.right(), rect.bottom()-1); // r
	
	bitBlt(pix, rect.right()-1, rect.y(), prBgShadow, 2, 0, 2, 3, Qt::CopyROP);
	bitBlt(pix, rect.x(), rect.y(), prBgShadow, 0, 0, 2, 3, Qt::CopyROP);
	bitBlt(pix, rect.right()-1, rect.bottom(), prBgShadow, 2, 3, 2, 1, Qt::CopyROP);
	bitBlt(pix, rect.x(), rect.bottom(), prBgShadow, 0, 3, 2, 1, Qt::CopyROP);
	
	p.end();
	return pix;
}



#include "eventfilter.cpp"
#include "domino.moc"
